SystemOrganization addCategory: #'KernelTests-Chronology'!
SystemOrganization addCategory: #'KernelTests-Classes'!
SystemOrganization addCategory: #'KernelTests-Exception'!
SystemOrganization addCategory: #'KernelTests-Methods'!
SystemOrganization addCategory: #'KernelTests-Numbers'!
SystemOrganization addCategory: #'KernelTests-Objects'!
SystemOrganization addCategory: #'KernelTests-Processes'!

ClassTestCase subclass: #BooleanTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!BooleanTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Boolean. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category
!

!BooleanTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:02'!
testBooleanInitializedInstance

	self assert: (Boolean initializedInstance = nil).! !

!BooleanTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:02'!
testBooleanNew

	self should: [Boolean new] raise: TestResult error. 
	self should: [True new] raise: TestResult error. 
	self should: [False new] raise: TestResult error. ! !

!BooleanTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:02'!
testNew

	self should: [Boolean new] raise: TestResult error. ! !

ClassTestCase subclass: #ClassDescriptionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!ClassDescriptionTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class ClassDescription. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ClassDescriptionTest methodsFor: 'tests' stamp: 'sd 5/10/2008 12:34'!
testMethods
	self assert: Object methods = Object methodDict values.  ! !

!ClassDescriptionTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:24'!
testOrganization

	| aClassOrganizer |
	aClassOrganizer := ClassDescription organization.
	self assert: (aClassOrganizer isKindOf: ClassOrganizer).! !

ClassTestCase subclass: #CompiledMethodTest
	instanceVariableNames: 'x y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!CompiledMethodTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class CompiledMethod. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!CompiledMethodTest methodsFor: 'examples' stamp: 'sd 4/6/2009 21:27'!
readX
	| tmp |
	tmp := x.
	^ tmp! !

!CompiledMethodTest methodsFor: 'examples' stamp: 'sd 4/6/2009 21:23'!
readXandY
	
	^ x + y
	! !

!CompiledMethodTest methodsFor: 'examples' stamp: 'md 2/18/2006 20:09'!
returnPlusOne: anInteger
	^anInteger + 1. ! !

!CompiledMethodTest methodsFor: 'examples' stamp: 'md 2/18/2006 20:09'!
returnTrue
	^true  ! !

!CompiledMethodTest methodsFor: 'tests - conversion' stamp: 'StephaneDucasse 4/11/2010 23:42'!
testCompiledMethodAsString
     "self debug: #testCompiledMethodAsString" 

	self shouldnt: [CompiledMethod allInstances first  asString] raise: Error! !

!CompiledMethodTest methodsFor: 'tests - decompiling' stamp: 'md 2/16/2006 20:29'!
testDecompile
	"self debug: #testDecompileTree"
	| method  cls stream |

	Smalltalk removeClassNamed: #TUTU.

	cls := Object subclass: #TUTU
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.
	stream := ReadWriteStream on: String new.
	method decompile printOn: stream.
	self assert: stream contents = 'foo
	^ 10'
	
	
	! !

!CompiledMethodTest methodsFor: 'tests - instance variable' stamp: 'sd 4/6/2009 21:30'!
testHasInstVarRef
	"self debug: #testHasInstVarRef"
	
	| method  |
	method := self class compiledMethodAt: #readX.
	self assert: (method hasInstVarRef).

	method := self class compiledMethodAt: #readXandY.
	self assert: (method hasInstVarRef).
	
	method := self class compiledMethodAt: #writeX.
	self assert: (method hasInstVarRef).
	
	method := self class compiledMethodAt: #writeXandY.
	self assert: (method hasInstVarRef).
	! !

!CompiledMethodTest methodsFor: 'tests - testing' stamp: 'md 2/19/2006 11:28'!
testHasNewPropertyFormat
		| method |
		method := (self class)>>#returnTrue.
		self assert: method hasNewPropertyFormat.
! !

!CompiledMethodTest methodsFor: 'tests - testing' stamp: 'md 2/18/2006 20:10'!
testIsInstalled
|  method cls |

	method := (self class)>>#returnTrue.
	self assert: method isInstalled.

	"now make an orphaned method by just deleting the class."

	Smalltalk removeClassNamed: #TUTU.

	cls := Object subclass: #TUTU
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.

	self deny: method isInstalled. ! !

!CompiledMethodTest methodsFor: 'tests - testing' stamp: 'md 4/16/2003 15:32'!
testIsQuick
	| method  |

	method := self class compiledMethodAt: #returnTrue.
	self assert: (method isQuick).

	method := self class compiledMethodAt: #returnPlusOne:.
	self deny: (method isQuick).

	! !

!CompiledMethodTest methodsFor: 'tests - accessing' stamp: 'md 2/18/2006 20:10'!
testMethodClass
	| method cls |
	method := self class >> #returnTrue.
	self assert: method selector = #returnTrue.
	"now make an orphaned method by just deleting the class.
	old: #unknown
	new semantics: return Absolete class"
	Smalltalk removeClassNamed: #TUTU.
	cls := Object
				subclass: #TUTU
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.
	self assert: method methodClass = cls! !

!CompiledMethodTest methodsFor: 'tests - instance variable' stamp: 'sd 4/6/2009 21:46'!
testReadsField
	"self debug: #testReadsField"
	
	| method |
	method := self class compiledMethodAt: #readX.
	self assert: (method readsField: 2).
	
	method := self class compiledMethodAt: #readXandY.
	self assert: (method readsField: 3).
	
	
	"read is not write"
	method := self class compiledMethodAt: #writeX.
	self deny: (method readsField: 2).
	
	method := self class compiledMethodAt: #writeXandY.
	self deny: (method readsField: 2).
	
	method := self class compiledMethodAt: #writeXandY.
	self deny: (method readsField: 3).! !

!CompiledMethodTest methodsFor: 'tests - accessing' stamp: 'md 2/15/2006 20:54'!
testSearchForClass
	|  method cls |

	method := (self class)>>#returnTrue.
	self assert: (method searchForClass = self class).
	
	"now make an orphaned method. we want to get nil as the class"	
	
	Smalltalk removeClassNamed: #TUTU.

	cls := Object subclass: #TUTU
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.
	
	self assert: method searchForClass = nil. 
! !

!CompiledMethodTest methodsFor: 'tests - accessing' stamp: 'md 2/15/2006 20:55'!
testSearchForSelector
	|  method cls |

	method := (self class)>>#returnTrue.
	self assert: (method searchForSelector = #returnTrue).

	"now make an orphaned method. we want to get nil as the selector"	
	
	Smalltalk removeClassNamed: #TUTU.

	cls := Object subclass: #TUTU
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.
	
	self assert: method searchForSelector = nil. 
! !

!CompiledMethodTest methodsFor: 'tests - accessing' stamp: 'md 2/16/2006 20:28'!
testSelector
	|  method cls |

	method := (self class)>>#returnTrue.
	self assert: (method selector = #returnTrue).

	"now make an orphaned method. new semantics: return corrent name"	
	
	Smalltalk removeClassNamed: #TUTU.

	cls := Object subclass: #TUTU
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Methods'.
	cls compile: 'foo ^ 10'.
	method := cls >> #foo.
	Smalltalk removeClassNamed: #TUTU.

	self assert: method selector = #foo. 
! !

!CompiledMethodTest methodsFor: 'tests - evaluating' stamp: 'md 4/16/2003 15:30'!
testValueWithReceiverArguments
	
	| method value |

	method := self class compiledMethodAt: #returnTrue.

	value := method valueWithReceiver: nil arguments: #().
	self assert: (value = true).

	method := self class compiledMethodAt: #returnPlusOne:.
	value := method valueWithReceiver: nil arguments: #(1).
	self assert: (value = 2).	! !

!CompiledMethodTest methodsFor: 'tests - instance variable' stamp: 'sd 4/6/2009 21:48'!
testWritesField
	"self debug: #testWritesField"

	| method |
	method := self class compiledMethodAt: #writeX.
	self assert: (method writesField: 2).
	
	method := self class compiledMethodAt: #writeXandY.
	self assert: (method writesField: 2).
	
	method := self class compiledMethodAt: #writeXandY.
	self assert: (method writesField: 3).
	
	"write is not read"
	
	method := self class compiledMethodAt: #readX.
	self deny: (method writesField: 2).
	
	method := self class compiledMethodAt: #readXandY.
	self deny: (method writesField: 2).
	
	method := self class compiledMethodAt: #readXandY.
	self deny: (method writesField: 3).! !

!CompiledMethodTest methodsFor: 'examples' stamp: 'sd 4/6/2009 21:23'!
writeX
	
	x := 33
	! !

!CompiledMethodTest methodsFor: 'examples' stamp: 'sd 4/6/2009 21:23'!
writeXandY
	
	x := 33.
	y := 66
	! !

ClassTestCase subclass: #DateAndTimeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!DateAndTimeTest methodsFor: 'coverage' stamp: 'brp 9/25/2003 09:25'!
classToBeTested

	^ DateAndTime

! !

!DateAndTimeTest methodsFor: 'failures' stamp: 'StephaneDucasse 4/24/2010 11:26'!
expectedFailures

	^ #(testNotSymmetricWithString)! !

!DateAndTimeTest methodsFor: 'coverage' stamp: 'StephaneDucasse 4/23/2010 21:08'!
selectorsToBeIgnored

	| private | 
	private := #( #printOn: ).
	^ super selectorsToBeIgnored, private
! !

!DateAndTimeTest methodsFor: 'test - arithmetic' stamp: 'StephaneDucasse 4/23/2010 21:07'!
testArithmeticAcrossDateBoundary

	| t1 t2 |
	t1 := '2004-01-07T11:55:00+00:00' asDateAndTime. 
	t2 := t1 - ( (42900+1) seconds).  
	self assert: t2 = ('2004-01-06T23:59:59+00:00' asDateAndTime)
		
! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/24/2010 11:18'!
testAsDateAndTime
	"self debug: #testAsDateAndTime"
	
		#('-1199-01-05T20:33:14.321-05:00' 
		' 2002-05-16T17:20:45.1+01:01' 

		' 2002-05-16T17:20:45.02+01:01' 

		' 2002-05-16T17:20:45.003+01:01' 

		' 2002-05-16T17:20:45.0004+01:01' 
  		' 2002-05-16T17:20:45.00005' 
		' 2002-05-16T17:20:45.000006+01:01' 

		' 2002-05-16T17:20:45.0000007+01:01' 
		' 2002-05-16T17:20:45.00000008-01:01'    
		' 2002-05-16T17:20:45.000000009+01:01'   
		' 2002-05-16T17:20:45.0000000001+01:01'   

 		' 2002-05-16T17:20' 
		' 2002-05-16T17:20:45' 
		' 2002-05-16T17:20:45+01:57' 
 		' 2002-05-16T17:20:45-02:34' 
 		' 2002-05-16T17:20:45+00:00' 
		' 1997-04-26T01:02:03+01:02:3' ) do: [:each | each printString = each]! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/23/2010 21:44'!
testDateTimeDenotation1
 	"DateAndTimeTest new testDateTimeDenotation1"
	
	"Detroit is 5 hours behind UTC, this offset to UTC is therefore written with a minus sign. This example tests the correct interpretation of the DateAndTime denotation. "

	| twoPmInLondon twoPmUTCInLocalTimeOfDetroit nineAmInDetroit |
	twoPmInLondon := DateAndTime
				year: 2004
				month: 11
				day: 2
				hour: 14
				minute: 0
				second: 0
				offset: 0 hours.
	twoPmUTCInLocalTimeOfDetroit := twoPmInLondon utcOffset: -5 hours.
	nineAmInDetroit  := '2004-11-02T09:00:00-05:00' asDateAndTime.
	self assert:  twoPmUTCInLocalTimeOfDetroit = nineAmInDetroit.
	
! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/23/2010 21:44'!
testDateTimeDenotation2
 	"DateAndTimeTest new testDateTimeDenotation2"
	"Moscow is 3 hours ahead UTC, this offset to UTC is therefore positive. This example tests the correct interpretation of the DateAndTime denotation."

	| lateEveningInLondon lateEveningInLocalTimeOfMoscow
	localMoscowTimeFromDenotation |
	lateEveningInLondon := DateAndTime
				year: 2004
				month: 11
				day: 30
				hour: 23
				minute: 30
				second: 0
				offset: 0 hours.
	lateEveningInLocalTimeOfMoscow := lateEveningInLondon utcOffset: 3 hours.
	localMoscowTimeFromDenotation  := '2004-12-01T02:30:00+03:00' asDateAndTime.
	self assert:  lateEveningInLocalTimeOfMoscow = localMoscowTimeFromDenotation.! !

!DateAndTimeTest methodsFor: 'test bogus date' stamp: 'StephaneDucasse 4/23/2010 21:45'!
testErrorWhenDayIsAfterMonthEnd

	self should: [DateAndTime year: 2004 month: 2 day: 30] raise: Error.
	self shouldnt: [DateAndTime year: 2004 month: 2 day: 29] raise: Error.
	! !

!DateAndTimeTest methodsFor: 'test bogus date' stamp: 'StephaneDucasse 4/23/2010 21:45'!
testErrorWhenDayIsBeforeMonthStart

	self should: [DateAndTime year: 2004 month: 2 day: -1] raise: Error.
 	self should: [DateAndTime year: 2004 month: 2 day: 0] raise: Error.
	self shouldnt: [DateAndTime year: 2004 month: 2 day: 1] raise: Error.
	! !

!DateAndTimeTest methodsFor: 'tests - instance' stamp: 'brp 1/7/2004 15:37'!
testInstanceCreation

	| t |
	t := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self 
		assert: (t julianDayNumber = 1721427);
		assert: (t offset = 6 hours);
		assert: (t hour = 2);
		assert: (t minute = 3);
		assert: (t second = 4);
		assert: (t nanoSecond = 5).
		
! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/23/2010 21:46'!
testMonotonicity

	| t1 t2 t3 t4 |
	t1 := DateAndTime now.
	t2 := DateAndTime now.
	(Delay forMilliseconds: 1000) wait.
	t3 := DateAndTime now.
	t4 := DateAndTime now.

	self
		assert: (t1 <= t2);
		assert: (t2 < t3);
		assert: (t3 <= t4).
! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/23/2010 22:13'!
testNotSymmetricWithString
	"self debug: #testNotSymmetricWithString"
	
	| t1 t2 |
	t1 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	t2 :=  '0001-01-02T02:03:04.000000005+06:00'.
	self deny: (t1 = t2) = (t2 = t1)
	
	
! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/23/2010 21:47'!
testPrintString
	"self debug: #testPrintString"

	| dt |
	dt :=DateAndTime
		year: 2004
		month: 11
		day: 2
		hour: 14
		minute: 3
		second: 5
		nanoSecond: 12345
		offset: (Duration seconds: (5 * 3600)).
	self assert: dt printString = '2004-11-02T14:03:05.000012345+05:00'


! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/23/2010 21:50'!
testReadFrom

	self assert: (DateAndTime readFrom: '1901-01-01T00:00:00+12:00' readStream) asString =  '1901-01-01T00:00:00+12:00'! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/24/2010 11:58'!
testReadFromFoolProofExtension
	"Convenient extension without a time, only a date"
	"self debug: #testReadFromFoolProofExtension"
	
	self assert: (DateAndTime fuzzyReadFrom: '2008' readStream) printString = '2008-01-01T00:00:00+00:00'.
	self assert: (DateAndTime fuzzyReadFrom: '2008-08' readStream)   printString = '2008-08-01T00:00:00+00:00'.
	self assert: (DateAndTime fuzzyReadFrom: '2006-08-28' readStream)  printString = '2006-08-28T00:00:00+00:00'.
	"Regular nanoseconds"
	self assert: (DateAndTime fuzzyReadFrom: '2006-08-28T00:00:00.123456789' readStream)  printString = '2006-08-28T00:00:00.123456789+00:00'.
	"Extra picoseconds precision should not spoil the DateAndTime"
	self assert: (DateAndTime fuzzyReadFrom: '2006-08-28T00:00:00.123456789000' readStream)  printString = '2006-08-28T00:00:00.123456789+00:00'.! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/24/2010 12:05'!
testReadFromOffset
	"self debug: #testReadFromOffset"
	
	self assert: (DateAndTime readFrom: '-1199-01-05T20:33:14.321-05:00' readStream) offset printString =  '-0:05:00:00'.
! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/24/2010 11:19'!
testReadFromSecond
	"self debug: #testReadFromSecond"
	
	self assert: ('-1199-01-05T20:33:14.321-05:00' asDateAndTime printString = '-1199-01-05T20:33:14.321-05:00').
	self assert: ('2002-05-16T17:20:45.1+01:01' asDateAndTime printString = '2002-05-16T17:20:45.1+01:01').
	self assert:	(' 2002-05-16T17:20:45.02+01:01' asDateAndTime printString = '2002-05-16T17:20:45.02+01:01').  
	self assert:	('2002-05-16T17:20:45.000000009+01:01' asDateAndTime printString =  '2002-05-16T17:20:45.000000009+01:01').
	self assert: (' 2002-05-16T17:20' asDateAndTime printString = '2002-05-16T17:20:00+00:00').
	self assert: ('2002-05-16T17:20:45' asDateAndTime printString = '2002-05-16T17:20:45+00:00' ).
	self assert: (' 2002-05-16T17:20:45+01:57' asDateAndTime printString = '2002-05-16T17:20:45+01:57').
	self assert: (' 2002-05-16T17:20:45-02:34' asDateAndTime = '2002-05-16T17:20:45-02:34').
	self assert: ('2002-05-16T17:20:45+00:00' asDateAndTime = '2002-05-16T17:20:45+00:00').
	self assert: ('1997-04-26T01:02:03+01:02:3' asDateAndTime = '1997-04-26T01:02:03+01:02:3'). ! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/23/2010 22:07'!
testReflexive

	| t |
	t := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self assert: t = t.
	
! !

!DateAndTimeTest methodsFor: 'tests - instance' stamp: 'StephaneDucasse 4/23/2010 21:47'!
testSimpleAccessors

	| t |
	t := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self 
		assert: (t hours = t hours);
		assert: (t minutes = t minute);
		assert: (t seconds = t second).
! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/23/2010 22:08'!
testSymmetric

	| t1 t2 |
	t1 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	t2 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self assert: (t2 = t1) = (t1 = t2).
	
	
! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'BG 11/7/2004 12:18'!
testTimeZoneEquivalence
  "DateAndTimeTest new testTimeZoneEquivalence"
	"When the clock on the wall in Detroit says 9:00am, the clock on the wall
	in London says 2:00pm. The Duration difference between the corresponding
	DateAndTime values should be zero."
	
	 " Detroit is 5 hours behind UTC, this offset to UTC is therefore written with a minus sign. This example tests both the correct interpretation of the DateAndTime denotation and correct DateAndTime arithmetics. "

	| twoPmInLondon nineAmInDetroit durationDifference |
	twoPmInLondon := '2004-11-02T14:00:00+00:00' asDateAndTime.
	nineAmInDetroit  := '2004-11-02T09:00:00-05:00' asDateAndTime.
	durationDifference := twoPmInLondon - nineAmInDetroit.
	self assert: durationDifference asSeconds = 0.
	self assert: twoPmInLondon = nineAmInDetroit
! !

!DateAndTimeTest methodsFor: 'tests' stamp: 'BG 11/7/2004 12:17'!
testTimeZoneEquivalence2
  "DateAndTimeTest new testTimeZoneEquivalence2"
	"This example demonstates the fact that
        2004-05-24T22:40:00  UTC  is
        2004-05-25T01:40:00  in Moscow
     (Moscow is 3 hours ahead of UTC)  "

	| thisMoment thisMomentInMoscow |
    thisMoment := DateAndTime year: 2004 month: 5 day: 24 hour: 22 minute: 40.
    thisMomentInMoscow := thisMoment utcOffset: 3 hours.
	self assert: (thisMoment - thisMomentInMoscow) asSeconds = 0.
	self assert: thisMoment = thisMomentInMoscow
! !

!DateAndTimeTest methodsFor: 'tests under design' stamp: 'StephaneDucasse 4/23/2010 22:09'!
testTransitive
	"self debug: #testTransitive"
	| t1 t2 t3 |
	t1 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	t2 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	t3 := DateAndTime 
			year: 1 month: 1 day: 2 
			hour: 2 minute: 3 second: 4 nanoSecond: 5 
			offset: 6 hours.
	self assert: ((t1 = t2) & (t2 = t3) ==> (t1 = t3)).
	
	
! !

ClassTestCase subclass: #DateTest
	instanceVariableNames: 'date aDate aTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!DateTest commentStamp: 'brp 7/26/2003 16:58' prior: 0!
This is the unit test for the class Date. !

!DateTest methodsFor: 'Coverage' stamp: 'brp 7/27/2003 13:01'!
classToBeTested

	^ self dateClass! !

!DateTest methodsFor: 'Private' stamp: 'brp 8/24/2003 00:10'!
dateClass

	^ Date! !

!DateTest methodsFor: 'Coverage' stamp: 'brp 1/30/2005 09:03'!
selectorsToBeIgnored

	 | deprecated private special |
	deprecated := #().
	private := #().
	special := #( #< #= #new #next #previous #printOn: #printOn:format: #storeOn: #fromString: ).

	^ super selectorsToBeIgnored, deprecated, private, special! !

!DateTest methodsFor: 'Running' stamp: 'brp 1/21/2004 18:46'!
setUp

	date := self dateClass newDay: 153 year: 1973.	"2 June 1973"

	aDate := Date readFrom: '01-23-2004' readStream.
	aTime := Time readFrom: '12:34:56 pm' readStream! !

!DateTest methodsFor: 'Tests' stamp: 'brp 8/23/2003 16:07'!
testAccessing

	self	
		assert: date day = 153;
		assert: date julianDayNumber = 2441836;
		assert: date leap = 0;
		assert: date monthIndex = 6;
		assert: date monthName = #June;
		assert: date weekday = #Saturday;
		assert: date weekdayIndex = 7;
		assert: date year = 1973.
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testAddDays
	self assert: (aDate addDays: 00) yyyymmdd =  '2004-01-23'.	
	self assert: (aDate addDays: 30) yyyymmdd =  '2004-02-22'.
	self assert: (aDate addDays: 60) yyyymmdd =  '2004-03-23'.
	self assert: (aDate addDays: 90) yyyymmdd =  '2004-04-22'.
	self assert: (aDate addDays:120) yyyymmdd =  '2004-05-22'! !

!DateTest methodsFor: 'testing' stamp: 'tbn 7/11/2006 10:37'!
testAddMonths
	self assert: (aDate addMonths: 0) yyyymmdd =  '2004-01-23'.	
	self assert: (aDate addMonths: 1) yyyymmdd =  '2004-02-23'.
	self assert: (aDate addMonths: 2) yyyymmdd =  '2004-03-23'.
	self assert: (aDate addMonths: 3) yyyymmdd =  '2004-04-23'.
	self assert: (aDate addMonths: 12) yyyymmdd =  '2005-01-23'.
	
	self assert: ((Date readFrom: '05-31-2017' readStream) addMonths: 1) yyyymmdd =  '2017-06-30'.
	self assert: ((Date readFrom: '02-29-2000' readStream) addMonths: 12) yyyymmdd =  '2001-02-28'! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:10'!
testArithmetic
	| d |
	d := date addDays: 32.		"4 July 1973"

	self 
		assert: d year = 1973;
		assert: d monthIndex = 7;
		assert: d dayOfMonth = 4.
	self 
		assert: (d subtractDate: date) = 32;
		assert: (date subtractDate: d) = -32.
	self	 
		assert: (d subtractDays: 32) = date.
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testAsDate
	self assert: (aDate asDate) = aDate
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testAsSeconds
	self assert: (aDate asSeconds) =   3252268800.
	self assert: (aDate asSeconds) =  ((103*365*24*60*60) + (22+25"leap days"*24*60*60)) .
	self assert: aDate  =  (Date fromSeconds: 3252268800).! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:54'!
testComparing
	| d1 d2 d3 |
	d1 := self dateClass newDay: 2 month: #June year: 1973.
	d2 := self dateClass newDay: 97 year: 2003. 		"7 April 2003"
	d3 := self dateClass newDay: 250 year: 1865. 		"7 September 1865"

	self
		assert: date = d1;
		assert: date = date copy;
		assert: date hash = d1 hash.
	self 
		assert: date < d2;
		deny: date < d3.
! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:15'!
testConverting

	self 
		assert: date asDate = date;
		assert: '2 June 1973' asDate = date;
		assert: date asSeconds = 2285280000.

	date dayMonthYearDo: [ :d :m :y | self assert: d = 2; assert: m = 6; assert: y = 1973 ].! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testDateAndTimeNow
	"Not a great test: could falsely fail if midnight come in between the two executions and doesnt catch time errors"
	self assert: Date dateAndTimeNow first  = Date today 
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testDayMonthYearDo
	self assert: (aDate dayMonthYearDo: [:day :month :year | day asString , month asString, year asString]) = '2312004'
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testDaysInMonthForYear
	self assert: (Date daysInMonth: 'February' forYear: 2008)  = 29.	
	self assert: (Date daysInMonth: 'February' forYear: 2000)  = 29.	
	self assert: (Date daysInMonth: 'February' forYear: 2100)  = 28.	
	self assert: (Date daysInMonth: 'July' forYear: 2100)  = 31.	! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testDaysInYear
	self assert: (Date daysInYear: 2008)  = 366.	
	self assert: (Date daysInYear: 2000)  = 366.	
	self assert: (Date daysInYear: 2100)  = 365	
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testDuration
	self assert: aDate duration = 24 hours! !

!DateTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testEqual
	self assert: aDate = (Date readFrom: 'January 23, 2004' readStream)! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testFirstWeekdayOfMonthYear
	self assert: (Date firstWeekdayOfMonth: 'January' year: 2004)  = 5.	
! !

!DateTest methodsFor: 'Tests' stamp: 'brp 1/21/2004 18:47'!
testFromDays
	| epoch d0 d1 d2 |
	epoch := self dateClass newDay: 1 year: 1901.
	d0 := self dateClass fromDays: 0. 			"1 January 1901"
	self assert: d0 = epoch.

	d1 := self dateClass fromDays:  26450. 	"2 June 1973"
	self assert: d1 = date.

	d2 := self dateClass fromDays: -100000.	"18 March 1627"
	self assert: d2 julianDayNumber = 2315386.

	self assert: aDate  =  (Date fromDays:  37642).
	self assert: aDate  =  (Date fromDays: 103*365 + 22 + 25 "leap days") .
	! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:17'!
testFromSeconds
	| d |
	d := self dateClass fromSeconds: 2285280000. 
	self
		assert: d = date.
! !

!DateTest methodsFor: 'Tests' stamp: 'brp 1/7/2004 16:37'!
testGeneralInquiries

	| shuffled indices names now | 

	shuffled := #(#January #February #March #April #May #June #July 
					#August #September #October #November #December) shuffled.
	indices := shuffled collect: [ :m | self dateClass indexOfMonth: m ].
	names := indices collect: [ :i | self dateClass nameOfMonth: i ].
	self assert: names = shuffled.

	shuffled := #(#Monday #Tuesday #Wednesday #Thursday #Friday #Saturday #Sunday) shuffled.
	indices := shuffled collect: [ :m | self dateClass dayOfWeek: m ].
	names := indices collect: [ :i | self dateClass nameOfDay: i ].
	self assert: names = shuffled.
	
	now  := self dateClass dateAndTimeNow.
	self 
		assert: now size = 2;
		assert: now first = self dateClass today.

	self assert: (self dateClass firstWeekdayOfMonth: #June year: 1973) = 6.

	self
		assert: (self dateClass leapYear: 1973) = 0;
		assert: (self dateClass leapYear: 1972) = 1;
		assert: (self dateClass daysInYear: 1973) = 365;
		assert: (self dateClass daysInYear: 1972) = 366;
		assert: (self dateClass daysInMonth: #February forYear: 1973) = 28;
		assert: (self dateClass daysInMonth: #February forYear: 1972) = 29.
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testIndexOfMonth
	self assert: (Date indexOfMonth: 'January')  = 1.	
	self assert: (Date indexOfMonth: 'December')  = 12.	! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:17'!
testInitialization

	self should: [ self dateClass initialize. true ].
! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:18'!
testInquiries

	self	
		assert: date dayOfMonth = 2;
		assert: date dayOfYear = 153;
		assert: date daysInMonth = 30;
		assert: date daysInYear = 365;
		assert: date daysLeftInYear = (365 - 153);
		assert: date firstDayOfMonth = 152.
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testJulianDayNumber
	self assert: aDate = (Date julianDayNumber: ((4713+2004)*365 +1323) ).  ! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testLeap
	self assert: aDate leap = 1.	

! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testLeapNot
	self assert: (aDate addDays: 365) leap = 0
! !

!DateTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testLessThan
	self assert: aDate < (Date readFrom: '01-24-2004' readStream)! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testMmddyyyy
	self assert: aDate mmddyyyy =  '1/23/2004'! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testNameOfMonth
	self assert: (Date nameOfMonth: 5) = 'May'.	
	self assert: (Date nameOfMonth: 8) = 'August' ! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:05'!
testNew
	| epoch |
	epoch := self dateClass newDay: 1 year: 1901.
	self assert: (self dateClass new = epoch).! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testNewDayMonthYear
	self assert: aDate = (Date newDay: 23 month: 1 year: 2004)	
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testNewDayYear
	self assert: aDate = (Date newDay: 23 year: 2004)	
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testPreviousFriday
	self assert: (aDate previous: 'Friday') yyyymmdd = '2004-01-16'

! !

!DateTest methodsFor: 'Tests' stamp: 'brp 1/9/2004 16:33'!
testPreviousNext
	| n p pt ps |
	n := date next.
	p := date previous.

	self
		assert: n year = 1973;
		assert: n dayOfYear = 154;
		assert: p year = 1973;
		assert: p dayOfYear = 152.

	pt := date previous: #Thursday.		"31 May 1973"
	self	
		assert: pt year = 1973;
		assert: pt dayOfYear = 151.

	ps := date previous: #Saturday.		" 26 May 1973"
	self	
		assert: ps year = 1973;
		assert: ps dayOfYear = (153-7).
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testPreviousThursday
	self assert: (aDate previous: 'Thursday') yyyymmdd = '2004-01-22'

! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testPrintFormat
	self assert: (aDate printFormat: #(1 2 3 $? 2 2)) =  '23?Jan?04'! !

!DateTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '23 January 2004' readStream.
	rw := ReadWriteStream on: ''.
	aDate printOn: rw.
	self assert: rw contents = cs contents! !

!DateTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOnFormat
	| cs rw |
	cs := '04*Jan*23' readStream.
	rw := ReadWriteStream on: ''.
	aDate 
		printOn: rw
		format: #(3 2 1 $* 2 2 ).
	self assert: rw contents = cs contents! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:21'!
testPrinting

	self	
		assert: date mmddyyyy = '6/2/1973';
		assert: date yyyymmdd = '1973-06-02';
		assert: (date printFormat: #(3 1 2 $!! 2 1 1)) = '1973!!2!!Jun'.
! !

!DateTest methodsFor: 'Tests' stamp: 'StephaneDucasse 4/24/2010 11:23'!
testReadFrom
	"self debug: #testReadFrom"
	| s1 s2 s3 s4 s5 | 
	s1 := '2 June 1973'.
	s2 := '2-JUN-73'.
	s3 := 'June 2, 1973'.
	s4 := '6/2/73'.
	s5 := '2JUN73'.

	self	
		assert: date = (self dateClass readFrom: s1 readStream);
		assert: date = (self dateClass readFrom: s2 readStream);
		assert: date = (self dateClass readFrom: s3 readStream);
		assert: date = (self dateClass readFrom: s4 readStream);
		assert: date = (self dateClass readFrom: s5 readStream).! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testStarting
	self assert: aDate = (Date starting: (DateAndTime fromString: '2004-01-23T12:12')).  ! !

!DateTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testStoreOn
	| cs rw |
	cs := '''23 January 2004'' asDate' readStream.
	rw := ReadWriteStream on: ''.
	aDate storeOn: rw.
	self assert: rw contents = cs contents! !

!DateTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:05'!
testStoring

	self	
		assert: date storeString = '''2 June 1973'' asDate';
		assert: date = ('2 June 1973' asDate).
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testSubtractDate
	self assert: (aDate subtractDate:(aDate addDays: 30)) = -30.	
	self assert: (aDate subtractDate:(aDate subtractDays: 00)) = 0.	
	self assert: (aDate subtractDate:(aDate subtractDays: 30)) = 30.

! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testSubtractDays
	self assert: (aDate subtractDays: 00) yyyymmdd =  '2004-01-23'.	
	self assert: (aDate subtractDays: 30) yyyymmdd =  '2003-12-24'.
	self assert: (aDate subtractDays: 60) yyyymmdd =  '2003-11-24'
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testTomorrow
	"Not a great test: could falsely fail if midnight come in between the two executions and doesnt catch many errors"
	self assert: Date tomorrow  > Date today 
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testWeekday
	self assert: aDate weekday = 'Friday'.	
	self assert: aDate weekdayIndex = 6. 
	self assert: (Date dayOfWeek: aDate weekday ) =6.
	self assert: (Date nameOfDay: 6 ) = 'Friday'	! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testYesterday
	"Not a great test:  doesnt catch many errors"
	self assert: Date yesterday  < Date today 
! !

!DateTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:48'!
testYyyymmdd
	self assert: aDate yyyymmdd =  '2004-01-23'! !

ClassTestCase subclass: #DurationTest
	instanceVariableNames: 'aDuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!DurationTest methodsFor: 'coverage' stamp: 'brp 9/25/2003 14:30'!
classToBeTested

	^ Duration


! !

!DurationTest methodsFor: 'coverage' stamp: 'brp 9/25/2003 14:30'!
selectorsToBeIgnored

	| private | 
	private := #( #printOn: ).

	^ super selectorsToBeIgnored, private
! !

!DurationTest methodsFor: 'setup' stamp: 'brp 1/21/2004 18:36'!
setUp
	aDuration := Duration days: 1 hours: 2 minutes: 3 seconds: 4 nanoSeconds: 5 ! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAbs
	self assert: aDuration abs = aDuration. 
	self assert: (Duration nanoSeconds: -5)  abs =  (Duration nanoSeconds: 5). 
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAsDelay
	self deny: aDuration asDelay =   aDuration.
	"want to come up with a more meaningful test"
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAsDuration
	self assert: aDuration asDuration =  aDuration
	
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAsMilliSeconds
	self assert: (Duration nanoSeconds: 1000000)  asMilliSeconds = 1.
	self assert: (Duration seconds: 1)  asMilliSeconds = 1000.	
	self assert: (Duration nanoSeconds: 1000000)  asMilliSeconds = 1.
	self assert: (Duration nanoSeconds: 1000000)  asMilliSeconds = 1.
	self assert: aDuration   asMilliSeconds = 93784000.! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAsNanoSeconds
	self assert: (Duration nanoSeconds: 1)  asNanoSeconds = 1.
	self assert: (Duration seconds: 1)  asNanoSeconds = 1000000000.	
	self assert: aDuration   asNanoSeconds = 93784000000005.! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testAsSeconds
	self assert: (Duration nanoSeconds: 1000000000)  asSeconds = 1.
	self assert: (Duration seconds: 1)  asSeconds = 1.	
	self assert: aDuration   asSeconds = 93784.! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/9/2004 06:32'!
testComparing

	| d1 d2 d3 |
	d1 := Duration seconds: 10 nanoSeconds: 1.
	d2 := Duration seconds: 10 nanoSeconds: 1.
	d3 := Duration seconds: 10 nanoSeconds: 2.
	
	self
		assert: (d1 = d1);
		assert: (d1 = d2);
		deny: (d1 = d3);
		assert: (d1 < d3)
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testDays
	self assert: aDuration   days = 1.
	self assert: (Duration   days: 1) days= 1.	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testDivide
	self assert: aDuration / aDuration = 1. 
	self assert: aDuration / 2 = (Duration days: 0 hours: 13 minutes: 1 seconds: 32 nanoSeconds: 2). 
	self assert: aDuration / (1/2) = (Duration days: 2 hours: 4 minutes: 6 seconds: 8 nanoSeconds: 10).
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testFromString
	self assert: aDuration = (Duration fromString: '1:02:03:04.000000005').
! !

!DurationTest methodsFor: 'testing' stamp: 'al 6/12/2008 21:57'!
testHash
	self assert: aDuration hash = (Duration days: 1 hours: 2 minutes: 3 seconds: 4 nanoSeconds: 5) hash! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testHours
	self assert: aDuration   hours = 2.
	self assert: (Duration   hours: 2) hours = 2.	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testIntegerDivision
	self assert: aDuration // aDuration = 1. 
	self assert: aDuration // 2 =  (aDuration / 2). 
	"is there ever a case where this is not true, since precision is always to the nano second?"! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testLessThan
	self assert: aDuration  < (aDuration + 1 day ).
	self deny: aDuration < aDuration.
	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testMilliSeconds
	self assert: (Duration milliSeconds: 5) nanoSeconds = 5000000.	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testMinus
	self assert: aDuration - aDuration = (Duration seconds: 0).
	self assert: aDuration - (Duration days: -1 hours: -2 minutes: -3 seconds: -4 nanoSeconds: -5) = 
						    (Duration days: 2  hours: 4  minutes: 6  seconds: 8  nanoSeconds: 10). 
	self assert: aDuration - (Duration days: 0  hours: 1  minutes: 2  seconds: 3  nanoSeconds: 4) = 
						    (Duration days: 1  hours: 1  minutes: 1  seconds: 1  nanoSeconds: 1). 
	self assert: aDuration - (Duration days: 0  hours: 3   minutes: 0  seconds: 5  nanoSeconds: 0) = 
						    (Duration days: 0  hours: 23  minutes: 2  seconds: 59  nanoSeconds: 5). ! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testMinutes
	self assert: aDuration   minutes = 3.
	self assert: (Duration minutes: 3) minutes = 3.	! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/21/2004 18:36'!
testModulo

	| d1 d2 d3 |
	d1 := 11.5 seconds.
	d2 := d1 \\ 3.
	self assert: d2 = (Duration nanoSeconds: 1).

	d3 := d1 \\ (3 seconds).
	self assert: d3 =  (Duration seconds: 2 nanoSeconds: 500000000).

	self assert: aDuration \\ aDuration = 
		(Duration days: 0 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0). 
	self assert: aDuration \\ 2 = 
		(Duration days: 0 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 1).
	

! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/16/2004 14:17'!
testMonthDurations

	| jan feb dec |
	jan := Duration month: #January.
	feb := Duration month: #February.
	dec := Duration month: #December.
	
	self 
		assert: jan = (Year current months first duration);
		assert: feb = (Year current months second duration);
		assert: dec = (Year current months last duration)

		
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testMultiply
	self assert: aDuration * 2 = (Duration days: 2 hours: 4 minutes: 6 seconds: 8 nanoSeconds: 10). ! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testNanoSeconds
	self assert: aDuration nanoSeconds = 5.
	self assert: (Duration nanoSeconds: 5) nanoSeconds = 5.	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testNegated
	self assert: aDuration + aDuration negated = (Duration seconds: 0). 
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testNegative
	self deny: aDuration negative.
	self assert: aDuration negated negative
! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testNew
	"self assert: Duration new =  (Duration seconds: 0)."
    "new is not valid as a creation method: MessageNotUnderstood: UndefinedObject>>quo:, where Duration seconds is nil"! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/9/2004 06:28'!
testNumberConvenienceMethods

	self
		assert: 1 week = (Duration days: 7);
		assert: -1 week = (Duration days: -7);
		assert: 1 day = (Duration days: 1);
		assert: -1 day = (Duration days: -1);
		assert: 1 hours = (Duration hours: 1);
		assert: -1 hour = (Duration hours: -1);
		assert: 1 minute = (Duration seconds: 60);
		assert: -1 minute = (Duration seconds: -60);
		assert: 1 second = (Duration seconds: 1);
		assert: -1 second = (Duration seconds: -1);
		assert: 1 milliSecond = (Duration milliSeconds: 1);
		assert: -1 milliSecond = (Duration milliSeconds: -1);
		assert: 1 nanoSecond = (Duration nanoSeconds: 1);
		assert: -1 nanoSecond = (Duration nanoSeconds: -1)
		! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testPlus
	self assert: (aDuration + 0 hours) = aDuration.
	self assert: (aDuration + aDuration) = (Duration days: 2 hours: 4 minutes: 6 seconds: 8 nanoSeconds: 10). ! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testPositive
	self assert: (Duration nanoSeconds: 0) positive.
	self assert: aDuration positive.
	self deny: aDuration negated positive
! !

!DurationTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '1:02:03:04.000000005' readStream.
	rw := ReadWriteStream on: ''.
	aDuration printOn: rw.
	self assert: rw contents = cs contents! !

!DurationTest methodsFor: 'tests' stamp: 'brp 9/25/2003 14:57'!
testQuotient

	| d1 d2 q |
	d1 := 11.5 seconds.
	d2 := d1 // 3.
	self assert: d2 = (Duration seconds: 3 nanoSeconds: 833333333).

	q := d1 // (3 seconds).
	self assert: q = 3.

! !

!DurationTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testReadFrom
	self assert: aDuration = (Duration readFrom: '1:02:03:04.000000005' readStream)! !

!DurationTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/24/2010 10:52'!
testReadFromBogus

	self should: [Duration readFrom: '+0:01:02' readStream] raise: Error.
	"Seconds should be supplied as per ANSI 5.8.2.16: [-]D:HH:MM:SS[.S]"! !

!DurationTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/23/2010 21:55'!
testReadFromMillisecond

	self assert: (Duration readFrom: '0:00:00:00.001 ' readStream) nanoSeconds = 1000000! !

!DurationTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/24/2010 10:53'!
testReadFromNoException
	"self debug: #testReadFromNoException"

	#(	'0:00:00:00' 
		'0:00:00:00.000000001' 
		'0:00:00:00.999999999' 
		'0:00:00:00.100000000' 
		'0:00:00:00.10' 
		'0:00:00:00.1' 
		'0:00:00:01' 
		'0:12:45:45' 
		'1:00:00:00' 
		'365:00:00:00' 
		'-7:09:12:06.10' 
		'+0:01:02:55' 
		'+0:01:02:3') do: [:each | each asDuration printString = each]! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/21/2004 18:38'!
testRoundTo

	self assert: ((5 minutes + 37 seconds) roundTo: (2 minutes)) = (6 minutes).
	
	self assert:  (aDuration roundTo: (Duration days: 1)) =
	               (Duration days: 1 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0).
	self assert:  (aDuration roundTo: (Duration hours: 1)) =
	               (Duration days: 1 hours: 2 minutes: 0 seconds: 0 nanoSeconds: 0).	
	self assert:  (aDuration roundTo: (Duration minutes: 1)) =
	               (Duration days: 1 hours: 2 minutes: 3 seconds: 0 nanoSeconds: 0).! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testSeconds
	self assert: aDuration seconds =   (800000001/200000000).
	self assert: (Duration  nanoSeconds: 2) seconds = (2/1000000000).	
	self assert: (Duration  seconds: 2) seconds = 2.	
	self assert: (Duration  days: 1 hours: 2 minutes: 3 seconds:4) seconds = (4).
	self deny: (Duration  days: 1 hours: 2 minutes: 3 seconds:4) seconds = (1*24*60*60+(2*60*60)+(3*60)+4).	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testSecondsNanoSeconds 
	self assert: (Duration   seconds: 0 nanoSeconds: 5)  = (Duration  nanoSeconds: 5).	
	"not sure I should include in sunit since its Private "
	self assert: (aDuration seconds: 0 nanoSeconds: 1) = (Duration nanoSeconds: 1). 
! !

!DurationTest methodsFor: 'testing' stamp: 'PeterHugossonMiller 9/3/2009 16:02'!
testStoreOn
	| stream |
	aDuration storeOn: (stream := (String new: 20) writeStream).
	self assert: stream contents = '(Duration seconds: 93784 nanoSeconds: 5)'.

! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testTicks
	self assert: aDuration ticks =  #(1 7384 5)! !

!DurationTest methodsFor: 'tests' stamp: 'brp 1/21/2004 18:37'!
testTruncateTo

	self assert: ((5 minutes + 37 seconds) truncateTo: (2 minutes)) = (4 minutes).
	self assert:  (aDuration truncateTo: (Duration days: 1)) =
	               (Duration days: 1 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0).
	self assert:  (aDuration truncateTo: (Duration hours: 1)) =
	               (Duration days: 1 hours: 2 minutes: 0 seconds: 0 nanoSeconds: 0).	
	self assert:  (aDuration truncateTo: (Duration minutes: 1)) =
	               (Duration days: 1 hours: 2 minutes: 3 seconds: 0 nanoSeconds: 0).! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testWeeks
	self assert: (Duration  weeks: 1) days= 7.	! !

!DurationTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:38'!
testZero
	self assert: (Duration zero) = (Duration seconds: 0).	! !

ClassTestCase subclass: #ExtendedNumberParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:25'!
testFractionPartWithoutIntegerPart
	"The integer part before the decimal is optional"
	
	self assert: (ExtendedNumberParser parse: '.5') = (1/2).
	self assert: (ExtendedNumberParser parse: '.5') isFloat.
	
	self assert: (ExtendedNumberParser parse: '.3e2') = 30.
	self assert: (ExtendedNumberParser parse: '.3e2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '-.4e2') = -40.
	self assert: (ExtendedNumberParser parse: '-.4e2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '+.5e2') = 50.
	self assert: (ExtendedNumberParser parse: '+.5e2') isFloat.

	self assert: (ExtendedNumberParser parse: '+.6e+2') = 60.
	self assert: (ExtendedNumberParser parse: '+.6e+2') isFloat.

	self assert: (ExtendedNumberParser parse: '-.7e+2') = -70.
	self assert: (ExtendedNumberParser parse: '-.7e+2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '+2r.1e-2') = (1/8).
	self assert: (ExtendedNumberParser parse: '+2r.1e-2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '-4r.1e-2') = (-1/64).
	self assert: (ExtendedNumberParser parse: '-4r.1e-2') isFloat.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:25'!
testIntegerPartWithoutFraction
	"The fraction part after the decimal is optional"
	
	self assert: (ExtendedNumberParser parse: '1.') = 1.
	self assert: (ExtendedNumberParser parse: '1.') isFloat.
	
	self assert: (ExtendedNumberParser parse: '3.e2') = 300.
	self assert: (ExtendedNumberParser parse: '3.e2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '-4.e2') = -400.
	self assert: (ExtendedNumberParser parse: '-4.e2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '+5.e2') = 500.
	self assert: (ExtendedNumberParser parse: '+5.e2') isFloat.

	self assert: (ExtendedNumberParser parse: '+6.e+2') = 600.
	self assert: (ExtendedNumberParser parse: '+6.e+2') isFloat.

	self assert: (ExtendedNumberParser parse: '-7.e+2') = -700.
	self assert: (ExtendedNumberParser parse: '-7.e+2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '+2r1.e-2') = (1/4).
	self assert: (ExtendedNumberParser parse: '+2r1.e-2') isFloat.
	
	self assert: (ExtendedNumberParser parse: '-4r1.e-2') = (-1/16).
	self assert: (ExtendedNumberParser parse: '-4r1.e-2') isFloat.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:23'!
testInvalidExponent
	"The leading number is returned, the invalid part is ignored"
	
	self assert: (ExtendedNumberParser parse: '1e') = 1.
	self assert: (ExtendedNumberParser parse: '1eZ') = 1.
	self assert: (ExtendedNumberParser parse: '+1eW') = 1.
	self assert: (ExtendedNumberParser parse: '-1eX') = -1.
	
	self assert: (ExtendedNumberParser parse: '2e-') = 2.
	self assert: (ExtendedNumberParser parse: '2e--1') = 2.
	self assert: (ExtendedNumberParser parse: '2e-+1') = 2.
	self assert: (ExtendedNumberParser parse: '2e-Z') = 2.
	self assert: (ExtendedNumberParser parse: '+2e-W') = 2.
	self assert: (ExtendedNumberParser parse: '-2e-X') = -2.
	
	self assert: (ExtendedNumberParser parse: '3e+') = 3.
	self assert: (ExtendedNumberParser parse: '3e+-') = 3.
	self assert: (ExtendedNumberParser parse: '3e+-1') = 3.
	self assert: (ExtendedNumberParser parse: '+3e+W') = 3.
	self assert: (ExtendedNumberParser parse: '-3e+Z') = -3.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:08'!
testInvalidRadix
	"The leading number is returned, the invalid part is ignored"
	
	self assert: (ExtendedNumberParser parse: '1r') = 1.
	self assert: (ExtendedNumberParser parse: '+1r') = 1.
	self assert: (ExtendedNumberParser parse: '-1r') = -1.
	self assert: (ExtendedNumberParser parse: '-1r+') = -1.
	self assert: (ExtendedNumberParser parse: '-1r-') = -1.
	
	self assert: (ExtendedNumberParser parse: '-2r.') = -2.
	self assert: (ExtendedNumberParser parse: '-2r-.') = -2.
	self assert: (ExtendedNumberParser parse: '+2r-.') = 2.
	
	self assert: (ExtendedNumberParser parse: '+2r3.') = 2.
	self assert: (ExtendedNumberParser parse: '+2r.3') = 2.
	self assert: (ExtendedNumberParser parse: '+2r-.3') = 2.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:24'!
testInvalidScale
	"The leading number is returned, the invalid part is ignored"
	
	self assert: (ExtendedNumberParser parse: '1s') = 1.
	self assert: (ExtendedNumberParser parse: '1sZ') = 1.
	self assert: (ExtendedNumberParser parse: '+1sW') = 1.
	self assert: (ExtendedNumberParser parse: '-1sX') = -1.
	
	self assert: (ExtendedNumberParser parse: '2s-') = 2.
	self assert: (ExtendedNumberParser parse: '2s--1') = 2.
	self assert: (ExtendedNumberParser parse: '2s-+1') = 2.
	self assert: (ExtendedNumberParser parse: '2s-1') = 2.
	self assert: (ExtendedNumberParser parse: '+2s-2') = 2.
	self assert: (ExtendedNumberParser parse: '-2s-3') = -2.
	
	self assert: (ExtendedNumberParser parse: '3s+') = 3.
	self assert: (ExtendedNumberParser parse: '3s+-') = 3.
	self assert: (ExtendedNumberParser parse: '3s+-1') = 3.
	self assert: (ExtendedNumberParser parse: '+3s+2') = 3.
	self assert: (ExtendedNumberParser parse: '-3s+3') = -3.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:26'!
testPositive
	"A leading + sign is allowed"
	
	self assert: (ExtendedNumberParser parse: '+1') = 1.
	self assert: (ExtendedNumberParser parse: '+22') = 22.
	self assert: (ExtendedNumberParser parse: '+2r11') = 3.
	self assert: (ExtendedNumberParser parse: '+2r+101') = 5.
	self assert: (ExtendedNumberParser parse: '+2r-101') = -5.
	self assert: (ExtendedNumberParser parse: '-2r+101') = -5.
	
	self assert: (ExtendedNumberParser parse: '+1.') isFloat.
	self assert: (ExtendedNumberParser parse: '+1.') = 1.
	self assert: (ExtendedNumberParser parse: '+21.') = 21.
	self assert: (ExtendedNumberParser parse: '+3r21.') = 7.
	self assert: (ExtendedNumberParser parse: '+3r+201.') = 19.
	self assert: (ExtendedNumberParser parse: '+3r-201.') = -19.
	self assert: (ExtendedNumberParser parse: '-3r+201.') = -19.! !

!ExtendedNumberParserTest methodsFor: 'testing' stamp: 'nice 2/13/2010 02:27'!
testPositiveExponent
	"A leading + sign is allowed in exponent"
	
	self assert: (ExtendedNumberParser parse: '1e+2') = 100.
	self assert: (ExtendedNumberParser parse: '1e+2') isInteger.
	self assert: (ExtendedNumberParser parse: '-1e+2') = -100.

	self assert: (ExtendedNumberParser parse: '1.e+2') = 100.
	self assert: (ExtendedNumberParser parse: '1.e+2') isFloat.
	self assert: (ExtendedNumberParser parse: '-1.0e+2') = -100.! !

ClassTestCase subclass: #FalseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!FalseTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class False. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category
!

!FalseTest methodsFor: 'tests' stamp: 'FabrizioPerin 3/13/2010 15:23'!
testAND

	self deny: (false & true).
	self deny: (false & false).! !

!FalseTest methodsFor: 'tests' stamp: 'FabrizioPerin 3/13/2010 15:23'!
testAnd

	self deny: (false and: ['alternativeBlock']).! !

!FalseTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/2/2010 13:49'!
testAsBit

       self assert: (false asBit = 0).! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:03'!
testIfFalse
	
	self assert: ((false ifFalse: ['alternativeBlock']) = 'alternativeBlock'). ! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:03'!
testIfFalseIfTrue

	self assert: (false ifFalse: ['falseAlternativeBlock'] 
                      ifTrue: ['trueAlternativeBlock']) = 'falseAlternativeBlock'. ! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:03'!
testIfTrue

	self assert: (false ifTrue: ['alternativeBlock']) = nil. ! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:03'!
testIfTrueIfFalse

	self assert: (false ifTrue: ['trueAlternativeBlock'] 
                      ifFalse: ['falseAlternativeBlock']) = 'falseAlternativeBlock'. ! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:03'!
testNew

	self should: [False new] raise: TestResult error. ! !

!FalseTest methodsFor: 'tests' stamp: 'StephaneDucasse 3/5/2010 15:32'!
testNot

	self assert: (false not).! !

!FalseTest methodsFor: 'tests' stamp: 'FabrizioPerin 3/13/2010 15:24'!
testOR

	self assert: (false | true).
	self deny: (false | false).! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:04'!
testOr

	self assert: (false or: ['alternativeBlock']) = 'alternativeBlock'.! !

!FalseTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:04'!
testPrintOn

	self assert: (String streamContents: [:stream | false printOn: stream]) = 'false'. ! !

!FalseTest methodsFor: 'tests' stamp: 'NikoSchwarz 10/17/2009 18:20'!
testXor
	self assert: (false xor: true) = true.
	self assert: (false xor: false) = false.
		
	self
		should: [(false xor: [false])
 			ifTrue: ["This should never be true, do not signal an Error and let the test fail"]
 			ifFalse: [self error: 'OK, this should be false, raise an Error']]
		raise: Error
		description: 'a Block argument is not allowed. If it were, answer would be false'.! !

ClassTestCase subclass: #FloatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!FloatTest commentStamp: 'fbs 3/8/2004 22:13' prior: 0!
I provide a test suite for Float values. Examine my tests to see how Floats should behave, and see how to use them.!

!FloatTest methodsFor: 'IEEE 754' stamp: 'nice 5/30/2006 02:34'!
test32bitGradualUnderflow
	"method asIEEE32BitWord did not respect IEEE gradual underflow"
	
	| conv expected exponentPart |
	
	"IEEE 32 bits Float have 1 bit sign/8 bit exponent/23 bits of mantissa after leading 1
	2r1.mmmmmmmmmmmmmmmmmmmmmmm * (2 raisedTo: 2reeeeeeee-127) * sign
	except when 2reeeeeeee isZero, which is a gradual underflow:
	2r0.mmmmmmmmmmmmmmmmmmmmmmm * (2 raisedTo: 2r00000000-126) * sign
	and when 2reeeeeeee = 255, which is infinity if mantissa all zero or nan otherwise"
	
	"case 1: This example is the first gradual underflow case"
	conv := 2r0.11111111111111111111111e-126 asIEEE32BitWord.
	
	"expected float encoded as sign/exponent/mantissa (whithout leading 1 or 0)"
	exponentPart := 0.
	expected := exponentPart bitOr: 2r11111111111111111111111.
	self assert: expected = conv.
	
	"case 2: smallest number"
	conv := 2r0.00000000000000000000001e-126 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r1.
	self assert: expected = conv.
	
	"case 3: round to nearest even also in underflow cases... here round to upper"
	conv := 2r0.000000000000000000000011e-126 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r10.
	self assert: expected = conv.
	
	"case 4: round to nearest even also in underflow cases... here round to lower"
	conv := 2r0.000000000000000000000101e-126 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r10.
	self assert: expected = conv.
	
	"case 5: round to nearest even also in underflow cases... here round to upper"
	conv := 2r0.0000000000000000000001011e-126 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r11.
	self assert: expected = conv.
	! !

!FloatTest methodsFor: 'IEEE 754' stamp: 'nice 5/30/2006 00:07'!
test32bitRoundingMode
	"method asIEEE32BitWord did not respect IEEE default rounding mode"
	
	| conv expected exponentPart |
	
	"IEEE 32 bits Float have 1 bit sign/8 bit exponent/23 bits of mantissa after leading 1
	2r1.mmmmmmmmmmmmmmmmmmmmmmm * (2 raisedTo: 2reeeeeeee-127) * sign
	except when 2reeeeeeee isZero, which is a gradual underflow:
	2r0.mmmmmmmmmmmmmmmmmmmmmmm * (2 raisedTo: 2r00000000-127) * sign
	and when 2reeeeeeee = 255, which is infinity if mantissa all zero or nan otherwise"
	
	"This example has two extra bits in mantissa for testing rounding mode
	case 1: should obviously round to upper"
	conv := 2r1.0000000000000000000000111e25 asIEEE32BitWord.
	
	"expected float encoded as sign/exponent/mantissa (whithout leading 1)"
	exponentPart := 25+127 bitShift: 23. "127 is 2r01111111 or 16r7F"
	expected := exponentPart bitOr: 2r10.
	self assert: expected = conv.
	
	"case 2: exactly in the mid point of two 32 bit float: round toward nearest even (to upper)"
	conv := 2r1.0000000000000000000000110e25 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r10.
	self assert: expected = conv.
	
	"case 3: exactly in the mid point of two 32 bit float: round toward nearest even (to lower)"
	conv := 2r1.0000000000000000000000010e25 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r0.
	self assert: expected = conv.
	
	"case 4: obviously round to upper"
	conv := 2r1.0000000000000000000000011e25 asIEEE32BitWord.
	expected := exponentPart bitOr: 2r1.
	self assert: expected = conv.
! !

!FloatTest methodsFor: 'testing' stamp: 'nice 10/30/2009 22:23'!
testArcTan

	self assert: ((100 arcTan: 100) closeTo: Float pi / 4).
	self assert: ((-100 arcTan: 100) closeTo: Float pi / -4).
	self assert: ((100 arcTan: -100) closeTo: Float pi * 3 / 4).
	self assert: ((-100 arcTan: -100) closeTo: Float pi * -3 / 4).
	self assert: ((0 arcTan: 100) closeTo: 0).
	self assert: ((0 arcTan: -100) closeTo: Float pi).
	self assert: ((100 arcTan: 0) closeTo: Float pi / 2).
	self assert: ((-100 arcTan: 0) closeTo: Float pi / -2).
	
	self assert: ((Float negativeZero arcTan: 100) closeTo: 0).
	self assert: ((Float negativeZero arcTan: -100) closeTo: Float pi * -1).
	
	self assert: (0 arcTan: 0) = 0.
	self assert: (Float negativeZero arcTan: 0) = 0.
	self assert: ((0 arcTan: Float negativeZero) closeTo: Float pi).
	self assert: ((Float negativeZero arcTan: Float negativeZero) closeTo: Float pi negated). ! !

!FloatTest methodsFor: 'characterization' stamp: 'nice 6/11/2009 20:47'!
testCharacterization

	"Test the largest finite representable floating point value"
	self assert: Float fmax successor = Float infinity.
	self assert: Float infinity predecessor = Float fmax.
	self assert: Float fmax negated predecessor = Float infinity negated.
	self assert: Float infinity negated successor = Float fmax negated.
	
	"Test the smallest positive representable floating point value"
	self assert: Float fmin predecessor = 0.0.
	self assert: 0.0 successor = Float fmin.
	self assert: Float fmin negated successor = 0.0.
	self assert: 0.0 predecessor = Float fmin negated.
	
	"Test the relative precision"
	self assert: Float one + Float epsilon > Float one.
	self assert: Float one + Float epsilon = Float one successor.
	self assert: Float one + (Float epsilon / Float radix) = Float one.
	
	"Test maximum and minimum exponent"
	self assert: Float fmax exponent = Float emax.
	self assert: Float fminNormalized exponent = Float emin.
	Float denormalized ifTrue: [
		self assert: Float fminDenormalized exponent = (Float emin + 1 - Float precision)].
	
	"Alternative tests for maximum and minimum"
	self assert: (Float radix - Float epsilon) * (Float radix raisedTo: Float emax) = Float fmax.
	self assert: Float epsilon * (Float radix raisedTo: Float emin) = Float fmin.
	
	"Test sucessors and predecessors"
	self assert: Float one predecessor successor = Float one.
	self assert: Float one successor predecessor = Float one.
	self assert: Float one negated predecessor successor = Float one negated.
	self assert: Float one negated successor predecessor = Float one negated.
	self assert: Float infinity successor = Float infinity.
	self assert: Float infinity negated predecessor = Float infinity negated.
	self assert: Float nan predecessor isNaN.
	self assert: Float nan successor isNaN.
	
	"SPECIFIC FOR IEEE 754 double precision - 64 bits"
	self assert: Float fmax hex = '7FEFFFFFFFFFFFFF'.
	self assert: Float fminDenormalized hex = '0000000000000001'.
	self assert: Float fminNormalized hex = '0010000000000000'.
	self assert: 0.0 hex = '0000000000000000'.
	self assert: Float negativeZero hex = '8000000000000000'.
	self assert: Float one hex = '3FF0000000000000'.
	self assert: Float infinity hex = '7FF0000000000000'.
	self assert: Float infinity negated hex = 'FFF0000000000000'.! !

!FloatTest methodsFor: 'testing compare' stamp: 'nice 7/19/2009 19:24'!
testCloseTo
	self deny: (Float nan closeTo: Float nan) description: 'NaN isn''t close to anything'.
	self deny: (Float nan closeTo: 1.0) description: 'NaN isn''t close to anything'.
	self deny: (1.0 closeTo: Float nan) description: 'NaN isn''t close to anything'.
	
	self deny: (-1.0 closeTo: 1.0).
	self deny: (1.0 closeTo: Float infinity).
	self assert: (Float infinity closeTo: Float infinity) description: 'since they are =, they also are closeTo:'.
	
	self assert: (1.0/3.0 closeTo: 1/3).
	self assert: (1.0e-8 closeTo: 0).
	self assert: (0 closeTo: 1.0e-8).
	self assert: (1+1.0e-8 closeTo: 1.0).
	
	self assert: (1000000001.0 closeTo: 1000000000.0).
	self deny: (1000000001 closeTo: 1000000000) description: 'exact representation are considered closeTo: only if equal'.! !

!FloatTest methodsFor: 'testing compare' stamp: 'nice 5/30/2008 01:23'!
testComparison
	
	"test equality when Float conversion loose bits"
	| a b c |
	a := 16r1FFFFFFFFFFFFF1.
	b := 16r1FFFFFFFFFFFFF3.
	c := a asFloat.
	self assert: ((a = c) & (b = c)) ==> (a = b).
	
	"Test equality when Float conversion exact"
	self assert: 16r1FFFFFFFFFFFFF = 16r1FFFFFFFFFFFFF asFloat.
	self assert: 16r1FFFFFFFFFFFFF = 16r1FFFFFFFFFFFFF asFloat asInteger.
	
	"Test inequality when Float conversion loose bits"
	self assert: (((1 bitShift: 54)+1)/(1 bitShift: 54)) > 1.
	self assert: (((1 bitShift: 54)+1)/(1 bitShift: 54)) > 1.0.
	
	self assert: (((1 bitShift: 54)-1)/(1 bitShift: 54)) < 1.
	self assert: (((1 bitShift: 54)-1)/(1 bitShift: 54)) < 1.0.
	
	"Test exact vs inexact arithmetic"
	(1 to: 100) do: [:i |
		i isPowerOfTwo
			ifTrue: [self assert: (1/i) = (1/i) asFloat]
			ifFalse: [self deny: (1/i) = (1/i) asFloat]].
	
	"Test overflow (compare to infinity)"
	a := (11 raisedTo: 400) / 2.
	b := (13 raisedTo: 400) / 2.
	c := a asFloat.
	self assert: ((a = c) & (b = c)) ==> (a = b).
	
	"every integer is smaller than infinity"
	self assert: a < Float infinity.
	self assert: a > Float infinity negated.
	
	"Test underflow"
	self deny: 1 / (11 raisedTo: 400) = 0.
	self deny: 1 / (11 raisedTo: 400) = 0.0.
	
	"Test hash code"
	self assert:
		((Set new: 3) add: 3; add: 3.0; size) =
		((Set new: 4) add: 3; add: 3.0; size).! !

!FloatTest methodsFor: 'testing compare' stamp: 'nice 7/10/2009 22:27'!
testComparisonWhenPrimitiveFails
	"This is related to http://bugs.squeak.org/view.php?id=7361"

	self deny: 0.5 < (1/4).
	self deny: 0.5 < (1/2).
	self assert: 0.5 < (3/4).
	
	self deny: 0.5 <= (1/4).
	self assert: 0.5 <= (1/2).
	self assert: 0.5 <= (3/4).
	
	self assert: 0.5 > (1/4).
	self deny: 0.5 > (1/2).
	self deny: 0.5 > (3/4).
	
	self assert: 0.5 >= (1/4).
	self assert: 0.5 >= (1/2).
	self deny: 0.5 >= (3/4).
	
	self deny: 0.5 = (1/4).
	self assert: 0.5 = (1/2).
	self deny: 0.5 = (3/4).
	
	self assert: 0.5 ~= (1/4).
	self deny: 0.5 ~= (1/2).
	self assert: 0.5 ~= (3/4).! !

!FloatTest methodsFor: 'testing - arithmetic' stamp: 'st 9/20/2004 17:04'!
testContinuedFractions
	self assert: (Float pi asApproximateFractionAtOrder: 1) = (22/7).
	self assert: (Float pi asApproximateFractionAtOrder: 3) = (355/113)! !

!FloatTest methodsFor: 'testing' stamp: 'nice 10/4/2009 23:13'!
testCopy
	"Elementary tests"
	self assert: 2.0 copy = 2.0.
	self assert: -0.5 copy = -0.5.
	
	"Are exceptional Floats preserved by the copy ?"
	self assert: Float nan copy isNaN.
	self assert: Float infinity copy = Float infinity.
	self assert: Float infinity negated copy = Float infinity negated.
	
	"Is the sign of zero preserved by the copy ?"
	self assert: 0.0 copy hex = 0.0 hex.
	self assert: Float negativeZero copy hex = Float negativeZero hex.! !

!FloatTest methodsFor: 'testing - arithmetic' stamp: 'GabrielOmarCotelli 6/6/2009 17:14'!
testDivide

	self assert: 1.5 / 2.0 = 0.75.
	
	self assert: 2.0 / 1 = 2.0.
	
	self should: [ 2.0 / 0 ] raise: ZeroDivide.
	self should: [ 2.0 / 0.0 ] raise: ZeroDivide.
	self should: [ 1.2 / Float negativeZero ] raise: ZeroDivide.
	self should: [ 1.2 / (1.3 - 1.3) ] raise: ZeroDivide
	! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 7/24/2008 02:04'!
testFloatRounded
	"5000000000000001 asFloat has an exact representation (no round off error).
	It should round to nearest integer without loosing bits.
	This is a no regression test on http://bugs.squeak.org/view.php?id=7134"
	
	| x y int r |
	
	"This is a preamble asserting exactness of representation
	and quality of various conversions"
	int := 5000000000000001.
	x := int asFloat.
	y := (5 asFloat squared squared squared squared timesTwoPower: 15) + 1.
	self assert: x = y.
	self assert: x asTrueFraction = int.
	
	"this one should be true for any float
	in order to conform to ISO/IEC 10967-2"
	self assert: x rounded = x asTrueFraction rounded.
	self assert: x negated rounded = x negated asTrueFraction rounded.

	"a random test"
	r := Random new.
	10000 timesRepeat: [
		x := r next * 1.9999e16 + 1.0e12 .
		self assert: x rounded = x asTrueFraction rounded.
		self assert: x negated rounded = x negated asTrueFraction rounded]! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 4/26/2006 05:21'!
testFloatTruncated
	"(10 raisedTo: 16) asFloat has an exact representation (no round off error).
	It should convert back to integer without loosing bits.
	This is a no regression test on http://bugs.impara.de/view.php?id=3504"
	
	| x y int r |
	int := 10 raisedTo: 16.
	x := int asFloat.
	y := (5 raisedTo: 16) asFloat timesTwoPower: 16.
	self assert: x = y.
	
	self assert: x asInteger = int.
	
	"this one should be true for any float"
	self assert: x asInteger = x asTrueFraction asInteger.

	"a random test"
	r := Random new.
	10000 timesRepeat: [
		x := r next * 1.9999e16 + 1.0e12 .
		self assert: x truncated = x asTrueFraction truncated]! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 5/7/2006 16:22'!
testFractionAsFloat
	"use a random test"
	
	| r m frac err collec |
	r := Random new seed: 1234567.
	m := (2 raisedTo: 54) - 1.
	200 timesRepeat: [
		frac := ((r nextInt: m) * (r nextInt: m) + 1) / ((r nextInt: m) * (r nextInt: m) + 1).
		err := (frac - frac asFloat asTrueFraction) * frac reciprocal * (1 bitShift: 52).
		self assert: err < (1/2)].
	
	collec := #(16r10000000000000 16r1FFFFFFFFFFFFF 1 2 16r20000000000000 16r20000000000001 16r3FFFFFFFFFFFFF 16r3FFFFFFFFFFFFE 16r3FFFFFFFFFFFFD).
	collec do: [:num |
		collec do: [:den |
			frac := Fraction numerator: num denominator: den.
			err := (frac - frac asFloat asTrueFraction) * frac reciprocal * (1 bitShift: 52).
			self assert: err <= (1/2)]].! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 1/10/2007 02:29'!
testFractionAsFloat2
	"test rounding to nearest even"
		
	self assert: ((1<<52)+0+(1/4)) asFloat asTrueFraction = ((1<<52)+0).
	self assert: ((1<<52)+0+(1/2)) asFloat asTrueFraction = ((1<<52)+0).
	self assert: ((1<<52)+0+(3/4)) asFloat asTrueFraction = ((1<<52)+1).
	self assert: ((1<<52)+1+(1/4)) asFloat asTrueFraction = ((1<<52)+1).
	self assert: ((1<<52)+1+(1/2)) asFloat asTrueFraction = ((1<<52)+2).
	self assert: ((1<<52)+1+(3/4)) asFloat asTrueFraction = ((1<<52)+2).! !

!FloatTest methodsFor: 'tests' stamp: 'nice 6/11/2009 01:36'!
testHash
	self assert: (2 = 2.0) ==> (2 hash = 2.0 hash).
	self assert: (1/2 = 0.5) ==> ((1/2) hash = 0.5 hash).
	
	self shouldnt: [Float nan hash] raise: Error.
	self shouldnt: [Float infinity hash] raise: Error.! !

!FloatTest methodsFor: 'infinity behavior' stamp: 'nice 7/14/2009 09:32'!
testHugeIntegerCloseTo
	"This is a test for bug http://bugs.squeak.org/view.php?id=7368"
	
 	"FloatTest new testHugeIntegerCloseTo"

	self deny: (1.0 closeTo: 200 factorial).
	self deny: (200 factorial closeTo: 1.0).
	self assert: (Float infinity closeTo: 200 factorial) = (200 factorial closeTo: Float infinity).! !

!FloatTest methodsFor: 'infinity behavior' stamp: 'sd 6/5/2005 08:30'!
testInfinity1
   "FloatTest new testInfinity1"

	| i1  i2 |

	i1 := 10000 exp.
	i2 := 1000000000 exp.
	self assert: i1 isInfinite & i2 isInfinite & (i1 = i2).
	"All infinities are equal. (This is a very substantial difference to NaN's, which are never equal."
! !

!FloatTest methodsFor: 'infinity behavior' stamp: 'sd 6/5/2005 08:30'!
testInfinity2
   "FloatTest new testInfinity2"

	| i1  i2 |
	i1 := 10000 exp.
	i2 := 1000000000 exp.
	i2 := 0 - i2. " this is entirely ok. You can compute with infinite values."

	self assert: i1 isInfinite & i2 isInfinite & i1 positive & i2 negative.
	self deny: i1 = i2.
  	"All infinities are signed. Negative infinity is not equal to Infinity"
! !

!FloatTest methodsFor: 'infinity behavior' stamp: 'nice 10/17/2007 23:54'!
testInfinityCloseTo
	"This is a test for bug http://bugs.squeak.org/view.php?id=6729:"
	
 	"FloatTest new testInfinityCloseTo"

	self deny: (Float infinity closeTo: Float infinity negated).
	self deny: (Float infinity negated closeTo: Float infinity).! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 5/6/2006 22:13'!
testIntegerAsFloat
	"assert IEEE 754 round to nearest even mode is honoured"
	
	self deny: 16r1FFFFFFFFFFFF0801 asFloat = 16r1FFFFFFFFFFFF0800 asFloat. "this test is on 65 bits"
	self deny: 16r1FFFFFFFFFFFF0802 asFloat = 16r1FFFFFFFFFFFF0800 asFloat. "this test is on 64 bits"
	self assert: 16r1FFFFFFFFFFF1F800 asFloat = 16r1FFFFFFFFFFF20000 asFloat. "nearest even is upper"
	self assert: 16r1FFFFFFFFFFFF0800 asFloat = 16r1FFFFFFFFFFFF0000 asFloat. "nearest even is lower"
! !

!FloatTest methodsFor: 'zero behavior' stamp: 'md 4/16/2003 15:02'!
testIsZero
	self assert: 0.0 isZero.
	self deny:  0.1 isZero.! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'sd 6/5/2005 08:31'!
testNaN1
   	"FloatTest new testNaN1"

	self assert: Float nan == Float nan.
	self deny: Float nan = Float nan.
	"a NaN is not equal to itself."
! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'dtl 10/1/2004 18:26'!
testNaN2
	"Two NaN values are always considered to be different.
	On an little-endian machine (32 bit Intel), Float nan is 16rFFF80000 16r00000000.
	On a big-endian machine (PowerPC), Float nan is 16r7FF80000 16r00000000. Changing
	the bit pattern of the first word of a NaN produces another value that is still
	considered equal to NaN. This test should work on both little endian and big
	endian machines. However, it is not guaranteed to work on future 64 bit versions
	of Squeak, for which Float may have different internal representations."

	"FloatTest new testNaN2"

	| nan1 nan2 |
	nan1 := Float nan copy.
	nan2 := Float nan copy.

	"test two instances of NaN with the same bit pattern"
	self deny: nan1 = nan2.
	self deny: nan1 == nan2.
	self deny: nan1 = nan1.
	self assert: nan1 == nan1.

	"change the bit pattern of nan1"
	self assert: nan1 size == 2.
	self assert: (nan1 at: 2) = 0.
	nan1 at: 1 put: (nan1 at: 1) + 999.
	self assert: nan1 isNaN.
	self assert: nan2 isNaN.
	self deny: (nan1 at: 1) = (nan2 at: 1).

	"test two instances of NaN with different bit patterns"
	self deny: nan1 = nan2.
	self deny: nan1 == nan2.
	self deny: nan1 = nan1.
	self assert: nan1 == nan1
! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'sd 6/5/2005 08:32'!
testNaN3
   "FloatTest new testNaN3"

   	| set item identitySet |
	set := Set new.
	set add: (item := Float nan).
	self deny: (set includes: item).
	identitySet := IdentitySet new.
	identitySet add: (item := Float nan).
	self assert: (identitySet includes: item).
	"as a NaN is not equal to itself, it can not be retrieved from a set"
! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'sd 6/5/2005 08:32'!
testNaN4
   	"FloatTest new testNaN4"

	| dict |
	dict := Dictionary new.
	dict at: Float nan put: #NaN.
	self deny: (dict includes: Float nan).
	"as a NaN is not equal to itself, it can not be retrieved when it is used as a dictionary key"
! !

!FloatTest methodsFor: 'IEEE 754' stamp: 'al 6/22/2008 11:52'!
testNaN5
	self assert: ((Float nan asIEEE32BitWord printPaddedWith: $0 to: 32 base: 2)
		copyFrom: 2 to: 9) = '11111111'.
	self assert: (Float fromIEEE32Bit:
		(Integer readFrom: '01111111110000000000000000000000' readStream base: 2)) isNaN! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'nice 10/11/2007 00:09'!
testNaNCompare
	"IEEE 754 states that NaN cannot be ordered.
	As a consequence, every arithmetic comparison involving a NaN SHOULD return false.
	Except the is different test (~=).
	This test does verify this rule"
	
	| compareSelectors theNaN anotherNaN comparand brokenMethods warningMessage |
	compareSelectors := #(#< #<= #> #>= #=).
	theNaN := Float nan.
	anotherNaN := Float infinity - Float infinity.
	comparand := {1. 2.3. Float infinity. 2/3. 1.25s2. 2 raisedTo: 50}.
	comparand := comparand , (comparand collect: [:e | e negated]).
	comparand := comparand , {theNaN. anotherNaN}.

"do a first pass to collect all broken methods"
	brokenMethods := Set new.
	comparand do: [:comp |
		compareSelectors do: [:op |
			(theNaN perform: op with: comp) ifTrue: [brokenMethods add: (theNaN class lookupSelector: op)].
			(comp perform: op with: theNaN) ifTrue: [brokenMethods add: (comp class lookupSelector: op)]].
		(theNaN ~= comp) ifFalse: [brokenMethods add: (theNaN class lookupSelector: #~=)].
		(comp ~= theNaN) ifFalse: [brokenMethods add: (comp class lookupSelector: #~=)]].
	
"build a warning message to tell about all broken methods at once"
	warningMessage := String streamContents: [:s |
			s nextPutAll: 'According to IEEE 754 comparing with a NaN should always return false, except ~= that should return true.'; cr.
			s nextPutAll: 'All these methods failed to do so. They are either broken or call a broken one'.
			brokenMethods do: [:e | s cr; print: e methodClass; nextPutAll: '>>'; print: e selector]].
		
"Redo the tests so as to eventually open a debugger on one of the failures"
	brokenMethods := Set new.
	comparand do: [:comp2 |
		compareSelectors do: [:op2 |
			self deny: (theNaN perform: op2 with: comp2) description: warningMessage.
			self deny: (comp2 perform: op2 with: theNaN) description: warningMessage].
		self assert: (theNaN ~= comp2) description: warningMessage.
		self assert: (comp2 ~= theNaN) description: warningMessage].! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'nice 3/14/2008 23:42'!
testNaNisLiteral
	self deny: Float nan isLiteral description: 'there is no literal representation of NaN'! !

!FloatTest methodsFor: 'testing - arithmetic' stamp: 'nice 12/1/2007 17:59'!
testRaisedTo
	"this is a test related to http://bugs.squeak.org/view.php?id=6781"
	
	self should: [0.0 raisedTo: -1] raise: ZeroDivide.
	self should: [0.0 raisedTo: -1.0] raise: ZeroDivide.! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'nice 3/14/2008 23:59'!
testReadFromManyDigits
	"A naive algorithm may interpret these representations as Infinity or NaN.
	This is http://bugs.squeak.org/view.php?id=6982"
	
	| s1 s2 |
	s1 := '1' , (String new: 321 withAll: $0) , '.0e-321'.
	s2 := '0.' , (String new: 320 withAll: $0) , '1e321'.
	self assert: (Number readFrom: s1) = 1.
	self assert: (Number readFrom: s2) = 1.! !

!FloatTest methodsFor: 'NaN behavior' stamp: 'GabrielOmarCotelli 5/23/2009 20:38'!
testReciprocal

	self 
		assert: 1.0 reciprocal = 1.0;
		assert: 2.0 reciprocal = 0.5;
		assert: -1.0 reciprocal = -1.0;
		assert: -2.0 reciprocal = -0.5.
		
	self should: [ 0.0 reciprocal ] raise: ZeroDivide! !

!FloatTest methodsFor: 'testing compare' stamp: 'StephaneDucasse 2/13/2010 11:18'!
testSetOfFloat
      "Classical disagreement between hash and = did lead to a bug.
       This is a non regression test from http://bugs.squeak.org/view.php?id=3360"
	"self debug: #testSetOfFloat"

       | size3 size4 |
       size3 := (Set new: 3) add: 3; add: 3.0; size.
       size4 := (Set new: 4) add: 3; add: 3.0; size.
       self assert: size3 = size4 description: 'The size of a Set should not depend on its capacity.'! !

!FloatTest methodsFor: 'printing' stamp: 'nice 10/11/2008 21:45'!
testStoreBase16
	"This bug was reported in mantis http://bugs.squeak.org/view.php?id=6695"

	self
		assert: (20.0 storeStringBase: 16) = '16r14.0'
		description: 'the radix prefix should not be omitted, except in base 10'! !

!FloatTest methodsFor: 'testing - conversion' stamp: 'dtl 9/18/2004 12:40'!
testStringAsNumber
	"This covers parsing in Number>>readFrom:"

	| aFloat |
	aFloat := '10r-12.3456' asNumber.
	self assert: -12.3456 = aFloat.
	aFloat := '10r-12.3456e2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '10r-12.3456d2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '10r-12.3456q2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '-12.3456q2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '12.3456q2' asNumber.
	self assert: 1234.56 = aFloat.
! !

!FloatTest methodsFor: 'zero behavior' stamp: 'sd 6/5/2005 08:33'!
testZero1
	"FloatTest new testZero1"

	self assert: Float negativeZero = 0 asFloat.
	self assert: (Float negativeZero at: 1) ~= (0 asFloat at: 1).

	"The negative zero has a bit representation that is different from the bit representation of the positive zero. Nevertheless, both values are defined to be equal."
! !

!FloatTest methodsFor: 'zero behavior' stamp: 'nice 3/23/2008 16:00'!
testZeroSignificandAsInteger
	"This is about http://bugs.squeak.org/view.php?id=6990"
	
	self assert: 0.0 significandAsInteger = 0! !

ClassTestCase subclass: #FractionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!FractionTest methodsFor: 'tests - sinuses' stamp: 'sd 3/4/2004 21:13'!
testDegreeCos
	"self run: #testDegreeCos"
	
	self shouldnt: [ (4/3) degreeCos] raise: Error.
	self assert: (1/3) degreeCos printString =  '0.999983076857744'! !

!FractionTest methodsFor: 'tests - sinuses' stamp: 'sd 3/5/2004 14:54'!
testDegreeSin
	"self run: #testDegreeSin"
	
	self shouldnt: [ (4/3) degreeSin] raise: Error.
	self assert: (1/3) degreeSin printString =  '0.005817731354993834'.! !

!FractionTest methodsFor: 'tests - printing' stamp: 'nice 3/5/2010 22:35'!
testFractionPrinting

	self assert: (353/359) printString = '(353/359)'.
	self assert: ((2/3) printStringBase: 2) = '(10/11)'.
	self assert: ((2/3) storeStringBase: 2) = '(2r10/2r11)'.
	self assert: ((5/7) printStringBase: 3) = '(12/21)'.
	self assert: ((5/7) storeStringBase: 3) = '(3r12/3r21)'.
	self assert: ((11/13) printStringBase: 4) = '(23/31)'.
	self assert: ((11/13) storeStringBase: 4) = '(4r23/4r31)'.
	self assert: ((17/19) printStringBase: 5) = '(32/34)'.
	self assert: ((17/19) storeStringBase: 5) = '(5r32/5r34)'.
	self assert: ((23/29) printStringBase: 6) = '(35/45)'.
	self assert: ((23/29) storeStringBase: 6) = '(6r35/6r45)'.
	self assert: ((31/37) printStringBase: 7) = '(43/52)'.
	self assert: ((31/37) storeStringBase: 7) = '(7r43/7r52)'.
	self assert: ((41/43) printStringBase: 8) = '(51/53)'.
	self assert: ((41/43) storeStringBase: 8) = '(8r51/8r53)'.
	self assert: ((47/53) printStringBase: 9) = '(52/58)'.
	self assert: ((47/53) storeStringBase: 9) = '(9r52/9r58)'.
	self assert: ((59/61) printStringBase: 10) = '(59/61)'.
	self assert: ((59/61) storeStringBase: 10) = '(59/61)'.
	self assert: ((67/71) printStringBase: 11) = '(61/65)'.
	self assert: ((67/71) storeStringBase: 11) = '(11r61/11r65)'.
	self assert: ((73/79) printStringBase: 12) = '(61/67)'.
	self assert: ((73/79) storeStringBase: 12) = '(12r61/12r67)'.
	self assert: ((83/89) printStringBase: 13) = '(65/6B)'.
	self assert: ((83/89) storeStringBase: 13) = '(13r65/13r6B)'.
	self assert: ((97/101) printStringBase: 14) = '(6D/73)'.
	self assert: ((97/101) storeStringBase: 14) = '(14r6D/14r73)'.
	self assert: ((103/107) printStringBase: 15) = '(6D/72)'.
	self assert: ((103/107) storeStringBase: 15) = '(15r6D/15r72)'.
	self assert: ((109/113) printStringBase: 16) = '(6D/71)'.
	self assert: ((109/113) storeStringBase: 16) = '(16r6D/16r71)'.
	self assert: ((127/131) printStringBase: 17) = '(78/7C)'.
	self assert: ((127/131) storeStringBase: 17) = '(17r78/17r7C)'.
	self assert: ((137/139) printStringBase: 18) = '(7B/7D)'.
	self assert: ((137/139) storeStringBase: 18) = '(18r7B/18r7D)'.
	self assert: ((149/151) printStringBase: 19) = '(7G/7I)'.
	self assert: ((149/151) storeStringBase: 19) = '(19r7G/19r7I)'.
	self assert: ((157/163) printStringBase: 20) = '(7H/83)'.
	self assert: ((157/163) storeStringBase: 20) = '(20r7H/20r83)'.
	self assert: ((167/173) printStringBase: 21) = '(7K/85)'.
	self assert: ((167/173) storeStringBase: 21) = '(21r7K/21r85)'.
	self assert: ((179/181) printStringBase: 22) = '(83/85)'.
	self assert: ((179/181) storeStringBase: 22) = '(22r83/22r85)'.
	self assert: ((191/193) printStringBase: 23) = '(87/89)'.
	self assert: ((191/193) storeStringBase: 23) = '(23r87/23r89)'.
	self assert: ((197/199) printStringBase: 24) = '(85/87)'.
	self assert: ((197/199) storeStringBase: 24) = '(24r85/24r87)'.
	self assert: ((211/223) printStringBase: 25) = '(8B/8N)'.
	self assert: ((211/223) storeStringBase: 25) = '(25r8B/25r8N)'.
	self assert: ((227/229) printStringBase: 26) = '(8J/8L)'.
	self assert: ((227/229) storeStringBase: 26) = '(26r8J/26r8L)'.
	self assert: ((233/239) printStringBase: 27) = '(8H/8N)'.
	self assert: ((233/239) storeStringBase: 27) = '(27r8H/27r8N)'.
	self assert: ((241/251) printStringBase: 28) = '(8H/8R)'.
	self assert: ((241/251) storeStringBase: 28) = '(28r8H/28r8R)'.
	self assert: ((257/263) printStringBase: 29) = '(8P/92)'.
	self assert: ((257/263) storeStringBase: 29) = '(29r8P/29r92)'.
	self assert: ((269/271) printStringBase: 30) = '(8T/91)'.
	self assert: ((269/271) storeStringBase: 30) = '(30r8T/30r91)'.
	self assert: ((277/281) printStringBase: 31) = '(8T/92)'.
	self assert: ((277/281) storeStringBase: 31) = '(31r8T/31r92)'.
	self assert: ((283/293) printStringBase: 32) = '(8R/95)'.
	self assert: ((283/293) storeStringBase: 32) = '(32r8R/32r95)'.
	self assert: ((307/311) printStringBase: 33) = '(9A/9E)'.
	self assert: ((307/311) storeStringBase: 33) = '(33r9A/33r9E)'.
	self assert: ((313/317) printStringBase: 34) = '(97/9B)'.
	self assert: ((313/317) storeStringBase: 34) = '(34r97/34r9B)'.
	self assert: ((331/337) printStringBase: 35) = '(9G/9M)'.
	self assert: ((331/337) storeStringBase: 35) = '(35r9G/35r9M)'.
	self assert: ((347/349) printStringBase: 36) = '(9N/9P)'.
	self assert: ((347/349) storeStringBase: 36) = '(36r9N/36r9P)'.

	self assert: ((-2/3) printStringBase: 2) = '(-10/11)'.
	self assert: ((-2/3) storeStringBase: 2) = '(-2r10/2r11)'.
	self assert: ((5 / -7) printStringBase: 3) = '(-12/21)'.
	self assert: ((5 / -7) storeStringBase: 3) = '(-3r12/3r21)'.
! !

!FractionTest methodsFor: 'tests - sinuses' stamp: 'GabrielOmarCotelli 5/23/2009 20:19'!
testReciprocal

	self 
		assert: (1/2) reciprocal = 2;
		assert: (3/4) reciprocal = (4/3);
		assert: (-1/3) reciprocal = -3;
		assert: (-3/5) reciprocal = (-5/3)! !

ClassTestCase subclass: #InstructionPrinterTest
	instanceVariableNames: 'tt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!InstructionPrinterTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class InstructionPrinter. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!InstructionPrinterTest methodsFor: 'examples' stamp: 'md 4/8/2003 12:28'!
example1
	| ff|
	(1 < 2) ifTrue: [tt ifNotNil: [ff := 'hallo']].
	^ ff.! !

!InstructionPrinterTest methodsFor: 'tests' stamp: 'marcus.denker 8/24/2008 13:19'!
testInstructions
	"just print all of methods of Object and see if no error accours"

	| printer  |

	printer  := InstructionPrinter.	

	Object methods do: [:method |
					self shouldnt: [ 
						String streamContents: [:stream | 
							(printer on: method) printInstructionsOn: stream]] raise: Error.
			].
! !

ClassTestCase subclass: #LargeNegativeIntegerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!LargeNegativeIntegerTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:36'!
testEmptyTemplate
	"Check that an uninitialized instance behaves reasonably."

	| i |
	i := LargeNegativeInteger new: 4.
	self assert: i size == 4.
	self assert: i printString = '-0'.
	self assert: i normalize == 0! !

ClassTestCase subclass: #LargePositiveIntegerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!LargePositiveIntegerTest methodsFor: 'tests' stamp: 'md 3/17/2003 15:20'!
testBitShift

	"Check bitShift from and back to SmallInts"
	
	1 to: 257 do: [:i | self should: [((i bitShift: i) bitShift: 0-i) == i]].! !

!LargePositiveIntegerTest methodsFor: 'tests' stamp: 'dtl 5/26/2004 18:33'!
testEmptyTemplate

	"Check that an uninitialized instance behaves reasonably."

	| i |
	i := LargePositiveInteger new: 4.
	self assert: i size == 4.
	self assert: i printString = '0'.
	self assert: i normalize == 0! !

!LargePositiveIntegerTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:52'!
testMultDicAddSub
	"self run: #testMultDicAddSub"

	| n f f1 |	
	n := 100.
	f := 100 factorial.
	f1 := f*(n+1).
	n timesRepeat: [f1 := f1 - f].
	self assert: (f1 = f). 

	n timesRepeat: [f1 := f1 + f].
	self assert: (f1 // f = (n+1)). 
	self assert: (f1 negated = (Number readFrom: '-' , f1 printString)).! !

!LargePositiveIntegerTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:53'!
testNormalize
	"self run: #testNormalize"
	"Check normalization and conversion to/from SmallInts"

	self assert: ((SmallInteger maxVal + 1 - 1) == SmallInteger maxVal).
	self assert: (SmallInteger maxVal + 3 - 6) == (SmallInteger maxVal-3).
	self should: ((SmallInteger minVal - 1 + 1) == SmallInteger minVal).
	self assert: (SmallInteger minVal - 3 + 6) == (SmallInteger minVal+3).! !

ClassTestCase subclass: #MonthTest
	instanceVariableNames: 'month'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!MonthTest commentStamp: 'brp 7/26/2003 22:44' prior: 0!
This is the unit test for the class Month.
!

!MonthTest methodsFor: 'Coverage' stamp: 'brp 7/27/2003 12:42'!
classToBeTested

	^ Month! !

!MonthTest methodsFor: 'Coverage' stamp: 'brp 7/26/2003 23:29'!
selectorsToBeIgnored

	| deprecated private special |
	deprecated := #().
	private := #( #printOn: ).
	special := #( #next ).

	^ super selectorsToBeIgnored, deprecated, private, special.! !

!MonthTest methodsFor: 'Running' stamp: 'brp 8/6/2003 19:37'!
setUp

	super setUp.
	month := Month month: 7 year: 1998.! !

!MonthTest methodsFor: 'Running' stamp: 'brp 8/6/2003 19:37'!
tearDown

	super tearDown.
	month := nil.! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 22:52'!
testConverting

	self assert: month asDate = '1 July 1998' asDate! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 1/30/2005 09:35'!
testEnumerating
	| weeks |
	weeks := OrderedCollection new.
	month weeksDo: [ :w | weeks add: w start ].
	0 to: 4 do: [ :i | weeks remove: (Week starting:  ('29 June 1998' asDate addDays: i * 7)) start ].
	self assert: weeks isEmpty! !

!MonthTest methodsFor: 'Tests' stamp: 'lr 3/7/2010 11:59'!
testIndexOfMonth
	"self debug: #testIndexOfMonth"
       | m |
       m := #(#January #February #March #April #May #June #July #August
#September #October #November #December).

       m withIndexDo: [:item :index | self assert: (Month indexOfMonth:
item) = index].

       self should: [Month indexOfMonth: 1] raise: TestResult error.
       self should: [Month indexOfMonth: #Marsh] raise: TestResult error.
"notice the misspell!!!!"
       self should: [Month indexOfMonth: #UnexistingMonth] raise:
TestResult error.! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 8/23/2003 16:08'!
testInquiries

	self 
		assert: month index = 7;
		assert: month name = #July;
		assert: month duration = (31 days).
! !

!MonthTest methodsFor: 'Tests' stamp: 'nk 7/30/2004 17:52'!
testInstanceCreation
	| m1 m2 |
	m1 := Month starting:  '4 July 1998' asDate.
	m2 := Month month: #July year: 1998.
	self
		assert: month = m1;
		assert: month = m2! !

!MonthTest methodsFor: 'Tests' stamp: 'StephaneDucasse 2/2/2010 13:45'!
testNameOfMonth

       | m |
       m := #(#January #February #March #April #May #June #July #August
#September #October #November #December).

       m withIndexDo: [:item :index | self assert: (Month nameOfMonth:
index) = item].

       self should: [Month nameOfMonth: 0] raise: TestResult error.
       self should: [Month nameOfMonth: 13] raise: TestResult error.
       self should: [Month nameOfMonth: #January] raise: TestResult error.! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 23:02'!
testPreviousNext
	| n p |
	n := month next.
	p := month previous.

	self
		assert: n year = 1998;
		assert: n index = 8;
		assert: p year = 1998;
		assert: p index = 6.

! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 22:50'!
testPrinting

	self 
		assert: month printString = 'July 1998'.
! !

!MonthTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 22:46'!
testReadFrom

	| m |
	m := Month readFrom: 'July 1998' readStream.
	self 
		assert: m = month! !

ClassTestCase subclass: #NumberTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/26/2009 21:57'!
testFractionPart

	self 
		assert: 2 fractionPart = 0;
		assert: (1/2) fractionPart = (1/2);
		assert: (4/3) fractionPart = (1/3);
		assert: 2.0 fractionPart = 0.0;
		assert: 0.5 fractionPart = 0.5;
		assert: 2.5 fractionPart = 0.5
! !

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/26/2009 21:55'!
testIntegerPart

	self 
		assert: 2 integerPart = 2;
		assert: (1/2) integerPart = 0;
		assert: (4/3) integerPart = 1;
		assert: 2.0 integerPart = 2.0;
		assert: 0.5 integerPart = 0.0;
		assert: 2.5 integerPart = 2.0
! !

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/23/2009 20:49'!
testOne

	self 
		assert: Integer one = 1;
		assert: Float one = 1.0;
		assert: Fraction one = 1! !

!NumberTest methodsFor: 'tests' stamp: 'adrian_lienhard 1/7/2009 17:55'!
testPrintShowingDecimalPlaces
	self assert: (111.2 printShowingDecimalPlaces: 2) = '111.20'.
	self assert: (111.2 printShowingDecimalPlaces: 0) = '111'.
	self assert: (111 printShowingDecimalPlaces: 0) = '111'.
	self assert: (111111111111111 printShowingDecimalPlaces: 2) = '111111111111111.00'.
	self assert: (10 printShowingDecimalPlaces: 20) ='10.00000000000000000000'.
	self assert: (0.98 printShowingDecimalPlaces: 2) = '0.98'.
	self assert: (-0.98 printShowingDecimalPlaces: 2) = '-0.98'.
	self assert: (2.567 printShowingDecimalPlaces: 2) = '2.57'.
	self assert: (-2.567 printShowingDecimalPlaces: 2) = '-2.57'.
	self assert: (0.01 printShowingDecimalPlaces: 2) = '0.01'.
	self assert: (-0.001 printShowingDecimalPlaces: 2) = '0.00'.! !

!NumberTest methodsFor: 'tests' stamp: 'nice 3/24/2008 16:50'!
testPrintShowingDecimalPlaces2
	"This tests problems related to Float>>rounded and Float>>roundTo::
	- Float>>#rounded is inexact
	- Float>>#roundTo: might overflow"

	"5000000000000001.0 asTrueFraction = 5000000000000001.
	5000000000000001 highBit = 53.
	This number is represented exactly asFloat, it should print exactly.
	Beware, 5000000000000001.0 rounded is inexact and will answer 5000000000000002"
	self assert: (5000000000000001.0 printShowingDecimalPlaces: 0) = '5000000000000001'.
	
	"50000000000001.25 asTrueFraction = (200000000000005/4).
	200000000000005 highBit = 48, 4 isPowerOfTwo,
	So this number is also represented exactly as Float, it should print exactly.
	Beware: (50000000000001.25 / 0.01) rounded exhibit the same problem as above."
	self assert: (50000000000001.25 printShowingDecimalPlaces: 2) = '50000000000001.25'.
	
	"This number is close to maximum float value"
	self shouldnt: [1.0e306 printShowingDecimalPlaces: 3] raise: Error.! !

!NumberTest methodsFor: 'tests' stamp: 'nice 4/24/2008 00:58'!
testPrintShowingDecimalPlaces3
	"This problem were reported at http://bugs.squeak.org/view.php?id=7028
	unfortunate inversion of left / right padding"

	self assert: (1.009 printShowingDecimalPlaces: 3) = '1.009'.
	self assert: (35.900 printShowingDecimalPlaces: 3) = '35.900'.
	self assert: (-0.097 printShowingDecimalPlaces: 3) = '-0.097'.! !

!NumberTest methodsFor: 'tests' stamp: 'nice 12/6/2007 21:24'!
testRaisedTo
	"this is a test related to http://bugs.squeak.org/view.php?id=6781"
	
	self should: [0 raisedTo: -1] raise: ZeroDivide.
	self should: [0 raisedTo: -1.0] raise: ZeroDivide.! !

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2009 16:41'!
testRaisedToInteger

	self 
		assert: (2 raisedToInteger: 0) = 1;
		assert: (2 raisedToInteger: 1) = 2;
		assert: (2 raisedToInteger: 4) = 16;
		assert: (0 raisedToInteger: 0) = 1;
		assert: (0 raisedToInteger: 2) = 0;
		assert: (2 raisedToInteger: -1) = (1/2);
		assert: (2 raisedToInteger: -4) = (1/16).
	
	self 
		assert: (-3 raisedTo: 0) = 1;
		assert: (-3 raisedTo: 1) = -3;
		assert: (-3 raisedTo: 2) = 9;
		assert: (-3 raisedTo: 3) = -27;
		assert: (-3 raisedTo: -2) = (1/9);
		assert: (-3 raisedTo: -3) = (-1/27).
	
	self should: [ 0 raisedTo: -1 ] raise: ZeroDivide! !

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/25/2009 16:46'!
testRaisedToIntegerWithFloats

	self 
		assert: (2.0 raisedToInteger: 0) = 1.0;
		assert: (2.0 raisedToInteger: 1) = 2.0;
		assert: (2.0 raisedToInteger: 4) = 16.0;
		assert: (0.0 raisedToInteger: 0) = 1.0;
		assert: (0.0 raisedToInteger: 2) = 0.0;
		assert: (2.0 raisedToInteger: -1) = 0.5;
		assert: (2.0 raisedToInteger: -4) = 0.0625.
	
	self 
		assert: (-3.0 raisedTo: 0) = 1.0;
		assert: (-3.0 raisedTo: 1) = -3.0;
		assert: (-3.0 raisedTo: 2) = 9.0;
		assert: (-3.0 raisedTo: 3) = -27.0;
		assert: (-2.0 raisedTo: -2) = 0.25;
		assert: (-2.0 raisedTo: -3) = -0.125.
	
	self should: [ 0.0 raisedTo: -1 ] raise: ZeroDivide! !

!NumberTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:56'!
testReadFrom
	
	self assert: 1.0e-14	= (Number readFrom: '1.0e-14').
	self assert: 2r1e26	= (Number readFrom: '2r1e26').! !

!NumberTest methodsFor: 'tests' stamp: 'GabrielOmarCotelli 5/23/2009 19:26'!
testReciprocal

	self 
		assert: 1 reciprocal = 1;
		assert: 2 reciprocal = (1/2);
		assert: -1 reciprocal = -1;
		assert: -3 reciprocal = (-1/3).
		
	self should: [ 0 reciprocal ] raise: ZeroDivide! !

ClassTestCase subclass: #ObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!ObjectTest methodsFor: 'private' stamp: 'md 10/15/2004 13:45'!
a
	self b.! !

!ObjectTest methodsFor: 'private' stamp: 'md 10/15/2004 13:46'!
a1
	self b1.! !

!ObjectTest methodsFor: 'private' stamp: 'md 10/15/2004 13:45'!
b
	self haltIf: #testHaltIf.! !

!ObjectTest methodsFor: 'private' stamp: 'md 10/15/2004 13:46'!
b1
	self haltIf: #testasdasdfHaltIf.! !

!ObjectTest methodsFor: 'tests - debugging' stamp: 'sd 6/5/2005 09:05'!
testAssert

	self shouldnt: [Object assert: [true]] raise: Error.
	self shouldnt: [Object assert: true] raise: Error.
	self should: [Object assert: [false]] raise: AssertionFailure.
	self should: [Object assert: false] raise: AssertionFailure.! !

!ObjectTest methodsFor: 'tests' stamp: 'md 11/26/2004 16:37'!
testBecome
	"self debug: #testBecome"
	"this test should that all the variables pointing to an object are pointing now to another one, and all
      object pointing to the other are pointing to the object"

	| pt1 pt2 pt3 |
	pt1 := 0@0.
	pt2 := pt1.
	pt3 := 100@100.

	pt1 become: pt3.
	self assert: pt2 = (100@100).
	self assert: pt3 = (0@0).
	self assert: pt1 = (100@100).! !

!ObjectTest methodsFor: 'tests' stamp: 'md 11/26/2004 16:36'!
testBecomeForward
	"self debug: #testBecomeForward"
	"this test should that all the variables pointing to an object are pointing now to another one.
	Not that this inverse is not true. This kind of become is called oneWayBecome in VW"

	| pt1 pt2 pt3 |
	pt1 := 0@0.
	pt2 := pt1.
	pt3 := 100@100.
	pt1 becomeForward: pt3.
	self assert: pt2 = (100@100).
	self assert: pt3 == pt2.
	self assert: pt1 = (100@100)! !

!ObjectTest methodsFor: 'tests - debugging' stamp: 'sd 6/5/2005 09:05'!
testHaltIf

	self should: [self haltIf: true] raise: Halt.
	self shouldnt: [self haltIf: false] raise: Halt.

	self should: [self haltIf: [true]] raise: Halt.
	self shouldnt: [self haltIf: [false]] raise: Halt.

	self should: [self haltIf: #testHaltIf.] raise: Halt.
	self shouldnt: [self haltIf: #teadfasdfltIf.] raise: Halt.

	self should: [self a] raise: Halt.
	self shouldnt: [self a1] raise: Halt.

	self should: [self haltIf: [:o | o class = self class]] raise: Halt.
	self shouldnt: [self haltIf: [:o | o class ~= self class]] raise: Halt.
! !

ClassTestCase subclass: #ProtoObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!ProtoObjectTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class ProtoObject. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:05'!
testFlag
	
	self shouldnt: [ProtoObject new flag: #hallo] raise: Error.! !

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'ul 12/18/2009 15:40'!
testIfNil
	
	| object block |
	object := ProtoObject new.
	self shouldnt: [ object ifNil: [ self halt ]] raise: Halt.
	self assert: (object ifNil: [ nil ]) == object.
	"Now the same without inlining."
	block := [ self halt ].
	self shouldnt: [ object ifNil: block ] raise: Halt.
	block := [ nil ].
	self assert: (object ifNil: block) == object.
	
! !

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'ul 12/18/2009 15:49'!
testIfNilIfNotNil

	| object returnValue block |
	object := ProtoObject new.
	returnValue := Object new.
	self should: [ object ifNil: [ self error ] ifNotNil: [ self halt ] ] raise: Halt.
	self should: [ object ifNil: [ self error ] ifNotNil: [ :o | self halt ] ] raise: Halt.
	self assert: (object ifNil: [ false ] ifNotNil: [ :o | o == object ]).
	self assert: (object ifNil: [ nil ] ifNotNil: [ returnValue ]) == returnValue.
	self assert: (object ifNil: [ nil ] ifNotNil: [ :o | returnValue ]) == returnValue.
	"Now the same without inlining."
	block := [ self halt ].
	self should: [ object ifNil: [ self error ] ifNotNil: block ] raise: Halt.
	block := [ :o | self halt ].
	self should: [ object ifNil: [ self error ] ifNotNil: block ] raise: Halt.
	block := [ :o | o == object ].
	self assert: (object ifNil: [ false ] ifNotNil: block).
	block := [ returnValue ].
	self assert: (object ifNil: [ nil ] ifNotNil: block) = returnValue.
	block := [ :o | returnValue ].
	self assert: (object ifNil: [ nil ] ifNotNil: block) = returnValue! !

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'ul 12/18/2009 15:48'!
testIfNotNil

	| object returnValue block |
	object := ProtoObject new.
	returnValue := Object new.
	self should: [ object ifNotNil: [ self halt ] ] raise: Halt.
	self should: [ object ifNotNil: [ :o | self halt ] ] raise: Halt.
	self assert: (object ifNotNil: [ :o | o == object ]).
	self assert: (object ifNotNil: [ returnValue ]) == returnValue.
	self assert: (object ifNotNil: [ :o | returnValue ]) == returnValue.	
	"Now the same without inlining."
	block := [ self halt ].
	self should: [ object ifNotNil: block ] raise: Halt.
	block := [ :o | self halt ].
	self should: [ object ifNotNil: block ] raise: Halt.
	block := [ :o | o == object ].
	self assert: (object ifNotNil: block).
	block := [ returnValue ].
	self assert: (object ifNotNil: block) = returnValue.
	block := [ :o | returnValue ].
	self assert: (object ifNotNil: block) = returnValue! !

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'ul 12/18/2009 15:48'!
testIfNotNilIfNil

	| object returnValue block |
	object := ProtoObject new.
	returnValue := Object new.
	self should: [ object ifNotNil: [ self halt ] ifNil: [ self error ]  ] raise: Halt.
	self should: [ object ifNotNil: [ :o | self halt ] ifNil: [ self error ] ] raise: Halt.
	self assert: (object ifNotNil: [ :o | o == object ] ifNil: [ false ]).
	self assert: (object ifNotNil: [ returnValue ] ifNil: [ false ]) == returnValue.
	self assert: (object ifNotNil: [ :o | returnValue ] ifNil: [ false ]) == returnValue.
	"Now the same without inlining."
	block := [ self halt ].
	self should: [ object ifNotNil: block ifNil: [ self error ]  ] raise: Halt.
	block := [ :o | self halt ].
	self should: [ object ifNotNil: block ifNil: [ self error ] ] raise: Halt.
	block := [ :o | o == object ].
	self assert: (object ifNotNil: block ifNil: [ false ]).
	block := [ returnValue ].
	self assert: (object ifNotNil: block ifNil: [ false ]) == returnValue.
	block := [ :o | returnValue ].
	self assert: (object ifNotNil: block ifNil: [ false ]) == returnValue! !

!ProtoObjectTest methodsFor: 'tests - testing' stamp: 'ul 12/18/2009 15:48'!
testIsNil

	self deny: ProtoObject new isNil! !

ClassTestCase subclass: #RandomTest
	instanceVariableNames: 'gen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!RandomTest methodsFor: 'setup' stamp: 'md 4/2/2003 12:32'!
setUp
	gen := Random seed: 112629.! !

!RandomTest methodsFor: 'tests' stamp: 'md 4/2/2003 12:50'!
testNext

	10000 timesRepeat: [
			| next | 
			next := gen next.
			self assert: (next >= 0).
			self assert: (next < 1).
	].! !

ClassTestCase subclass: #ScaledDecimalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!ScaledDecimalTest commentStamp: '<historical>' prior: 0!
I provide a test suite for ScaledDecimal values. Examine my tests to see how SmallIntegers should behave, and see how to use them.!

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 12:22'!
testAsNumber
	"Ensure no loss of precision"

	| sd |
	sd := '1.40s2' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 2.
	self assert: '1.40s2' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 15:51'!
testAsNumberNegatedWithoutDecimalPoint

	| sd |
	sd := '-123s0' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 0.
	self assert: '-123s0' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 15:51'!
testAsNumberNegatedWithoutDecimalPoint2

	| sd |
	sd := '-123s2' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 2.
	self assert: '-123.00s2' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 12:21'!
testAsNumberWithExtendedScale

	| sd |
	sd := '123s2' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 2.
	self assert: '123.00s2' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 12:28'!
testAsNumberWithRadix

	| sd |
	sd := '10r-22.2s5' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 5.
	self assert: '-22.20000s5' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 6/11/2009 03:54'!
testAsNumberWithSuperfluousDecimalPoint

	| sd |
	sd := '123.s2' asNumber.
	self deny: ScaledDecimal == sd class description: 'It used to, but this syntax is not valid Smalltalk'.
"	self assert: sd scale == 2.
	self assert: '123.00s2' = sd printString."

! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 15:49'!
testAsNumberWithoutDecimalPoint

	| sd |
	sd := '123s0' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 0.
	self assert: '123s0' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 15:51'!
testAsNumberWithoutDecimalPoint2

	| sd |
	sd := '123s2' asNumber.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 2.
	self assert: '123.00s2' = sd printString.
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/16/2009 23:48'!
testCoercion
	#( #* #+ #- #/) do: [:op |
		self assert: (1.0s1 perform: op with: 2) class = ScaledDecimal.
		self assert: (1.0s1 perform: op with: 1/2) class = ScaledDecimal.
		self deny: (1.0s1 perform: op with: 1.0) class = ScaledDecimal.
		
		self assert: (1 perform: op with: 2.0s1) class = ScaledDecimal.
		self assert: (1/2 perform: op with: 2.0s1) class = ScaledDecimal.
		self deny: (1.0 perform: op with: 1.0s1) class = ScaledDecimal]! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/16/2009 22:33'!
testConvertFromFloat

	| aFloat sd f2 diff |
	aFloat := 11/13 asFloat.
	sd := aFloat asScaledDecimal: 2.
	self assert: 2 == sd scale.
	self assert: '0.85s2' = sd printString.
	f2 := sd asFloat.
	diff := f2 - aFloat.
	self assert: diff < 1.0e-9. "actually, f = f2, but this is not a requirement"
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 12:24'!
testConvertFromFraction

	| sd |
	sd := (13 / 11) asScaledDecimal: 6.
	self assert: ScaledDecimal == sd class.
	self assert: ('1.181818s6' = sd printString).
	self assert: 6 == sd scale
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 12/3/2007 20:35'!
testIsLiteral
	"This test is related to http://bugs.squeak.org/view.php?id=6796"
	
	self assert: 1.00s2 isLiteral description: 'every literal obviously isLiteral'.
	
	"Note that (1 / 3.00s2) is not a well behaved literal,
	because it does not re-evaluate to self...
	Every literal should be evaluated as self (see isSelfEvaluating).
	There is currently no way to print it as a literal.
	So i propose it shall not answer true."
	self deny: (1/3.00s2) isLiteral description: 'this number cannot represent itself as a literal'.! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'dtl 9/18/2004 13:38'!
testLiteral

	| sd |
	sd := 1.40s2.
	self assert: ScaledDecimal == sd class.
	self assert: sd scale == 2.
	self assert: '1.40s2' = sd printString! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/17/2009 00:38'!
testOneRaisedToInteger
	"One might be handled specially"
	
	self assert: (1.0s1 raisedToInteger: -1) scale = 1.
	self assert: (1.0s1 raisedToInteger: -1) = 1.
	self assert: (1.0s1 raisedToInteger: 0) scale = 1.
	self assert: (1.0s1 raisedToInteger: 0) = 1.
	self assert: (1.0s1 raisedToInteger: 1) scale = 1.
	self assert: (1.0s1 raisedToInteger: 1) = 1.
	self assert: (1.0s1 raisedToInteger: 2) scale = 1.
	self assert: (1.0s1 raisedToInteger: 2) = 1.! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/16/2009 22:28'!
testPrintString
	"The printed representation of a ScaledDecimal is rounded.
	Note that old implementation was truncated."

	| sd |
	sd := (13 / 11) asScaledDecimal: 6.
	self assert: ('1.181818s6' = sd printString).
	sd := (13 / 11) asScaledDecimal: 5.
	self assert: ('1.18182s5' = sd printString).
	sd := (13 / 11) asScaledDecimal: 5.
	self deny: ('1.18181s5' = sd printString)
! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/17/2009 00:35'!
testRaisedToInteger
	"Raising to integer should preserve class and scale"
	
	self assert: (3.0s1 raisedToInteger: -1) scale = 1.
	self assert: (3.0s1 raisedToInteger: -1) = (1/3).
	self assert: (3.0s1 raisedToInteger: 0) scale = 1.
	self assert: (3.0s1 raisedToInteger: 0) = 1.
	self assert: (3.0s1 raisedToInteger: 1) scale = 1.
	self assert: (3.0s1 raisedToInteger: 1) = 3.
	self assert: (3.0s1 raisedToInteger: 2) scale = 1.
	self assert: (3.0s1 raisedToInteger: 2) = 9.! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 11/22/2007 22:42'!
testReadFrom
	"This is related to http://bugs.squeak.org/view.php?id=6779"
	
	self should: [(ScaledDecimal readFrom: '5.3') isKindOf: ScaledDecimal]
		description: 'Reading a ScaledDecimal should answer a ScaledDecimal'.
	self should: [((ScaledDecimal readFrom: '5.3') asScaledDecimal: 1) = (53/10 asScaledDecimal: 1)]
		description: 'ScaledDecimal readFrom: should not use Float intermediate because it would introduce round off errors'.! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/17/2009 00:31'!
testScaleExtension
	"The scale is extended to the larger one in case of arithmetic operation"
	
	#( #* #+ #- #/) do: [:op |
		self assert: (2.5s1 perform: op with: 1.000s3) scale = 3.
		self assert: (3.5000s4 perform: op with: 1.0s1) scale = 4.]! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/12/2007 00:34'!
testStoreOn
	"this is http://bugs.squeak.org/view.php?id=4378"
	
	"Both results should be 1.
	ScaledDecimal representations are exact
	(though only scale digits or fractional part are printed)"

	self assert:
    		(Compiler evaluate: (0.5s1 squared storeString)) * 4
		= (0.5s1 squared * 4)! !

!ScaledDecimalTest methodsFor: 'tests' stamp: 'nice 5/17/2009 00:40'!
testZeroRaisedToInteger
	"Zero might be handle specially"
	
	self should: [0.0s1 raisedToInteger: -1] raise: Error.
	self assert: (0.0s1 raisedToInteger: 0) = 1.
	self assert: (0.0s1 raisedToInteger: 0) scale = 1.
	self assert: (0.0s1 raisedToInteger: 1) = 0.
	self assert: (0.0s1 raisedToInteger: 1) scale = 1.
	self assert: (0.0s1 raisedToInteger: 2) = 0.
	self assert: (0.0s1 raisedToInteger: 2) scale = 1.! !

ClassTestCase subclass: #ScheduleTest
	instanceVariableNames: 'firstEvent aSchedule restoredTimeZone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!ScheduleTest methodsFor: 'Coverage' stamp: 'brp 9/25/2003 16:03'!
classToBeTested

	^ Schedule

! !

!ScheduleTest methodsFor: 'Coverage' stamp: 'brp 9/25/2003 16:03'!
selectorsToBeIgnored

	| private | 
	private := #( #printOn: ).

	^ super selectorsToBeIgnored, private
! !

!ScheduleTest methodsFor: 'running' stamp: 'brp 9/26/2004 19:30'!
setUp
 	 "Schedule is a type of Timespan representing repeated occurences of the same event. 
	The beginning of the schedule is the first occurrence of the event.
	A schedule maintains an array of Durations. 
	Each durations specify the offset to the next scheduled each. 
	The duration of each occurence of the event is not specified. 
	Nor are any other attributes such as name"

	restoredTimeZone := DateAndTime localTimeZone.
	DateAndTime localTimeZone: (TimeZone timeZones detect: [:tz | tz abbreviation = 'GMT']).

	"Create aSchedule with an event scheduled for 8:30pm every Saturday 
	and Sunday for the year 2003. "
	"Create the first event occurring on the first Saturday at 8:30 pm: 1/4/03"
	firstEvent :=  DateAndTime year: 2003 month: 1 day: 4 hour: 20 minute: 30. 
	
	"Create a schedule for one year starting with the first event" 
	aSchedule := Schedule starting: firstEvent duration: 52 weeks.

	"Schedule the recurring events by scheduling the time in between each one.  
	One day for Sat-Sun. 6 days for Sun-Sat" 
	aSchedule schedule: { Duration days: 1. Duration days: 6 }.
! !

!ScheduleTest methodsFor: 'running' stamp: 'brp 9/26/2004 19:30'!
tearDown

	DateAndTime localTimeZone: restoredTimeZone.
! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testBetweenAndDoDisjointWithSchedule
	| count |
	count := 0.
	aSchedule
		between: (DateAndTime
				year: 2004
				month: 4
				day: 1)
		and: (DateAndTime
				year: 2004
				month: 4
				day: 30)
		do: [:each | count := count + 1].
	self assert: count = 0! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testBetweenAndDoIncludedInSchedule
	| count |
	count := 0.
	aSchedule
		between: (DateAndTime
				year: 2003
				month: 4
				day: 1)
		and: (DateAndTime
				year: 2003
				month: 4
				day: 30)
		do: [:each | count := count + 1].
	self assert: count = 8! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testBetweenAndDoOverlappingSchedule
	| count |
	count := 0.
	aSchedule
		between: (DateAndTime
				year: 2002
				month: 12
				day: 1)
		and: (DateAndTime
				year: 2003
				month: 1
				day: 31)
		do: [:each | count := count + 1].
	self assert: count = 8! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testDateAndTimes
	| answer |
	self assert: aSchedule dateAndTimes size  = 104.
	self assert: aSchedule dateAndTimes first = firstEvent.
	answer := true.
	aSchedule dateAndTimes do: [:each | (each dayOfWeekName = 'Saturday'
		or: [each dayOfWeekName = 'Sunday']) ifFalse: [^false]].
	self assert: answer
! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testExampleFromSwikiPage
	"It is often neccessary to schedule repeated events, like airline flight schedules, TV programmes, and file backups.
	 Schedule is a Timespan which maintains an array of Durations.
	 The durations specify the offset to the next scheduled DateAndTime. "
	"Consider a TV programme scheduled for 8:30pm every Saturday and Sunday for the current year. "
	"Find the first Saturday and set its time to 20h30"
	| sat shows |
	sat := Year current asMonth dates detect: [ :d | d dayOfWeekName = #Saturday ].
	sat := sat start + (Duration hours: 20.5).
	"Create a schedule" 
	shows := Schedule starting: sat ending: Year current end.
	shows schedule: { Duration days: 1. Duration days: 6 }.
	"To inspect:"
	shows dateAndTimes.
	shows dateAndTimes collect: [ :dt | dt dayOfWeekName ].

! !

!ScheduleTest methodsFor: 'Tests' stamp: 'brp 9/25/2003 16:08'!
testFromDateAndTime

	| oc1 oc2 |
	oc1 := OrderedCollection new.
	DateAndTime today to: DateAndTime tomorrow by: 10 hours do: [ :dt | oc1 add: dt ].

	oc2 := { DateAndTime today. 
			(DateAndTime today + 10 hours). 
				(DateAndTime today + 20 hours) }.

	self assert: (oc1 asArray = oc2)! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testIncludes
	self assert: (aSchedule includes: (DateAndTime year: 2003 month: 6 day: 15 hour: 20 minute: 30 second: 0 offset: 0 hours))
	
! !

!ScheduleTest methodsFor: 'Tests' stamp: 'nk 3/30/2004 10:34'!
testMonotonicity

	| t1 t2 t3 t4 |
	t1 := DateAndTime now.
	t2 := DateAndTime now.
	t3 := DateAndTime now.
	t4 := DateAndTime now.

	self 
		assert: (	t1 <= t2);
		assert: (	t2 <= t3);
		assert: (	t3 <= t4).
! !

!ScheduleTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:44'!
testSchedule
	self assert: aSchedule schedule size = 2.
	self assert: aSchedule schedule first = 1 days.	
	self assert: aSchedule schedule second = 6 days.
! !

ClassTestCase subclass: #SemaphoreTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!SemaphoreTest commentStamp: 'tlk 5/5/2006 13:32' prior: 0!
A SemaphoreTest is sunit test for simple and multiEx semaphores

Instance Variables none; does not have common test fixture accross all tests (because its testing differenct sorts of semaphores (could refactor into muliple testcases if there were more test conditions.
!

!SemaphoreTest methodsFor: 'private' stamp: 'tlk 5/5/2006 13:26'!
criticalError
	Processor activeProcess terminate! !

!SemaphoreTest methodsFor: 'testing' stamp: 'tlk 5/5/2006 13:27'!
testCritical
	| lock |
	lock := Semaphore forMutualExclusion.
	[lock critical: [self criticalError]] forkAt: Processor userInterruptPriority.
	self assert: lock isSignaled! !

!SemaphoreTest methodsFor: 'testing' stamp: 'tlk 5/5/2006 13:28'!
testCriticalIfError
	| lock |
	lock := Semaphore forMutualExclusion.
	[lock critical: [self criticalError ifError:[]]] forkAt: Processor userInterruptPriority.
	self assert: lock isSignaled! !

!SemaphoreTest methodsFor: 'testing' stamp: 'ar 10/5/2007 18:09'!
testSemaAfterCriticalWait	"self run: #testSemaAfterCriticalWait"
	"This tests whether a semaphore that has just left the wait in Semaphore>>critical:
	leaves it with signaling the associated semaphore."
	| s p |
	s := Semaphore new.
	p := [s critical:[]] forkAt: Processor activePriority-1.
	"wait until p entered the critical section"
	[p suspendingList == s] whileFalse:[(Delay forMilliseconds: 10) wait].
	"Now that p entered it, signal the semaphore. p now 'owns' the semaphore
	but since we are running at higher priority than p it will not get to do
	anything."
	s signal.
	p terminate.
	self assert:[(s instVarNamed: #excessSignals) = 1]! !

!SemaphoreTest methodsFor: 'testing' stamp: 'ar 10/5/2007 18:06'!
testSemaInCriticalWait	"self run: #testSemaInCriticalWait"
	"This tests whether a semaphore that has entered the wait in Semaphore>>critical:
	leaves it without signaling the associated semaphore."
	| s p |
	s := Semaphore new.
	p := [s critical:[]] fork.
	Processor yield.
	self assert:[p suspendingList == s].
	p terminate.
	self assert:[(s instVarNamed: #excessSignals) = 0]! !

!SemaphoreTest methodsFor: 'testing' stamp: 'jf 12/2/2003 19:31'!
testWaitAndWaitTimeoutTogether
	| semaphore value waitProcess waitTimeoutProcess |
	semaphore := Semaphore new.
	
	waitProcess := [semaphore wait. value := #wait] fork.

	waitTimeoutProcess := [semaphore waitTimeoutMSecs: 50. value := #waitTimeout] fork.

	"Wait for the timeout to happen"
	(Delay forMilliseconds: 100) wait.

	"The waitTimeoutProcess should already have timed out.  This should release the waitProcess"
	semaphore signal.

	[waitProcess isTerminated and: [waitTimeoutProcess isTerminated]]
		whileFalse: [(Delay forMilliseconds: 100) wait].

	self assert: value = #wait.
	! !

!SemaphoreTest methodsFor: 'testing' stamp: 'ar 3/23/2009 17:01'!
testWaitTimeoutMSecs
	"Ensure that waitTimeoutMSecs behaves properly"

	"Ensure that a timed out waitTimeoutMSecs: returns true from the wait"
	self assert: (Semaphore new waitTimeoutMSecs: 50) == true.

	"Ensure that a signaled waitTimeoutMSecs: returns false from the wait"
	self assert: (Semaphore new signal waitTimeoutMSecs: 50) == false.
! !

!SemaphoreTest methodsFor: 'tests not working' stamp: 'stephane.ducasse 5/25/2008 17:59'!
waitAndWaitTimeoutTogether
	"self run: #testWaitAndWaitTimeoutTogether" 
	| semaphore value waitProcess waitTimeoutProcess |
	self halt: 'WatchOut Hang the image'.
	
	semaphore := Semaphore new.
	
	waitProcess := [semaphore wait. value := #wait] fork.

	waitTimeoutProcess := [semaphore waitTimeoutMSecs: 50. value := #waitTimeout] fork.

	"Wait for the timeout to happen"
	(Delay forMilliseconds: 100) wait.

	"The waitTimeoutProcess should already have timed out.  This should release the waitProcess"
	semaphore signal.

	[waitProcess isTerminated and: [waitTimeoutProcess isTerminated]]
		whileFalse: [(Delay forMilliseconds: 100) wait].

	self assert: value = #wait.
	! !

ClassTestCase subclass: #SmallIntegerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!SmallIntegerTest commentStamp: 'fbs 3/8/2004 22:13' prior: 0!
I provide a test suite for SmallInteger values. Examine my tests to see how SmallIntegers should behave, and see how to use them.!

!SmallIntegerTest methodsFor: 'testing - Class Methods' stamp: 'sd 6/5/2005 08:59'!
testBasicNew

	self should: [SmallInteger basicNew] raise: TestResult error. ! !

!SmallIntegerTest methodsFor: 'testing - arithmetic' stamp: 'sd 6/5/2005 08:59'!
testDivide

	self assert: 2 / 1 = 2.
	self assert: (3 / 2) isFraction.
	self assert: 4 / 2 = 2.
	self should: [ 1 / 0 ] raise: ZeroDivide.! !

!SmallIntegerTest methodsFor: 'testing - Class Methods' stamp: 'sd 6/5/2005 08:59'!
testMaxVal

	self assert: (SmallInteger maxVal = 16r3FFFFFFF).! !

!SmallIntegerTest methodsFor: 'testing - Class Methods' stamp: 'sd 6/5/2005 08:59'!
testMinVal

	self assert: (SmallInteger minVal = -16r40000000).! !

!SmallIntegerTest methodsFor: 'testing - Class Methods' stamp: 'sd 6/5/2005 08:59'!
testNew

	self should: [SmallInteger new] raise: TestResult error. ! !

!SmallIntegerTest methodsFor: 'testing - printing' stamp: 'fbs 12/8/2005 12:58'!
testPrintPaddedWith

self assert: (123 printPaddedWith: $0 to: 10 base: 2)  = '0001111011'.
self assert: (123 printPaddedWith: $0 to: 10 base: 8)  = '0000000173'.
self assert: (123 printPaddedWith: $0 to: 10 base: 10) = '0000000123'.
self assert: (123 printPaddedWith: $0 to: 10 base: 16) = '000000007B'.! !

!SmallIntegerTest methodsFor: 'testing - printing' stamp: 'al 7/21/2008 21:48'!
testPrintString
	self assert: 1 printString  = '1'.
	self assert: -1 printString  = '-1'.
	self assert: SmallInteger minVal printString  = '-1073741824'.
	self assert: SmallInteger maxVal printString  = '1073741823'.
	self assert: 12345 printString  = '12345'.
	self assert: -54321 printString  = '-54321'.

	self assert: 0 decimalDigitLength = 1.
	self assert: 4 decimalDigitLength = 1.
	self assert: 12 decimalDigitLength = 2.
	self assert: 123 decimalDigitLength = 3.
	self assert: 1234 decimalDigitLength = 4.
	self assert: 56789 decimalDigitLength = 5.
	self assert: 657483 decimalDigitLength = 6.
	self assert: 6571483 decimalDigitLength = 7.
	self assert: 65174383 decimalDigitLength = 8.
	self assert: 625744831 decimalDigitLength = 9.
	self assert: 1000001111 decimalDigitLength = 10.
	self assert: SmallInteger maxVal decimalDigitLength = 10.! !

ClassTestCase subclass: #SqNumberParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!SqNumberParserTest commentStamp: 'nice 5/7/2006 17:54' prior: 0!
Provide tests for new clas aimed at parsing numbers.

It duplicates NumberParsingTest, with few more tests.!

!SqNumberParserTest methodsFor: 'utility' stamp: 'nice 3/14/2010 23:02'!
areLowercaseDigitsAllowed
	"Answer true if lowercase letter are allowed digits."
	
	^(SqNumberParser parse: '16re' onError: [-1]) = 16rE! !

!SqNumberParserTest methodsFor: 'tests - fail' stamp: 'NikoSchwarz 10/23/2009 12:39'!
testFail
	"Verify that the value of a failblock is returned."
	self assert: (SqNumberParser parse: 'blablabla' onError: [42]) equals: 42! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'nice 5/7/2006 17:46'!
testFloatFromStreamAsNumber
	"This covers parsing in Number>>readFrom:"

	| rs aFloat |
	rs := '10r-12.3456' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -12.3456 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456e2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456e2e2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs upToEnd = 'e2'.

	rs := '10r-12.3456d2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456q2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '-12.3456q2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '12.3456q2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '12.3456z2' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 12.3456 = aFloat.
	self assert: rs upToEnd = 'z2'.
! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'nice 5/7/2006 17:46'!
testFloatFromStreamWithExponent
	"This covers parsing in Number>>readFrom:"

	| rs aFloat |
	rs := '1.0e-14' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs atEnd.

	rs := '1.0e-14 1' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs upToEnd = ' 1'.

	rs := '1.0e-14eee' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs upToEnd = 'eee'.

	rs := '1.0e14e10' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0e14 = aFloat.
	self assert: rs upToEnd = 'e10'.

	rs := '1.0e+14e' readStream. "Plus sign is not parseable"
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0 = aFloat.
	self assert: rs upToEnd = 'e+14e'.

	rs := '1.0e' readStream.
	aFloat := SqNumberParser parse: rs.
	self assert: 1.0 = aFloat.
	self assert: rs upToEnd = 'e'.! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'nice 3/14/2010 23:03'!
testFloatGradualUnderflow
	"Gradual underflow are tricky.
	This is a non regression test for http://bugs.squeak.org/view.php?id=6976"

	| float trueFraction str |
	
	"as a preamble, use a base 16 representation to avoid round off error and check that number parsing is correct"
	trueFraction := 16r2D2593D58B4FC4 / (16 raisedTo: 256+13).
	"Parse the number in base 16 if possible - it is impossible if lowercase letter are allowed digits due to exponent letter ambiguity."
	float := self areLowercaseDigitsAllowed
		ifNil: [SqNumberParser parse: '16r2.D2593D58B4FC4e-256']
		ifNotNil: [trueFraction asFloat]..
	self assert: float asTrueFraction = trueFraction.
	self assert: float = trueFraction asFloat.

	"now print in base 10"
	str := (String new: 32) writeStream.
	float absPrintExactlyOn: str base: 10.
	
	"verify if SqNumberParser can read it back"
	self assert: (SqNumberParser parse: str contents) = float. ! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'nice 3/14/2010 23:04'!
testFloatPrintString
	"self debug: #testFloatPrintString"
	
	| f r bases |
	f := Float basicNew: 2.
	r := Random new seed: 1234567.
	"printing a Float in base other than 10 is broken if lowercase digits are allowed"
	bases := self areLowercaseDigitsAllowed
		ifTrue: [#(10)]
		ifFalse: [#(2 8 10 16)].
	100
		timesRepeat: [f basicAt: 1 put: (r nextInt: 16r100000000)- 1.
			f basicAt: 2 put: (r nextInt: 16r100000000) - 1.
			bases
				do: [:base | | str |
						str := (String new: 64) writeStream.
						f negative ifTrue: [str nextPut: $-].
						str print: base; nextPut: $r.
						f absPrintExactlyOn: str base: base.
						self assert: (SqNumberParser parse: str contents) = f]].
	"test big num near infinity"
	10
		timesRepeat: [f basicAt: 1 put: 16r7FE00000 + ((r nextInt: 16r100000) - 1).
			f basicAt: 2 put: (r nextInt: 16r100000000) - 1.
			bases
				do: [:base | | str |
						str := (String new: 64) writeStream.
						f negative ifTrue: [str nextPut: $-].
						str print: base; nextPut: $r.
						f absPrintExactlyOn: str base: base.
						self assert: (SqNumberParser parse: str contents) = f]].
	"test infinitesimal (gradual underflow)"
	10
		timesRepeat: [f basicAt: 1 put: 0 + ((r nextInt: 16r100000) - 1).
			f basicAt: 2 put: (r nextInt: 16r100000000) - 1.
			bases
				do: [:base | | str |
						str := (String new: 64) writeStream.
						f negative ifTrue: [str nextPut: $-].
						str print: base; nextPut: $r.
						f absPrintExactlyOn: str base: base.
						self assert: (SqNumberParser parse: str contents) = f]].! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'nice 5/1/2006 00:40'!
testFloatReadError
	"This covers parsing in Number>>readFrom:"

	| rs num |
	rs := '1e' readStream.
	num := SqNumberParser parse: rs.
	self assert: 1 = num.
	self assert: rs upToEnd = 'e'.
	
	rs := '1s' readStream.
	num := SqNumberParser parse: rs.
	self assert: 1 = num.
	self assert: rs upToEnd = 's'.

	rs := '1.' readStream.
	num := SqNumberParser parse: rs.
	self assert: 1 = num.
	self assert: num isInteger.
	self assert: rs upToEnd = '.'.
	
	rs := '' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.
	
	rs := 'foo' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.

	rs := 'radix' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.
	
	rs := '.e0' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.
	
	rs := '-.e0' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.
	
	rs := '--1' readStream.
	self should: [SqNumberParser parse: rs] raise: Error.! !

!SqNumberParserTest methodsFor: 'tests - Float' stamp: 'damiencassou 5/30/2008 11:09'!
testFloatReadWithRadix
	"This covers parsing in Number>>readFrom:
	Note: In most Smalltalk dialects, the radix notation is not used for numbers
	with exponents. In Squeak, a string with radix and exponent can be parsed,
	and the exponent is always treated as base 10 (not the base indicated in the
	radix prefix). I am not sure if this is a feature, a bug, or both, but the
	Squeak behavior is documented in this test. -dtl"
	| aNumber rs |
	aNumber := '2r1.0101e9' asNumber.
	self assert: 672.0 = aNumber.
	self assert: (SqNumberParser parse: '2r1.0101e9') = (1.3125 * (2 raisedTo: 9)).
	rs := '2r1.0101e9e9' readStream.
	self assert: (SqNumberParser parse: rs) = 672.0.
	self assert: rs upToEnd = 'e9'! !

!SqNumberParserTest methodsFor: 'tests - Integer' stamp: 'damiencassou 5/30/2008 11:09'!
testIntegerReadFrom
	"Ensure remaining characters in a stream are not lost when parsing an integer."
	| rs i s |
	rs := '123s could be confused with a ScaledDecimal' readStream.
	i := SqNumberParser parse: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: 's could be confused with a ScaledDecimal' = s.
	rs := '123.s could be confused with a ScaledDecimal' readStream.
	i := SqNumberParser parse: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: '.s could be confused with a ScaledDecimal' = s! !

!SqNumberParserTest methodsFor: 'tests - Integer' stamp: 'nice 5/7/2006 17:46'!
testIntegerReadWithRadix
	"This covers parsing in Number>>readFrom:
	Note: In most Smalltalk dialects, the radix notation is not used for numbers
	with exponents. In Squeak, a string with radix and exponent can be parsed,
	and the exponent is always treated as base 10 (not the base indicated in the
	radix prefix). I am not sure if this is a feature, a bug, or both, but the
	Squeak behavior is documented in this test. -dtl"

	| aNumber rs |
	aNumber := '2r1e26' asNumber.
	self assert: 67108864 = aNumber.
	self assert: (SqNumberParser parse: '2r1e26') = (2 raisedTo: 26).
	rs := '2r1e26eee' readStream.
	self assert: (SqNumberParser parse: rs) = 67108864.
	self assert: rs upToEnd = 'eee'
! !

!SqNumberParserTest methodsFor: 'tests - ScaledDecimal' stamp: 'nice 8/29/2008 22:03'!
testScaledDecimalWithTrailingZeroes
	"This is a non regression tests for http://bugs.squeak.org/view.php?id=7169"
	
	self assert: (SqNumberParser parse: '0.50s2') = (1/2).
	self assert: (SqNumberParser parse: '0.500s3') = (1/2).
	self assert: (SqNumberParser parse: '0.050s3') = (1/20).! !

!SqNumberParserTest methodsFor: 'tests - squeezing' stamp: 'TestRunner 10/24/2009 09:18'!
testSqueezingOutNumbers
	"test that SqNumberParser squeezeNumberOutOfString finds numbers."
	self assert: '123blabla' squeezeOutNumber equals: 123.
	self assert: 'blabla123' squeezeOutNumber equals: 123.
	self assert: 'blabla12blabla' squeezeOutNumber equals: 12.
	self assert: ('12.3bla' squeezeOutNumber -12.3 ) abs < 0.0001.
	self assert: '.1' squeezeOutNumber > 0.
	
! !

!SqNumberParserTest methodsFor: 'tests - Integer' stamp: 'AdrianLienhard 11/6/2009 20:02'!
testcheckForCoverage
	"Tests for old semantics of Number>>readFrom:"
	
	self should: [ '.' asNumber = 0 ].
	self should: ['1.' asNumber = 1].
	self should: ['.1' asNumber = 0]! !

ClassTestCase subclass: #StopwatchTest
	instanceVariableNames: 'aStopwatch aDelay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!StopwatchTest class methodsFor: 'testing' stamp: 'JorgeRessia 3/16/2010 20:24'!
isUnitTest
	^false! !

!StopwatchTest methodsFor: 'Coverage' stamp: 'brp 9/24/2003 22:49'!
classToBeTested

	^ Stopwatch

! !

!StopwatchTest methodsFor: 'Coverage' stamp: 'brp 9/24/2003 23:01'!
selectorsToBeIgnored

	| private | 
	private := #( #printOn: #state: ).

	^ super selectorsToBeIgnored, private
! !

!StopwatchTest methodsFor: 'running' stamp: 'brp 1/21/2004 18:49'!
setUp
	aStopwatch := Stopwatch new.
	aDelay := Delay forMilliseconds: 1.! !

!StopwatchTest methodsFor: 'Tests' stamp: 'brp 9/25/2003 11:45'!
testActive

	| sw |
	sw := Stopwatch new.
	sw activate.
	
	1 seconds asDelay wait.
	self 
		assert: (sw duration >= 1 seconds).

	2 seconds asDelay wait.
	self 
		assert: (sw duration >= 3 seconds).

	sw suspend.! !

!StopwatchTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:49'!
testChangingStatus
	aStopwatch activate.
	self assert: aStopwatch isActive.
	self assert: aStopwatch timespans size = 1.
	aStopwatch suspend.
	self assert: aStopwatch isSuspended.
	self assert: aStopwatch timespans size = 1.
	aStopwatch activate.
	aStopwatch reActivate.
	self assert: aStopwatch isActive.
	self assert: aStopwatch timespans size = 3.
	aStopwatch reset.
	self assert: aStopwatch isSuspended.
	self assert: aStopwatch timespans size = 0.! !

!StopwatchTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:49'!
testInitialStatus
	self assert: aStopwatch isSuspended.
	self deny: aStopwatch isActive.
	self assert: aStopwatch duration = 0 seconds! !

!StopwatchTest methodsFor: 'testing' stamp: 'brp 9/26/2004 19:32'!
testMultipleTimings
	aStopwatch activate.
	aDelay wait.
	aStopwatch suspend.
	aStopwatch activate.
	aDelay wait.
	aStopwatch suspend.
	self assert: aStopwatch timespans size = 2. 
	self assert: aStopwatch timespans first asDateAndTime <= 
					aStopwatch timespans last asDateAndTime.
! !

!StopwatchTest methodsFor: 'Tests' stamp: 'brp 9/24/2003 22:56'!
testNew

	| sw |
	sw := Stopwatch new.
	
	self 
		assert: (sw isSuspended);
		assert: (sw state = #suspended);
		deny: (sw isActive);
		assert: (sw timespans isEmpty)

! !

!StopwatchTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := 'a Stopwatch(suspended:0:00:00:00)' readStream.
	rw := ReadWriteStream on: ''.
	aStopwatch printOn: rw.
	self assert: rw contents = cs contents! !

!StopwatchTest methodsFor: 'Tests' stamp: 'brp 9/25/2003 12:02'!
testReActivate

	| sw |
	sw := Stopwatch new.
	sw 
		activate;
		suspend;
		reActivate.
	
	self 
		assert: (sw isActive).
! !

!StopwatchTest methodsFor: 'Tests' stamp: 'brp 9/25/2003 11:56'!
testReset

	| sw |
	sw := Stopwatch new.
	sw activate.
	
	sw reset.
	self 
		assert: (sw isSuspended);
		assert: (sw timespans isEmpty)
! !

!StopwatchTest methodsFor: 'testing' stamp: 'brp 9/26/2004 19:32'!
testSingleTiming
	| timeBefore |
	timeBefore := DateAndTime now.
	aStopwatch activate.
	aDelay wait.
	aStopwatch suspend.
	self assert: aStopwatch timespans size = 1. 
	self assert: aStopwatch timespans first asDateAndTime >= timeBefore. 
	self assert: aStopwatch timespans first asDateAndTime <= aStopwatch end.
! !

!StopwatchTest methodsFor: 'Tests' stamp: 'brp 9/26/2004 19:36'!
testStartStop

	| sw t1 t2 t3 t4 |
	sw := Stopwatch new.
	t1 := DateAndTime now.
	(Delay forMilliseconds: 10) wait.
	sw activate; activate.
	(Delay forMilliseconds: 10) wait.
	t2 := DateAndTime now.
	
	self 
		deny: (sw isSuspended);
		assert: (sw isActive);
		assert: (sw timespans size = 1);
		assert: (t1 <= sw start);
		assert: (sw start <= t2).

	(Delay forMilliseconds: 10) wait.
	t3 := DateAndTime now.
	(Delay forMilliseconds: 10) wait.
	sw suspend; suspend.
	(Delay forMilliseconds: 10) wait.
	t4 := DateAndTime now.

	self 
		assert: (sw isSuspended);
		deny: (sw isActive);
		assert: (sw timespans size = 1);
		assert: (sw end between: t3 and: t4);
		assert: (t3 <= sw end);
		assert: (sw end <= t4).
! !

ClassTestCase subclass: #TimeStampTest
	instanceVariableNames: 'timestamp aTimeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!TimeStampTest commentStamp: 'brp 7/26/2003 22:44' prior: 0!
This is the unit test for the class TimeStamp.!

!TimeStampTest methodsFor: 'Coverage' stamp: 'brp 7/27/2003 13:50'!
classToBeTested

	^ self timestampClass! !

!TimeStampTest methodsFor: 'Coverage' stamp: 'brp 1/30/2005 09:13'!
selectorsToBeIgnored

	| deprecated private special |

	deprecated := #().
	private := #( #printOn: ).
	special := #().

	^ super selectorsToBeIgnored, deprecated, private, special.! !

!TimeStampTest methodsFor: 'Running' stamp: 'brp 1/21/2004 18:41'!
setUp

	timestamp := self timestampClass date: ('1-10-2000' asDate) time: ('11:55:00 am' asTime).

	aTimeStamp := TimeStamp readFrom: '1-02-2004 12:34:56 am' readStream! !

!TimeStampTest methodsFor: 'Running' stamp: 'brp 7/26/2003 21:53'!
tearDown

	timestamp := nil.! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 21:56'!
testAccessing

	| d t |
	d := '1-10-2000' asDate.
	t := '11:55:00 am' asTime.

	self
		assert: timestamp date = d;
		assert: timestamp time = t.
! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 22:05'!
testArithmetic

	| ts |
	ts := timestamp minusDays: 123.  	"9 September 1999, 11:55 am"
	ts := ts minusSeconds: 1056.			"9 September 1999, 11:37:24 am"
	ts := ts plusDays: 123.				"10 January 2000, 11:37:24 am"
	ts := ts plusSeconds: 1056.			"10 January 2000, 11:55 am"
	self
		assert: ts  = timestamp.

	! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 1/9/2004 06:34'!
testArithmeticAcrossDateBoundary

	| ts |
	ts := timestamp minusSeconds: ((11*3600) + (55*60) + 1).
	self
		assert: ts = ('1-9-2000 11:59:59 pm' asTimeStamp).

	! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 3/12/2004 15:54'!
testComparing

	| ts1 ts2 ts3 c1 c2 le |
	ts1 := self timestampClass date: ('01-10-2000' asDate) time: ('11:55:00 am' asTime).
	ts2 := self timestampClass date: ('07-26-2003' asDate) time: ('22:09:45 am' asTime).
	ts3 := self timestampClass date: ('05-28-1972' asDate) time: ('04:31:14 pm' asTime).

	self
		assert: ts1 = timestamp;
		assert: ts1 hash = timestamp hash;
		assert: timestamp = timestamp copy;
		assert: ts1 < ts2;
		deny: ts1 < ts3.

	c1 := self timestampClass current.
	c2 := self timestampClass current.
	le := (c1 <= c2).
	self assert: le.

! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 7/26/2003 22:19'!
testConverting

	| d t |
	d := '1-10-2000' asDate.
	t := '11:55:00 am' asTime.

	self
		assert: timestamp asSeconds = (d asSeconds + t asSeconds);
		assert: timestamp asDate = d;
		assert: timestamp asTime = t;
		assert: timestamp asTimeStamp == timestamp;
		assert: timestamp dateAndTime = {d. t}.
! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testDate
	self assert: aTimeStamp date = '01-02-2004' asDate! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testDateAndTime
	self assert: aTimeStamp dateAndTime
			= (Array with: '01-02-2004' asDate with: '00:34:56' asTime)! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 7/27/2003 13:55'!
testFromSeconds

	self
		assert: (self timestampClass fromSeconds: 3124958100) = timestamp.! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 8/23/2003 15:02'!
testFromString
	"This should signal an exception in 3.6beta as Time>>fromString: does not exist."

	self should: [ timestamp = (self timestampClass fromString: '1-10-2000 11:55:00 am') ] 

! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 1/30/2005 09:12'!
testInstanceCreation

	self 
		should: [ self timestampClass midnight asDuration = (0 hours) ];
		should: [ self timestampClass noon asDuration = (12 hours) ].
! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testMinusDays
	self assert: (aTimeStamp minusDays: 5) dateAndTime
			= (Array with: '12-28-2003' asDate with: '00:34:56' asTime)! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testMinusSeconds
	self assert: (aTimeStamp minusSeconds: 34 * 60 + 56) dateAndTime
			= (Array with: '01-02-2004' asDate with: '00:00:00' asTime)! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testMinusSecondsOverMidnight
	self assert: (aTimeStamp minusSeconds: 34 * 60 + 57) dateAndTime
			= (Array with: '01-01-2004' asDate with: '23:59:59' asTime)
	"Bug The results are actual results are: #(1 January 2005 11:25:03 pm)"! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testPlusDays
	self assert: (aTimeStamp plusDays: 366) dateAndTime
			= (Array with: '01-02-2005' asDate with: '00:34:56' asTime)! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testPlusSeconds
	self assert: (aTimeStamp plusSeconds: 60 * 60 ) dateAndTime
			= (Array with: '01-02-2004' asDate with: '01:34:56' asTime)! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testPlusSecondsOverMidnight
	self assert: (aTimeStamp plusSeconds: 24 * 60 * 60 + 1) dateAndTime
			= (Array with: '01-03-2004' asDate with: '00:34:57' asTime)! !

!TimeStampTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '2 January 2004 12:34:56 am' readStream.
	rw := ReadWriteStream on: ''.
	aTimeStamp printOn: rw.
	self assert: rw contents = cs contents! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 8/23/2003 15:02'!
testPrinting

	self	
		assert: timestamp printString = '10 January 2000 11:55 am'.
! !

!TimeStampTest methodsFor: 'testing' stamp: 'cbc 2/4/2004 21:18'!
testReadFromA1
	|ts|
	ts := TimeStamp current.
 	self assert: (ts = (TimeStamp fromString: ts asString)).! !

!TimeStampTest methodsFor: 'Tests' stamp: 'brp 8/23/2003 17:47'!
testSorting

	| c1 c2 |
	c1 := self timestampClass current.
	c2 := self timestampClass current.

	self
		assert: (self timestampClass current) <= (self timestampClass current);
		assert: (c1 <= c2).


! !

!TimeStampTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testStoreOn
	| cs rw |
	cs := '''2 January 2004 12:34:56 am'' asTimeStamp' readStream.
	rw := ReadWriteStream on: ''.
	aTimeStamp storeOn: rw.
	self assert: rw contents = cs contents! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testTime
	self assert: aTimeStamp time =  '00:34:56' asTime! !

!TimeStampTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:41'!
testTimeStamp
	self assert: aTimeStamp = aTimeStamp asTimeStamp
! !

!TimeStampTest methodsFor: 'Private' stamp: 'brp 7/27/2003 13:50'!
timestampClass

	^ TimeStamp! !

ClassTestCase subclass: #TimeTest
	instanceVariableNames: 'time aTime localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!TimeTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class Time.

!

!TimeTest methodsFor: 'coverage' stamp: 'brp 7/27/2003 13:31'!
classToBeTested

	^ self timeClass! !

!TimeTest methodsFor: 'coverage' stamp: 'brp 1/30/2005 09:09'!
selectorsToBeIgnored

	 | deprecated private special primitives timing benchmarks |

	deprecated := #().
	private := #( #print24:on: #print24:showSeconds:on: ).
	special := #( #< #= #new #printOn: #storeOn: ).
	primitives := #( #primMillisecondClock #primSecondsClock ).
	timing := #( #millisecondClockValue #milliseconds:since: #millisecondsSince: ).
	benchmarks := #( #benchmarkMillisecondClock #benchmarkPrimitiveResponseDelay ). 

	^ super selectorsToBeIgnored, deprecated, private, special, primitives, timing, benchmarks.! !

!TimeTest methodsFor: 'setup' stamp: 'nk 3/30/2004 09:40'!
setUp

	localTimeZoneToRestore := DateAndTime localTimeZone.
	DateAndTime localTimeZone: TimeZone default.
	time := self timeClass fromSeconds: 14567.		"4:02:47 am"
	aTime := Time readFrom: '12:34:56 pm' readStream
! !

!TimeTest methodsFor: 'setup' stamp: 'nk 3/30/2004 09:40'!
tearDown
	DateAndTime localTimeZone: localTimeZoneToRestore.
! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:33'!
testAccessing

	self
		assert: time hours = 4;
		assert: time minutes = 2;
		assert: time seconds = 47;
		assert: time asSeconds = 14567.
! !

!TimeTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testAddSeconds
	self assert: (aTime addSeconds: 1) = (Time readFrom: '12:34:57' readStream).
	self assert: (aTime addSeconds: 60) = (Time readFrom: '12:35:56' readStream).
	self assert: (aTime addSeconds: 3600) = (Time readFrom: '13:34:56' readStream).
	self assert: (aTime addSeconds: 24 * 60 * 60) = (Time readFrom: '12:34:56' readStream)! !

!TimeTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testAddTime
	self assert: (aTime addTime: aTime) = (Time readFrom: '01:09:52' readStream)! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:35'!
testArithmetic
	| t1 t2 t3 |
	t1 := time addSeconds: 70.		"4:03:57 am"
	self
		assert: t1 hours = 4;
		assert: t1 minutes = 3;
		assert: t1 seconds = 57.

	t2 := t1 addTime: (self timeClass fromSeconds: (60*60*5)).
	self
		assert: t2 hours = 9;
		assert: t2 minutes = 3;
		assert: t2 seconds = 57.

	t3 := t2 subtractTime: (self timeClass fromSeconds: (60*60*5) + 70).
	self
		assert: t3 = time.
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsDate
	self assert: (aTime asDate) = (Date current)
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsDateAndTime
	self assert: (aTime asDateAndTime) = (DateAndTime current midnight + aTime)
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsDuration
	self assert: (aTime asDuration) = (Duration days: 0 hours: 12 minutes: 34 seconds: 56)
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsNanoSeconds
	self assert: (aTime asNanoSeconds) = 45296000000000

! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsSeconds
	self assert: (aTime asSeconds) = 45296
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsTime
	self assert: (aTime asTime) = aTime

! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsTimeStamp
	self assert: (aTime asTimeStamp) = (DateAndTime current midnight + aTime) asTimeStamp
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsWeek
	self assert: aTime asWeek = (DateAndTime current midnight + aTime) asWeek
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testAsYear
	self assert: aTime asYear = (DateAndTime current midnight + aTime) asYear
! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:36'!
testComparing
	| t1 t2 t3 |
	t1 := self timeClass fromSeconds: 14567.		"4:02:47 am"
	t2 := self timeClass fromSeconds: 5000.		"1:23:20 am"
	t3 := self timeClass fromSeconds: 80000.		"10:13:20 pm"

	self
		assert: time = t1;
		assert: time hash = t1 hash;
		assert: time = time copy.
	self
		deny: t1 < t2;
		assert: t1 < t3.! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:37'!
testConverting

	self
		assert: time asSeconds = 14567.! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testDuration
	self assert: aTime duration = 0 seconds! !

!TimeTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/24/2010 12:24'!
testEqual
	"self debug: #testEqual"
	
	self assert: aTime = (Time readFrom: '12:34:56' readStream).
	self assert: aTime = (Time readFrom: '12:34:56.00' readStream).
	self assert: aTime = (Time readFrom: '12:34:56.0000' readStream)! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:38'!
testFromSeconds
	| t |
	t := self timeClass fromSeconds: 14567.
	self
		assert: t = time
! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:52'!
testGeneralInquiries
	| now d t dt |

	now  := self timeClass dateAndTimeNow.
	self 
		assert: now size = 2;
		assert: now last <= self timeClass now.

	self should: [ self timeClass timeWords ] raise: MessageNotUnderstood.

	d := '2 June 1973' asDate.
	t := '4:02:47 am' asTime.
	dt := self timeClass dateAndTimeFromSeconds: (2285280000 + 14567).
	self
		assert: dt = {d. t.}.
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testHhmm24
	self assert: aTime hhmm24 = '1234'! !

!TimeTest methodsFor: 'testing' stamp: 'nk 3/30/2004 09:42'!
testHour
	self assert: aTime hour =  12.
	self assert: aTime hour12 =  12.
	self assert: aTime hour24 =  12.
	self assert: aTime hours =  12.! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testHumanWordsForSecondsAgo
	self assert: (Time humanWordsForSecondsAgo: 0.999999999)
			= 'a second ago'.
	self assert: (Time humanWordsForSecondsAgo: 44.99999999)
			= '44.99999999 seconds ago'.
	self assert: (Time humanWordsForSecondsAgo: 89.999999999)
			= 'a minute ago'.
	self assert: (Time humanWordsForSecondsAgo: 2699.999999999)
			= '44 minutes ago'.
	self assert: (Time humanWordsForSecondsAgo: 5399.999999999)
			= 'an hour ago'.
	self assert: (Time humanWordsForSecondsAgo: 64799.999999999)
			= '17 hours ago'.
	! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testHumanWordsForSecondsAgoWithDays

	self assert: (Time humanWordsForSecondsAgo: 18 * 60 * 60)
					= 'yesterday'.
	self assert: (Time humanWordsForSecondsAgo: 24 * 60 * 60)
					= 'yesterday'.
! !

!TimeTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testLessThan
	self assert: aTime < (Time readFrom: '12:34:57' readStream)! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testMeridianAbbreviation
	self assert: aTime meridianAbbreviation =  'PM'.
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testMinute
	self assert: aTime minute =  34.
	self assert: aTime minutes =  34
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testNanoSecond
	self assert: aTime nanoSecond = 0
	"Right now all times all seconds"
! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:44'!
testNew
	
	self assert: self timeClass new asSeconds = 0! !

!TimeTest methodsFor: 'test printing' stamp: 'brp 1/21/2004 18:52'!
testPrint24
	self assert: aTime print24 = '12:34:56'! !

!TimeTest methodsFor: 'test printing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrint24On
	| cs rw |
	cs := '12:34:56' readStream.
	rw := ReadWriteStream on: ''.
	aTime 
		print24: true
		on: rw.
	self assert: rw contents = cs contents! !

!TimeTest methodsFor: 'test printing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrint24OnWithPM
	| cs rw |
	cs := '12:34:56 pm' readStream.
	rw := ReadWriteStream on: ''.
	aTime 
		print24: false
		on: rw.
	^ self assert: rw contents = cs contents! !

!TimeTest methodsFor: 'test printing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrint24OnWithoutSeconds
	| cs rw |
	cs := '12:34:56' readStream.
	rw := ReadWriteStream on: ''.
	aTime 
		print24: true
		showSeconds: true
		on: rw.
	self assert: rw contents = cs contents! !

!TimeTest methodsFor: 'test printing' stamp: 'brp 1/21/2004 18:52'!
testPrintMinutes
	self assert: aTime printMinutes = '12:34 pm'! !

!TimeTest methodsFor: 'test printing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '12:34:56 pm' readStream.
	rw := ReadWriteStream on: ''.
	aTime printOn: rw.
	self assert: rw contents = cs contents! !

!TimeTest methodsFor: 'testing' stamp: 'GabrielCalero 11/21/2009 12:20'!
testPrintStringNanos
	self assert: (Time hour: 15 minute: 15 second: 15  nanoSecond: (150 *
1000000)) printString = '3:15:15.15 pm'! !

!TimeTest methodsFor: 'test printing' stamp: 'brp 8/23/2003 22:27'!
testPrinting

	self	
		assert: time printString = '4:02:47 am';
		assert: time intervalString =  '4 hours 2 minutes 47 seconds';
		assert: time print24 = '04:02:47';
		assert: time printMinutes = '4:02 am';
		assert: time hhmm24 = '0402'.
! !

!TimeTest methodsFor: 'test input' stamp: 'StephaneDucasse 4/24/2010 12:33'!
testReadFrom
	"self debug: #testReadFrom"
	
	| string t |
	string := '4:02:47 am'.
	t := self timeClass readFrom: string readStream.
	self assert: time printString = t printString.
! !

!TimeTest methodsFor: 'test input' stamp: 'StephaneDucasse 4/24/2010 12:48'!
testReadFromWithNanos
	"self debug: #testReadFromWithNanos"
	
	#('4:02:47.5 am'
	'4:02:55.521 pm'
		"there is a bug with 520 instead of 521"
	
	) do: [:each |
			|  t |
			t := self timeClass readFrom: each readStream.
			self assert: t printString = each]
! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testSecond
	self assert: aTime second =  56.
	self assert: aTime seconds =  56
! !

!TimeTest methodsFor: 'tests' stamp: 'gk 8/31/2006 00:47'!
testSqueakInquiries
	| timewords totalseconds condensed corrected |
	self assert: 
		(self timeClass namesForTimes: #(2 10000023 10000026))
			= #('January, 1901' 'April, 1901, 4/26/1901, 5:47 pm' 'April, 1901, 4/26/1901, 5:47 pm').

	timewords := #(0.5 30 62 130 4000 10000 60000 86401) 
		collect: [ :ss | self timeClass humanWordsForSecondsAgo: ss ].
	self assert: 
		timewords = #('a second ago' '30 seconds ago' 'a minute ago' '2 minutes ago' 
			'an hour ago' '2 hours ago' '16 hours ago' 'yesterday').

	totalseconds :=  self timeClass totalSeconds.
	condensed := self timeClass condenseBunches: 
		(#(20 400 401  20000 20200 20300 40000 45000  200000 201000 202000) 
			collect: [:tt | totalseconds - tt]).
	corrected := condensed collect: [ :e | totalseconds - e ].
	self
		assert: (corrected includesAllOf: #(20 400 401 20000 40000 45000 200000)).
! !

!TimeTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testStoreOn
	| cs rw |
	cs := '''12:34:56 pm'' asTime' readStream.
	rw := ReadWriteStream on: ''.
	aTime storeOn: rw.
	self assert: rw contents = cs contents! !

!TimeTest methodsFor: 'tests' stamp: 'brp 7/27/2003 13:47'!
testStoring

	self	
		assert: time storeString = '''4:02:47 am'' asTime';
		assert: time = ('4:02:47 am' asTime).
! !

!TimeTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testSubtractTime
	self assert: (aTime subtractTime: aTime) = (Time readFrom: '00:00:00' readStream)! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testTicks
	self assert: aTime ticks = #(0 45296 0).
	self assert: aTime  = (Time new ticks: #(0 45296 0))! !

!TimeTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:52'!
testTimeStamp
	self assert: aTime = aTime asTimeStamp asTime! !

!TimeTest methodsFor: 'setup' stamp: 'brp 7/27/2003 13:32'!
timeClass

	^ Time! !

ClassTestCase subclass: #TimespanTest
	instanceVariableNames: 'timespan aTimespan anOverlappingTimespan anIncludedTimespan aDisjointTimespan aDay aWeek dec31 jan01 jan08 localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!TimespanTest methodsFor: 'Coverage' stamp: 'brp 9/15/2003 14:15'!
classToBeTested

	^ Timespan
! !

!TimespanTest methodsFor: 'Running' stamp: 'nk 3/30/2004 09:21'!
setUp

	localTimeZoneToRestore := DateAndTime localTimeZone.
	DateAndTime localTimeZone: TimeZone default.

	"100 hours starting noon 22 March 2003"
	timespan := Timespan starting:
					(DateAndTime year: 2003 month: 03 day: 22 hour: 12 minute: 0 second: 0)
						duration: (Duration hours: 100).

	dec31 := (DateAndTime year: 2004 month: 12 day: 31 hour: 0 minute: 0 second: 0).
	jan01 := (DateAndTime year: 2005 month: 1 day: 1 hour: 0 minute: 0 second: 0).
	jan08 := (DateAndTime year: 2005 month: 1 day: 8 hour: 0 minute: 0 second:0).
	aDay := Duration days: 1 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0.
	aWeek := Duration days: 7 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0.
	aTimespan := Timespan starting: jan01 duration: aWeek.
	anOverlappingTimespan := Timespan starting: dec31 duration: aWeek.
	anIncludedTimespan := Timespan starting: jan01 duration: aDay.
	aDisjointTimespan := Timespan starting: jan08 duration: aWeek.



! !

!TimespanTest methodsFor: 'Running' stamp: 'nk 3/30/2004 09:22'!
tearDown
	DateAndTime localTimeZone: localTimeZoneToRestore.
	timespan := nil
! !

!TimespanTest methodsFor: 'Tests' stamp: 'brp 1/7/2004 16:25'!
testAccessing

	self 
		assert: (timespan start =
				 (DateAndTime year: 2003 month: 03 day: 22 hour: 12 minute: 0 second: 0));
		assert: timespan duration = (Duration hours: 100);
		assert: timespan month = 3;
		assert: timespan monthName = 'March';
		assert: timespan monthAbbreviation = 'Mar'
		

! !

!TimespanTest methodsFor: 'Tests' stamp: 'brp 9/15/2003 14:29'!
testArithmetic

	| ts1 ts2 d |
	ts1 := timespan + 2 days.
	ts2 := ts1 - 2 days.
	d := ts1 - (DateAndTime year: 2003 month: 03 day: 20).

	self 
		assert: (ts1 start = 
				 (DateAndTime year: 2003 month: 03 day: 24 hour: 12 minute: 0 second: 0));
		assert: (ts1 duration = timespan duration);
		assert: (ts2 start = timespan start);
		assert: (ts2 duration = timespan duration).

	self
		assert: d = (Duration days: 4 hours: 12 minutes: 0 seconds: 0)

! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsDate
	self assert: aTimespan asDate =   jan01 asDate.
	"MessageNotUnderstood: Date class>>starting:"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsDateAndTime
	self assert: aTimespan asDateAndTime =   jan01.
	"MessageNotUnderstood: Date class>>starting:"
	
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsDuration
	self assert: aTimespan asDuration =  aWeek.

	
	
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsMonth
	self assert: aTimespan asMonth =   jan01 asMonth.
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsTime
	self assert: aTimespan asTime =  jan01 asTime
	"MessageNotUnderstood: Time class>>seconds:nanoSeconds:"
 ! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsTimeStamp
	self assert: aTimespan asTimeStamp =  ((TimeStamp readFrom: '1-01-2005 0:00 am' readStream) offset: 0 hours).
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsWeek
	self assert: aTimespan asWeek =   jan01 asWeek.
	"DateAndTime new asWeek
	 MessageNotUnderstood: Week class>>starting:"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testAsYear
	self assert: aTimespan asYear =   jan01 asYear.

! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 9/23/2004 09:58'!
testClockPrecisionDuration
	| ts |
	ts := Timespan starting: Date today duration: DateAndTime clockPrecision.
	self
		assert: ts start = ts end! !

!TimespanTest methodsFor: 'testing' stamp: 'nk 3/30/2004 09:26'!
testCurrent
	self assert: (Timespan starting: DateAndTime current)
			<= Timespan current.
	self assert:  Timespan current
			<= (Timespan starting: DateAndTime current)! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDay
	self assert: aTimespan day =   jan01 day
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDayOfMonth
	self assert: aTimespan dayOfMonth  = 1.
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDayOfWeek
	self assert: aTimespan  dayOfWeek  = 7.
	self assert: aTimespan  dayOfWeekName = 'Saturday'.
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDayOfYear
	self assert: aTimespan  dayOfYear  = 1.
	"MessageNotUnderstood: UndefinedObject>>year:, Undefined object is Year class"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDaysInMonth
	self assert: aTimespan  daysInMonth  = 31.
	"MessageNotUnderstood: Month class>>starting:"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDaysInYear
	self assert: aTimespan  daysInYear  = 365.
	"MessageNotUnderstood: UndefinedObject>>starting:  UndefinedObject is Year class"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDaysLeftInYear
	self assert: aTimespan  daysLeftInYear  = 364.
	"MessageNotUnderstood: UndefinedObject>>starting:  UndefinedObject is Year class"
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDoWith
	| count |
	count := 0.
	aTimespan
		do: [:each | count := count + 1]
		with: (Timespan starting: jan01 duration: aDay).
	self assert: count = 7! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDoWithWhen
	| count |
	count := 0.
	aTimespan
		do: [:each | count := count + 1]
		with: (Timespan starting: jan01 duration: aDay)
		when: [:each | count < 5].
	self assert: count = 5! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testDuration
	self assert: aTimespan duration  = aWeek.
	aTimespan duration: aDay.
	self assert: aTimespan duration =  aDay.

! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testEnd
	self assert: aTimespan end 	+ (Duration  nanoSeconds:1)  =  aDisjointTimespan
	"self assert: aTimespan end 	(DateAndTime year: 2005 month: 1 day: 7 hour: 23 minute: 59 second: 59 nanoSecond: 999999999 offset: 0 hours). "
	"This should work once DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset: is fixed"

! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testEveryDo
	| count duration |
	count := 0.
	duration := 7 days.
	aTimespan
		every: duration
		do: [:each | count := count + 1].
	self assert: count = 1! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testFirstDayOfMonth
	self assert: aTimespan firstDayOfMonth =   1. 
	self assert: aDisjointTimespan firstDayOfMonth =   1
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIncludes
	self assert: (aTimespan includes: jan01).
	self deny: (aTimespan includes: jan08)
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIncludesAllOf
	self assert: (aTimespan includesAllOf: (Bag with: jan01)).
	self deny: (aTimespan includesAllOf: (Bag with: jan01 with: jan08))
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIncludesAnyOf
	self deny: (aTimespan includesAnyOf: (Bag with: dec31)).
	self assert: (aTimespan includesAnyOf: (Bag with: jan01 with: jan08))
	"Error is due to bug in Timespan 
includesAnyOf: aCollection "
	"Answer whether any element of aCollection is included in the receiver"
	"aCollection do: [ :elem | (self includes: elem) ifTrue: [^ true]].
Shouldn't this return false if none are included?
"
! !

!TimespanTest methodsFor: 'Tests' stamp: 'brp 1/9/2004 06:43'!
testInclusion

	| t1 t2 t3 t4 |
	t1 := timespan start.
	t2 := timespan start + (timespan duration / 2).
	t3 := timespan end.
	t4 := timespan start + (timespan duration).

	self 
		assert: (timespan includes: t1);
		assert: (timespan includes: t2);
		assert: (timespan includes: t3)";
		deny: (timespan includes: t4).
	self
		assert: (timespan includes: (t1 to: t2));
		assert: (timespan includes: (t1 to: t4));
		deny: (timespan includes: (Timespan starting: t2 duration: (timespan duration * 2))).
	self 
		assert: (timespan includesAllOf: { t1. t2. t3 } );
		deny: (timespan includesAllOf: { t1. t2. t3. t4} ).
	self 
		assert: (timespan includesAnyOf: { t1. t2. t3 } );
		deny: (timespan includesAnyOf: { t4 } ).
"! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIntersectionWithDisjoint
	self assert: (aTimespan intersection: aDisjointTimespan) isNil.
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIntersectionWithIncluded
	self assert: (aTimespan intersection: anIncludedTimespan)  = 
	(Timespan starting: jan01 duration: (Duration days: 0 hours: 23 minutes: 59 seconds: 59 nanoSeconds: 999999999)).		
	self deny: (aTimespan intersection: anIncludedTimespan)	= anIncludedTimespan
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIntersectionWithOverlapping
	self assert: (aTimespan intersection: anOverlappingTimespan)  = 
	(Timespan starting: jan01 duration: (Duration days: 5 hours: 23 minutes: 59 seconds: 59 nanoSeconds: 999999999)).		

! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIntersectionWithSelf
	self assert: (aTimespan intersection: aTimespan)  = 
	(Timespan starting: jan01 duration: (Duration days: 6 hours: 23 minutes: 59 seconds: 59 nanoSeconds: 999999999)).		
	self deny: (aTimespan intersection: anIncludedTimespan)	= aTimespan
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIntersectionWithSeparate
	self assert: (aTimespan intersection: aDisjointTimespan) isNil.
	self deny: (aTimespan intersection: anOverlappingTimespan) isNil.
	self assert: (aTimespan intersection: anIncludedTimespan)  = 
	(Timespan starting: jan01 duration: (Duration days: 0 hours: 23 minutes: 59 seconds: 59 nanoSeconds: 999999999)).		
	self deny: (aTimespan intersection: anIncludedTimespan)	= anIncludedTimespan
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testIsLeapYear
	"self assert: anOverlappingTimespan isLeapYear."
	"not sure why this fails"
	self deny: aTimespan isLeapYear
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testJulianDayNumber
	self assert: aTimespan julianDayNumber =  (jan01 julianDayNumber).
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testLessThan
	self assert: aTimespan  < aDisjointTimespan.
	self deny: anIncludedTimespan < aTimespan
	! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testMinusADateAndTime
	"It appears that subtracting a date from a Timespan gives you a duration = to the difference between the start of the timespan and the date "
	self assert: aTimespan - dec31 =  aDay.
	self assert: aDisjointTimespan - jan01 =  aWeek.


! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testMinusADuration
	"It appears that subtracting a duration from a Timespan gives you a Timespan shifted by the duration"
	self assert: aTimespan - aDay =  anOverlappingTimespan.
	self assert: aDisjointTimespan - aWeek =  aTimespan.	


! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testMonth
	self assert: aTimespan month  = 1.
	self assert: aTimespan monthName = 'January'.
	self assert: aTimespan monthIndex = 1.! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testNew
	self assert: Timespan new = (Timespan starting: '01-01-1901' asDate)! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testNext
	self assert: aTimespan next = aDisjointTimespan
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testPlus
	self assert: aTimespan + aWeek = aDisjointTimespan.
	self assert: anOverlappingTimespan + aDay = aTimespan.
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testPrevious
	self assert: aTimespan  = aDisjointTimespan previous.
	self assert: aTimespan next previous = aTimespan 
! !

!TimespanTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := 'a Timespan(2005-01-01T00:00:00+00:00D7:00:00:00)' readStream.
	rw := ReadWriteStream on: ''.
	aTimespan printOn: rw.
	self assert: rw contents = cs contents! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testStart
	self assert: aTimespan start =   jan01.
	aTimespan start: jan08.
	self assert: aTimespan start =   jan08.! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testStartingEnding
	self assert: aTimespan  = (Timespan starting: jan01 ending: jan08)
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testTo
	self assert: (anIncludedTimespan to: jan08) = aTimespan 
! !

!TimespanTest methodsFor: 'Tests' stamp: 'brp 1/9/2004 16:49'!
testUnion

	| union |
	union := timespan union: timespan.
	
	self 
		assert: (union start = timespan start);
		assert: (union duration = timespan duration)
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testUnionWithDisjoint

	self assert: (aTimespan union: aDisjointTimespan)  = 
		(Timespan starting: jan01 duration: (14 days)).	
			
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testUnionWithIncluded

	self 
		assert: (aTimespan union: anIncludedTimespan) = aTimespan 	! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testUnionWithOverlapping

	self 
		assert: (aTimespan union: anOverlappingTimespan)  = 
				(Timespan starting: dec31 duration: (8 days))! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testUnionWithSelf
	self assert: (aTimespan union: aTimespan) = aTimespan
	! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testUnionWithSeparate

	self 
		assert: (anOverlappingTimespan union: aDisjointTimespan) = 
			(Timespan 
				starting: anOverlappingTimespan start
				ending:  (aDisjointTimespan end + DateAndTime clockPrecision))
			
! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testWorkDatesDo
	| count |
	count := 0.
	aTimespan
		workDatesDo: [:each | count := count + 1].
	self assert: count = 5! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 1/21/2004 18:31'!
testYear
	self assert: aTimespan year = 2005.

	! !

!TimespanTest methodsFor: 'testing' stamp: 'brp 9/23/2004 09:57'!
testZeroDuration
	| ts |
	ts := Timespan starting: Date today duration: Duration zero.
	self
		assert: ts start = ts end! !

ClassTestCase subclass: #TrueTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!TrueTest methodsFor: 'testing' stamp: 'StephaneDucasse 3/5/2010 15:32'!
testAND

	self assert: (true & true).
	self deny: (true & false).! !

!TrueTest methodsFor: 'testing' stamp: 'StephaneDucasse 2/2/2010 13:49'!
testAsBit

       self assert: (true asBit = 1).! !

!TrueTest methodsFor: 'testing' stamp: 'sd 6/5/2005 09:06'!
testNew

	self should: [True new] raise: Error. ! !

!TrueTest methodsFor: 'testing' stamp: 'StephaneDucasse 3/6/2010 22:31'!
testNot

	self assert: (false not).! !

!TrueTest methodsFor: 'testing' stamp: 'sd 6/5/2005 09:06'!
testPrintOn

	self assert: (String streamContents: [:stream | true printOn: stream]) = 'true'. ! !

!TrueTest methodsFor: 'testing' stamp: 'NikoSchwarz 10/17/2009 18:21'!
testXor
	self assert: (true xor: true) = false.
	self assert: (true xor: false) = true.
	
	self
		should: [(true xor: [true])
 			ifTrue: ["This should never be true, do not signal an Error and let the test fail"]
 			ifFalse: [self error: 'OK, this should be false, raise an Error']]
		raise: Error
		description: 'a Block argument is not allowed. If it were, answer would be false'.! !

ClassTestCase subclass: #UndefinedObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!UndefinedObjectTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class UndefinedObject. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!UndefinedObjectTest methodsFor: 'tests - Class Methods' stamp: 'nice 11/14/2009 19:25'!
testAllInstances
	self assert: UndefinedObject allInstances size = 1 description: 'There should be a single instance of UndefinedObject'.
	self assert: (UndefinedObject allInstances includes: nil) description: 'nil should be an instance of UndefinedObject'.! !

!UndefinedObjectTest methodsFor: 'tests - copying' stamp: 'sd 6/5/2005 09:07'!
testDeepCopy

	self assert:  (nil deepCopy = nil).! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:07'!
testHaltIfNil

	self should: [ nil haltIfNil] raise: Halt.! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:07'!
testIfNil

	self should: [ nil ifNil: [self halt]] raise: Halt.


! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIfNilIfNotNil

	self should: [ nil ifNil: [self halt] ifNotNil: [self error] ] raise: Halt.


! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIfNotNil

	self shouldnt: [ nil ifNotNil: [self halt]] raise: Halt.


! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'marcus.denker 11/10/2008 10:04'!
testIfNotNilDo

	self shouldnt: [ nil ifNotNil: [self halt]] raise: Halt.
! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIfNotNilIfNil

	self should: [ nil ifNotNil: [self error] ifNil: [self halt] ] raise: Halt.


! !

!UndefinedObjectTest methodsFor: 'tests - Class Methods' stamp: 'sd 6/5/2005 09:08'!
testInitializedInstance

	self assert: ( UndefinedObject initializedInstance class == UndefinedObject).! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIsEmptyOrNil

	self assert: (nil isEmptyOrNil).! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIsLiteral

	self assert: (nil isLiteral).! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testIsNil

	self assert: (nil isNil).! !

!UndefinedObjectTest methodsFor: 'tests - Class Methods' stamp: 'sd 6/5/2005 09:09'!
testNew

	self should: [ UndefinedObject new] raise: Error.! !

!UndefinedObjectTest methodsFor: 'tests - testing' stamp: 'sd 6/5/2005 09:08'!
testNotNil

	self deny: (nil notNil).! !

!UndefinedObjectTest methodsFor: 'tests - printing' stamp: 'sd 6/5/2005 09:09'!
testPrintOn

	| string |
	string := String streamContents: [:stream | nil printOn: stream].
	self assert: (string = 'nil').! !

!UndefinedObjectTest methodsFor: 'tests - copying' stamp: 'sd 6/5/2005 09:07'!
testShallowCopy

	self assert: (nil shallowCopy = nil).! !

!UndefinedObjectTest methodsFor: 'tests - printing' stamp: 'sd 6/5/2005 09:09'!
testStoreOn

	| string |
	string := String streamContents: [:stream | nil storeOn: stream].
	self assert: ((Compiler evaluate: string) = nil).! !

!UndefinedObjectTest methodsFor: 'tests - copying' stamp: 'sd 6/5/2005 09:07'!
testVeryDeepCopyWith

	self assert: ((nil veryDeepCopyWith: nil) = nil).! !

ClassTestCase subclass: #WeakMessageSendTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!WeakMessageSendTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:10'!
testNoArguments
	"self run: #testNoArguments"

	| m |
	m := WeakMessageSend
			receiver: true
			selector: #yourself.
	self assert: (m value).
! !

!WeakMessageSendTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:11'!
testOneArgument
	"self run: #testOneArgument"	

	| m |
	m := WeakMessageSend
		receiver: Array
		selector: #with:
		argument: 1.
	Smalltalk garbageCollectMost.
	self assert: (m value  = { 1 })
! !

!WeakMessageSendTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:11'!
testOneArgumentWithGC

	| m |
	m := WeakMessageSend
		receiver: Array
		selector: #with:
		arguments: { Object new }.
	Smalltalk garbageCollectMost.
	self assert: (m value isNil)! !

!WeakMessageSendTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:11'!
testReceiverWithGC

	| m |
	m := WeakMessageSend
		receiver: Object new
		selector: #isNil.
	Smalltalk garbageCollectMost.
	self assert: (m value isNil).! !

!WeakMessageSendTest methodsFor: 'tests' stamp: 'sd 6/5/2005 09:11'!
testTwoArguments

	| m |
	m := WeakMessageSend
		receiver: Array
		selector: #with:with:
		arguments: { 1 . 2 }.
	Smalltalk garbageCollectMost.
	self assert: (m value = { 1 . 2 }).
! !

ClassTestCase subclass: #WeekTest
	instanceVariableNames: 'week restoredStartDay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!WeekTest methodsFor: 'Coverage' stamp: 'brp 7/27/2003 12:42'!
classToBeTested

	^ Week! !

!WeekTest methodsFor: 'Coverage' stamp: 'brp 1/30/2005 09:21'!
selectorsToBeIgnored

	| deprecated private special |

	deprecated := #().
	private := #( #printOn: ).
	special := #( #next #do: ).

	^ super selectorsToBeIgnored, deprecated, private, special.! !

!WeekTest methodsFor: 'Running' stamp: 'brp 9/26/2004 18:52'!
setUp
	"June 1998, 5th week"

	super setUp.
	restoredStartDay := Week startDay.
	Week startDay: #Sunday.
	week := Week starting: '4 July 1998' asDate! !

!WeekTest methodsFor: 'Running' stamp: 'brp 9/26/2004 18:53'!
tearDown

	super tearDown.
	Week startDay: restoredStartDay.
	week := nil.

! !

!WeekTest methodsFor: 'Tests' stamp: 'it 7/14/2009 19:47'!
testDayNames
	self assert: (Week dayNames) = #(#Sunday #Monday #Tuesday #Wednesday #Thursday #Friday #Saturday).! !

!WeekTest methodsFor: 'Tests' stamp: 'brp 1/30/2005 09:32'!
testEnumerating

	| days |
	days := OrderedCollection new.
	0 to: 6 do: [ :i | days add: ('28 June 1998' asDate addDays: i) ].

	week datesDo: [ :d | days remove: d ].
	
	self assert: days isEmpty.
! !

!WeekTest methodsFor: 'Tests' stamp: 'it 7/14/2009 19:52'!
testIndexOfDay
	| days |
	days := #(#Sunday #Monday #Tuesday #Wednesday #Thursday #Friday #Saturday).
	
	days withIndexDo: [:item :index | self assert: (Week indexOfDay: item) = index].
	
	"This should probably raise an error rather than returning 0."
	self assert: (Week indexOfDay: 0) = 0.
	self assert: (Week indexOfDay: 1) = 0.
	self assert: (Week indexOfDay: 7) = 0.
	self assert: (Week indexOfDay: 8) = 0.
	self assert: (Week indexOfDay: #Sunnyday) = 0.! !

!WeekTest methodsFor: 'Tests' stamp: 'brp 1/30/2005 09:30'!
testInquiries

	self
		assert: week start asDate = '28 June 1998' asDate;
		assert: week end asDate = '4 July 1998' asDate;
		assert: week index = 5;
		assert: week duration = (7 days).
! !

!WeekTest methodsFor: 'Tests' stamp: 'it 7/14/2009 19:46'!
testNameOfDay
	| days |
	days := #(#Sunday #Monday #Tuesday #Wednesday #Thursday #Friday #Saturday).
	
	days withIndexDo: [:item :index | self assert: (Week nameOfDay: index) = item].
	
	self should: [Week nameOfDay: 0] raise: TestResult error.
	self should: [Week nameOfDay: 8] raise: TestResult error.
	self should: [Week nameOfDay: #Sunday] raise: TestResult error.! !

!WeekTest methodsFor: 'Tests' stamp: 'nk 7/30/2004 17:52'!
testPreviousNext
	self
		assert: week next = (Week starting: '6 July 1998' asDate);
		assert: week previous = (Week starting:  '22 June 1998' asDate)! !

ClassTestCase subclass: #YearTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!YearTest methodsFor: 'Coverage' stamp: 'brp 9/11/2003 14:31'!
classToBeTested

	^ Year! !

!YearTest methodsFor: 'tests' stamp: 'StephaneDucasse 3/13/2010 13:31'!
testPreviousInLeapYear
	"self debug: #testPreviousInLeapYear"
	
	| leap |
	leap := Year year: 2008.
	self assert: leap isLeapYear. 
	self assert: (Year year: (leap year - 1)) = leap previous ! !

!YearTest methodsFor: 'tests' stamp: 'StephaneDucasse 3/13/2010 13:30'!
testStart
	"self debug: #testStart"
	
	| yyyy |
	yyyy := DateAndTime now year.
	self assert: Year current start = (DateAndTime year: yyyy month: 1 day: 1)! !

ProcessLocalVariable subclass: #TestLocalVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!TestLocalVariable commentStamp: 'mvl 3/13/2007 13:52' prior: 0!
TestLocalVariable is a test class using in ProcessSpecificTest.!

!TestLocalVariable class methodsFor: 'as yet unclassified' stamp: 'mvl 3/13/2007 14:54'!
default
	"My default value for a new process is 0."
	
	^0! !

Error subclass: #ArithmeticError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

ArithmeticError subclass: #FloatingPointException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

Error subclass: #BlockCannotReturn
	instanceVariableNames: 'result deadHome'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!BlockCannotReturn commentStamp: '<historical>' prior: 0!
This class is private to the EHS implementation.  Its use allows for ensured execution to survive code such as:

[self doThis.
^nil]
	ensure: [self doThat]

Signaling or handling this exception is not recommended.!

!BlockCannotReturn methodsFor: 'accessing' stamp: 'ajh 2/6/2002 11:12'!
deadHome

	^ deadHome! !

!BlockCannotReturn methodsFor: 'accessing' stamp: 'ajh 2/6/2002 11:12'!
deadHome: context

	deadHome := context! !

!BlockCannotReturn methodsFor: 'exceptiondescription' stamp: 'tfei 3/30/1999 12:55'!
defaultAction

	self messageText: 'Block cannot return'.
	^super defaultAction! !

!BlockCannotReturn methodsFor: 'exceptiondescription' stamp: 'tfei 4/2/1999 15:49'!
isResumable

	^true! !

!BlockCannotReturn methodsFor: 'accessing' stamp: 'tfei 3/30/1999 12:54'!
result

	^result! !

!BlockCannotReturn methodsFor: 'accessing' stamp: 'tfei 3/30/1999 12:54'!
result: r

	result := r! !

TestCase subclass: #AllocationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Exception'!

!AllocationTest commentStamp: 'StephaneDucasse 12/18/2009 12:03' prior: 0!
Test originally from Andreas Raab!

!AllocationTest methodsFor: 'workarounds' stamp: 'StephaneDucasse 4/23/2010 16:14'!
expectedFailures 
	"should be fixed with latest VMs but unix is still pending"
	self isSafeVM
		ifTrue: [ ^#() ]
		ifFalse: [ ^#(testOneGigAllocation testOutOfMemorySignal) ]! !

!AllocationTest methodsFor: 'workarounds' stamp: 'StephaneDucasse 4/23/2010 16:18'!
isSafeVM
	"False if the VM is a Unix VM that has not been updated to version 4.
	Earlier versions may be subject to a bug that can crash the VM when
	running AllocationTest."

	self flag: #toRemove. "April 2010 dtl - remove this about one year from now"
	^ Smalltalk isUnix not
		or: [((Smalltalk vmFullPath
				findTokens: FileDirectory slash)
				select: [:e | e beginsWith: '4.']) isEmpty not]! !

!AllocationTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/23/2010 16:11'!
testOneGigAllocation
	"Documentating a weird bug in the allocator"
	
	| sz array failed |
	failed := false.
	sz := 1024*1024*1024.
	array := [Array new: sz] on: OutOfMemory do: [:ex| failed := true].
	self assert: (failed or:[array size = sz]).
	! !

!AllocationTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/23/2010 16:12'!
testOneMegAllocation
	"Documentating a weird bug in the allocator"
	
	| sz array failed |
	failed := false.
	sz := 1024*1024.
	array := [Array new: sz] on: OutOfMemory do: [:ex| failed := true].
	self assert: (failed or:[array size = sz]).
	! !

!AllocationTest methodsFor: 'testing' stamp: 'StephaneDucasse 3/17/2010 21:32'!
testOutOfMemorySignal
	"Ensure that OOM is signaled eventually"
	| sz |
	Smalltalk isUnix ifTrue: [ "avoid crashing"
		^self assert: false ].
	sz := 512*1024*1024. "work around the 1GB alloc bug"
	self should: [(1 to: 2000) collect: [:i| Array new: sz]] raise: OutOfMemory.

	"Call me when this test fails, I want your machine"
	sz := 1024*1024*1024*1024.
	self should:[Array new: sz] raise: OutOfMemory.
! !

TestCase subclass: #BasicBehaviorClassMetaclassTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!BasicBehaviorClassMetaclassTest commentStamp: '<historical>' prior: 0!
This class contains some tests regarding the classes 
	Behavior
		ClassDescription
			Class
			Metaclass
---
	!

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:19'!
testBehaviorClassClassDescriptionMetaclassHierarchy
	"self run: #testBehaviorClassClassDescriptionMetaclassHierarchy"
	
	self assert: Class superclass  == ClassDescription.
	self assert: Metaclass superclass == ClassDescription.

	self assert: ClassDescription superclass  == Behavior.
	self assert: Behavior superclass  = Object.

	self assert: Class class class ==  Metaclass.
	self assert: Metaclass class class  == Metaclass.
	self assert: ClassDescription class class == Metaclass.
	self assert: Behavior class class == Metaclass.




	
	
	



	
	

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'md 7/24/2009 15:29'!
testClassDescriptionAllSubInstances
	"self run: #testClassDescriptionAllSubInstances"

	| cdNo clsNo metaclsNo |
	
	Smalltalk garbageCollect. 
	cdNo := ClassDescription allSubInstances size.
	clsNo := Class allSubInstances size .
	metaclsNo := Metaclass allSubInstances size.

	self assert: cdNo = (clsNo + metaclsNo).

	
	
	



	
	

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:17'!
testMetaclass
	"self run: #testMetaclass"
	
	self assert: OrderedCollection class class == Metaclass.
	self assert: Dictionary class class == Metaclass.
	self assert: Object class class == Metaclass.




	
	
	



	
	

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:12'!
testMetaclassName
	"self run: #testMetaclassName"

	self assert: Dictionary class  name = 'Dictionary class'.
	self assert: OrderedCollection class name = 'OrderedCollection class'.
	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:12'!
testMetaclassNumberOfInstances
	"self run: #testMetaclassNumberOfInstances"

	self assert: Dictionary class allInstances size  = 1.
	self assert: OrderedCollection class allInstances size  = 1.! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:18'!
testMetaclassPointOfCircularity
	"self run: #testMetaclassPointOfCircularity"

	self assert: Metaclass class instanceCount = 1.
	self assert: Metaclass class someInstance == Metaclass.


	
	
	



	
	

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:13'!
testMetaclassSuperclass
	"self run: #testMetaclassSuperclass"

	self assert: Dictionary class superclass == Set class.
	self assert: OrderedCollection class superclass == SequenceableCollection class.

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:14'!
testMetaclassSuperclassHierarchy
	"self run: #testMetaclassSuperclassHierarchy"

	| s |
	self assert: SequenceableCollection class instanceCount  = 1.
	self assert: Collection class instanceCount  = 1.
	self assert: Object class instanceCount  = 1.
	self assert: ProtoObject class instanceCount  = 1.
	
	s := OrderedCollection new.
	s add: SequenceableCollection class.
	s add: Collection class.
	s add: Object class.
	s add: ProtoObject class.

	s add: Class.
	s add: ClassDescription.
	s add: Behavior.
	s add: Object.
	s add: ProtoObject.

	self assert: OrderedCollection class allSuperclasses  = s.



	
	

	! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:21'!
testObjectAllSubclasses
	"self run: #testObjectAllSubclasses"

	| n2 |
	n2 := Object allSubclasses size.
	self assert: n2 = (Object allSubclasses
			select: [:cls | cls class class == Metaclass
					or: [cls class == Metaclass]]) size! !

!BasicBehaviorClassMetaclassTest methodsFor: 'testing' stamp: 'sd 6/5/2005 08:13'!
testSuperclass
	"self run: #testSuperclass"

	| s |
	self assert: Dictionary superclass == Set.
	self assert: OrderedCollection superclass == SequenceableCollection.

	s := OrderedCollection new.
	s add: SequenceableCollection.
	s add: Collection.
	s add: Object.
	s add: ProtoObject.

	self assert: OrderedCollection allSuperclasses = s.

	
	! !

TestCase subclass: #BehaviorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!BehaviorTest methodsFor: 'tests' stamp: 'nice 12/25/2009 18:58'!
sampleMessageWithFirstArgument: firstArgument "This is a comment intended to explain arg1"
   andInterleavedCommentBeforeSecondArgument: secondArgument

	"This method is here to test a few utilities like formalParametersAt:"
	
	| thisIsAnUnusedTemp |
	thisIsAnUnusedTemp := self.
	^thisIsAnUnusedTemp! !

!BehaviorTest methodsFor: 'tests' stamp: 'JB 2/19/2010 16:06'!
testAllCallsOn
	"
	self debug: #testAllCallsOn
	"

	| t tmp |
	((SystemNavigation >> #allCallsOn:) sendsSelector: #deprecated:)
		ifFalse: [ ^ self ].
	t := false.
	[ (SystemNavigation default allCallsOn: Point binding) ]
		on: Deprecation do: [:ex | t := true ].
	self assert: t.
	
	"We refactored allCallsOn: and introduced allReferencesTo:. The remaining of this test checked what the refactoring does not break senders of allCallsOn:"
	t := true.
	[ Point allCallsOn ] on: Deprecation do: [:ex | t := false ].
	self assert: t.
	
	tmp := Point allCallsOn.
	self assert: tmp notEmpty.
	self assert: (tmp includes: (Point>> #isSelfEvaluating) methodReference).
	
	t := true.
	[ SystemNavigation default allUnreferencedClassVariablesOf: Object ] on: Deprecation do: [:ex | t := false ].
	self assert: t.
! !

!BehaviorTest methodsFor: 'tests' stamp: 'JB 2/19/2010 16:13'!
testAllLocalCallsOn
	" self debug: #testAllLocalCallsOn"
	
	self assert: (( Point allLocalCallsOn: #asPoint )  notEmpty).
	self assert: (( Point allLocalCallsOn: #asPoint )  includes: (Point>> #roundDownTo:) methodReference).
	self assert: (( Point allLocalCallsOn: #asPoint )  includes: (Point >> #roundUpTo:) methodReference).
	self assert: (( Point allLocalCallsOn: #asPoint )  includes: (Point >> #roundTo:) methodReference).
	self assert: (( Point allLocalCallsOn: #asPoint )  includes: (Point >> #truncateTo: ) methodReference).
	! !

!BehaviorTest methodsFor: 'tests' stamp: 'AlexandreBergel 10/28/2009 11:02'!
testAllMethods
	| allMethods nonOverridenMethods |
	allMethods := IdentitySet new addAll: Object allMethods; yourself.
	self assert: (allMethods includesAllOf: Object methods).
	
	"We fetch all compiled methods that are not redefined in Object"
	nonOverridenMethods := OrderedCollection new.
	ProtoObject methodDictionary 
		associationsDo: [:assoc | (Object includesSelector: assoc key) 
											ifFalse: [ nonOverridenMethods add: assoc value ] ].
										
	self assert: (allMethods includesAllOf: nonOverridenMethods).! !

!BehaviorTest methodsFor: 'tests' stamp: 'TestRunner 11/21/2009 11:45'!
testAllReferencesTo
	"
	self debug: #testAllReferencesTo
	"

	| t tmp |
	t := SystemNavigation default allReferencesTo: MessageSet binding.
	self assert: t notEmpty.
	self assert: (t includes: (ChangeList >> #browseCurrentVersionsOfSelections) methodReference).

	self assert: ((ChangeList >> #browseCurrentVersionsOfSelections) hasLiteral: MessageSet binding).

	tmp := false.
	[ SystemNavigation default allReferencesTo: #printOn: ]
		on: Error do: [:ex | tmp := true].
	self assert: tmp.! !

!BehaviorTest methodsFor: 'tests' stamp: 'nice 12/3/2009 23:58'!
testAllSelectors
	self assert: ProtoObject allSelectors asSet = ProtoObject selectors asSet.
	self assert: Object allSelectors asSet = (Object selectors asSet union: ProtoObject selectors).
	self assert: (Object allSelectorsBelow: ProtoObject) asSet = (Object selectors) asSet.! !

!BehaviorTest methodsFor: 'tests' stamp: 'sd 1/28/2009 14:32'!
testAllSelectorsAbove
	"self debug: #testAllSelectorsAbove"
	
	|sels |
	sels := Morph allSelectorsAbove.
	self deny: (sels includes: #submorphs). 
	self deny: (sels includes: #submorphs).
	self assert: (sels includes: #clearHaltOnce).
	self assert: (sels includes: #cannotInterpret: )
	! !

!BehaviorTest methodsFor: 'tests' stamp: 'sd 1/28/2009 14:31'!
testAllSelectorsAboveUntil
	"self debug: #testAllSelectorsAboveUntil"
	
	|sels |
	sels := Morph allSelectorsAboveUntil: Object..
	self deny: (sels includes: #submorphs). 
	self deny: (sels includes: #submorphs).
	self assert: (sels includes: #clearHaltOnce).
	self deny: (sels includes: #cannotInterpret: )
	! !

!BehaviorTest methodsFor: 'tests' stamp: 'sd 3/14/2004 18:11'!
testBehaviorSubclasses
	"self run: #testBehaviorSubclasses"
	
	| b b2 |
	b := Behavior new.
	b superclass: OrderedCollection.
	b methodDictionary: MethodDictionary new.
	self shouldnt: [b subclasses ] raise: Error.
	self shouldnt: [b withAllSubclasses] raise: Error.
	self shouldnt: [b allSubclasses] raise: Error.
	b2 := Behavior new.
	b2 superclass: b.
	b2 methodDictionary: MethodDictionary new.
	self assert: (b subclasses includes: b2).
	self assert: (b withAllSubclasses includes: b).! !

!BehaviorTest methodsFor: 'tests' stamp: 'sd 11/19/2004 15:38'!
testBehaviornewnewShouldNotCrash

	Behavior new new.
	"still not working correctly but at least does not crash the image"
	! !

!BehaviorTest methodsFor: 'tests' stamp: 'marcus.denker 9/14/2008 21:14'!
testBinding
	self assert: Object binding value = Object.
	self assert: Object binding key = #Object.
	
	self assert: Object class binding value = Object class.
	
	"returns nil for Metaclasses... like Encoder>>#associationFor:"
	
	self assert: Object class binding key isNil.! !

!BehaviorTest methodsFor: 'tests' stamp: 'ar 9/27/2005 21:43'!
testChange
	"self debug: #testChange"

	| behavior model |
	behavior := Behavior new.
	behavior superclass: Model.
	behavior setFormat: Model format.
	model := Model new.
	model primitiveChangeClassTo: behavior new.
	behavior compile: 'thisIsATest  ^ 2'.
	self assert: model thisIsATest = 2.
	self should: [Model new thisIsATest] raise: MessageNotUnderstood.


! !

!BehaviorTest methodsFor: 'tests' stamp: 'nice 12/25/2009 19:10'!
testFormalParameterNames
	| method |
	method := #sampleMessageWithFirstArgument:andInterleavedCommentBeforeSecondArgument:.
	self assert: (self class formalParametersAt: method) size = 2.
	self assert: (self class formalParametersAt: method) asArray = #('firstArgument' 'secondArgument').
	Object selectorsDo: [:e |
		self assert: (Object formalParametersAt: e) size = e numArgs].! !

!BehaviorTest methodsFor: 'tests' stamp: 'TestRunner 10/28/2009 11:16'!
testIsAbstractClass
	self deny: Object isAbstractClass.
	self deny: Object class isAbstractClass.
		
	"Behavior has abstract methods, for example hasTraitComposition, basicLocalSelectors:"
	self assert: Behavior isAbstractClass.
	self assert: Behavior class isAbstractClass.
	
	"AbstractEvent does not have an abstract method, but its class yes"
	"AbstractEvent class>>changeKind is abstract, therefore the class is abstract"
	self assert: AbstractEvent isAbstractClass.
	self assert: AbstractEvent class isAbstractClass.
	
	! !

!BehaviorTest methodsFor: 'tests - testing method dictionary' stamp: 'marcus.denker 9/29/2008 15:11'!
testWhichSelectorsAccess
	self assert: ((Point whichSelectorsAccess: 'x') includes: #x).
	self deny:  ((Point whichSelectorsAccess: 'y') includes: #x).! !

!BehaviorTest methodsFor: 'tests' stamp: 'sd 1/28/2009 14:25'!
testallSuperclassesIncluding
	"self debug: #testallSuperclassesIncluding"
	
	|cls |
	cls := ArrayedCollection allSuperclassesIncluding: Collection.
	self deny: (cls includes: ArrayedCollection).
	self deny: (cls includes: Object). 
	self assert: (cls includes: Collection). 
	self assert: (cls includes: SequenceableCollection). ! !

TestCase subclass: #BlockClosureTest
	instanceVariableNames: 'aBlockContext contextOfaBlockContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!BlockClosureTest methodsFor: 'setup' stamp: 'MarcusDenker 2/24/2010 12:28'!
setUp
	super setUp.
	aBlockContext := [100@100 corner: 200@200].
	contextOfaBlockContext := thisContext.! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:32'!
testCull

       self shouldnt: [ [ ] cull: 1 ] raise: Error.
       self shouldnt: [ [ :x | ] cull: 1 ] raise: Error.
       self should: [ [ :x :y | ] cull: 1 ] raise: Error.
       self should: [ [ :x :y :z | ] cull: 1 ] raise: Error.
       self should: [ [ :x :y :z :a | ] cull: 1 ] raise: Error.
       self should: [ [ :x :y :z :a :b | ] cull: 1 ] raise: Error.
       self assert: ([ 0 ] cull: 1) = 0.
       self assert: ([ :x | x ] cull: 1) = 1! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:31'!
testCullCull

       self shouldnt: [ [ ] cull: 1 cull: 2 ] raise: Error.
       self shouldnt: [ [ :x | ] cull: 1 cull: 2 ] raise: Error.
       self shouldnt: [ [ :x :y | ] cull: 1 cull: 2 ] raise: Error.
       self should: [ [ :x :y :z | ] cull: 1 cull: 2 ] raise: Error.
       self should: [ [ :x :y :z :a | ] cull: 1 cull: 2 ] raise: Error.
       self should: [ [ :x :y :z :a :b | ] cull: 1 cull: 2 ] raise: Error.
       self assert: ([ 0 ] cull: 1 cull: 2) = 0.
       self assert: ([ :x | x ] cull: 1 cull: 2) = 1.
       self assert: ([ :x :y | y ] cull: 1 cull: 2) = 2.! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:32'!
testCullCullCull

       self shouldnt: [ [ ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self shouldnt: [ [ :x | ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self shouldnt: [ [ :x :y | ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self shouldnt: [ [ :x :y :z | ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self should: [ [ :x :y :z :a | ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self should: [ [ :x :y :z :a :b | ] cull: 1 cull: 2 cull: 3 ] raise: Error.
       self assert: ([ 0 ] cull: 1 cull: 2 cull: 3) = 0.
       self assert: ([ :x | x ] cull: 1 cull: 2 cull: 3) = 1.
       self assert: ([ :x :y | y ] cull: 1 cull: 2 cull: 3) = 2.
       self assert: ([ :x :y :z | z ] cull: 1 cull: 2 cull: 3) = 3.! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:31'!
testCullCullCullCull

       self shouldnt: [ [ ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self shouldnt: [ [ :x | ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self shouldnt: [ [ :x :y | ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self shouldnt: [ [ :x :y :z | ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self shouldnt: [ [ :x :y :z :a | ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self should: [ [ :x :y :z :a :b | ] cull: 1 cull: 2 cull: 3 cull: 4 ] raise: Error.
       self assert: ([ 0 ] cull: 1 cull: 2 cull: 3 cull: 4) = 0.
       self assert: ([ :x | x ] cull: 1 cull: 2 cull: 3 cull: 4) = 1.
       self assert: ([ :x :y | y ] cull: 1 cull: 2 cull: 3 cull: 4) = 2.
       self assert: ([ :x :y :z | z ] cull: 1 cull: 2 cull: 3 cull: 4) = 3.
       self assert: ([ :x :y :z :a | a ] cull: 1 cull: 2 cull: 3 cull: 4) = 4.! !

!BlockClosureTest methodsFor: 'tests - printing' stamp: 'MarcusDenker 2/24/2010 12:28'!
testDecompile
	self assert: ([3 + 4] decompile printString = '{[3 + 4]}').! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testNew
	self	should: [ContextPart new: 5] raise: Error.
	[ContextPart new: 5]
		ifError: [:error :receiver | error = 'Error: Contexts must only be created with newForMethod:'].
	[ContextPart new]
		ifError: [:error :receiver | error = 'Error: Contexts must only be created with newForMethod:'].	
	[ContextPart basicNew]
		ifError: [:error :receiver | error = 'Error: Contexts must only be created with newForMethod:'].				

! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testNoArguments
	[10
		timesRepeat: [:arg | 1 + 2]]
		ifError: [:err :rcvr | self deny: err = 'This block requires 1 arguments.'].
	[10
		timesRepeat: [:arg1 :arg2 | 1 + 2]]
		ifError: [:err :rcvr | self deny: err = 'This block requires 2 arguments.'] ! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testOneArgument
	| c |
	c := OrderedCollection new.
	c add: 'hello'.
	[c
		do: [1 + 2]]
		ifError: [:err :rcvr | self deny: err = 'This block requires 0 arguments.'].
	[c
		do: [:arg1 :arg2 | 1 + 2]]
		ifError: [:err :rcvr | self deny: err = 'This block requires 2 arguments.'] ! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testRunSimulated
	self assert: (ContextPart runSimulated: aBlockContext) class = Rectangle.! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSetUp
	"Note: In addition to verifying that the setUp worked the way it was expected to, testSetUp is used to illustrate the meaning of the simple access methods, methods that are not normally otherwise 'tested'"
	self deny: aBlockContext isPseudoContext.
	self assert: aBlockContext home = contextOfaBlockContext.
	self assert: aBlockContext receiver = self.
	self assert: (aBlockContext method isKindOf: CompiledMethod).! !

!BlockClosureTest methodsFor: 'testing' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerOfFillInTheBlank

	self should: ['blue' = ([UIManager default request: 'Your favorite color?'] 
		valueSupplyingAnswer: #('Your favorite color?' 'blue'))]! !

!BlockClosureTest methodsFor: 'testing' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerOfFillInTheBlankUsingDefaultAnswer

	self should: ['red' = ([UIManager default  request: 'Your favorite color?' initialAnswer: 'red'] 
		valueSupplyingAnswer: #('Your favorite color?' #default))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerThroughNestedBlocks

	self should: [true = ([[self confirm: 'You like Smalltalk?'] 
		valueSupplyingAnswer: #('Blub' false)] valueSupplyingAnswer: #('Smalltalk' true))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerUsingOnlySubstringOfQuestion

	self should: [false = ([self confirm: 'You like Smalltalk?'] 
		valueSupplyingAnswer: #('like' false))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerUsingRegexMatchOfQuestion

	(String includesSelector: #matchesRegex:) ifFalse: [^ self].
	
	self should: [true = ([self confirm: 'You like Smalltalk?'] 
		valueSupplyingAnswer: #('.*Smalltalk\?' true))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplyAnswerUsingTraditionalMatchOfQuestion

	self should: [true = ([self confirm: 'You like Smalltalk?'] 
		valueSupplyingAnswer: #('*Smalltalk#' true))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplySameAnswerToAllQuestions

	self should: [true = ([self confirm: 'You like Smalltalk?'] valueSupplyingAnswer: true)].
	
	self should: [#(true true) = ([{self confirm: 'One'. self confirm: 'Two'}] valueSupplyingAnswer: true)].! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplySeveralAnswersToSeveralQuestions

	self should: [#(false true) = ([{self confirm: 'One'. self confirm: 'Two'}] 
		valueSupplyingAnswers: #( ('One' false) ('Two' true) ))].
	
	self should: [#(true false) = ([{self confirm: 'One'. self confirm: 'Two'}] 
		valueSupplyingAnswers: #( ('One' true) ('Two' false) ))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSupplySpecificAnswerToQuestion

	self should: [false = ([self confirm: 'You like Smalltalk?'] 
		valueSupplyingAnswer: #('You like Smalltalk?' false))]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSuppressInform

	self should: [[nil inform: 'Should not see this message or this test failed!!'] valueSuppressingAllMessages isNil]! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testSuppressInformUsingStringMatchOptions

	self should: [([nil inform: 'Should not see this message or this test failed!!'] valueSuppressingMessages: #('Should not see this message or this test failed!!')) isNil].
	
	self should: [([nil inform: 'Should not see this message or this test failed!!'] valueSuppressingMessages: #('not see this message')) isNil].
	
	self should: [([nil inform: 'Should not see this message or this test failed!!'] valueSuppressingMessages: #('*message*failed#')) isNil].
! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testTallyInstructions
	self assert: (ContextPart tallyInstructions: aBlockContext) size = 15.! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testTallyMethods
	self assert: (ContextPart tallyMethods: aBlockContext) size = 3.! !

!BlockClosureTest methodsFor: 'tests' stamp: 'MarcusDenker 2/24/2010 12:28'!
testTrace
	self assert: (ContextPart trace: aBlockContext) class = Rectangle.! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:28'!
testValueWithArguments
	self
		should: [aBlockContext
				valueWithArguments: #(1 )]
		raise: Error.
	self
		shouldnt: [aBlockContext
				valueWithArguments: #()]
		raise: Error.
	[aBlockContext
		valueWithArguments: #(1 )]
		ifError: [:err :rcvr | self assert: err = 'Error: This block accepts 0 arguments, but was called with 1 argument.'].
	[[:i | 3 + 4]
		valueWithArguments: #(1 2)]
		ifError: [:err :rcvr | self assert: err = 'Error: This block accepts 1 argument, but was called with 2 arguments.']! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:28'!
testValueWithExitBreak

	| val |	

	[ :break |
	    1 to: 10 do: [ :i |
			val := i.
			i = 4 ifTrue: [break value].
		] 
	] valueWithExit.

	self assert: val = 4.! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:28'!
testValueWithExitContinue

	| val last |	
	val := 0. 

	1 to: 10 do: [ :i |
		[ :continue |
			i = 4 ifTrue: [continue value].
			val := val + 1.
			last := i
		] valueWithExit.
	].

	self assert: val = 9.
	self assert: last = 10.	! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:28'!
testValueWithPossibleArgs
	| block  blockWithArg blockWith2Arg |

	block := [1].
	blockWithArg  := [:arg | arg].
	blockWith2Arg := [:arg1 :arg2 | {arg1. arg2}].

	self assert: (block valueWithPossibleArgs: #()) = 1.
	self assert: (block valueWithPossibleArgs: #(1)) = 1.
	
	self assert: (blockWithArg valueWithPossibleArgs: #()) = nil.
	self assert: (blockWithArg valueWithPossibleArgs: #(1)) = 1.
	self assert: (blockWithArg valueWithPossibleArgs: #(1 2)) = 1.

	self assert: (blockWith2Arg valueWithPossibleArgs: #()) = {nil .nil}.
	self assert: (blockWith2Arg valueWithPossibleArgs: #(1)) =  {1 . nil}.
	self assert: (blockWith2Arg valueWithPossibleArgs: #(1 2)) =  #(1 2).
	self assert: (blockWith2Arg valueWithPossibleArgs: #(1 2 3)) = #(1 2).
		

	! !

!BlockClosureTest methodsFor: 'tests - evaluating' stamp: 'MarcusDenker 2/24/2010 12:28'!
testValueWithPossibleArgument
	| block  blockWithArg blockWith2Arg |

	block := [1].
	blockWithArg  := [:arg | arg].
	blockWith2Arg := [:arg1 :arg2 | {arg1. arg2}].

	self assert: (block valueWithPossibleArgument: 1) = 1.
	
	self assert: (blockWithArg valueWithPossibleArgument: 1) = 1.
	
	self assert: (blockWith2Arg valueWithPossibleArgument: 1) = {1 . nil}.
	

	! !

TestCase subclass: #BlockClosuresTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!BlockClosuresTestCase commentStamp: '<historical>' prior: 0!
This test case collects examples for block uses that require full block closures.!

!BlockClosuresTestCase methodsFor: 'jensen device examples' stamp: 'BG 1/25/2002 10:01'!
comment

  "  The Jensen device was something very sophisticated in the days of Algol 60.
Essentially it was tricky use of a parameter passing policy that was called 'call by name'. In modern terminology, a call by name parameter was a pair of blocks (in a system with full block closures, of course.)

For the lovers of Algol 60, here is a short example:

 BEGIN
  REAL PROCEDURE JensenSum (A, I, N);
     REAL  A;   INTEGER   I, N;
  BEGIN
    REAL  S;
    S := 0.0;
    FOR I := 1 STEP 1 UNTIL N DO  S := S + A;
  JensenSum := S;
  END;

  ARRAY X [1:10], Y[1:10, 1:10];
  COMMENT Do array initialization here ;

  JensenSum (X[I], I, 10);
  JensenSum (Y[I, I], I, 10);
  JensenSum(JensenSum(Y[I, J], J, 10), I, 10);
END;

The first call sums the elements of X, the second sums the diagonal elements of Y and the third call sums up all elements of Y.

It is possible to reimplement all this with blocks only and that is what is done in the jensen device examples.

Additional remark:
The Jensen device was something for clever minds. I remember an artice written by Donald Knuth and published in the Communications of the ACM (I think in 1962, but I may err) about that programming trick. That article showed how a simple procedure (called the general problem solver) could be used to do almost anything. The problem was of course to find out the right parameters. I seached my collection of photocopies for that article, but regrettably I could not find it. Perhaps I can find it later.
 "! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'rw 1/26/2002 01:20'!
constructCannotReturnBlockInDeadFrame

	^ [:arg | ^arg].
! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'rw 1/26/2002 01:14'!
constructFibonacciBlockInDeadFrame

	| fib |
	fib := [:val | 
		(val <= 0) ifTrue: [self error: 'not a natural number'].
		(val <= 2) ifTrue: [1]
		     ifFalse: [(fib value: (val - 1)) + (fib value: (val - 2))]].
	^fib
! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'rw 1/26/2002 01:14'!
constructFibonacciBlockWithBlockArgumentInDeadFrame

	^ [:val :blk | 
		(val <= 0) ifTrue: [self error: 'not a natural number'].
		(val <= 2) ifTrue: [1]
		    ifFalse: [(blk value: (val - 1) value: blk) + (blk value: (val - 2) value: blk)]].
! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'rw 1/26/2002 01:15'!
constructSharedClosureEnvironmentInDeadFrame

	|array result| 
	result := 10.
	array := Array new: 2.
	array at: 1 put: [:arg | result := arg].
	array at: 2 put: [result].
	^array
! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'BG 1/24/2002 17:44'!
continuationExample1: aCollection

  " see comment below.
    Here we simply collect the values of a value with continuation block "


     | streamCreator collector |

   streamCreator := [:collection | | i localBlock |
                i := 1.
                localBlock :=
                    [  | current |
                     current := collection at: i.
                     i := i + 1.
                     Array with: current
                           with: (i<= collection size ifTrue: [localBlock]
                                                      ifFalse: [nil])
                    ].
             ].
 

 collector := [:valueWithContinuation |  | oc |
                 oc := OrderedCollection new.
                 [ | local |
                  local := valueWithContinuation value.
                  oc add: local first.
                  local last notNil]
                 whileTrue: [].
                 oc.
               ].

  ^collector value: (streamCreator value: aCollection).

"The continuation examples are examples of a 'back to LISP' style.
These examples use blocks to process the elements of a collection in a
fashion that is similar to streaming.
The creator block creates a blocks that act like a stream. In the
following, this block is called a 'value with continuation block'.
When such a value with continuation block receives the message
value, it returns a Array of two elements, the value and the
continuation
 1. the next collection element
 2. a so-called continuation, which is either nil or a block
    that can return the next value with continuation.
    
To collect all elements of a value with continuation stream,
use the collector block. "
! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'BG 1/24/2002 17:44'!
continuationExample2: aCollection

  " see comment in method continuationExample1:.
    The block named 'processor' takes a value with contiuation
    and a processing block. It creates a new value with continuation.
   Again we use a collector to collect all values.  "

     | stream processor collector |

   stream := [:collection | | i localBlock |
                i := 1.
                localBlock :=
                    [  | current |
                     current := collection at: i.
                     i := i + 1.
                     Array with: current
                           with: (i<= collection size ifTrue: [localBlock]
                                                      ifFalse: [nil])
                    ].
             ].
 
  processor := 
            [:valueWithContinuation :activity | | localBlock |
                localBlock :=
                 [ | current |
                   current := valueWithContinuation value. 
                   Array with: (activity value: current first)
                         with: (current last notNil ifTrue: [localBlock])].
               localBlock
           ].

 collector := [:valueWithContinuation |  | oc |
                 oc := OrderedCollection new.
                 [ | local |
                  local := valueWithContinuation value.
                  oc add: local first.
                  local last notNil]
                 whileTrue: [].
                 oc.
               ].

  ^collector value: (processor value: (stream value: aCollection)
                                           value: [:x | x * x]).! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'BG 1/24/2002 17:27'!
continuationExample3: aCollection

  " see comment in method continuationExample1:.
    The block named 'processor' takes a value with contiuation
    and a processing block. It creates a new value with continuation.
    Here we set up a chain of three values with continuation:
    one data source and two value processors.
    Again we use a collector to collect all values.  "

    | stream processor collector |

   stream := [:collection | | i localBlock |
                i := 1.
                localBlock :=
                    [  | current |
                     current := collection at: i.
                     i := i + 1.
                     Array with: current
                           with: (i<= collection size ifTrue: [localBlock]
                                                      ifFalse: [nil])
                    ].
             ].
 
  processor := 
            [:valueWithContinuation :activity | | localBlock |
                localBlock :=
                 [ | current |
                   current := valueWithContinuation value. 
                   Array with: (activity value: current first)
                         with: (current last notNil ifTrue: [localBlock])].
               localBlock
           ].

 collector := [:valueWithContinuation |  | oc |
                 oc := OrderedCollection new.
                 [ | local |
                  local := valueWithContinuation value.
                  oc add: local first.
                  local last notNil]
                 whileTrue: [].
                 oc.
               ].

  ^collector value: (processor value: (processor value: (stream value: aCollection)
                                                                     value: [:x | x * x])
                                            value: [:x | x - 10]).! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'BG 1/24/2002 17:44'!
example1: anInteger

  " this example is very simple. A named block recursively computes the factorial.
    The example tests whether the value of x is still available after the recursive call.
    Note that the recursive call precedes the multiplication. For the purpose of the test
    this is essential. (When you commute the factors, the example will work also in
    some system without block closures, but not in Squeak.) "

    | factorial |

   factorial := [:x | x = 1 ifTrue: [1]
                            ifFalse: [(factorial value: x - 1)* x]].
  ^ factorial value: anInteger


  ! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'lr 3/31/2009 09:31'!
example2: anInteger

  " BlockClosuresTestCase new example2: 6"
  " to complicate the example1, we set up a dynamic reference chain that is
    used to dump all calls of facorial when recursion depth is maximal.
    The return value is an instance of orderedCollection, the trace. "

    | factorial trace |

   trace := OrderedCollection new.
   factorial :=
       [:x :dumper :trace2 |  | localDumper |
          localDumper := [ :collection |
                             collection add: x. 
                             dumper value: collection.].
            x = 1 ifTrue:
                    [localDumper value: trace2.
                     1]
                  ifFalse:
                    [(factorial value: x - 1
                                 value: localDumper
                                 value: trace2)* x.
                   ]
         ].
  factorial value: anInteger
             value: [ :collection | ]
             value: trace.
  ^trace! !

!BlockClosuresTestCase methodsFor: 'jensen device examples' stamp: 'BG 1/24/2002 18:00'!
gpsExample1: aCollection

   " BlockClosuresTestCase new gpsExample1: (1 to: 100) asArray"

 | gps i  s |

  gps := [:idx :exp :sum | | cnt |
               cnt := 1.
               sum first value: 0.
               [idx first value: cnt.
                sum first value: (sum last value + exp last value). 
                cnt := cnt + 1.
                cnt <= aCollection size] whileTrue: [   ].
              sum last value
             ].


  ^gps value: (Array with: [:val | i := val]
                    with: [ i])
      value: (Array with: [:val | aCollection at: i put:  val]
                    with: [ aCollection at: i])
      value: (Array with: [:val | s := val]
                    with: [ s])
! !

!BlockClosuresTestCase methodsFor: 'jensen device examples' stamp: 'BG 1/25/2002 10:03'!
gpsExample2: aCollection

   " BlockClosuresTestCase new 
       gpsExample2: #(#(1 2 3 4 5) #(6 7 8 9 10) #(11 12 13 14 15) #(16 17 18 19 20) #(21 22 23 24 25))"

    | js i j |

  "  js is the translation of the Algol procedure from method comment. "
  js := [:a :idx :n | | sum |
               sum := 0.
               idx first value: 1.
               [idx last value <= n last value]
                 whileTrue:
                   [sum :=  sum  + a last value. 
                    idx first value:  idx last value + 1.].
              sum
             ].

  "  This is the most complicated call that is mentioned in method comment. Note that  js  is called recursively. "

  ^ js value: (Array with: [:val | self error: 'can not assign to procedure']
                        with: [ js value: (Array with: [:val | (aCollection at: i) at: j put: val]
                                                     with: [ (aCollection at: i) at: j])
                                   value: (Array with:[:val | j := val]
                                                     with: [ j])
                                   value: (Array with: [:val | self error: 'can not assign to constant']
                                                     with: [ aCollection size])
                               ]
               )
    value: (Array with:[:val | i := val]
                  with: [ i])
    value: (Array with: [:val | self error: 'can not assign to constant']
                  with: [ aCollection size])

! !

!BlockClosuresTestCase methodsFor: 'examples' stamp: 'PeterHugossonMiller 9/2/2009 16:19'!
nestedLoopsExample: arrays

 " A while ago, Hans Baveco asked for a way to
   dynamically nest loops. Better solutions than this one
   were proposed, but this one is a beautiful test for
   recursive block usage. "
 
  | result sizeOfResult streams block |


"arrays := OrderedCollection new.
arrays add: #(#a #b);
       add: #(1 2 3 4);
       add: #('w' 'x' 'y' 'z')."
sizeOfResult :=
   arrays inject: 1 into:
          [:prod :array | prod * array size].
streams := arrays collect:
			[:a | a readStream]. " This is an OrderedCollection of Streams "

result := OrderedCollection new: sizeOfResult.
block :=
 [:r :tupel :allStreams | | innerBlock |
   innerBlock :=
    [:myIdx |
       [myIdx = allStreams size
	  ifTrue: [1 to: allStreams size do:
		   [:i | tupel at: i put: (allStreams at: i) peek].
			 r addLast: tupel shallowCopy]
	 ifFalse:  [innerBlock value: myIdx + 1].
	(allStreams at: myIdx) next.     
	(allStreams at: myIdx) atEnd
      ] 
       whileFalse: [].
     (allStreams at: myIdx) reset.
    ].
    innerBlock value: 1.
   r   
  ].
 block value: result 
         value: (Array new: streams size) " this is a buffer "
         value: streams.

  ^result
! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'rw 1/26/2002 01:22'!
testCannotReturn

	| blk |
	blk := self constructCannotReturnBlockInDeadFrame.
	self 
		should: [blk value: 1]
		raise: Exception
! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 17:45'!
testContinuationExample1

   | array |
    array := (1 to: 20) asOrderedCollection.
   self assert: ((self continuationExample1: array) = array)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 17:45'!
testContinuationExample2

   | array |
    array := (1 to: 20) asOrderedCollection.
   self assert: ((self continuationExample2: array) = (array collect: [:x | x * x]))
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 17:45'!
testContinuationExample3

   | array |
    array := (1 to: 20) asOrderedCollection.
   self assert: ((self continuationExample3: array) = (array collect: [:x | x * x - 10]))
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 16:05'!
testExample1

   self assert: ((self example1: 5) = 5 factorial)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 18:28'!
testExample2

   self assert: ((self example2: 5) = (1 to: 5) asOrderedCollection)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 17:59'!
testGpsExample1

  | result array |

  array := (1 to: 100) asArray.
  result := array inject: 0
              into: [:sum :val | sum + val].
 self assert: ((self gpsExample1: array) = result)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/25/2002 09:57'!
testGpsExample2

  | result array |

  "  integer matrix elements should be used for the purpose of this test. " 
  array := #(#(1 2 3 4 5) #(6 7 8 9 10) #(11 12 13 14 15) #(16 17 18 19 20) #(21 22 23 24 25)).
  result := array inject: 0
              into: [:sum :subarray |
                      sum + (subarray inject: 0
                                          into: [:s :elem | s + elem])].
 self assert: ((self gpsExample2: array) = result)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'BG 1/24/2002 16:40'!
testNestedLoopsExample1

   | arrays result |
   arrays := OrderedCollection new.
   arrays add: #(#a #b);
             add: #(1 2 3 4);
             add: #('w' 'x' 'y' 'z').
    result := OrderedCollection new.
    CollectionCombinator new
          forArrays:  arrays
          processWith: [:item |result addLast: item].
   self assert: ((self nestedLoopsExample: arrays) = result)
  ! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'rw 1/26/2002 01:17'!
testReentrantBlock

	| fib |
	fib := [:val | 
		(val <= 0) ifTrue: [self error: 'not a natural number'].
		(val <= 2) 
			ifTrue: [1]
			ifFalse: [(fib value: (val - 1)) + (fib value: (val - 2))]].

	self 
		should: [fib value: 0] 
		raise: TestResult error.
	self assert: ((fib value: 1) == 1).
	self assert: ((fib value: 2) == 1).
	self assert: ((fib value: 3) == 2).
	self assert: ((fib value: 4) == 3).
	self assert: ((fib value: 5) == 5).
	self assert: ((fib value: 6) == 8).
! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'rw 1/26/2002 01:17'!
testReentrantBlockOldEnvironment

	| fib |
	fib := self constructFibonacciBlockInDeadFrame.
	self 
		should: [fib value: 0] 
		raise: TestResult error.
	self assert: ((fib value: 1) == 1).
	self assert: ((fib value: 2) == 1).
	self assert: ((fib value: 3) == 2).
	self assert: ((fib value: 4) == 3).
	self assert: ((fib value: 5) == 5).
	self assert: ((fib value: 6) == 8).
! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'rw 1/26/2002 01:18'!
testReentrantBlockOldEnvironmentWithBlockArguement

	| fib |
	fib := self constructFibonacciBlockWithBlockArgumentInDeadFrame.
	self 
		should: [fib value: 0 value: fib] 
		raise: TestResult error.
	self assert: ((fib value: 1 value: fib) == 1).
	self assert: ((fib value: 2 value: fib) == 1).
	self assert: ((fib value: 3 value: fib) == 2).
	self assert: ((fib value: 4 value: fib) == 3).
	self assert: ((fib value: 5 value: fib) == 5).
	self assert: ((fib value: 6 value: fib) == 8).
! !

!BlockClosuresTestCase methodsFor: 'testing' stamp: 'rw 1/26/2002 01:18'!
testSharedClosureEnvironment
	|blockArray|
	blockArray := self constructSharedClosureEnvironmentInDeadFrame.
	self assert: ((blockArray at: 2) value == 10).
	self assert: (((blockArray at: 1) value: 5) == 5).
	self assert: ((blockArray at: 2) value == 5).
! !

TestCase subclass: #ClassBuilderChangeClassTypeTest
	instanceVariableNames: 'baseClass subClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!ClassBuilderChangeClassTypeTest methodsFor: 'utilities' stamp: 'BG 1/5/2004 22:49'!
baseClassName

   ^'TestClassForClassChangeTest'! !

!ClassBuilderChangeClassTypeTest methodsFor: 'utilities' stamp: 'BG 1/5/2004 22:51'!
cleanup
	baseClass ifNotNil:[baseClass removeFromSystem].! !

TestCase subclass: #ClassBuilderFormatTests
	instanceVariableNames: 'baseClass subClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!ClassBuilderFormatTests class methodsFor: 'testing' stamp: 'JorgeRessia 3/16/2010 20:23'!
isUnitTest
	^false! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:15'!
baseClassName
	^#DummyClassBuilderFormatTestSuperClass! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'SergeStinckwich 1/10/2010 18:13'!
cleanup
	subClass ifNotNil:[subClass removeFromSystem].
	baseClass ifNotNil:[baseClass removeFromSystem].
	SystemOrganization removeCategory:'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:15'!
makeByteVariableSubclassOf: aClass
	subClass := aClass variableByteSubclass: self subClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:15'!
makeIVarsSubclassOf: aClass
	subClass := aClass subclass: self subClassName
		instanceVariableNames: 'var3 var4'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:15'!
makeNormalSubclassOf: aClass
	subClass := aClass subclass: self subClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:15'!
makeVariableSubclassOf: aClass
	subClass := aClass variableSubclass: self subClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:16'!
makeWeakSubclassOf: aClass
	subClass := aClass weakSubclass: self subClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:16'!
makeWordVariableSubclassOf: aClass
	subClass := aClass variableWordSubclass: self subClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'! !

!ClassBuilderFormatTests methodsFor: 'utilities' stamp: 'ar 1/4/2004 20:16'!
subClassName
	^#DummyClassBuilderFormatTestSubClass! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:21'!
testByteVariableSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object variableByteSubclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[

	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self assert: (subClass isBytes).
	subClass removeFromSystem.

	"pointer classes"
	self should:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self should:[self makeVariableSubclassOf: baseClass] raise: Error.
	self should:[self makeWeakSubclassOf: baseClass] raise: Error.

	"bit classes"
	self shouldnt:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self assert: (subClass isBytes).
	subClass removeFromSystem.

	self should:[self makeWordVariableSubclassOf: baseClass] raise: Error.

	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 7/13/2009 21:18'!
testChangeToVariableSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object subclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
		self shouldnt:[baseClass := Object variableSubclass: self baseClassName
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: 'Kernel-Tests-ClassBuilder'] raise: Error.

	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:20'!
testSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object subclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self deny: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"pointer classes"
	self shouldnt:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self deny: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeVariableSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert:(subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeWeakSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert:(subClass isVariable).
	self assert:(subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"bit classes"
	self shouldnt:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self assert: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeWordVariableSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.
	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:21'!
testSubclassWithInstanceVariables
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object subclass: self baseClassName
		instanceVariableNames: 'var1 var2'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self deny: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"pointer classes"
	self shouldnt:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self deny: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeVariableSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeWeakSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self assert: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"bit classes"
	self should:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self should:[self makeWordVariableSubclassOf: baseClass] raise: Error.
	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:20'!
testVariableSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object variableSubclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
	"pointer classes"
	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeVariableSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeWeakSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self assert: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"bit classes"
	self should:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self should:[self makeWordVariableSubclassOf: baseClass] raise: Error.
	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:20'!
testWeakSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object weakSubclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
	"pointer classes"
	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self assert: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self assert: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeVariableSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	self shouldnt:[self makeWeakSubclassOf: baseClass] raise: Error.
	self assert: (subClass isPointers).
	self assert: (subClass isVariable).
	self assert: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"bit classes"
	self should:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self should:[self makeWordVariableSubclassOf: baseClass] raise: Error.
	] ensure:[self cleanup].! !

!ClassBuilderFormatTests methodsFor: 'testing' stamp: 'ar 1/4/2004 20:20'!
testWordVariableSubclass
	"Ensure that the invariants for superclass/subclass format are preserved"
	baseClass := Object variableWordSubclass: self baseClassName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Kernel-Tests-ClassBuilder'.
	[
	self shouldnt:[self makeNormalSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	"pointer classes"
	self should:[self makeIVarsSubclassOf: baseClass] raise: Error.
	self should:[self makeVariableSubclassOf: baseClass] raise: Error.
	self should:[self makeWeakSubclassOf: baseClass] raise: Error.

	"bit classes"
	self should:[self makeByteVariableSubclassOf: baseClass] raise: Error.
	self shouldnt:[self makeWordVariableSubclassOf: baseClass] raise: Error.
	self deny: (subClass isPointers).
	self assert: (subClass isVariable).
	self deny: (subClass isWeak).
	self deny: (subClass isBytes).
	subClass removeFromSystem.

	] ensure:[self cleanup].! !

TestCase subclass: #ClassTest
	instanceVariableNames: 'className renamedName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!ClassTest methodsFor: 'setup' stamp: 'lr 3/14/2010 21:13'!
deleteClass
	| cl |
	cl := Smalltalk globals at: className ifAbsent: [ ^ self ].
	cl
		removeFromChanges;
		removeFromSystemUnlogged! !

!ClassTest methodsFor: 'setup' stamp: 'lr 3/14/2010 21:13'!
deleteRenamedClass
	| cl |
	cl := Smalltalk globals at: renamedName ifAbsent: [ ^ self ].
	cl
		removeFromChanges;
		removeFromSystemUnlogged! !

!ClassTest methodsFor: 'setup' stamp: 'rw 10/17/2006 22:05'!
setUp
	className := #TUTU.
	renamedName := #RenamedTUTU.
	self deleteClass.
	self deleteRenamedClass.
	Object subclass: className
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'KernelTests-Classes'! !

!ClassTest methodsFor: 'setup' stamp: 'rw 10/17/2006 22:08'!
tearDown
	self deleteClass.
	self deleteRenamedClass! !

!ClassTest methodsFor: 'testing' stamp: 'lr 3/14/2010 21:13'!
testAddInstVarName
	"self run: #testAddInstVarName"

	| tutu |
	tutu := Smalltalk globals at: #TUTU.
	tutu addInstVarName: 'x'.
	self assert: tutu instVarNames = #('x').
	tutu addInstVarName: 'y'.
	self assert: tutu instVarNames = #('x' 'y')! !

!ClassTest methodsFor: 'testing - class variables' stamp: 'marcus.denker 12/4/2008 11:12'!
testClassVarNames

	self assert: (Object classVarNames includes: #DependentsFields).
	
	"A class and it's meta-class share the class variables"
	self assert: (Object classVarNames = Object class classVarNames).! !

!ClassTest methodsFor: 'testing - compiling' stamp: 'sd 6/5/2005 08:25'!
testCompileAll

	self shouldnt: [ClassTest compileAll] raise: Error.! !

!ClassTest methodsFor: 'testing - access' stamp: 'sd 4/24/2008 22:11'!
testHaSharedPools
	"self run: #testHaSharedPools"

	self deny: Point hasSharedPools.
	self assert: Date hasSharedPools! !

!ClassTest methodsFor: 'testing' stamp: 'lr 3/14/2010 21:13'!
testRenaming
	"self debug: #testRenaming"

	"self run: #testRenaming"

	| oldName newMetaclassName class |
	oldName := className.
	newMetaclassName := (renamedName , #' class') asSymbol.
	class := Smalltalk globals at: oldName.
	class class compile: 'dummyMeth'.
	class rename: renamedName.
	self assert: class name = renamedName.
	self assert: (ChangeSet current changedClassNames includes: renamedName).
	self assert: (ChangeSet current changedClassNames includes: newMetaclassName)! !

TestCase subclass: #CompiledMethodTrailerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:13'!
testEmbeddingSourceCode

	| trailer newTrailer code |
	
	trailer := CompiledMethodTrailer new.
	
	code := 'foo'.
	trailer sourceCode: code.
	newTrailer := trailer testEncoding.
	
	self assert: (trailer kind == #EmbeddedSourceQCompress ).
	self assert: (newTrailer sourceCode = code).

	"the last bytecode index must be at 0"
	self assert: (newTrailer endPC = 0).

	code := 'testEmbeddingSourceCode

	| trailer newTrailer code |
	
	trailer := CompiledMethodTrailer new.
	
	trailer sourceCode: code.
	newTrailer := trailer testEncoding.
	
	self assert: (newTrailer sourceCode = code).'.

	trailer sourceCode: code.
	self assert: (trailer kind == #EmbeddedSourceZip ).
	newTrailer := trailer testEncoding.
	
	self assert: (newTrailer sourceCode = code).
	"the last bytecode index must be at 0"
	self assert: (newTrailer endPC = 0).
! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:13'!
testEmbeddingTempNames

	| trailer newTrailer code |
	
	trailer := CompiledMethodTrailer new.
	
	code := 'foo'.
	trailer tempNames: code.
	newTrailer := trailer testEncoding.
	
	self assert: (trailer kind == #TempsNamesQCompress ).
	self assert: (newTrailer tempNames = code).
	"the last bytecode index must be at 0"
	self assert: (newTrailer endPC = 0).
	

	code := 'testEmbeddingSourceCode

	| trailer newTrailer code |
	
	trailer := CompiledMethodTrailer new.
	
	trailer sourceCode: code.
	newTrailer := trailer testEncoding.
	
	self assert: (newTrailer sourceCode = code).'.

	trailer tempNames: code.
	self assert: (trailer kind == #TempsNamesZip ).
	newTrailer := trailer testEncoding.
	
	self assert: (newTrailer tempNames = code).
	"the last bytecode index must be at 0"
	self assert: (newTrailer endPC = 0).
! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:17'!
testEncodingNoTrailer

	| trailer |
	
	trailer := CompiledMethodTrailer new.
	
	"by default it should be a no-trailer"	
	self assert: (trailer kind == #NoTrailer ).
	self assert: (trailer size = 1).
	
	trailer := trailer testEncoding.
	
	self assert: (trailer kind == #NoTrailer ).
	self assert: (trailer size = 1).
	"the last bytecode index must be at 0"
	self assert: (trailer endPC = 0).
! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:14'!
testEncodingSourcePointer

	| trailer |
	
	trailer := CompiledMethodTrailer new.
	
	CompiledMethod allInstancesDo: [:method | | ptr |
		trailer method: method.
		self assert: ( (ptr := method sourcePointer) == trailer sourcePointer).
		"the last bytecode index must be at 0"
		ptr ~= 0 ifTrue: [
			self assert: (method endPC = trailer endPC) ].
	 ].! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:15'!
testEncodingVarLengthSourcePointer

	| trailer newTrailer |
	
	trailer := CompiledMethodTrailer new.
	
	trailer sourcePointer: 1.
	newTrailer := trailer testEncoding.
	
	self assert: (newTrailer sourcePointer = 1).
	
	trailer sourcePointer: 16r100000000000000.
	newTrailer := trailer testEncoding.
	self assert: (newTrailer sourcePointer = 16r100000000000000).
	"the last bytecode index must be at 0"
	self assert: (newTrailer endPC = 0).
! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:15'!
testSourceByIdentifierEncoding

	| trailer id |
	
	trailer := CompiledMethodTrailer new.
	
	id := UUID new asString.
	trailer sourceIdentifier: id.
	
	self assert: (trailer kind == #SourceByStringIdentifier ).
	
	trailer := trailer testEncoding.
	
	self assert: (trailer kind == #SourceByStringIdentifier ).
	self assert: (trailer sourceIdentifier = id).
	"the last bytecode index must be at 0"
	self assert: (trailer endPC = 0).
! !

!CompiledMethodTrailerTest methodsFor: 'tests' stamp: 'Igor.Stasenko 12/13/2009 21:49'!
testSourceBySelectorEncoding

	| trailer |
	
	trailer := CompiledMethodTrailer new.
	
	trailer setSourceBySelector.
	
	self assert: (trailer kind == #SourceBySelector ).
	self assert: (trailer size = 1).
	
	trailer := trailer testEncoding.
	
	self assert: (trailer kind == #SourceBySelector ).
	self assert: (trailer size = 1).
	"the last bytecode index must be at 0"
	self assert: (trailer endPC = 0).
! !

TestCase subclass: #ComplexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 15:58'!
testAbs
	"self run: #testAbs"
	"self debug: #testAbs"
	
	| c |
	c := (6 - 6 i).
	self assert: c abs  = 72 sqrt.
	self assert: (3+4i) abs = 5.! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:00'!
testAdding
	"self run: #testAdding"
	
	| c |
	c := (5 - 6 i) + (-5 + 8 i).     "Complex with Complex"
	self assert: (c =  (0 + 2 i)).
	self assert: (1+1i) + 2 = (3+1i).
	self assert: 2 + (1+1i) = (3+1i).
	self assert: (1+1i) + 2i = (1+3i).
	self assert: 2i + (1+1i) = (1+3i).! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:00'!
testArg
	"self run: #testArg"
	"self debug: #testArg"
	
	| c |
	c := (0 + 5 i) .
	self assert: c arg  = (Float pi / 2).
	! !

!ComplexTest methodsFor: 'testing - bugs' stamp: 'md 2/18/2006 16:53'!
testBug1

	self assert: (0.5 * (2+0i) ln) exp = (0.5 * 2 ln) exp.! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:01'!
testCompare
	"Complex objects with zero imaginary are Real numbers and should behave as such"
	self assert: ((1 negated asFloat) < (1 asFloat)).
	self assert: ((1 negated asFraction) < (1 asFraction)).
	self assert: ((1 negated asNumber) < (1 asNumber)).
	self assert: ((1 negated asComplex) < (1 asComplex)).
	self should: [((1 negated)i) < (1i)] raise: ArithmeticError.	self should: [-1i < 1i] raise: ArithmeticError.
	self assert: (-1 asComplex < 1 asComplex).
	self  should: [0 < (1 + 0.002i)]   raise: ArithmeticError.
	self  should: [(0 + 0.001i) < 1/2] raise: ArithmeticError.
	self  should: [1/2 < (1 + 0.002i)] raise: ArithmeticError.
	self  should: [(1/2 + 0.001i) < 1] raise: ArithmeticError.! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 7/17/2004 14:13'!
testComplexCollection
	"self run: #testComplexCollection"
	"self debug: #testComplexCollection"
	
	| array array2 |
	array := Array with: 1 + 2i with:  3 + 4i with: 5 + 6i.
	array2 := 2 * array.
	array with:  array2 do: [:one :two | self assert: (2 * one) = two ] ! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/26/2009 16:44'!
testConversion
	"self run: #testConversion"
	"self debug: #testConversion"
	
	self assert: ((1 + 2i) + 1) =  (2 + 2 i).
	self assert: (1 + (1 + 2i)) =  (2 + 2 i).
	self assert: ((1 + 2i) + 1.0) =  (2.0 + 2 i).
	self assert: (1.0 + (1 + 2i)) =  (2.0 + 2 i).
	self assert: ((1 + 2i) + (2/3)) = ((5/3) + 2 i ).
	self assert: ((2/3) + (1 + 2i)) = ((5/3) + 2 i ).
	self assert: (1 + 0i) = 1 asComplex.! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 7/17/2004 13:59'!
testCreation
	"self run: #testCreation"
	
	| c |
	c := 5 i.
	self assert: (c real = 0).
	self assert: (c imaginary = 5).
	
	c := 6 + 7 i.
	self assert: (c real = 6).
	self assert: ( c imaginary = 7).
	
	c := 5.6 - 8 i.
	self assert: (c real = 5.6).
	self assert: (c imaginary = -8).
	
	c := Complex real: 10 imaginary: 5.
	self assert: (c real = 10).
	self assert: (c imaginary = 5).
	
	c := Complex abs: 5 arg: (Float pi/2).
	self assert: (c real rounded = 0).
	self assert: (c imaginary = 5).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:29'!
testDivision1
	"self run: #testDivision1"
	"self debug: #testDivision1"
	
	| c1 c2 quotient |
	c1 := 2.0e252 + 3.0e70 i.
	c2 := c1.
	quotient := c1 / c2.
 	self deny: (quotient - 1) isZero.
	
	"This test fails due to the wonders of floating point arithmetic. 
	 Please have a look at Complex>>divideSecureBy: and #divideFastAndSecureBy:
	how this can be avoided."
	
! !

!ComplexTest methodsFor: 'testing' stamp: 'KenDickey 8/28/2009 16:02'!
testEquality
	"self run: #testEquality"
	"self debug: #testEquality"
	
	self assert: 0i = 0.
	self assert: (2 - 5i) = ((1 -4 i) + (1 - 1i)).
	self assert: 0i isZero.
	self deny: (1 + 3 i) = 1.
	self deny: (1 + 3 i) = (1 + 2i).

	"Some more stuff"
	self deny: (1 i) = nil.
	self deny: nil = (1 i).

	self deny: (1 i) = #(1 2 3).
	self deny: #(1 2 3) = (1 i).

	self deny: (1 i) = 0.
	self deny: 0 = (1 i).

	self assert:  (1 + 0 i) = 1.
	self assert:  1 = (1 + 0 i).

	self assert:  (1 + 0 i) = 1.0.
	self assert:  1.0 = (1+ 0 i).

	self assert:  (1/2 + 0 i) = (1/2).
	self assert:  (1/2) = (1/2+ 0 i).! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/26/2009 16:52'!
testExpLn
	"self run: #testMath"

	| a epsilon |
	a := ((1/2) + 2.1i).
	epsilon := 0.00001.
	"Because of rounding, the following do not compare exactly."
 	self assert: ((3 ln)  roundTo: epsilon) = (1.09861 roundTo: epsilon).
 	self assert: ((-3 asComplex ln) roundTo: epsilon) = ((1.09861 + 3.14159i) roundTo: epsilon).
 	self assert: ((a ln)  roundTo: epsilon) = ((0.769508 + 1.33705i) roundTo: epsilon).
 	self assert: ((3 exp) roundTo: epsilon) = (20.08553692318767 roundTo: epsilon).
 	self assert: ((a exp) roundTo: epsilon) 
			= ((-0.832350511083887 + 1.423191643861584i) roundTo: epsilon).! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/27/2009 16:33'!
testHyperbolicMath
	| a epsilon |
	a := ((1/2) + 2.1i).
	epsilon := 0.00001.
	"Because of rounding, the following do not compare exactly."
 	self assert: ((a sinH) roundTo: epsilon) = ((-0.263073 + 0.973377i) roundTo: epsilon).
 	self assert: ((a cosH) roundTo: epsilon) = ((-0.569278 + 0.449814i) roundTo: epsilon).
 	self assert: ((a tanH) roundTo: epsilon) = ((1.11624 + -0.827849i) roundTo: epsilon).
 	self assert: ((a sinH arcSinH negated + ((Float pi)i)) roundTo: epsilon) = a.
 	self assert: ((a cosH arcCosH) roundTo: epsilon)  = a.
 	self assert: ((a tanH arcTanH + ((Float pi)i)) roundTo: epsilon) = a.! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:03'!
testLn
	self assert: (Float e + 0 i) ln = (Float e ln). "See Bug 1815 on Mantis"
	self assert: (-1 asComplex ln) = (Float pi i).! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:04'!
testMagAngle

	"magnitude & angle"
	| pi |
	pi := Float pi.
  	self assert: ( (3+4i) magnitude = 5.0).
 	self assert: ((1+1i) angle * 4 = pi).
 	self assert: (-1 asComplex angle = pi).
 	self assert: (1 asComplex angle = 0).
 	self assert: (0 asComplex angle = 0). "Defined as per X3J13"
 	self assert: (-1i angle * 2 = pi negated).
 	self assert: (1i angle * 2 = pi).
	self assert: (1 angle = 1 asComplex angle).
	self assert: (1 magnitude = 1).
	self assert: (-1 angle = -1 asComplex angle).
	self assert: (0 angle = 0 asComplex angle).
! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 12:36'!
testMath

	| a b epsilon |
	a := ((1/2) + 2.1i).
     b := ((3/4) + 5.7i).
	epsilon := 0.00001.
	"Because of rounding, the following do not compare exactly."
 	self assert: ((a + b) roundTo: epsilon) = (((5/4) + 7.8i) roundTo: epsilon).
 	self assert: ((a - b) roundTo: epsilon) = (((-1/4) + -3.6i) roundTo: epsilon).
 	self assert: ((a * b) roundTo: epsilon) = ((-11.595 + 4.425i) roundTo: epsilon).
 	self assert: ((a * 3) roundTo: epsilon) = ((1.5 + 6.3i) roundTo: epsilon).
 	self assert: ((3 * b) roundTo: epsilon) = ((2.25 + 17.1i) roundTo: epsilon).
 	self assert: ((a / b) roundTo: epsilon) 
			= ((0.3734967097798956 + -0.0385749943272067i) roundTo: epsilon).
 	self assert: ((a / 3) roundTo: epsilon) 
			= ((0.1666666666666666 + 0.7i) roundTo: epsilon).
 	self assert: ((3 / b) roundTo: epsilon)
			 = ((0.068073519400953 -0.517358747447243i) roundTo: epsilon).
	
! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 15:55'!
testNegated
	"self run: #testNegated"
	"self debug: #testNegated"
	
	| c |
	c := (2 + 5 i) .
	self assert: c negated  = (-2 - 5i).
	self assert: (-2 -(1/2)i) negated = (2+(1/2)i).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/26/2009 16:48'!
testPrivate
	self assert: (0 signum = 1).
	self assert: (-3 signum = -1).
	self assert: (3 signum = 1).
	self assert: (2 asComplex signum) = 1 asComplex.
	self assert: (0+0i signum = 1)! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:08'!
testReciprocal
	"self run: #testReciprocal"
	"self debug: #testReciprocal"
	
	| c |
	c := (2 + 5 i).
	self assert: c reciprocal  = ((2/29) - (5/29)i).
	self assert: c reciprocal  = (1/c).
	! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 7/17/2004 14:07'!
testReciprocalError
	"self run: #testReciprocalError"
	"self debug: #testReciprocalError"
	
	| c |
	c := (0 i).
	self should: [c reciprocal] raise: ZeroDivide
	! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:29'!
testSecureDivision1
	"self run: #testSecureDivision1"
	"self debug: #testSecureDivision1"
	
	| c1 c2 quotient |
	c1 := 2.0e252 + 3.0e70 i.
	c2 := c1.
	quotient := c1 divideSecureBy: c2.
	self assert: (quotient - 1) isZero.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'sd 6/5/2005 08:29'!
testSecureDivision2
	"self run: #testSecureDivision2"
	"self debug: #testSecureDivision2"
	
	| c1 c2 quotient |
 	c1 := 2.0e252 + 3.0e70 i.
 	c2 := c1.
 	quotient := c1 divideFastAndSecureBy: c2.
	self assert: (quotient - 1) isZero.
	! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 15:57'!
testSqrt
	self assert: (-4 asComplex sqrt) = 2i.
	self assert: (4 asComplex sqrt) = 2.! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 16:09'!
testSquared
	"self run: #testSquared"
	"self debug: #testSquared"
	
	| c c2 |
	c := (6 - 6 i).
	c2 := (c squared).
	self assert: c2 imaginary = -72.
	self assert: c2 real = 0.
	self assert: (1i squared) = -1.
	self assert: c squared = (c * c).! !

!ComplexTest methodsFor: 'tests' stamp: 'KenDickey 8/28/2009 15:47'!
testTrig
	| a epsilon pi pi2 pi2neg |
	a := ((1/2) + 2.1i).
	epsilon := 0.00001.
	pi := Float pi.
	pi2 := pi / 2.
	pi2neg := pi2 negated.
	"Because of rounding, the following do not compare exactly."
 	self assert: ((a sin) roundTo: epsilon) 
			= ((1.986889573868458 + 3.529511343338503i) roundTo: epsilon).
 	self assert: ((a cos) roundTo: epsilon) 
			= ((3.63698 + -1.92818i) roundTo: epsilon).
 	self assert: ((a tan) roundTo: epsilon) = ((0.0248288 + 0.983615i) roundTo: epsilon).
 	self assert: ((a sin arcSin) roundTo: epsilon) = a.
 	self assert: ((a cos arcCos) roundTo: epsilon) = a.
 	self assert: ((a tan arcTan) roundTo: epsilon) = a.
     self assert: (-1 arcCos) = pi.
     self assert: (-1 arcSin) = (pi / 2 negated).
	self assert: (Float pi) cos = -1.0.
	self assert: (Float pi / 2) sin = 1.0.
	self assert: (((Float pi / 4) tan) roundTo: epsilon) = 1.0.
	self assert: (1 arcCos) = 0.0.
	self assert: (1 arcSin) = (pi / 2).
	self assert: (1 arcTan) = (pi / 4).
	self assert: (-1 arcCos) = pi.
	self assert: (((pi2 tanH arcTanH) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2 tan arcTan) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2 sinH arcSinH) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2 sin arcSin) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2 cosH arcCosH) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2 cos arcCos) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2neg asComplex tanH arcTanH) roundTo: epsilon) = (pi2neg roundTo: epsilon)).
	self assert: (((pi2neg asComplex tan arcTan) roundTo: epsilon) = (pi2neg roundTo: epsilon)).
	self assert: (((pi2neg asComplex sinH arcSinH) roundTo: epsilon) = (pi2neg roundTo: epsilon)).
	self assert: (((pi2neg asComplex sin arcSin) roundTo: epsilon) = (pi2neg roundTo: epsilon)).
	self assert: (((pi2neg asComplex cosH arcCosH) roundTo: epsilon) = (pi2 roundTo: epsilon)).
	self assert: (((pi2neg asComplex cos arcCos) roundTo: epsilon) = (pi2 roundTo: epsilon)).
! !

TestCase subclass: #DateAndTimeEpochTest
	instanceVariableNames: 'aDateAndTime aDuration aTimeZone localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!DateAndTimeEpochTest commentStamp: 'tlk 1/6/2004 18:27' prior: 0!
I represent one of several Sunit test Cases intentended to provide complete coverage  for the Chronology set of classes as part of the external testing. The other Chronology sunit test cases are:
 DateTestCase
 DateAndTimeLeapTestCase,
 DurationTestCase,
 ScheduleTestCase
 TimeStampTestCase
 TimespanDoTestCase, 
 TimespanDoSpanAYearTestCase, 
 TimespanTestCase, 
 YearMonthWeekTestCase.  
These tests attempt to exercise all public and private methods.  Except, they do not explicitly depreciated methods. tlk
My fixtures are:
aDateAndTime = January 01, 1901 midnight (the start of the Squeak epoch) with localTimeZone = Grenwhich Meridian (local offset = 0 hours)
aDuration = 1 day, 2 hours, 3, minutes, 4 seconds and 5 nano seconds.
aTimeZone =  'Epoch Test Time Zone', 'ETZ' , offset: 12 hours, 15 minutes. !

!DateAndTimeEpochTest methodsFor: 'running' stamp: 'tlk 1/2/2004 10:58'!
setUp
     localTimeZoneToRestore := DateAndTime localTimeZone.
	aDateAndTime :=  DateAndTime localTimeZone: TimeZone default; epoch.
	aTimeZone := TimeZone offset: (Duration minutes: 135) name: 'Epoch Test Time Zone' abbreviation: 'ETZ'.
	aDuration := Duration days: 1 hours: 2 minutes: 3 seconds: 4 nanoSeconds: 5 ! !

!DateAndTimeEpochTest methodsFor: 'running' stamp: 'tlk 1/2/2004 11:04'!
tearDown
     DateAndTime localTimeZone: localTimeZoneToRestore.
     "wish I could remove the time zones I added earlier, tut there is no method for that"
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testAsDate
	self assert: aDateAndTime asDate =   'January 1, 1901' asDate.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:31'!
testAsDateAndTime
	self assert: aDateAndTime asDateAndTime =  aDateAndTime
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:34'!
testAsDuration
	self assert: aDateAndTime asDuration =  0 asDuration
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:06'!
testAsLocal
	self assert: aDateAndTime asLocal =  aDateAndTime.
	self assert: aDateAndTime asLocal = (aDateAndTime utcOffset: aDateAndTime class localOffset)
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:27'!
testAsMonth
	self assert: aDateAndTime asMonth = (Month month: 'January' year: 1901). 
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:59'!
testAsNanoSeconds
	self assert: aDateAndTime asNanoSeconds =  0 asDuration asNanoSeconds
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 14:01'!
testAsSeconds
	self assert: aDateAndTime asSeconds =  0 asDuration asSeconds
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:32'!
testAsTime
	self assert: aDateAndTime asTime =  Time midnight.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 14:51'!
testAsTimeStamp
	self assert: aDateAndTime asTimeStamp =  TimeStamp new.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:07'!
testAsUTC
	self assert: aDateAndTime asUTC =  aDateAndTime
          ! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:43'!
testAsWeek
	self assert: aDateAndTime asWeek = (Week starting: '12-31-1900' asDate). 

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:43'!
testAsYear
	self assert: aDateAndTime asYear =   (Year starting: '01-01-1901' asDate). 
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:28'!
testCurrent
	self deny: aDateAndTime =  (DateAndTime current).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:46'!
testDateTime
	self assert: aDateAndTime =  (DateAndTime date: '01-01-1901' asDate time: '00:00:00' asTime)
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testDay
	self assert: aDateAndTime day =   DateAndTime new day
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 11:08'!
testDayMonthYearDo
	|iterations|
	iterations := 0.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  iterations := iterations + 1])  = 1.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachYear])  = 1901.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachMonth]) = 1.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachDay]) = 1.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 15:45'!
testDayOfMonth
	self assert: aDateAndTime dayOfMonth  = 1.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:47'!
testDayOfWeek
	self assert: aDateAndTime dayOfWeek  = 3.
	self assert: aDateAndTime dayOfWeekAbbreviation = 'Tue'.
	self assert: aDateAndTime dayOfWeekName = 'Tuesday'.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:01'!
testDayOfYear
	self assert: aDateAndTime dayOfYear  = 1.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysInMonth
	self assert: aDateAndTime daysInMonth  = 31.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysInYear
	self assert: aDateAndTime daysInYear  = 365.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testDaysLeftInYear
	self assert: aDateAndTime daysLeftInYear  = 364.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 16:24'!
testDuration
	self assert: aDateAndTime duration  = 0 asDuration.

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:25'!
testEpoch
	self assert: aDateAndTime =  '1901-01-01T00:00:00+00:00'.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:44'!
testFirstDayOfMonth
	self assert: aDateAndTime firstDayOfMonth =   1
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:25'!
testFromSeconds
	self assert: aDateAndTime =  (DateAndTime fromSeconds: 0).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:26'!
testFromString
	self assert: aDateAndTime =  (DateAndTime fromString: ' 1901-01-01T00:00:00+00:00').
	self assert: aDateAndTime =  (DateAndTime fromString: ' 1901-01-01T00:00:00').
	self assert: aDateAndTime =  (DateAndTime fromString: ' 1901-01-01T00:00').
	self assert: aDateAndTime =  (DateAndTime fromString: ' 1901-01-01T00:00:00+00:00').
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'al 6/12/2008 21:56'!
testHash
	self assert: aDateAndTime hash = DateAndTime new hash! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 16:59'!
testHour
	self assert: aDateAndTime hour =    aDateAndTime hour24.
	self assert: aDateAndTime hour =    0.
	self assert: aDateAndTime hour =    aDateAndTime hours
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 3/12/2004 15:21'!
testHour12
	self assert: aDateAndTime hour12  = DateAndTime new hour12.
	self assert: aDateAndTime hour12  = 12
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:02'!
testIsLeapYear
	self deny: aDateAndTime isLeapYear
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:18'!
testJulianDayNumber
	self assert: aDateAndTime =  (DateAndTime julianDayNumber: 2415386).
	self assert: aDateAndTime julianDayNumber = 2415386.! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 13:20'!
testLessThan
	self assert: aDateAndTime  < (aDateAndTime + '1:00:00:00').
	self assert: aDateAndTime + -1 < aDateAndTime.
	! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:40'!
testMeridianAbbreviation
	self assert: aDateAndTime meridianAbbreviation = 'AM'.

	! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:37'!
testMiddleOf
	self assert: (aDateAndTime middleOf: '2:00:00:00' asDuration) = 
	 (Timespan starting: '12-31-1900' asDate duration: 2 days).
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:39'!
testMidnight
	self assert: aDateAndTime midnight =  aDateAndTime
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:03'!
testMinus
	self assert: aDateAndTime - aDateAndTime =  '0:00:00:00' asDuration.
	self assert: aDateAndTime - '0:00:00:00' asDuration = aDateAndTime.
	self assert: aDateAndTime - aDuration =  (DateAndTime year: 1900 month: 12 day: 30 hour: 21 minute: 56 second: 55 nanoSecond: 999999995 offset: 0 hours ).
	" I believe this Failure is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)" ! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:35'!
testMinute
	self assert: aDateAndTime minute =  0

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:41'!
testMinutes
	self assert: aDateAndTime minutes = 0
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:46'!
testMonth
	self assert: aDateAndTime month  = 1.
	self assert: aDateAndTime monthAbbreviation = 'Jan'.
	self assert: aDateAndTime monthName = 'January'.
	self assert: aDateAndTime monthIndex = 1.! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:47'!
testNanoSecond
	self assert: aDateAndTime nanoSecond =  0

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:27'!
testNew
	self assert: aDateAndTime =  (DateAndTime new).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 19:49'!
testNoon
	self assert: aDateAndTime noon =  '1901-01-01T12:00:00+00:00'.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:28'!
testNow
	self deny: aDateAndTime =  (DateAndTime now).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'StephaneDucasse 4/24/2010 10:13'!
testOffset
	"self debug: #testOffset"
	self assert: aDateAndTime offset =  '0:00:00:00' asDuration.
	self assert: (aDateAndTime offset: '0:12:00:00') =  '1901-01-01T00:00:00+12:00'.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 11:03'!
testPlus
	self assert: aDateAndTime + '0:00:00:00' = aDateAndTime.
	self assert: aDateAndTime + 0 = aDateAndTime.
	self assert: aDateAndTime + aDuration = (DateAndTime year: 1901 month: 1 day: 2 hour: 2 minute: 3 second: 4 nanoSecond: 5 offset: 0 hours )
	" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"
	
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '1901-01-01T00:00:00+00:00' readStream.
	rw := ReadWriteStream on: ''.
	aDateAndTime printOn: rw.
	self assert: rw contents = cs contents.
	cs := 'a TimeZone(ETZ)' readStream.
	rw := ReadWriteStream on: ''.
	aTimeZone printOn: rw.
	self assert: rw contents = cs contents! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:22'!
testSecond
	self assert: aDateAndTime second =  0

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:22'!
testSeconds
	self assert: aDateAndTime seconds =  0

! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:25'!
testTicks
	self assert: aDateAndTime ticks =  (DateAndTime julianDayNumber: 2415386) ticks.
	self assert: aDateAndTime ticks = #(2415386 0 0)! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:31'!
testTicksOffset
	self assert: aDateAndTime =  (aDateAndTime ticks:  #(2415386 0 0) offset: DateAndTime localOffset).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:42'!
testTo
	self assert: (aDateAndTime to: aDateAndTime) = (DateAndTime new to: DateAndTime new) 
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:43'!
testToBy
	self assert: (aDateAndTime to: aDateAndTime + 10 days by: 5 days) = 
				(DateAndTime new to: DateAndTime new + 10 days by: 5 days ) 
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:53'!
testToByDo
	"self assert: (aDateAndTime to: aDateAndTime + 10 days by: 5 days do: []) =  "
	"MessageNotUnderstood: UndefinedObject>>starting:ending:  where UndefinedObject is Timespan "! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 17:35'!
testToday
	self deny: aDateAndTime =  (DateAndTime today).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:45'!
testTommorrow
	self assert: (DateAndTime today + 24 hours) =  (DateAndTime tomorrow).
	self deny: aDateAndTime =  (DateAndTime tomorrow).
     "MessageNotUnderstood: Date class>>starting:"! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 20:58'!
testUtcOffset
     self assert: (aDateAndTime utcOffset: '0:12:00:00') =  '1901-01-01T12:00:00+12:00'.
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 21:00'!
testYear
	self assert: aDateAndTime year = 1901.

	! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:30'!
testYearDay
	self assert: aDateAndTime =  (DateAndTime year: 1901 day: 1).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearDayHourMinuteSecond
	self assert: aDateAndTime =  (DateAndTime year: 1901 day: 1 hour: 0 minute: 0 second: 0).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearMonthDay
	self assert: aDateAndTime =  (DateAndTime year: 1901 month: 1 day: 1).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/1/2004 12:31'!
testYearMonthDayHourMinuteSecond
	self assert: aDateAndTime =  (DateAndTime year: 1901 month: 1 day: 1 hour: 0 minute: 0 second: 0).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:23'!
testYearMonthDayHourMinuteSecondNanosSecondOffset
	self assert: aDateAndTime =  (DateAndTime year: 1901 month: 1 day: 1 hour: 0 minute: 0 second: 0 nanoSecond: 0 offset:0 hours ).
	self assert: ((DateAndTime year: 1 month: 1 day: 1 hour: 0 minute: 0 second: 0 nanoSecond: 0 offset: 0 hours ) +
				(Duration days: 1 hours: 2 minutes: 3 seconds: 4  nanoSeconds: 5) ) =  	
				(DateAndTime year: 1 month: 1 day: 2 hour: 2 minute: 3 second: 4 nanoSecond: 5 offset: 0 hours ) 
	" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"" I believe this is a bug in the nanosecond part of (DateAndTime >> year:month:day:hour:minute:second:nanoSecond:offset:)"   
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 09:47'!
testYesterday
	self deny: aDateAndTime =  (DateAndTime yesterday).
! !

!DateAndTimeEpochTest methodsFor: 'testing' stamp: 'nk 3/12/2004 11:26'!
testtimeZone
	self assert: aDateAndTime timeZoneName	= 'Universal Time'.
	self assert: aDateAndTime timeZoneAbbreviation	=  'UTC'

! !

TestCase subclass: #DateAndTimeLeapTest
	instanceVariableNames: 'aDateAndTime aDuration aTimeZone localTimeZoneToRestore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!DateAndTimeLeapTest commentStamp: 'tlk 1/6/2004 17:54' prior: 0!
I represent one of several Sunit test Cases intentended to provide complete coverage for the Chronology set of classes as part of the external testing. tlk.
My fixtures are:
aDateAndTime = February 29, 2004 1:33 PM with offset: 2 hours
aDuration = 15 days, 14 hours, 13 minutes, 12 seconds and 11 nano seconds.
aTimeZone =  Grenwhich Meridian (local offset = 0 hours) !

!DateAndTimeLeapTest methodsFor: 'running' stamp: 'nk 3/12/2004 11:00'!
setUp
	localTimeZoneToRestore := DateAndTime localTimeZone.
	DateAndTime localTimeZone: TimeZone default.
	aDateAndTime := (DateAndTime year: 2004 month: 2 day: 29 hour: 13 minute: 33 second: 0 offset: 2 hours).
	aTimeZone := TimeZone default.
	aDuration := Duration days: 0 hours: 13 minutes: 33 seconds: 0 nanoSeconds: 0
! !

!DateAndTimeLeapTest methodsFor: 'running' stamp: 'tlk 1/2/2004 21:30'!
tearDown
     DateAndTime localTimeZone: localTimeZoneToRestore.
     "wish I could remove the time zones I added earlier, tut there is no method for that"
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:00'!
testAsDate
	self assert: aDateAndTime asDate =   'February 29, 2004' asDate.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:55'!
testAsDuration
	self assert: aDateAndTime asDuration =  aDuration
	
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 14:00'!
testAsLocal
	self assert: aDateAndTime asLocal =  aDateAndTime.
	self assert: aDateAndTime asLocal = (aDateAndTime utcOffset: aDateAndTime class localOffset)

	
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:24'!
testAsMonth
	self assert: aDateAndTime asMonth = (Month month: 'February' year: 2004).
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:59'!
testAsNanoSeconds
	self assert: aDateAndTime asNanoSeconds =  aDuration asNanoSeconds.
	self assert: aDateAndTime asNanoSeconds = 48780000000000
	
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 22:05'!
testAsSeconds
	self assert: aDuration asSeconds =  48780.
	self assert: aDateAndTime asSeconds =  3255507180
	
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:26'!
testAsTime
	self assert: aDateAndTime asTime = (Time hour: 13 minute: 33 second: 0)
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:31'!
testAsTimeStamp
	self assert: aDateAndTime asTimeStamp =  ((TimeStamp readFrom: '2-29-2004 1:33 pm' readStream) offset: 2 hours).

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:59'!
testAsUTC
	self assert: aDateAndTime asUTC =  aDateAndTime

          ! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:30'!
testAsWeek
	self assert: aDateAndTime asWeek =    (Week starting: '02-29-2004' asDate).
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 12:36'!
testAsYear
	self assert: aDateAndTime asYear =   (Year starting: '02-29-2004' asDate).  
	self deny: aDateAndTime asYear =   (Year starting: '01-01-2004' asDate)  
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:23'!
testDay
	self assert: aDateAndTime day =   60. 
	self deny: aDateAndTime day =   29 ! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 22:16'!
testDayMonthYearDo
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachYear])  = 2004.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachMonth]) = 2.
	self assert: (aDateAndTime dayMonthYearDo: [:eachDay :eachMonth :eachYear |  eachDay]) = 29.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 22:17'!
testDayOfMonth
	self assert: aDateAndTime dayOfMonth  = 29.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:34'!
testDayOfWeek
	self assert: aDateAndTime dayOfWeek  = 1.
	self assert: aDateAndTime dayOfWeekAbbreviation = 'Sun'.
	self assert: aDateAndTime dayOfWeekName = 'Sunday'.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:59'!
testDayOfYear
	self assert: aDateAndTime dayOfYear  = 60.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysInMonth
	self assert: aDateAndTime daysInMonth  = 29.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysInYear
	self assert: aDateAndTime daysInYear  = 366.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:58'!
testDaysLeftInYear
	self assert: aDateAndTime daysLeftInYear  = 306.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:38'!
testFirstDayOfMonth
	self deny: aDateAndTime firstDayOfMonth =  1.
	self assert: aDateAndTime firstDayOfMonth = 32
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 10:43'!
testFromString
	self assert: aDateAndTime =  (DateAndTime fromString: ' 2004-02-29T13:33:00+02:00').

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 10:48'!
testHour
	self assert: aDateAndTime hour =    aDateAndTime hour24.
	self assert: aDateAndTime hour =    13.
	self assert: aDateAndTime hour =    aDateAndTime hours
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'brp 3/12/2004 15:19'!
testHour12
	self assert: aDateAndTime hour12  =   1.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:35'!
testIsLeapYear
	self assert: aDateAndTime isLeapYear
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testLessThan
	self assert: aDateAndTime  < (aDateAndTime + '1:00:00:00').
	self assert: aDateAndTime + -1 < aDateAndTime.
	! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 10:42'!
testMeridianAbbreviation
	self assert: aDateAndTime meridianAbbreviation = 'PM'.

	! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:12'!
testMiddleOf
	self assert: (aDateAndTime middleOf: aDuration)  = 
	 (Timespan starting: (DateAndTime year: 2004 month: 2 day: 29 hour: 6 minute: 46 second: 30 offset: 2 hours)
	duration: (Duration days: 0 hours: 13 minutes: 33 seconds: 0 nanoSeconds: 0 ))
	! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:57'!
testMidnight
	self assert: aDateAndTime midnight =  '2004-02-29T00:00:00+00:00'.
	self deny: aDateAndTime midnight =  '2004-02-29T00:00:00+02:00'
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:00'!
testMinute
	self assert: aDateAndTime minute =  33

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:44'!
testMinutes
	self assert: aDateAndTime minutes = 33
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:02'!
testMonth
	self assert: aDateAndTime month  = 2.
	self assert: aDateAndTime monthAbbreviation = 'Feb'.
	self assert: aDateAndTime monthName = 'February'.
	self assert: aDateAndTime monthIndex = 2.! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testNanoSecond
	self assert: aDateAndTime nanoSecond =  0

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:03'!
testNoon
	self assert: aDateAndTime noon =  '2004-02-29T12:00:00+00:00'.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:07'!
testOffset
	self assert: aDateAndTime offset =  '0:02:00:00' asDuration.
     self assert: (aDateAndTime offset: '0:12:00:00') =  '2004-02-29T13:33:00+12:00'.
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testPrintOn
	| cs rw |
	cs := '2004-02-29T13:33:00+02:00' readStream.
	rw := ReadWriteStream on: ''.
	aDateAndTime printOn: rw.
	self assert: rw contents = cs contents.
	cs := 'a TimeZone(UTC)' readStream.
	rw := ReadWriteStream on: ''.
	aTimeZone printOn: rw.
	self assert: rw contents = cs contents! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testSecond
	self assert: aDateAndTime second =  0

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/2/2004 21:30'!
testSeconds
	self assert: aDateAndTime seconds =  0

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:12'!
testTicks
	self assert: aDateAndTime ticks =  ((DateAndTime julianDayNumber: 2453065) + 48780 seconds) ticks.
	self assert: aDateAndTime ticks =  #(2453065 48780 0)! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:52'!
testTicksOffset
	self assert: aDateAndTime =  (aDateAndTime ticks:  #(2453065 48780 0) offset: DateAndTime localOffset).

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/4/2004 13:51'!
testUtcOffset
     self assert: (aDateAndTime utcOffset: '0:02:00:00') =  '2004-02-29T13:33:00+02:00'.

! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:17'!
testYear
	self assert: aDateAndTime year = 2004.

	! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:21'!
testYearDayHourMinuteSecond
	self assert: aDateAndTime =  ((DateAndTime year: 2004 day: 60 hour: 13 minute: 33 second: 0) offset: 2 hours).
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 11:23'!
testYearMonthDayHourMinuteSecond
	self assert: aDateAndTime =  ((DateAndTime year: 2004 month: 2 day: 29 hour: 13 minute: 33 second: 0) offset: 2 hours).
! !

!DateAndTimeLeapTest methodsFor: 'testing' stamp: 'nk 3/12/2004 11:26'!
testtimeZone
	self assert: aDateAndTime timeZoneName	= 'Universal Time'.
	self assert: aDateAndTime timeZoneAbbreviation	=  'UTC'

! !

TestCase subclass: #DelayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!DelayTest methodsFor: 'testing' stamp: 'nice 4/19/2009 21:21'!
testBounds
	"self run: #testBounds"
	
	self should: [Delay forMilliseconds: -1] raise: Error.
	self shouldnt: [Delay forMilliseconds: SmallInteger maxVal // 2 + 1] raise: Error.
	self shouldnt: [Delay forMilliseconds: SmallInteger maxVal + 1] raise: Error.
	self shouldnt: [(Delay forMilliseconds: Float pi) wait] raise: Error. "Wait 3ms"
! !

!DelayTest methodsFor: 'testing-limits' stamp: 'ar 9/21/2009 22:14'!
testMultiProcessWaitOnSameDelay
	"Ensure that waiting on the same delay from multiple processes raises an error"
	| delay p1 p2 wasRun |
	delay := Delay forSeconds: 1.
	wasRun := false.
	p1 := [delay wait] forkAt: Processor activePriority+1.
	p2 := [
		self should:[delay wait] raise: Error.
		wasRun := true.
	] forkAt: Processor activePriority+1.
	p1 terminate.
	p2 terminate.
	self assert: wasRun.

! !

!DelayTest methodsFor: 'testing-limits' stamp: 'ar 9/21/2009 22:12'!
testMultiSchedule
	"Ensure that scheduling the same delay twice raises an error"
	| delay |
	delay := Delay forSeconds: 1.
	delay schedule.
	self should:[delay schedule] raise: Error.
! !

!DelayTest methodsFor: 'testing' stamp: 'nice 1/5/2010 15:59'!
testSemaphore
	"When we provide our own semaphore for a Delay, it should be used"
	"See http://bugs.squeak.org/view.php?id=6834"

	"self run: #testSemaphore"
	
	| sem |
	sem := Semaphore new.
	[ | process |
		process := [Delay timeoutSemaphore: sem afterMSecs: 0. sem wait] newProcess.
		process priority: Processor highIOPriority.
		process resume.
		self assert: process isTerminated.
	] ensure: [sem signal]! !

TestCase subclass: #DependentsArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!DependentsArrayTest methodsFor: 'test' stamp: 'GabrielOmarCotelli 5/25/2009 16:16'!
testSize

	self 
		assert: (DependentsArray with: nil) size = 0;
		assert: (DependentsArray with: nil with: 1 with: nil) size = 1;
		assert: (DependentsArray with: 1 with: 3) size = 2;
		assert: (DependentsArray with: nil with: nil with: nil) size = 0! !

TestCase subclass: #IVsAndClassVarNamesConflictTest
	instanceVariableNames: 'class className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Classes'!

!IVsAndClassVarNamesConflictTest methodsFor: 'setup' stamp: 'oscar.nierstrasz 10/18/2009 17:11'!
setUp
	super setUp.
	className := #ClassForTestToBeDeleted.! !

!IVsAndClassVarNamesConflictTest methodsFor: 'setup' stamp: 'lr 3/14/2010 21:13'!
tearDown
	| cl |
	super tearDown.
	cl := Smalltalk globals at: className ifAbsent: [ ^ self ].
	cl
		removeFromChanges;
		removeFromSystemUnlogged! !

!IVsAndClassVarNamesConflictTest methodsFor: 'tests' stamp: 'Noury 10/26/2008 18:40'!
testOneCanProceedWhenIntroducingCapitalizedInstanceVariables
	self shouldnt: [
		[Object subclass: className
		instanceVariableNames: 'X Y'
		classVariableNames: ''
		poolDictionaries: ''
		category: self class category] on: Exception do: [:ex|	ex resume]
		] raise: Exception.
	self assert: (Smalltalk keys includes: className)
! !

!IVsAndClassVarNamesConflictTest methodsFor: 'tests' stamp: 'adrian_lienhard 3/7/2009 17:54'!
testOneCanProceedWhenIntroducingClasseVariablesBeginingWithLowerCaseCharacters
	self shouldnt: [
		[Object subclass: className
		instanceVariableNames: ''
		classVariableNames: 'a BVariableName'
		poolDictionaries: ''
		category: self class category] on: Exception do: [:ex|	ex resume]
		] raise: Exception.
	self assert: (Smalltalk keys includes: className)
! !

TestCase subclass: #InstVarRefLocatorTest
	instanceVariableNames: 'tt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!InstVarRefLocatorTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class InstVarRefLocator. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!InstVarRefLocatorTest methodsFor: 'examples' stamp: 'sd 6/5/2005 08:27'!
example1
	| ff |
	(1 < 2) ifTrue: [tt ifNotNil: [ff := 'hallo']].
	^ ff.! !

!InstVarRefLocatorTest methodsFor: 'examples' stamp: 'md 4/8/2003 12:31'!
example2
	| ff|	
	ff := 1.
	(1 < 2) ifTrue: [ff ifNotNil: [ff := 'hallo']].
	^ ff.! !

!InstVarRefLocatorTest methodsFor: 'private' stamp: 'md 4/8/2003 12:39'!
hasInstVarRef: aMethod
	"Answer whether the receiver references an instance variable."

	| scanner end printer |

	scanner := InstructionStream on: aMethod.
	printer := InstVarRefLocator new.
	end := scanner method endPC.

	[scanner pc <= end] whileTrue: [
		(printer interpretNextInstructionUsing: scanner) ifTrue: [^true].
	].
	^false! !

!InstVarRefLocatorTest methodsFor: 'tests' stamp: 'md 4/8/2003 12:42'!
testExample1
	| method |

	method := self class compiledMethodAt: #example1.
	self assert: (self hasInstVarRef: method).! !

!InstVarRefLocatorTest methodsFor: 'tests' stamp: 'md 4/8/2003 12:42'!
testExample2
	| method |

	method := self class compiledMethodAt: #example2.
	self deny: (self hasInstVarRef: method).! !

!InstVarRefLocatorTest methodsFor: 'tests' stamp: 'nice 1/5/2010 15:59'!
testInstructions

	

	Object methods do: [:method | | scanner printer end |
		scanner := InstructionStream on: method.
		printer := InstVarRefLocator new.
		end := scanner method endPC.

		[scanner pc <= end] whileTrue: [
			self shouldnt: [printer interpretNextInstructionUsing: scanner] raise: Error.
		].
	].! !

TestCase subclass: #InstructionClientTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!InstructionClientTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class InstructionClient. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category!

!InstructionClientTest methodsFor: 'tests' stamp: 'nice 1/5/2010 15:59'!
testInstructions
	"just interpret all of methods of Object"

	| client|
	
	client := InstructionClient new.	

	Object methods do: [:method | | scanner |
			scanner := (InstructionStream on: method).
			[scanner pc <= method endPC] whileTrue: [
					self shouldnt: [scanner interpretNextInstructionFor: client] raise: Error.
			].
	].
! !

TestCase subclass: #IntegerDigitLogicTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:12'!
testAndSingleBitWithMinusOne
	"And a single bit with -1 and test for same value"
	1 to: 100 do: [:i | self assert: ((1 bitShift: i) bitAnd: -1) = (1 bitShift: i)].! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'StephaneDucasse 2/11/2010 21:06'!
testLargeShift
	"A sanity check for LargeInteger bitShifts"
	"self debug: #testLargeShift"
	| suite |
	suite := #(     "some numbers on 64 bits or less"
               '101101011101001100110111110110011101101101000001110110011'
               '1101101001100010011001101110100000111011011010100011101100'
               '101101101011110011001100110011011101011001111000100011101000'
               '10101101101000101001111111111100101101011001011000100011100000'
               '1000101010101001111011101010111001011111110011110001000110000000'
               '1100101010101000010011101000110010111110110011110000000000000001' ).
	"65 bits or less"
	suite := suite , (suite collect: [:e | '1' , e reversed ]).
	"129 bits or less"
	suite := suite , (suite collect: [:e | e ,e ]).
	suite do: [:bits | | num ls rs |
		num := Integer readFrom: bits readStream base: 2.
		0 to: bits size-1 do: [:shift |
			ls := (num bitShift: shift) printStringBase: 2.
			rs := (num bitShift: 0-shift) printStringBase: 2.
			self assert: ls = (bits , (String new: shift withAll: $0)).
			self assert: rs = (bits copyFrom: 1 to: bits size - shift).]]! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:13'!
testMixedSignDigitLogic
	"Verify that mixed sign logic with large integers works."
	self assert: (-2 bitAnd: 16rFFFFFFFF) = 16rFFFFFFFE! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:12'!
testNBitAndNNegatedEqualsN
	"Verify that (n bitAnd: n negated) = n for single bits"
	| n |
	1 to: 100 do: [:i | n := 1 bitShift: i.
				self assert: (n bitAnd: n negated) = n]! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:12'!
testNNegatedEqualsNComplementedPlusOne
	"Verify that n negated = (n complemented + 1) for single bits"
	| n |
	1 to: 100 do: [:i | n := 1 bitShift: i.
				self assert: n negated = ((n bitXor: -1) + 1)]! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:13'!
testShiftMinusOne1LeftThenRight
	"Shift -1 left then right and test for 1"
	1 to: 100 do: [:i | self assert: ((-1 bitShift: i) bitShift: i negated) = -1].
! !

!IntegerDigitLogicTest methodsFor: 'tests' stamp: 'hmm 1/7/2002 21:12'!
testShiftOneLeftThenRight
	"Shift 1 bit left then right and test for 1"
	1 to: 100 do: [:i | self assert: ((1 bitShift: i) bitShift: i negated) = 1].
! !

TestCase subclass: #IntegerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!IntegerTest methodsFor: 'tests - printing' stamp: 'al 7/21/2008 22:36'!
testBadBase
	"This used to get into an endless loop.
	See Pharo #114"
	
	self should: [2 printStringBase: 1] raise: Error.! !

!IntegerTest methodsFor: 'tests - benchmarks' stamp: 'sd 6/5/2005 08:37'!
testBenchFib

	self assert: (0 benchFib = 1).
	self assert: (1 benchFib = 1).
	self assert: (2 benchFib = 3).
	! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'nice 1/5/2010 15:59'!
testBitAt
	| trials bitSequence2 |

	self
		assert: ((1 to: 100) allSatisfy: [:i | (0 bitAt: i) = 0])
		description: 'all bits of zero are set to zero'.
	
	self
		assert: ((1 to: 100) allSatisfy: [:i | (-1 bitAt: i) = 1])
		description: 'In two complements, all bits of -1 are set to 1'.
		
	
	trials := #(
		'2r10010011'
		'2r11100100'
		'2r10000000'
		'2r0000101011011001'
		'2r1000101011011001'
		'2r0101010101011000'
		'2r0010011110110010'
		'2r0010011000000000'
		'2r00100111101100101000101011011001'
		'2r01110010011110110010100110101101'
		'2r10101011101011001010000010110110'
		'2r10101000000000000000000000000000'
		'2r0010101110101001110010100000101101100010011110110010100010101100'
		'2r1010101110101100101000001011011000100111101100101000101011011001'
		'2r1010101110101000000000000000000000000000000000000000000000000000').
	trials do: [:bitSequence | | aNumber |
		aNumber := Number readFrom: bitSequence.
		bitSequence2 := (bitSequence size - 2 to: 1 by: -1) inject: '2r' into: [:string :i | string copyWith: (Character digitValue: (aNumber bitAt: i))].
		self assert: bitSequence2 = bitSequence].
	
	trials do: [:bitSequence | | bitInvert |
		bitInvert := -1 - (Number readFrom: bitSequence).
		bitSequence2 := (bitSequence size - 2 to: 1 by: -1) inject: '2r' into: [:string :i | string copyWith: (Character digitValue: 1 - (bitInvert bitAt: i))].
		self assert: bitSequence2 = bitSequence description: '-1-x is similar to a bitInvert operation in two complement']! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'sd 6/5/2005 08:43'!
testBitLogic  
	"This little suite of tests is designed to verify correct operation of most
	of Squeak's bit manipulation code, including two's complement
	representation of negative values.  It was written in a hurry and
	is probably lacking several important checks."

	"Shift 1 bit left then right and test for 1"
	"self run: #testBitLogic"
	| n |
	1 to: 100 do: [:i | self assert: ((1 bitShift: i) bitShift: i negated) = 1].

	"Shift -1 left then right and test for 1"
	1 to: 100 do: [:i | self assert: ((-1 bitShift: i) bitShift: i negated) = -1].

	"And a single bit with -1 and test for same value"
	1 to: 100 do: [:i | self assert: ((1 bitShift: i) bitAnd: -1) = (1 bitShift: i)].

	"Verify that (n bitAnd: n negated) = n for single bits"
	1 to: 100 do: [:i |  n := 1 bitShift: i. self assert: (n bitAnd: n negated) = n].

	"Verify that n negated = (n complemented + 1) for single bits"
	1 to: 100 do: [:i | 
					n := 1 bitShift: i. 
					self assert: n negated = ((n bitXor: -1) + 1)].

	"Verify that (n + n complemented) = -1 for single bits"
	1 to: 100 do: [:i | 
					n := 1 bitShift: i.
					self assert: (n + (n bitXor: -1)) = -1].

	"Verify that n negated = (n complemented +1) for single bits"
	1 to: 100 do: [:i | 
					n := 1 bitShift: i.
					self assert: n negated = ((n bitXor: -1) + 1)].

	self assert: (-2 bitAnd: 16rFFFFFFFF) = 16rFFFFFFFE.! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 08:48'!
testCreationFromBytes1
	"self run: #testCreationFromBytes1"
	"it is illegal for a LargeInteger to be less than SmallInteger maxVal." 
	"here we test that Integer>>byte!!byte2:byte3:byte4: resconstructs SmallInteger maxVal as an instance of SmallInteger. "
  
	| maxSmallInt hexString byte1 byte2 byte3 byte4 
	builtInteger |
	maxSmallInt := SmallInteger maxVal.
	hexString := maxSmallInt printStringHex.
	self assert: hexString size = 8.
	byte4 := Number readFrom: (hexString copyFrom: 1 to: 2) base: 16.
	byte3 := Number readFrom: (hexString copyFrom: 3 to: 4) base: 16.
	byte2 := Number readFrom: (hexString copyFrom: 5 to: 6) base: 16.
	byte1 := Number readFrom: (hexString copyFrom: 7 to: 8) base: 16.
	builtInteger := Integer byte1: byte1 byte2: byte2 byte3: byte3 byte4: byte4.
	self assert: builtInteger = maxSmallInt.
	self assert: builtInteger class = SmallInteger
! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 08:48'!
testCreationFromBytes2
 	"self run: #testCreationFromBytes2"

	"it is illegal for a LargeInteger to be less than SmallInteger maxVal." 
	"here we test that Integer>>byte!!byte2:byte3:byte4: resconstructs (SmallInteger maxVal + 1) as an instance of LargePositiveInteger. "
	| maxSmallInt hexString byte1 byte2 byte3 byte4 builtInteger |
	maxSmallInt := SmallInteger maxVal.
	hexString := (maxSmallInt + 1) printStringHex.
	self assert: hexString size = 8.
	byte4 := Number readFrom: (hexString copyFrom: 1 to: 2) base: 16.
	byte3 := Number readFrom: (hexString copyFrom: 3 to: 4) base: 16.
	byte2 := Number readFrom: (hexString copyFrom: 5 to: 6) base: 16.
	byte1 := Number readFrom: (hexString copyFrom: 7 to: 8) base: 16.
	builtInteger := Integer byte1: byte1 byte2: byte2 byte3: byte3 byte4: byte4.
	self assert: builtInteger = (maxSmallInt + 1).
	self deny: builtInteger class = SmallInteger
! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 08:49'!
testCreationFromBytes3
	"self run: #testCreationFromBytes3"

	"it is illegal for a LargeInteger to be less than SmallInteger maxVal." 
	"here we test that Integer>>byte!!byte2:byte3:byte4: resconstructs (SmallInteger maxVal - 1) as an instance of SmallInteger. "
	| maxSmallInt hexString byte1 byte2 byte3 byte4 
    builtInteger |
	maxSmallInt := SmallInteger maxVal.
	hexString := (maxSmallInt - 1) printStringHex.
	self assert: hexString size = 8.
	byte4 := Number readFrom: (hexString copyFrom: 1 to: 2) base: 16.
	byte3 := Number readFrom: (hexString copyFrom: 3 to: 4) base: 16.
	byte2 := Number readFrom: (hexString copyFrom: 5 to: 6) base: 16.
	byte1 := Number readFrom: (hexString copyFrom: 7 to: 8) base: 16.
	builtInteger := Integer byte1: byte1 byte2: byte2 byte3: byte3 byte4: byte4.
	self assert: builtInteger = (maxSmallInt - 1).
	self assert: builtInteger class = SmallInteger
! !

!IntegerTest methodsFor: 'testing - arithmetic' stamp: 'mga 5/11/2006 15:41'!
testCrossSumBase
	"self run: #testCrossSumBase"

	self assert: (
		((-20 to: 20) collect: [:each | each crossSumBase: 10]) asArray = 
		#(2 10 9 8 7 6 5 4 3 2 1 9 8 7 6 5 4 3 2 1 0 1 2 3 4 5 6 7 8 9 1 2 3 4 5 6 7 8 9 10 2)).
	self assert: (
		((-20 to: 20) collect: [:each | each crossSumBase: 2]) asArray = 
		#(2 3 2 2 1 4 3 3 2 3 2 2 1 3 2 2 1 2 1 1 0 1 1 2 1 2 2 3 1 2 2 3 2 3 3 4 1 2 2 3 2)).
	self should: [10 crossSumBase: 1] raise: AssertionFailure! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'sd 3/5/2004 14:50'!
testDegreeCos
	"self run: #testDegreeCos"
	
	self shouldnt: [ 45 degreeCos] raise: Error.
	self assert: 45  degreeCos printString =  (2 sqrt / 2) asFloat printString ! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'sd 6/5/2005 08:46'!
testDifferentBases
	"self run: #testDifferentBases"
	"| value |
	2 to: 36 do: [:each|
		value := 0.
		1 to: each-1 do: [:n| value := value + (n * (each raisedToInteger: n))].
		value := value negated.
		Transcript tab; show: 'self assert: (', value printString, ' printStringBase: ', each printString, ') = ''', (value printStringBase: each), '''.'; cr.
		Transcript tab; show: 'self assert: (', value printString, ' radix: ', each printString, ') = ''', (value radix: each), '''.'; cr.
		Transcript tab; show: 'self assert: ', value printString, ' printStringHex = ''', (value printStringBase: 16), '''.'; cr.
		Transcript tab; show: 'self assert: (', value printString, ' storeStringBase: ', each printString, ') = ''', (value storeStringBase: each), '''.'; cr.
		Transcript tab; show: 'self assert: ', value printString, ' storeStringHex = ''', (value storeStringBase: 16), '''.'; cr.


].
	"

	self assert: 2r10 = 2.
	self assert: 3r210 = 21.
	self assert: 4r3210 = 228.
	self assert: 5r43210 = 2930.
	self assert: 6r543210 = 44790.
	self assert: 7r6543210 = 800667.
	self assert: 8r76543210 = 16434824.
	self assert: 9r876543210 = 381367044.
	self assert: 10r9876543210 = 9876543210.
	self assert: 11rA9876543210 = 282458553905.
	self assert: 12rBA9876543210 = 8842413667692.
	self assert: 13rCBA9876543210 = 300771807240918.
	self assert: 14rDCBA9876543210 = 11046255305880158.
	self assert: 15rEDCBA9876543210 = 435659737878916215.
	self assert: 16rFEDCBA9876543210 = 18364758544493064720.
	self assert: 17rGFEDCBA9876543210 = 824008854613343261192.
	self assert: 18rHGFEDCBA9876543210 = 39210261334551566857170.
	self assert: 19rIHGFEDCBA9876543210 = 1972313422155189164466189.
	self assert: 20rJIHGFEDCBA9876543210 = 104567135734072022160664820.
	self assert: 21rKJIHGFEDCBA9876543210 = 5827980550840017565077671610.
	self assert: 22rLKJIHGFEDCBA9876543210 = 340653664490377789692799452102.
	self assert: 23rMLKJIHGFEDCBA9876543210 = 20837326537038308910317109288851.
	self assert: 24rNMLKJIHGFEDCBA9876543210 = 1331214537196502869015340298036888.
	self assert: 25rONMLKJIHGFEDCBA9876543210 = 88663644327703473714387251271141900.
	self assert: 26rPONMLKJIHGFEDCBA9876543210 = 6146269788878825859099399609538763450.
	self assert: 27rQPONMLKJIHGFEDCBA9876543210 = 442770531899482980347734468443677777577.
	self assert: 28rRQPONMLKJIHGFEDCBA9876543210 = 33100056003358651440264672384704297711484.
	self assert: 29rSRQPONMLKJIHGFEDCBA9876543210 = 2564411043271974895869785066497940850811934.
	self assert: 30rTSRQPONMLKJIHGFEDCBA9876543210 = 205646315052919334126040428061831153388822830.
	self assert: 31rUTSRQPONMLKJIHGFEDCBA9876543210 = 17050208381689099029767742314582582184093573615.
	self assert: 32rVUTSRQPONMLKJIHGFEDCBA9876543210 = 1459980823972598128486511383358617792788444579872.
	self assert: 33rWVUTSRQPONMLKJIHGFEDCBA9876543210 = 128983956064237823710866404905431464703849549412368.
	self assert: 34rXWVUTSRQPONMLKJIHGFEDCBA9876543210 = 11745843093701610854378775891116314824081102660800418.
	self assert: 35rYXWVUTSRQPONMLKJIHGFEDCBA9876543210 = 1101553773143634726491620528194292510495517905608180485.
	self assert: 36rZYXWVUTSRQPONMLKJIHGFEDCBA9876543210 = 106300512100105327644605138221229898724869759421181854980.

	self assert: -2r10 = -2.
	self assert: -3r210 = -21.
	self assert: -4r3210 = -228.
	self assert: -5r43210 = -2930.
	self assert: -6r543210 = -44790.
	self assert: -7r6543210 = -800667.
	self assert: -8r76543210 = -16434824.
	self assert: -9r876543210 = -381367044.
	self assert: -10r9876543210 = -9876543210.
	self assert: -11rA9876543210 = -282458553905.
	self assert: -12rBA9876543210 = -8842413667692.
	self assert: -13rCBA9876543210 = -300771807240918.
	self assert: -14rDCBA9876543210 = -11046255305880158.
	self assert: -15rEDCBA9876543210 = -435659737878916215.
	self assert: -16rFEDCBA9876543210 = -18364758544493064720.
	self assert: -17rGFEDCBA9876543210 = -824008854613343261192.
	self assert: -18rHGFEDCBA9876543210 = -39210261334551566857170.
	self assert: -19rIHGFEDCBA9876543210 = -1972313422155189164466189.
	self assert: -20rJIHGFEDCBA9876543210 = -104567135734072022160664820.
	self assert: -21rKJIHGFEDCBA9876543210 = -5827980550840017565077671610.
	self assert: -22rLKJIHGFEDCBA9876543210 = -340653664490377789692799452102.
	self assert: -23rMLKJIHGFEDCBA9876543210 = -20837326537038308910317109288851.
	self assert: -24rNMLKJIHGFEDCBA9876543210 = -1331214537196502869015340298036888.
	self assert: -25rONMLKJIHGFEDCBA9876543210 = -88663644327703473714387251271141900.
	self assert: -26rPONMLKJIHGFEDCBA9876543210 = -6146269788878825859099399609538763450.
	self assert: -27rQPONMLKJIHGFEDCBA9876543210 = -442770531899482980347734468443677777577.
	self assert: -28rRQPONMLKJIHGFEDCBA9876543210 = -33100056003358651440264672384704297711484.
	self assert: -29rSRQPONMLKJIHGFEDCBA9876543210 = -2564411043271974895869785066497940850811934.
	self assert: -30rTSRQPONMLKJIHGFEDCBA9876543210 = -205646315052919334126040428061831153388822830.
	self assert: -31rUTSRQPONMLKJIHGFEDCBA9876543210 = -17050208381689099029767742314582582184093573615.
	self assert: -32rVUTSRQPONMLKJIHGFEDCBA9876543210 = -1459980823972598128486511383358617792788444579872.
	self assert: -33rWVUTSRQPONMLKJIHGFEDCBA9876543210 = -128983956064237823710866404905431464703849549412368.
	self assert: -34rXWVUTSRQPONMLKJIHGFEDCBA9876543210 = -11745843093701610854378775891116314824081102660800418.
	self assert: -35rYXWVUTSRQPONMLKJIHGFEDCBA9876543210 = -1101553773143634726491620528194292510495517905608180485.
	self assert: -36rZYXWVUTSRQPONMLKJIHGFEDCBA9876543210 = -106300512100105327644605138221229898724869759421181854980.! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'md 4/21/2003 16:17'!
testEven
	
	self deny: (1073741825 even).
	self assert: (1073741824  even).
	! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'stephane.ducasse 5/25/2008 15:46'!
testHex
	self assert: 0 hex = '0'.
	self assert: 12 hex = 'C'.
	self assert: 1234 hex = '4D2'.! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'nice 7/8/2008 02:47'!
testHighBit
	| suite |

	suite := (0 to: 1024) asArray , #(16rFDFD 16rFFFF 16r1000 16r1000000 16r1000001 16r70000000 16r7AFAFAFA ) , {SmallInteger maxVal . SmallInteger maxVal+1}.
	suite := suite , (suite collect: [:e | e raisedTo: 20]).
	
	suite do: [:anInteger |
		| highBit shifted |
		highBit := 0.
		shifted := 1.
		[shifted > anInteger] whileFalse: [highBit := highBit+1. shifted := shifted bitShift: 1].
		self assert: anInteger highBit = highBit].! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'nice 7/8/2008 02:44'!
testHighBitOfMagnitude
	| suite |

	suite := (0 to: 1024) asArray , #(16rFDFD 16rFFFF 16r1000 16r1000000 16r1000001 16r70000000 16r7AFAFAFA ) , {SmallInteger maxVal . SmallInteger maxVal+1}.
	suite := suite , (suite collect: [:e | e raisedTo: 20]).
	
	suite do: [:anInteger |
		| highBit shifted |
		highBit := 0.
		shifted := 1.
		[shifted > anInteger] whileFalse: [highBit := highBit+1. shifted := shifted bitShift: 1].
		self assert: anInteger highBitOfMagnitude = highBit.
		self assert: anInteger negated highBitOfMagnitude = highBit].! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'laza 3/29/2004 18:16'!
testIntegerPadding
	"self run: #testIntegerPadding"

	self assert: (1 printStringBase: 10 length: 0 padded: false) = '1'.
	self assert: (1 printStringBase: 10 length: 1 padded: false) = '1'.
	self assert: (1 printStringBase: 10 length: 2 padded: false) = ' 1'.
	self assert: (1024 printStringBase: 10 length: 19 padded: false) = '               1024'.
	self assert: (1024 printStringBase: 10 length: -1 padded: false) = '1024'.
	self assert: (1024 printStringBase: 10 length: 5 padded: false) =  ' 1024'.
	self assert: (-1024 printStringBase: 10 length: 5 padded: false) =   '-1024'.
	self assert: (-1024 printStringBase: 10 length: 19 padded: false) =  '              -1024'.

	self assert: (1 printStringBase: 10 length: 0 padded: true) = '1'.
	self assert: (1 printStringBase: 10 length: 1 padded: true) = '1'.
	self assert: (1 printStringBase: 10 length: 2 padded: true) = '01'.
	self assert: (1024 printStringBase: 10 length: 19 padded: true) = '0000000000000001024'.
	self assert: (1024 printStringBase: 10 length: -1 padded: true) = '1024'.
	self assert: (1024 printStringBase: 10 length: 5 padded: true) =  '01024'.
	self assert: (-1024 printStringBase: 10 length: 5 padded: true) =   '-1024'.
	self assert: (-1024 printStringBase: 10 length: 19 padded: true) =  '-000000000000001024'.

	self assert: (1 printStringBase: 16 length: 0 padded: false) = '1'.
	self assert: (1 printStringBase: 16 length: 1 padded: false) = '1'.
	self assert: (1 printStringBase: 16 length: 2 padded: false) = ' 1'.
	self assert: (2047 printStringBase: 16 length: 19 padded: false) =  '                7FF'.
	self assert: (2047 printStringBase: 16 length: -1 padded: false) =  '7FF'.
	self assert: (2047 printStringBase: 16 length: 4 padded: false) =  ' 7FF'.
	self assert: (-2047 printStringBase: 16 length: 4 padded: false) = '-7FF'.
	self assert: (-2047 printStringBase: 16 length: 19 padded: false) =  '               -7FF'.

	self assert: (1 printStringBase: 16 length: 0 padded: true) = '1'.
	self assert: (1 printStringBase: 16 length: 1 padded: true) = '1'.
	self assert: (1 printStringBase: 16 length: 2 padded: true) = '01'.
	self assert: (2047 printStringBase: 16 length: 19 padded: true) =  '00000000000000007FF'.
	self assert: (2047 printStringBase: 16 length: -1 padded: true) =  '7FF'.
	self assert: (2047 printStringBase: 16 length: 4 padded: true) =  '07FF'.
	self assert: (-2047 printStringBase: 16 length: 4 padded: true) = '-7FF'.
	self assert: (-2047 printStringBase: 16 length: 19 padded: true) =  '-0000000000000007FF'.

	self assert: (1 storeStringBase: 10 length: 0 padded: false) = '1'.
	self assert: (1 storeStringBase: 10 length: 1 padded: false) = '1'.
	self assert: (1 storeStringBase: 10 length: 2 padded: false) = ' 1'.
	self assert: (1024 storeStringBase: 10 length: 19 padded: false) = '               1024'.
	self assert: (1024 storeStringBase: 10 length: -1 padded: false) = '1024'.
	self assert: (1024 storeStringBase: 10 length: 5 padded: false) =  ' 1024'.
	self assert: (-1024 storeStringBase: 10 length: 5 padded: false) =   '-1024'.
	self assert: (-1024 storeStringBase: 10 length: 19 padded: false) =  '              -1024'.

	self assert: (1 storeStringBase: 10 length: 0 padded: true) = '1'.
	self assert: (1 storeStringBase: 10 length: 1 padded: true) = '1'.
	self assert: (1 storeStringBase: 10 length: 2 padded: true) = '01'.
	self assert: (1024 storeStringBase: 10 length: 19 padded: true) = '0000000000000001024'.
	self assert: (1024 storeStringBase: 10 length: -1 padded: true) = '1024'.
	self assert: (1024 storeStringBase: 10 length: 5 padded: true) =  '01024'.
	self assert: (-1024 storeStringBase: 10 length: 5 padded: true) =   '-1024'.
	self assert: (-1024 storeStringBase: 10 length: 19 padded: true) =  '-000000000000001024'.

	self assert: (1 storeStringBase: 16 length: 0 padded: false) = '16r1'.
	self assert: (1 storeStringBase: 16 length: 4 padded: false) = '16r1'.
	self assert: (1 storeStringBase: 16 length: 5 padded: false) = ' 16r1'.
	self assert: (2047 storeStringBase: 16 length: 19 padded: false) =  '             16r7FF'.
	self assert: (2047 storeStringBase: 16 length: -1 padded: false) =  '16r7FF'.
	self assert: (2047 storeStringBase: 16 length: 7 padded: false) =  ' 16r7FF'.
	self assert: (-2047 storeStringBase: 16 length: 7 padded: false) = '-16r7FF'.
	self assert: (-2047 storeStringBase: 16 length: 19 padded: false) =  '            -16r7FF'.

	self assert: (1 storeStringBase: 16 length: 0 padded: true) = '16r1'.
	self assert: (1 storeStringBase: 16 length: 4 padded: true) = '16r1'.
	self assert: (1 storeStringBase: 16 length: 5 padded: true) = '16r01'.
	self assert: (2047 storeStringBase: 16 length: 19 padded: true) =  '16r00000000000007FF'.
	self assert: (2047 storeStringBase: 16 length: -1 padded: true) =  '16r7FF'.
	self assert: (2047 storeStringBase: 16 length: 7 padded: true) =  '16r07FF'.
	self assert: (-2047 storeStringBase: 16 length: 7 padded: true) = '-16r7FF'.
	self assert: (-2047 storeStringBase: 16 length: 19 padded: true) =  '-16r0000000000007FF'.
! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'laza 11/14/2004 00:14'!
testIntegerReadFrom
	self assert: (Integer readFrom: '123' readStream base: 10) = 123.
	self assert: (Integer readFrom: '-123' readStream base: 10) = -123.
	self assert: (Integer readFrom: 'abc' readStream base: 10) = 0.
	self assert: (Integer readFrom: 'D12' readStream base: 10) = 0.
	self assert: (Integer readFrom: '1two3' readStream base: 10) = 1.
! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'sd 6/5/2005 08:45'!
testIsInteger

	self assert: (0 isInteger).
	! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'md 4/15/2003 20:40'!
testIsPowerOfTwo

	self assert: (0 isPowerOfTwo).
	self assert: (1 isPowerOfTwo).
	self assert: (2 isPowerOfTwo).
	self deny:  (3 isPowerOfTwo).
	self assert: (4 isPowerOfTwo).
	! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'nice 1/25/2008 22:51'!
testIsPowerOfTwoM6873
	"This is a non regression test for http://bugs.squeak.org/view.php?id=6873"

	self deny: ((1 to: 80) anySatisfy: [:n | (2 raisedTo: n) negated isPowerOfTwo])
		description: 'A negative integer cannot be a power of two'.! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'md 2/12/2006 14:40'!
testIsPrime

	"The following tests should return 'true'"
	self assert: 17 isPrime.
	self assert: 78901 isPrime.
	self assert: 104729 isPrime.
	self assert: 15485863 isPrime.
	self assert: 2038074743 isPrime.
	self assert: 29996224275833 isPrime.
	
	"The following tests should return 'false' (first 5 are Carmichael integers)"
	self deny: 561 isPrime.
	self deny: 2821 isPrime.
	self deny: 6601 isPrime.
	self deny: 10585 isPrime.
	self deny: 15841 isPrime.
	self deny: 256 isPrime.
	self deny: 29996224275831 isPrime.! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'StephaneDucasse 12/1/2009 14:40'!
testIsPrime2

	"Not primes:"
	#(-100 -5 -3 -2 -1 0 1) do: [ :each |
		self deny: each isPrime ].

	"The following tests should return 'true'"
	#(17 78901 104729 15485863 2038074743) do: [ :each |
		self assert: each isPrime ].
	
	"The following tests should return 'false' (first 5 are Carmichael integers)"
	#(561 2821 6601 10585 15841 256 29996224275831) do: [ :each |
		self deny: each isPrime ].! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'md 2/12/2006 14:36'!
testLargePrimesUpTo

	| nn | 
	nn := (2 raisedTo: 17) - 1. 
	self deny: (Integer primesUpTo: nn) last = nn.
	self assert: (Integer primesUpTo: nn + 1) last  = nn.
	
	
! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'nice 7/8/2008 02:21'!
testLowBit
	| suite |

	suite := (0 to: 1024) asArray , #(16rFDFD 16rFFFF 16r1000 16r1000000 16r1000001 16r70000000 16r7AFAFAFA ) , {SmallInteger maxVal . SmallInteger maxVal+1}.
	suite := suite , (suite collect: [:e | e raisedTo: 20]).
	
	suite do: [:anInteger |
		| lowBit |
		lowBit := (anInteger respondsTo: #bitAt:)
			ifTrue: [(1 to: anInteger highBit) detect: [:bitIndex | (anInteger bitAt: bitIndex) ~= 0] ifNone: [0]]
			ifFalse: [(1 to: anInteger highBit) detect: [:bitIndex | (anInteger bitAnd: (1 bitShift: bitIndex-1)) ~= 0] ifNone: [0]].
		self assert: anInteger lowBit = lowBit.
		self assert: anInteger negated lowBit = lowBit].! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'laza 3/30/2004 14:20'!
testNegativeIntegerPrinting
	"self run: #testnegativeIntegerPrinting"

	self assert: (-2 printStringBase: 2) = '-10'.
	self assert: (-2 radix: 2) = '-10'.
	self assert: -2 printStringHex = '-2'.
	self assert: (-2 storeStringBase: 2) = '-2r10'.
	self assert: -2 storeStringHex = '-16r2'.
	self assert: (-21 printStringBase: 3) = '-210'.
	self assert: (-21 radix: 3) = '-210'.
	self assert: -21 printStringHex = '-15'.
	self assert: (-21 storeStringBase: 3) = '-3r210'.
	self assert: -21 storeStringHex = '-16r15'.
	self assert: (-228 printStringBase: 4) = '-3210'.
	self assert: (-228 radix: 4) = '-3210'.
	self assert: -228 printStringHex = '-E4'.
	self assert: (-228 storeStringBase: 4) = '-4r3210'.
	self assert: -228 storeStringHex = '-16rE4'.
	self assert: (-2930 printStringBase: 5) = '-43210'.
	self assert: (-2930 radix: 5) = '-43210'.
	self assert: -2930 printStringHex = '-B72'.
	self assert: (-2930 storeStringBase: 5) = '-5r43210'.
	self assert: -2930 storeStringHex = '-16rB72'.
	self assert: (-44790 printStringBase: 6) = '-543210'.
	self assert: (-44790 radix: 6) = '-543210'.
	self assert: -44790 printStringHex = '-AEF6'.
	self assert: (-44790 storeStringBase: 6) = '-6r543210'.
	self assert: -44790 storeStringHex = '-16rAEF6'.
	self assert: (-800667 printStringBase: 7) = '-6543210'.
	self assert: (-800667 radix: 7) = '-6543210'.
	self assert: -800667 printStringHex = '-C379B'.
	self assert: (-800667 storeStringBase: 7) = '-7r6543210'.
	self assert: -800667 storeStringHex = '-16rC379B'.
	self assert: (-16434824 printStringBase: 8) = '-76543210'.
	self assert: (-16434824 radix: 8) = '-76543210'.
	self assert: -16434824 printStringHex = '-FAC688'.
	self assert: (-16434824 storeStringBase: 8) = '-8r76543210'.
	self assert: -16434824 storeStringHex = '-16rFAC688'.
	self assert: (-381367044 printStringBase: 9) = '-876543210'.
	self assert: (-381367044 radix: 9) = '-876543210'.
	self assert: -381367044 printStringHex = '-16BB3304'.
	self assert: (-381367044 storeStringBase: 9) = '-9r876543210'.
	self assert: -381367044 storeStringHex = '-16r16BB3304'.
	self assert: (-9876543210 printStringBase: 10) = '-9876543210'.
	self assert: (-9876543210 radix: 10) = '-9876543210'.
	self assert: -9876543210 printStringHex = '-24CB016EA'.
	self assert: (-9876543210 storeStringBase: 10) = '-9876543210'.
	self assert: -9876543210 storeStringHex = '-16r24CB016EA'.
	self assert: (-282458553905 printStringBase: 11) = '-A9876543210'.
	self assert: (-282458553905 radix: 11) = '-A9876543210'.
	self assert: -282458553905 printStringHex = '-41C3D77E31'.
	self assert: (-282458553905 storeStringBase: 11) = '-11rA9876543210'.
	self assert: -282458553905 storeStringHex = '-16r41C3D77E31'.
	self assert: (-8842413667692 printStringBase: 12) = '-BA9876543210'.
	self assert: (-8842413667692 radix: 12) = '-BA9876543210'.
	self assert: -8842413667692 printStringHex = '-80AC8ECF56C'.
	self assert: (-8842413667692 storeStringBase: 12) = '-12rBA9876543210'.
	self assert: -8842413667692 storeStringHex = '-16r80AC8ECF56C'.
	self assert: (-300771807240918 printStringBase: 13) = '-CBA9876543210'.
	self assert: (-300771807240918 radix: 13) = '-CBA9876543210'.
	self assert: -300771807240918 printStringHex = '-1118CE4BAA2D6'.
	self assert: (-300771807240918 storeStringBase: 13) = '-13rCBA9876543210'.
	self assert: -300771807240918 storeStringHex = '-16r1118CE4BAA2D6'.
	self assert: (-11046255305880158 printStringBase: 14) = '-DCBA9876543210'.
	self assert: (-11046255305880158 radix: 14) = '-DCBA9876543210'.
	self assert: -11046255305880158 printStringHex = '-273E82BB9AF25E'.
	self assert: (-11046255305880158 storeStringBase: 14) = '-14rDCBA9876543210'.
	self assert: -11046255305880158 storeStringHex = '-16r273E82BB9AF25E'.
	self assert: (-435659737878916215 printStringBase: 15) = '-EDCBA9876543210'.
	self assert: (-435659737878916215 radix: 15) = '-EDCBA9876543210'.
	self assert: -435659737878916215 printStringHex = '-60BC6392F366C77'.
	self assert: (-435659737878916215 storeStringBase: 15) = '-15rEDCBA9876543210'.
	self assert: -435659737878916215 storeStringHex = '-16r60BC6392F366C77'.
	self assert: (-18364758544493064720 printStringBase: 16) = '-FEDCBA9876543210'.
	self assert: (-18364758544493064720 radix: 16) = '-FEDCBA9876543210'.
	self assert: -18364758544493064720 printStringHex = '-FEDCBA9876543210'.
	self assert: (-18364758544493064720 storeStringBase: 16) = '-16rFEDCBA9876543210'.
	self assert: -18364758544493064720 storeStringHex = '-16rFEDCBA9876543210'.
	self assert: (-824008854613343261192 printStringBase: 17) = '-GFEDCBA9876543210'.
	self assert: (-824008854613343261192 radix: 17) = '-GFEDCBA9876543210'.
	self assert: -824008854613343261192 printStringHex = '-2CAB6B877C1CD2D208'.
	self assert: (-824008854613343261192 storeStringBase: 17) = '-17rGFEDCBA9876543210'.
	self assert: -824008854613343261192 storeStringHex = '-16r2CAB6B877C1CD2D208'.
	self assert: (-39210261334551566857170 printStringBase: 18) = '-HGFEDCBA9876543210'.
	self assert: (-39210261334551566857170 radix: 18) = '-HGFEDCBA9876543210'.
	self assert: -39210261334551566857170 printStringHex = '-84D97AFCAE81415B3D2'.
	self assert: (-39210261334551566857170 storeStringBase: 18) = '-18rHGFEDCBA9876543210'.
	self assert: -39210261334551566857170 storeStringHex = '-16r84D97AFCAE81415B3D2'.
	self assert: (-1972313422155189164466189 printStringBase: 19) = '-IHGFEDCBA9876543210'.
	self assert: (-1972313422155189164466189 radix: 19) = '-IHGFEDCBA9876543210'.
	self assert: -1972313422155189164466189 printStringHex = '-1A1A75329C5C6FC00600D'.
	self assert: (-1972313422155189164466189 storeStringBase: 19) = '-19rIHGFEDCBA9876543210'.
	self assert: -1972313422155189164466189 storeStringHex = '-16r1A1A75329C5C6FC00600D'.
	self assert: (-104567135734072022160664820 printStringBase: 20) = '-JIHGFEDCBA9876543210'.
	self assert: (-104567135734072022160664820 radix: 20) = '-JIHGFEDCBA9876543210'.
	self assert: -104567135734072022160664820 printStringHex = '-567EF3C9636D242A8C68F4'.
	self assert: (-104567135734072022160664820 storeStringBase: 20) = '-20rJIHGFEDCBA9876543210'.
	self assert: -104567135734072022160664820 storeStringHex = '-16r567EF3C9636D242A8C68F4'.
	self assert: (-5827980550840017565077671610 printStringBase: 21) = '-KJIHGFEDCBA9876543210'.
	self assert: (-5827980550840017565077671610 radix: 21) = '-KJIHGFEDCBA9876543210'.
	self assert: -5827980550840017565077671610 printStringHex = '-12D4CAE2B8A09BCFDBE30EBA'.
	self assert: (-5827980550840017565077671610 storeStringBase: 21) = '-21rKJIHGFEDCBA9876543210'.
	self assert: -5827980550840017565077671610 storeStringHex = '-16r12D4CAE2B8A09BCFDBE30EBA'.
	self assert: (-340653664490377789692799452102 printStringBase: 22) = '-LKJIHGFEDCBA9876543210'.
	self assert: (-340653664490377789692799452102 radix: 22) = '-LKJIHGFEDCBA9876543210'.
	self assert: -340653664490377789692799452102 printStringHex = '-44CB61B5B47E1A5D8F88583C6'.
	self assert: (-340653664490377789692799452102 storeStringBase: 22) = '-22rLKJIHGFEDCBA9876543210'.
	self assert: -340653664490377789692799452102 storeStringHex = '-16r44CB61B5B47E1A5D8F88583C6'.
	self assert: (-20837326537038308910317109288851 printStringBase: 23) = '-MLKJIHGFEDCBA9876543210'.
	self assert: (-20837326537038308910317109288851 radix: 23) = '-MLKJIHGFEDCBA9876543210'.
	self assert: -20837326537038308910317109288851 printStringHex = '-1070108876456E0EF115B389F93'.
	self assert: (-20837326537038308910317109288851 storeStringBase: 23) = '-23rMLKJIHGFEDCBA9876543210'.
	self assert: -20837326537038308910317109288851 storeStringHex = '-16r1070108876456E0EF115B389F93'.
	self assert: (-1331214537196502869015340298036888 printStringBase: 24) = '-NMLKJIHGFEDCBA9876543210'.
	self assert: (-1331214537196502869015340298036888 radix: 24) = '-NMLKJIHGFEDCBA9876543210'.
	self assert: -1331214537196502869015340298036888 printStringHex = '-41A24A285154B026B6ED206C6698'.
	self assert: (-1331214537196502869015340298036888 storeStringBase: 24) = '-24rNMLKJIHGFEDCBA9876543210'.
	self assert: -1331214537196502869015340298036888 storeStringHex = '-16r41A24A285154B026B6ED206C6698'.
	self assert: (-88663644327703473714387251271141900 printStringBase: 25) = '-ONMLKJIHGFEDCBA9876543210'.
	self assert: (-88663644327703473714387251271141900 radix: 25) = '-ONMLKJIHGFEDCBA9876543210'.
	self assert: -88663644327703473714387251271141900 printStringHex = '-111374860A2C6CEBE5999630398A0C'.
	self assert: (-88663644327703473714387251271141900 storeStringBase: 25) = '-25rONMLKJIHGFEDCBA9876543210'.
	self assert: -88663644327703473714387251271141900 storeStringHex = '-16r111374860A2C6CEBE5999630398A0C'.
	self assert: (-6146269788878825859099399609538763450 printStringBase: 26) = '-PONMLKJIHGFEDCBA9876543210'.
	self assert: (-6146269788878825859099399609538763450 radix: 26) = '-PONMLKJIHGFEDCBA9876543210'.
	self assert: -6146269788878825859099399609538763450 printStringHex = '-49FBA7F30B0F48BD14E6A99BD8ADABA'.
	self assert: (-6146269788878825859099399609538763450 storeStringBase: 26) = '-26rPONMLKJIHGFEDCBA9876543210'.
	self assert: -6146269788878825859099399609538763450 storeStringHex = '-16r49FBA7F30B0F48BD14E6A99BD8ADABA'.
	self assert: (-442770531899482980347734468443677777577 printStringBase: 27) = '-QPONMLKJIHGFEDCBA9876543210'.
	self assert: (-442770531899482980347734468443677777577 radix: 27) = '-QPONMLKJIHGFEDCBA9876543210'.
	self assert: -442770531899482980347734468443677777577 printStringHex = '-14D1A80A997343640C1145A073731DEA9'.
	self assert: (-442770531899482980347734468443677777577 storeStringBase: 27) = '-27rQPONMLKJIHGFEDCBA9876543210'.
	self assert: -442770531899482980347734468443677777577 storeStringHex = '-16r14D1A80A997343640C1145A073731DEA9'.
	self assert: (-33100056003358651440264672384704297711484 printStringBase: 28) = '-RQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-33100056003358651440264672384704297711484 radix: 28) = '-RQPONMLKJIHGFEDCBA9876543210'.
	self assert: -33100056003358651440264672384704297711484 printStringHex = '-6145B6E6DACFA25D0E936F51D25932377C'.
	self assert: (-33100056003358651440264672384704297711484 storeStringBase: 28) = '-28rRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -33100056003358651440264672384704297711484 storeStringHex = '-16r6145B6E6DACFA25D0E936F51D25932377C'.
	self assert: (-2564411043271974895869785066497940850811934 printStringBase: 29) = '-SRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-2564411043271974895869785066497940850811934 radix: 29) = '-SRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -2564411043271974895869785066497940850811934 printStringHex = '-1D702071CBA4A1597D4DD37E95EFAC79241E'.
	self assert: (-2564411043271974895869785066497940850811934 storeStringBase: 29) = '-29rSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -2564411043271974895869785066497940850811934 storeStringHex = '-16r1D702071CBA4A1597D4DD37E95EFAC79241E'.
	self assert: (-205646315052919334126040428061831153388822830 printStringBase: 30) = '-TSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-205646315052919334126040428061831153388822830 radix: 30) = '-TSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -205646315052919334126040428061831153388822830 printStringHex = '-938B4343B54B550989989D02998718FFB212E'.
	self assert: (-205646315052919334126040428061831153388822830 storeStringBase: 30) = '-30rTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -205646315052919334126040428061831153388822830 storeStringHex = '-16r938B4343B54B550989989D02998718FFB212E'.
	self assert: (-17050208381689099029767742314582582184093573615 printStringBase: 31) = '-UTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-17050208381689099029767742314582582184093573615 radix: 31) = '-UTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -17050208381689099029767742314582582184093573615 printStringHex = '-2FC8ECB1521BA16D24A69E976D53873E2C661EF'.
	self assert: (-17050208381689099029767742314582582184093573615 storeStringBase: 31) = '-31rUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -17050208381689099029767742314582582184093573615 storeStringHex = '-16r2FC8ECB1521BA16D24A69E976D53873E2C661EF'.
	self assert: (-1459980823972598128486511383358617792788444579872 printStringBase: 32) = '-VUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-1459980823972598128486511383358617792788444579872 radix: 32) = '-VUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -1459980823972598128486511383358617792788444579872 printStringHex = '-FFBBCDEB38BDAB49CA307B9AC5A928398A418820'.
	self assert: (-1459980823972598128486511383358617792788444579872 storeStringBase: 32) = '-32rVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -1459980823972598128486511383358617792788444579872 storeStringHex = '-16rFFBBCDEB38BDAB49CA307B9AC5A928398A418820'.
	self assert: (-128983956064237823710866404905431464703849549412368 printStringBase: 33) = '-WVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-128983956064237823710866404905431464703849549412368 radix: 33) = '-WVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -128983956064237823710866404905431464703849549412368 printStringHex = '-584120A0328DE272AB055A8AA003CE4A559F223810'.
	self assert: (-128983956064237823710866404905431464703849549412368 storeStringBase: 33) = '-33rWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -128983956064237823710866404905431464703849549412368 storeStringHex = '-16r584120A0328DE272AB055A8AA003CE4A559F223810'.
	self assert: (-11745843093701610854378775891116314824081102660800418 printStringBase: 34) = '-XWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-11745843093701610854378775891116314824081102660800418 radix: 34) = '-XWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -11745843093701610854378775891116314824081102660800418 printStringHex = '-1F64D4FC76000F7B92CF0CD5D0F350139AB9F25D8FA2'.
	self assert: (-11745843093701610854378775891116314824081102660800418 storeStringBase: 34) = '-34rXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -11745843093701610854378775891116314824081102660800418 storeStringHex = '-16r1F64D4FC76000F7B92CF0CD5D0F350139AB9F25D8FA2'.
	self assert: (-1101553773143634726491620528194292510495517905608180485 printStringBase: 35) = '-YXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-1101553773143634726491620528194292510495517905608180485 radix: 35) = '-YXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -1101553773143634726491620528194292510495517905608180485 printStringHex = '-B8031AD55AD1FAA89E07A271CA1ED2F420415D1570305'.
	self assert: (-1101553773143634726491620528194292510495517905608180485 storeStringBase: 35) = '-35rYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -1101553773143634726491620528194292510495517905608180485 storeStringHex = '-16rB8031AD55AD1FAA89E07A271CA1ED2F420415D1570305'.
	self assert: (-106300512100105327644605138221229898724869759421181854980 printStringBase: 36) = '-ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (-106300512100105327644605138221229898724869759421181854980 radix: 36) = '-ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -106300512100105327644605138221229898724869759421181854980 printStringHex = '-455D441E55A37239AB4C303189576071AF5578FFCA80504'.
	self assert: (-106300512100105327644605138221229898724869759421181854980 storeStringBase: 36) = '-36rZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: -106300512100105327644605138221229898724869759421181854980 storeStringHex = '-16r455D441E55A37239AB4C303189576071AF5578FFCA80504'.! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'md 3/25/2003 23:14'!
testNew
	self should: [Integer new] raise: TestResult error. ! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'nice 2/15/2008 22:23'!
testNumberOfDigits
	
	2 to: 32 do: [:b |
		1 to: 1000//b do: [:n |
			| bRaisedToN |
			bRaisedToN := b raisedTo: n.
			self assert: (bRaisedToN - 1 numberOfDigitsInBase: b) = n.
			self assert: (bRaisedToN numberOfDigitsInBase: b) = (n+1).
			self assert: (bRaisedToN + 1 numberOfDigitsInBase: b) = (n+1).
			
			self assert: (bRaisedToN negated + 1 numberOfDigitsInBase: b) = n.
			self assert: (bRaisedToN negated numberOfDigitsInBase: b) = (n+1).
			self assert: (bRaisedToN negated - 1 numberOfDigitsInBase: b) = (n+1).]].
! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'laza 3/30/2004 11:52'!
testPositiveIntegerPrinting
	"self run: #testPositiveIntegerPrinting"

	self assert: 0 printString = '0'.
	self assert: 0 printStringHex = '0'.
	self assert: 0 storeStringHex = '16r0'.

	self assert: (2 printStringBase: 2) = '10'.
	self assert: (2 radix: 2) = '10'.
	self assert: 2 printStringHex = '2'.
	self assert: (2 storeStringBase: 2) = '2r10'.
	self assert: 2 storeStringHex = '16r2'.
	self assert: (21 printStringBase: 3) = '210'.
	self assert: (21 radix: 3) = '210'.
	self assert: 21 printStringHex = '15'.
	self assert: (21 storeStringBase: 3) = '3r210'.
	self assert: 21 storeStringHex = '16r15'.
	self assert: (228 printStringBase: 4) = '3210'.
	self assert: (228 radix: 4) = '3210'.
	self assert: 228 printStringHex = 'E4'.
	self assert: (228 storeStringBase: 4) = '4r3210'.
	self assert: 228 storeStringHex = '16rE4'.
	self assert: (2930 printStringBase: 5) = '43210'.
	self assert: (2930 radix: 5) = '43210'.
	self assert: 2930 printStringHex = 'B72'.
	self assert: (2930 storeStringBase: 5) = '5r43210'.
	self assert: 2930 storeStringHex = '16rB72'.
	self assert: (44790 printStringBase: 6) = '543210'.
	self assert: (44790 radix: 6) = '543210'.
	self assert: 44790 printStringHex = 'AEF6'.
	self assert: (44790 storeStringBase: 6) = '6r543210'.
	self assert: 44790 storeStringHex = '16rAEF6'.
	self assert: (800667 printStringBase: 7) = '6543210'.
	self assert: (800667 radix: 7) = '6543210'.
	self assert: 800667 printStringHex = 'C379B'.
	self assert: (800667 storeStringBase: 7) = '7r6543210'.
	self assert: 800667 storeStringHex = '16rC379B'.
	self assert: (16434824 printStringBase: 8) = '76543210'.
	self assert: (16434824 radix: 8) = '76543210'.
	self assert: 16434824 printStringHex = 'FAC688'.
	self assert: (16434824 storeStringBase: 8) = '8r76543210'.
	self assert: 16434824 storeStringHex = '16rFAC688'.
	self assert: (381367044 printStringBase: 9) = '876543210'.
	self assert: (381367044 radix: 9) = '876543210'.
	self assert: 381367044 printStringHex = '16BB3304'.
	self assert: (381367044 storeStringBase: 9) = '9r876543210'.
	self assert: 381367044 storeStringHex = '16r16BB3304'.
	self assert: (9876543210 printStringBase: 10) = '9876543210'.
	self assert: (9876543210 radix: 10) = '9876543210'.
	self assert: 9876543210 printStringHex = '24CB016EA'.
	self assert: (9876543210 storeStringBase: 10) = '9876543210'.
	self assert: 9876543210 storeStringHex = '16r24CB016EA'.
	self assert: (282458553905 printStringBase: 11) = 'A9876543210'.
	self assert: (282458553905 radix: 11) = 'A9876543210'.
	self assert: 282458553905 printStringHex = '41C3D77E31'.
	self assert: (282458553905 storeStringBase: 11) = '11rA9876543210'.
	self assert: 282458553905 storeStringHex = '16r41C3D77E31'.
	self assert: (8842413667692 printStringBase: 12) = 'BA9876543210'.
	self assert: (8842413667692 radix: 12) = 'BA9876543210'.
	self assert: 8842413667692 printStringHex = '80AC8ECF56C'.
	self assert: (8842413667692 storeStringBase: 12) = '12rBA9876543210'.
	self assert: 8842413667692 storeStringHex = '16r80AC8ECF56C'.
	self assert: (300771807240918 printStringBase: 13) = 'CBA9876543210'.
	self assert: (300771807240918 radix: 13) = 'CBA9876543210'.
	self assert: 300771807240918 printStringHex = '1118CE4BAA2D6'.
	self assert: (300771807240918 storeStringBase: 13) = '13rCBA9876543210'.
	self assert: 300771807240918 storeStringHex = '16r1118CE4BAA2D6'.
	self assert: (11046255305880158 printStringBase: 14) = 'DCBA9876543210'.
	self assert: (11046255305880158 radix: 14) = 'DCBA9876543210'.
	self assert: 11046255305880158 printStringHex = '273E82BB9AF25E'.
	self assert: (11046255305880158 storeStringBase: 14) = '14rDCBA9876543210'.
	self assert: 11046255305880158 storeStringHex = '16r273E82BB9AF25E'.
	self assert: (435659737878916215 printStringBase: 15) = 'EDCBA9876543210'.
	self assert: (435659737878916215 radix: 15) = 'EDCBA9876543210'.
	self assert: 435659737878916215 printStringHex = '60BC6392F366C77'.
	self assert: (435659737878916215 storeStringBase: 15) = '15rEDCBA9876543210'.
	self assert: 435659737878916215 storeStringHex = '16r60BC6392F366C77'.
	self assert: (18364758544493064720 printStringBase: 16) = 'FEDCBA9876543210'.
	self assert: (18364758544493064720 radix: 16) = 'FEDCBA9876543210'.
	self assert: 18364758544493064720 printStringHex = 'FEDCBA9876543210'.
	self assert: (18364758544493064720 storeStringBase: 16) = '16rFEDCBA9876543210'.
	self assert: 18364758544493064720 storeStringHex = '16rFEDCBA9876543210'.
	self assert: (824008854613343261192 printStringBase: 17) = 'GFEDCBA9876543210'.
	self assert: (824008854613343261192 radix: 17) = 'GFEDCBA9876543210'.
	self assert: 824008854613343261192 printStringHex = '2CAB6B877C1CD2D208'.
	self assert: (824008854613343261192 storeStringBase: 17) = '17rGFEDCBA9876543210'.
	self assert: 824008854613343261192 storeStringHex = '16r2CAB6B877C1CD2D208'.
	self assert: (39210261334551566857170 printStringBase: 18) = 'HGFEDCBA9876543210'.
	self assert: (39210261334551566857170 radix: 18) = 'HGFEDCBA9876543210'.
	self assert: 39210261334551566857170 printStringHex = '84D97AFCAE81415B3D2'.
	self assert: (39210261334551566857170 storeStringBase: 18) = '18rHGFEDCBA9876543210'.
	self assert: 39210261334551566857170 storeStringHex = '16r84D97AFCAE81415B3D2'.
	self assert: (1972313422155189164466189 printStringBase: 19) = 'IHGFEDCBA9876543210'.
	self assert: (1972313422155189164466189 radix: 19) = 'IHGFEDCBA9876543210'.
	self assert: 1972313422155189164466189 printStringHex = '1A1A75329C5C6FC00600D'.
	self assert: (1972313422155189164466189 storeStringBase: 19) = '19rIHGFEDCBA9876543210'.
	self assert: 1972313422155189164466189 storeStringHex = '16r1A1A75329C5C6FC00600D'.
	self assert: (104567135734072022160664820 printStringBase: 20) = 'JIHGFEDCBA9876543210'.
	self assert: (104567135734072022160664820 radix: 20) = 'JIHGFEDCBA9876543210'.
	self assert: 104567135734072022160664820 printStringHex = '567EF3C9636D242A8C68F4'.
	self assert: (104567135734072022160664820 storeStringBase: 20) = '20rJIHGFEDCBA9876543210'.
	self assert: 104567135734072022160664820 storeStringHex = '16r567EF3C9636D242A8C68F4'.
	self assert: (5827980550840017565077671610 printStringBase: 21) = 'KJIHGFEDCBA9876543210'.
	self assert: (5827980550840017565077671610 radix: 21) = 'KJIHGFEDCBA9876543210'.
	self assert: 5827980550840017565077671610 printStringHex = '12D4CAE2B8A09BCFDBE30EBA'.
	self assert: (5827980550840017565077671610 storeStringBase: 21) = '21rKJIHGFEDCBA9876543210'.
	self assert: 5827980550840017565077671610 storeStringHex = '16r12D4CAE2B8A09BCFDBE30EBA'.
	self assert: (340653664490377789692799452102 printStringBase: 22) = 'LKJIHGFEDCBA9876543210'.
	self assert: (340653664490377789692799452102 radix: 22) = 'LKJIHGFEDCBA9876543210'.
	self assert: 340653664490377789692799452102 printStringHex = '44CB61B5B47E1A5D8F88583C6'.
	self assert: (340653664490377789692799452102 storeStringBase: 22) = '22rLKJIHGFEDCBA9876543210'.
	self assert: 340653664490377789692799452102 storeStringHex = '16r44CB61B5B47E1A5D8F88583C6'.
	self assert: (20837326537038308910317109288851 printStringBase: 23) = 'MLKJIHGFEDCBA9876543210'.
	self assert: (20837326537038308910317109288851 radix: 23) = 'MLKJIHGFEDCBA9876543210'.
	self assert: 20837326537038308910317109288851 printStringHex = '1070108876456E0EF115B389F93'.
	self assert: (20837326537038308910317109288851 storeStringBase: 23) = '23rMLKJIHGFEDCBA9876543210'.
	self assert: 20837326537038308910317109288851 storeStringHex = '16r1070108876456E0EF115B389F93'.
	self assert: (1331214537196502869015340298036888 printStringBase: 24) = 'NMLKJIHGFEDCBA9876543210'.
	self assert: (1331214537196502869015340298036888 radix: 24) = 'NMLKJIHGFEDCBA9876543210'.
	self assert: 1331214537196502869015340298036888 printStringHex = '41A24A285154B026B6ED206C6698'.
	self assert: (1331214537196502869015340298036888 storeStringBase: 24) = '24rNMLKJIHGFEDCBA9876543210'.
	self assert: 1331214537196502869015340298036888 storeStringHex = '16r41A24A285154B026B6ED206C6698'.
	self assert: (88663644327703473714387251271141900 printStringBase: 25) = 'ONMLKJIHGFEDCBA9876543210'.
	self assert: (88663644327703473714387251271141900 radix: 25) = 'ONMLKJIHGFEDCBA9876543210'.
	self assert: 88663644327703473714387251271141900 printStringHex = '111374860A2C6CEBE5999630398A0C'.
	self assert: (88663644327703473714387251271141900 storeStringBase: 25) = '25rONMLKJIHGFEDCBA9876543210'.
	self assert: 88663644327703473714387251271141900 storeStringHex = '16r111374860A2C6CEBE5999630398A0C'.
	self assert: (6146269788878825859099399609538763450 printStringBase: 26) = 'PONMLKJIHGFEDCBA9876543210'.
	self assert: (6146269788878825859099399609538763450 radix: 26) = 'PONMLKJIHGFEDCBA9876543210'.
	self assert: 6146269788878825859099399609538763450 printStringHex = '49FBA7F30B0F48BD14E6A99BD8ADABA'.
	self assert: (6146269788878825859099399609538763450 storeStringBase: 26) = '26rPONMLKJIHGFEDCBA9876543210'.
	self assert: 6146269788878825859099399609538763450 storeStringHex = '16r49FBA7F30B0F48BD14E6A99BD8ADABA'.
	self assert: (442770531899482980347734468443677777577 printStringBase: 27) = 'QPONMLKJIHGFEDCBA9876543210'.
	self assert: (442770531899482980347734468443677777577 radix: 27) = 'QPONMLKJIHGFEDCBA9876543210'.
	self assert: 442770531899482980347734468443677777577 printStringHex = '14D1A80A997343640C1145A073731DEA9'.
	self assert: (442770531899482980347734468443677777577 storeStringBase: 27) = '27rQPONMLKJIHGFEDCBA9876543210'.
	self assert: 442770531899482980347734468443677777577 storeStringHex = '16r14D1A80A997343640C1145A073731DEA9'.
	self assert: (33100056003358651440264672384704297711484 printStringBase: 28) = 'RQPONMLKJIHGFEDCBA9876543210'.
	self assert: (33100056003358651440264672384704297711484 radix: 28) = 'RQPONMLKJIHGFEDCBA9876543210'.
	self assert: 33100056003358651440264672384704297711484 printStringHex = '6145B6E6DACFA25D0E936F51D25932377C'.
	self assert: (33100056003358651440264672384704297711484 storeStringBase: 28) = '28rRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 33100056003358651440264672384704297711484 storeStringHex = '16r6145B6E6DACFA25D0E936F51D25932377C'.
	self assert: (2564411043271974895869785066497940850811934 printStringBase: 29) = 'SRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (2564411043271974895869785066497940850811934 radix: 29) = 'SRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 2564411043271974895869785066497940850811934 printStringHex = '1D702071CBA4A1597D4DD37E95EFAC79241E'.
	self assert: (2564411043271974895869785066497940850811934 storeStringBase: 29) = '29rSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 2564411043271974895869785066497940850811934 storeStringHex = '16r1D702071CBA4A1597D4DD37E95EFAC79241E'.
	self assert: (205646315052919334126040428061831153388822830 printStringBase: 30) = 'TSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (205646315052919334126040428061831153388822830 radix: 30) = 'TSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 205646315052919334126040428061831153388822830 printStringHex = '938B4343B54B550989989D02998718FFB212E'.
	self assert: (205646315052919334126040428061831153388822830 storeStringBase: 30) = '30rTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 205646315052919334126040428061831153388822830 storeStringHex = '16r938B4343B54B550989989D02998718FFB212E'.
	self assert: (17050208381689099029767742314582582184093573615 printStringBase: 31) = 'UTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (17050208381689099029767742314582582184093573615 radix: 31) = 'UTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 17050208381689099029767742314582582184093573615 printStringHex = '2FC8ECB1521BA16D24A69E976D53873E2C661EF'.
	self assert: (17050208381689099029767742314582582184093573615 storeStringBase: 31) = '31rUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 17050208381689099029767742314582582184093573615 storeStringHex = '16r2FC8ECB1521BA16D24A69E976D53873E2C661EF'.
	self assert: (1459980823972598128486511383358617792788444579872 printStringBase: 32) = 'VUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (1459980823972598128486511383358617792788444579872 radix: 32) = 'VUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 1459980823972598128486511383358617792788444579872 printStringHex = 'FFBBCDEB38BDAB49CA307B9AC5A928398A418820'.
	self assert: (1459980823972598128486511383358617792788444579872 storeStringBase: 32) = '32rVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 1459980823972598128486511383358617792788444579872 storeStringHex = '16rFFBBCDEB38BDAB49CA307B9AC5A928398A418820'.
	self assert: (128983956064237823710866404905431464703849549412368 printStringBase: 33) = 'WVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (128983956064237823710866404905431464703849549412368 radix: 33) = 'WVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 128983956064237823710866404905431464703849549412368 printStringHex = '584120A0328DE272AB055A8AA003CE4A559F223810'.
	self assert: (128983956064237823710866404905431464703849549412368 storeStringBase: 33) = '33rWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 128983956064237823710866404905431464703849549412368 storeStringHex = '16r584120A0328DE272AB055A8AA003CE4A559F223810'.
	self assert: (11745843093701610854378775891116314824081102660800418 printStringBase: 34) = 'XWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (11745843093701610854378775891116314824081102660800418 radix: 34) = 'XWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 11745843093701610854378775891116314824081102660800418 printStringHex = '1F64D4FC76000F7B92CF0CD5D0F350139AB9F25D8FA2'.
	self assert: (11745843093701610854378775891116314824081102660800418 storeStringBase: 34) = '34rXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 11745843093701610854378775891116314824081102660800418 storeStringHex = '16r1F64D4FC76000F7B92CF0CD5D0F350139AB9F25D8FA2'.
	self assert: (1101553773143634726491620528194292510495517905608180485 printStringBase: 35) = 'YXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (1101553773143634726491620528194292510495517905608180485 radix: 35) = 'YXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 1101553773143634726491620528194292510495517905608180485 printStringHex = 'B8031AD55AD1FAA89E07A271CA1ED2F420415D1570305'.
	self assert: (1101553773143634726491620528194292510495517905608180485 storeStringBase: 35) = '35rYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 1101553773143634726491620528194292510495517905608180485 storeStringHex = '16rB8031AD55AD1FAA89E07A271CA1ED2F420415D1570305'.
	self assert: (106300512100105327644605138221229898724869759421181854980 printStringBase: 36) = 'ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: (106300512100105327644605138221229898724869759421181854980 radix: 36) = 'ZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 106300512100105327644605138221229898724869759421181854980 printStringHex = '455D441E55A37239AB4C303189576071AF5578FFCA80504'.
	self assert: (106300512100105327644605138221229898724869759421181854980 storeStringBase: 36) = '36rZYXWVUTSRQPONMLKJIHGFEDCBA9876543210'.
	self assert: 106300512100105327644605138221229898724869759421181854980 storeStringHex = '16r455D441E55A37239AB4C303189576071AF5578FFCA80504'.! !

!IntegerTest methodsFor: 'tests - basic' stamp: 'md 2/12/2006 14:36'!
testPrimesUpTo

	| primes nn|
	primes := Integer primesUpTo: 100.
	self assert: primes = #(2 3 5 7 11 13 17 19 23 29 31 37 41 43 47 53 59 61 67 71 73 79 83 89 97).
	
	"upTo: semantics means 'non-inclusive'"
	primes := Integer primesUpTo: 5.
	self assert: primes = #(2 3).
	
	"this test is green for nn>25000, see #testLargePrimesUpTo"
	nn := 5. 
	self deny: (Integer primesUpTo: nn) last = nn.
	self assert: (Integer primesUpTo: nn + 1) last  = nn.! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'fbs 2/9/2006 08:48'!
testPrintOnBaseShowRadix
	| s |
	s := ReadWriteStream on: ''.
	123 printOn: s base: 10 showRadix: false.
	self assert: (s contents = '123').
	
	s := ReadWriteStream on: ''.
	123 printOn: s base: 10 showRadix: true.
	self assert: (s contents = '10r123').
	
	s := ReadWriteStream on: ''.
	123 printOn: s base: 8 showRadix: false.
	self assert: (s contents = '173').
	
	s := ReadWriteStream on: ''.
	123 printOn: s base: 8 showRadix: true.
	self assert: (s contents = '8r173').! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'nice 2/15/2008 22:31'!
testPrintStringBase
	
	2 to: 32 do: [:b |
		1 to: 1000//b do: [:n |
			| bRaisedToN |
			bRaisedToN := b raisedTo: n.
			self assert: (bRaisedToN - 1 printStringBase: b) = (String new: n withAll: (Character digitValue: b-1)).
			self assert: (bRaisedToN printStringBase: b) = ('1' , (String new: n withAll: $0)).
			
			self assert: (bRaisedToN negated + 1 printStringBase: b) = ('-' , (String new: n withAll: (Character digitValue: b-1))).
			self assert: (bRaisedToN negated printStringBase: b) = ('-1' , (String new: n withAll: $0))]].
! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'damiencassou 5/30/2008 11:09'!
testReadFrom
	"Ensure remaining characters in a stream are not lost when parsing an integer."
	| rs i s |
	rs := '123s could be confused with a ScaledDecimal' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: 's could be confused with a ScaledDecimal' = s.
	rs := '123.s could be confused with a ScaledDecimal' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: '.s could be confused with a ScaledDecimal' = s! !

!IntegerTest methodsFor: 'tests - printing' stamp: 'laza 3/30/2004 09:23'!
testRomanPrinting
	self assert: 0 printStringRoman = ''. "No symbol for zero"
	self assert: 1 printStringRoman = 'I'.
	self assert: 2 printStringRoman = 'II'.
	self assert: 3 printStringRoman = 'III'.
	self assert: 4 printStringRoman = 'IV'.
	self assert: 5 printStringRoman = 'V'.
	self assert: 6 printStringRoman = 'VI'.
	self assert: 7 printStringRoman = 'VII'.
	self assert: 8 printStringRoman = 'VIII'.
	self assert: 9 printStringRoman = 'IX'.
	self assert: 10 printStringRoman = 'X'.
	self assert: 23 printStringRoman = 'XXIII'.
	self assert: 36 printStringRoman = 'XXXVI'.
	self assert: 49 printStringRoman = 'XLIX'.
	self assert: 62 printStringRoman = 'LXII'.
	self assert: 75 printStringRoman = 'LXXV'.
	self assert: 88 printStringRoman = 'LXXXVIII'.
	self assert: 99 printStringRoman = 'XCIX'.
	self assert: 100 printStringRoman = 'C'.
	self assert: 101 printStringRoman = 'CI'.
	self assert: 196 printStringRoman = 'CXCVI'.
	self assert: 197 printStringRoman = 'CXCVII'.
	self assert: 198 printStringRoman = 'CXCVIII'.
	self assert: 293 printStringRoman = 'CCXCIII'.
	self assert: 294 printStringRoman = 'CCXCIV'.
	self assert: 295 printStringRoman = 'CCXCV'.
	self assert: 390 printStringRoman = 'CCCXC'.
	self assert: 391 printStringRoman = 'CCCXCI'.
	self assert: 392 printStringRoman = 'CCCXCII'.
	self assert: 487 printStringRoman = 'CDLXXXVII'.
	self assert: 488 printStringRoman = 'CDLXXXVIII'.
	self assert: 489 printStringRoman = 'CDLXXXIX'.
	self assert: 584 printStringRoman = 'DLXXXIV'.
	self assert: 585 printStringRoman = 'DLXXXV'.
	self assert: 586 printStringRoman = 'DLXXXVI'.
	self assert: 681 printStringRoman = 'DCLXXXI'.
	self assert: 682 printStringRoman = 'DCLXXXII'.
	self assert: 683 printStringRoman = 'DCLXXXIII'.
	self assert: 778 printStringRoman = 'DCCLXXVIII'.
	self assert: 779 printStringRoman = 'DCCLXXIX'.
	self assert: 780 printStringRoman = 'DCCLXXX'.
	self assert: 875 printStringRoman = 'DCCCLXXV'.
	self assert: 876 printStringRoman = 'DCCCLXXVI'.
	self assert: 877 printStringRoman = 'DCCCLXXVII'.
	self assert: 972 printStringRoman = 'CMLXXII'.
	self assert: 973 printStringRoman = 'CMLXXIII'.
	self assert: 974 printStringRoman = 'CMLXXIV'.
	self assert: 1069 printStringRoman = 'MLXIX'.
	self assert: 1070 printStringRoman = 'MLXX'.
	self assert: 1071 printStringRoman = 'MLXXI'.
	self assert: 1166 printStringRoman = 'MCLXVI'.
	self assert: 1167 printStringRoman = 'MCLXVII'.
	self assert: 1168 printStringRoman = 'MCLXVIII'.
	self assert: 1263 printStringRoman = 'MCCLXIII'.
	self assert: 1264 printStringRoman = 'MCCLXIV'.
	self assert: 1265 printStringRoman = 'MCCLXV'.
	self assert: 1360 printStringRoman = 'MCCCLX'.
	self assert: 1361 printStringRoman = 'MCCCLXI'.
	self assert: 1362 printStringRoman = 'MCCCLXII'.
	self assert: 1457 printStringRoman = 'MCDLVII'.
	self assert: 1458 printStringRoman = 'MCDLVIII'.
	self assert: 1459 printStringRoman = 'MCDLIX'.
	self assert: 1554 printStringRoman = 'MDLIV'.
	self assert: 1555 printStringRoman = 'MDLV'.
	self assert: 1556 printStringRoman = 'MDLVI'.
	self assert: 1651 printStringRoman = 'MDCLI'.
	self assert: 1652 printStringRoman = 'MDCLII'.
	self assert: 1653 printStringRoman = 'MDCLIII'.
	self assert: 1748 printStringRoman = 'MDCCXLVIII'.
	self assert: 1749 printStringRoman = 'MDCCXLIX'.
	self assert: 1750 printStringRoman = 'MDCCL'.
	self assert: 1845 printStringRoman = 'MDCCCXLV'.
	self assert: 1846 printStringRoman = 'MDCCCXLVI'.
	self assert: 1847 printStringRoman = 'MDCCCXLVII'.
	self assert: 1942 printStringRoman = 'MCMXLII'.
	self assert: 1943 printStringRoman = 'MCMXLIII'.
	self assert: 1944 printStringRoman = 'MCMXLIV'.
	self assert: 2004 printStringRoman = 'MMIV'.

	self assert: -1 printStringRoman = '-I'.
	self assert: -2 printStringRoman = '-II'.
	self assert: -3 printStringRoman = '-III'.
	self assert: -4 printStringRoman = '-IV'.
	self assert: -5 printStringRoman = '-V'.
	self assert: -6 printStringRoman = '-VI'.
	self assert: -7 printStringRoman = '-VII'.
	self assert: -8 printStringRoman = '-VIII'.
	self assert: -9 printStringRoman = '-IX'.
	self assert: -10 printStringRoman = '-X'.
! !

!IntegerTest methodsFor: 'tests - instance creation' stamp: 'dtl 9/18/2004 17:07'!
testStringAsNumber
	"This covers parsing in Number>>readFrom:
	Trailing decimal points should be ignored."

	self assert: ('123' asNumber == 123).
	self assert: ('-123' asNumber == -123).
	self assert: ('123.' asNumber == 123).
	self assert: ('-123.' asNumber == -123).
	self assert: ('123This is not to be read' asNumber == 123).
	self assert: ('123s could be confused with a ScaledDecimal' asNumber == 123).
	self assert: ('123e could be confused with a Float' asNumber == 123).
! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'nice 1/26/2008 02:22'!
testTwoComplementBitLogicWithCarry
	"This is non regression test for http://bugs.squeak.org/view.php?id=6874"
	
	"By property of two complement, following operation is:
	...111110000 this is -16
	...111101111 this is -16-1
	...111100000 this is -32, the result of bitAnd: on two complement
	
	This test used to fail with n=31 39 47.... because of bug 6874"
	
	self assert: ((2 to: 80) allSatisfy: [:n | ((2 raisedTo: n) negated bitAnd: (2 raisedTo: n) negated - 1) = (2 raisedTo: n + 1) negated]).! !

!IntegerTest methodsFor: 'tests - bitLogic' stamp: 'sd 6/5/2005 08:44'!
testTwoComplementRightShift
	"self run: #testTwoComplementRightShift"

	| large small |
	small := 2 << 16.
	large := 2 << 32.	
	self assert: ((small negated bitShift: -1) ~= ((small + 1) negated bitShift: -1)
		== ((large negated bitShift: -1) ~= ((large + 1) negated bitShift: -1))).
		
     self assert: ((small bitShift: -1) ~= (small + 1 bitShift: -1)
		== ((large bitShift: -1) ~= (large + 1 bitShift: -1))).! !

TestCase subclass: #MethodContextTest
	instanceVariableNames: 'aCompiledMethod aReceiver aMethodContext aSender'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!MethodContextTest commentStamp: 'tlk 5/31/2004 16:07' prior: 0!
I am an SUnit Test of MethodContext and its super type, ContextPart. See also BlockContextTest.
See pages 430-437 of A. Goldberg and D. Robson's  Smalltalk-80 The Language (aka the purple book), which deal with Contexts. My fixtures are from their example. (The Squeak byte codes are not quite the same as Smalltalk-80.)
My fixtures are:
aReceiver         - just some arbitrary object, "Rectangle origin: 100@100 corner: 200@200"
aSender           - just some arbitrary object, thisContext
aCompiledMethod - just some arbitrary method, "Rectangle rightCenter".
aMethodContext   - just some arbitray context ...  

!

!MethodContextTest methodsFor: 'running' stamp: 'tlk 5/31/2004 16:18'!
setUp
	super setUp.
	aCompiledMethod := Rectangle methodDict at: #rightCenter.
	aReceiver := 100@100 corner: 200@200.
	aSender := thisContext.
	aMethodContext := MethodContext sender: aSender receiver: aReceiver method: aCompiledMethod arguments: #(). ! !

!MethodContextTest methodsFor: 'tests' stamp: 'tlk 5/30/2004 13:35'!
testActivateReturnValue
	self assert:  ((aSender activateReturn: aMethodContext value: #()) isKindOf: MethodContext).
	self assert:  ((aSender activateReturn: aMethodContext value: #()) receiver = aMethodContext).! !

!MethodContextTest methodsFor: 'tests' stamp: 'tlk 5/31/2004 17:10'!
testFindContextSuchThat
	self assert: (aMethodContext findContextSuchThat: [:each| true]) printString = aMethodContext printString.
	self assert: (aMethodContext hasContext: aMethodContext). ! !

!MethodContextTest methodsFor: 'tests' stamp: 'tlk 5/30/2004 10:57'!
testMethodContext
	self deny: aMethodContext isPseudoContext.
	self assert: aMethodContext home notNil.
	self assert: aMethodContext receiver notNil.
	self assert: (aMethodContext method isKindOf: CompiledMethod).! !

!MethodContextTest methodsFor: 'tests' stamp: 'tlk 5/31/2004 17:08'!
testMethodIsBottomContext
	self assert: aMethodContext bottomContext = aSender.
	self assert: aMethodContext secondFromBottom = aMethodContext.! !

!MethodContextTest methodsFor: 'tests' stamp: 'tlk 5/31/2004 16:55'!
testReturn
	"Why am I overriding setUp? Because sender must be thisContext, i.e, testReturn, not setUp."
	aMethodContext := MethodContext sender: thisContext receiver: aReceiver method: aCompiledMethod arguments: #(). 
	self assert: (aMethodContext return: 5) = 5.! !

!MethodContextTest methodsFor: 'tests' stamp: 'pmm 7/4/2009 15:45'!
testSetUp
	"Note: In addition to verifying that the setUp worked the way it was expected to, testSetUp is used to illustrate the meaning of the simple access methods, methods that are not normally otherwise 'tested'"
	self deny: aMethodContext isPseudoContext.
	self deny: aMethodContext isDead.
	self assert: aMethodContext home = aMethodContext.
	self assert: aMethodContext receiver = aReceiver.
	self assert: (aMethodContext method isKindOf: CompiledMethod).
	self assert: aMethodContext method = aCompiledMethod.
	self assert: aMethodContext methodNode selector = #rightCenter.
	self assert: aMethodContext client printString = 'MethodContextTest>>#testSetUp'.
! !

!MethodContextTest methodsFor: 'tests' stamp: 'md 2/9/2007 19:06'!
testTempNamed
	| oneTemp |
	
	oneTemp := 1.
	self assert: (thisContext tempNamed: 'oneTemp') = oneTemp.
	! !

!MethodContextTest methodsFor: 'tests' stamp: 'md 2/9/2007 19:08'!
testTempNamedPut
	| oneTemp |
	
	oneTemp := 1.
	self assert: (thisContext tempNamed: 'oneTemp') = oneTemp.
	thisContext tempNamed: 'oneTemp' put: 2.
	self assert: (thisContext tempNamed: 'oneTemp') = 2.! !

TestCase subclass: #MethodDictionaryTest
	uses: TIncludesTest + TDictionaryKeysValuesAssociationsAccess + TDictionaryRemovingTest
	instanceVariableNames: 'nonEmptyDict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

MethodDictionaryTest class
	uses: TIncludesTest classTrait + TDictionaryKeysValuesAssociationsAccess classTrait + TDictionaryRemovingTest classTrait
	instanceVariableNames: ''!

!MethodDictionaryTest methodsFor: 'helpers' stamp: 'Alexandre Bergel 12/25/2009 08:21'!
aValue

	^ self class >> #aValue! !

!MethodDictionaryTest methodsFor: 'helpers' stamp: 'Alexandre Bergel 12/25/2009 08:20'!
anIndex

	^ #aMethodName! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:29'!
anotherElementNotIn
" return an element different of 'elementNotIn'  not included in 'nonEmpty' "
	^ self class >> #anotherElementNotIn! !

!MethodDictionaryTest methodsFor: 'helpers' stamp: 'Alexandre Bergel 12/25/2009 08:21'!
anotherValue

	^ self class >> #anotherValue! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:29'!
elementNotIn
"return an element not included in 'nonEmpty' "

	^ self class >> #elementNotIn! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:25'!
empty
	^ MethodDictionary new! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:31'!
emptyDict
	^ self empty! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'TestRunner 12/25/2009 08:35'!
keyNotIn
" return a key not included in nonEmpty"
^ #bouba! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'TestRunner 12/25/2009 08:37'!
keyNotInNonEmptyDict
" return a key not included in nonEmptyDict"
	^ #keyNotInNonEmptyDict! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:31'!
newEmptyDict
	^ MethodDictionary new! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:27'!
nonEmpty
	^ nonEmptyDict! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'TestRunner 12/25/2009 08:30'!
nonEmptyDict
	^ nonEmptyDict ! !

!MethodDictionaryTest methodsFor: 'requirements' stamp: 'Alexandre Bergel 12/25/2009 08:32'!
nonEmptyDifferentFromNonEmptyDict
	" return a dictionary for which all keys are not included in nonEmptyDict"
	^ MethodDictionary new
		at: #nonEmptyDifferentFromNonEmptyDict put: (self class >> #nonEmptyDifferentFromNonEmptyDict)! !

!MethodDictionaryTest methodsFor: 'running' stamp: 'Alexandre Bergel 12/25/2009 08:28'!
setUp
	super setUp.
	nonEmptyDict := MethodDictionary new.
	nonEmptyDict at: #setUp put: (self class >> #setUp).
	nonEmptyDict at: #nonEmpty put: (self class >> #nonEmpty).! !

!MethodDictionaryTest methodsFor: 'requirement' stamp: 'Alexandre Bergel 12/25/2009 08:22'!
speciesClass
	^ MethodDictionary! !

!MethodDictionaryTest methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 2/21/2010 12:52'!
testBehaviorLocalyDefined
	"method not acquired from a trait or from its superclass"
	" self debug: #testBehavior "
	self assert: (Object methodDict isKindOf: MethodDictionary).
	self assert: (Object selectors asSortedCollection = Object selectors asSortedCollection).! !

!MethodDictionaryTest methodsFor: 'as yet unclassified' stamp: 'TestRunner 12/25/2009 08:23'!
testIncludesKeyLocalyDefined
	" self debug: #testIncludesKeyLocalyDefined "
	self assert: (self class methodDictionary includesKey: #testIncludesKeyLocalyDefined).! !

!MethodDictionaryTest methodsFor: 'as yet unclassified' stamp: 'TestRunner 12/25/2009 08:23'!
testKeyAtValueIfAbsentLocalyDefined
	" self debug: #testKeyAtValueIfAbsentLocalyDefined "
	self assert: (self class methodDict keyAtValue: (self class >> #testKeyAtValueIfAbsentLocalyDefined) ifAbsent: []) == #testKeyAtValueIfAbsentLocalyDefined.
	
	self assert: (self class methodDict keyAtValue: (Object >> #printOn:) ifAbsent: [#notFound]) == #notFound! !

TestCase subclass: #MethodPragmaTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!MethodPragmaTest methodsFor: 'utilities' stamp: 'md 2/18/2006 19:39'!
assertPragma: aString givesKeyword: aSymbol arguments: anArray
	| pragma decompiled |
	pragma := self pragma: aString selector: #zork.
	self assert: pragma keyword = aSymbol.
	self assert: pragma arguments = anArray.
	decompiled := (self class>>#zork) decompile.
	self assert: (decompiled properties pragmas includes: pragma). 
	self assert: (decompiled asString includesSubString: pragma asString).! !

!MethodPragmaTest methodsFor: 'utilities' stamp: 'lr 1/20/2006 02:23'!
compile: aString selector: aSelector
	self class 
		compileSilently: aSelector , String lf , aString
		classified: self methodCategory.
	^ self class >> aSelector.! !

!MethodPragmaTest methodsFor: 'utilities' stamp: 'lr 1/20/2006 11:50'!
methodCategory
	^ #generated! !

!MethodPragmaTest methodsFor: 'utilities' stamp: 'lr 2/6/2006 20:48'!
pragma: aString selector: aSelector
	^ (self compile: '<' , aString , '>' selector: aSelector)
		pragmas first.! !

!MethodPragmaTest methodsFor: 'utilities' stamp: 'lr 1/20/2006 08:11'!
pragma: aSymbol selector: aSelector times: anInteger
	^ (self 
		compile: (String streamContents: [ :stream | 
			(1 to: anInteger) asArray shuffled do: [ :each | 
				stream 
					nextPut: $<; nextPutAll: aSymbol; space;
					print: each; nextPut: $>; cr ] ])
		selector: aSelector)
			pragmas.! !

!MethodPragmaTest methodsFor: 'running' stamp: 'lr 1/20/2006 02:15'!
tearDown
	(self class organization listAtCategoryNamed: self methodCategory)
		do: [ :each | self class removeSelectorSilently: each ].
	self class organization removeCategory: self methodCategory.! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/20/2006 08:18'!
testAllNamedFromTo
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma allNamed: #foo: from: self class to: Object.
	self assert: pragmasDetected = pragmasCompiled.
	
	pragmasDetected := Pragma allNamed: #foo: from: Object to: Object.
	self assert: pragmasDetected isEmpty.! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/20/2006 08:17'!
testAllNamedFromToSortedByArgument
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma allNamed: #foo: from: self class to: Object sortedByArgument: 1.
	self assert: pragmasDetected = (pragmasCompiled 
		sort: [ :a :b | (a argumentAt: 1) < (b argumentAt: 1) ])! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/20/2006 08:17'!
testAllNamedFromToSortedUsing
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma 
		allNamed: #foo: from: self class to: Object 
		sortedUsing: [ :a :b | (a argumentAt: 1) > (b argumentAt: 1) ].
	self assert: pragmasDetected = (pragmasCompiled 
		sort: [ :a :b | (a argumentAt: 1) > (b argumentAt: 1) ]).! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/20/2006 08:19'!
testAllNamedIn
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma allNamed: #foo: in: self class.
	self assert: pragmasDetected = pragmasCompiled.
	
	pragmasDetected := Pragma allNamed: #foo: in: Object.
	self assert: pragmasDetected isEmpty.! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/20/2006 08:15'!
testAllNamedInSortedByArgument
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma allNamed: #foo: in: self class sortedByArgument: 1.
	self assert: pragmasDetected = (pragmasCompiled 
		sort: [ :a :b | (a argumentAt: 1) < (b argumentAt: 1) ])! !

!MethodPragmaTest methodsFor: 'testing-finding' stamp: 'lr 1/21/2006 13:01'!
testAllNamedInSortedUsing
	| pragmasCompiled pragmasDetected |
	pragmasCompiled := self pragma: #foo: selector: #bar times: 5.
	pragmasDetected := Pragma 
		allNamed: #foo: in: self class 
		sortedUsing: [ :a :b | (a argumentAt: 1) > (b argumentAt: 1) ].
	self assert: pragmasDetected = (pragmasCompiled 
		sort: [ :a :b | (a argumentAt: 1) > (b argumentAt: 1) ]).! !

!MethodPragmaTest methodsFor: 'testing' stamp: 'lr 3/19/2007 11:40'!
testArgumentAt
	| pragma |
	pragma := Pragma keyword: #value:value:value: arguments: #( 3 2 1 ).
	self assert: (pragma argumentAt: 1) = 3.
	self assert: (pragma argumentAt: 2) = 2.
	self assert: (pragma argumentAt: 3) = 1! !

!MethodPragmaTest methodsFor: 'testing-pragma' stamp: 'lr 1/20/2006 00:35'!
testArguments
	| pragma |
	pragma := Pragma keyword: #foo: arguments: #( 123 ).
	self assert: pragma arguments = #( 123 ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileArray
	self assertPragma: 'foo: #()' givesKeyword: #foo: arguments: #( () ).
	self assertPragma: 'foo: #( foo )' givesKeyword: #foo: arguments: #( ( foo ) ).
	self assertPragma: 'foo: #( foo: )' givesKeyword: #foo: arguments: #( ( foo: ) ).
	self assertPragma: 'foo: #( 12 )' givesKeyword: #foo: arguments: #( ( 12 ) ).
	self assertPragma: 'foo: #( true )' givesKeyword: #foo: arguments: #( ( true ) ).
	! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 8/19/2006 20:44'!
testCompileBinary
	self assertPragma: ' = 1' givesKeyword: #= arguments: #( 1 ).
	self assertPragma: ' , 3' givesKeyword: #, arguments: #( 3 ).
	self assertPragma: ' > 4' givesKeyword: #> arguments: #( 4 ).
	self assertPragma: ' < 5' givesKeyword: #< arguments: #( 5 ).

	self assertPragma: ' == 1' givesKeyword: #== arguments: #( 1 ).
	self assertPragma: ' <> 3' givesKeyword: #<> arguments: #( 3 ).
	self assertPragma: ' >< 4' givesKeyword: #>< arguments: #( 4 ).
	self assertPragma: ' ** 5' givesKeyword: #** arguments: #( 5 )! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileCharacter
	self assertPragma: 'foo: $a' givesKeyword: #foo: arguments: #( $a ).
	self assertPragma: 'foo: $ ' givesKeyword: #foo: arguments: #( $  ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileEmpty
	self assertPragma: 'foo' givesKeyword: #foo arguments: #().! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 07:39'!
testCompileFull
	self assertPragma: 'foo: 1' givesKeyword: #foo: arguments: #( 1 ).
	self assertPragma: 'foo: 1 bar: 2' givesKeyword: #foo:bar: arguments: #( 1 2 ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 10/5/2006 10:15'!
testCompileInvalid
	"Invalid pragmas should properly raise an error."

	self should: [ self compile: '<>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<1>' selector: #zork ] raise: SyntaxErrorNotification.	
	self should: [ self compile: '<#123>' selector: #zork ] raise: SyntaxErrorNotification.
	
	self should: [ self compile: '<foo bar>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<foo 1>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<foo bar zork>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<foo bar 1>' selector: #zork ] raise: SyntaxErrorNotification.
	
	self should: [ self compile: '<foo: bar:>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<foo: #bar: zork:>' selector: #zork ] raise: SyntaxErrorNotification.
	
	self should: [ self compile: '<<1>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<=2>' selector: #zork ] raise: SyntaxErrorNotification.

	self should: [ self compile: '< =1 = >' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '< =1 =2 >' selector: #zork ] raise: SyntaxErrorNotification.
	
	self should: [ self compile: '<foo: String>' selector: #zork ] raise: SyntaxErrorNotification.
	self should: [ self compile: '<foo: Pragma>' selector: #zork ] raise: SyntaxErrorNotification! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileNumber
	self assertPragma: 'foo: 123' givesKeyword: #foo: arguments: #( 123 ).
	self assertPragma: 'foo: -123' givesKeyword: #foo: arguments: #( -123 ).
	self assertPragma: 'foo: 12.3' givesKeyword: #foo: arguments: #( 12.3 ).
	self assertPragma: 'foo: -12.3' givesKeyword: #foo: arguments: #( -12.3 ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileString
	self assertPragma: 'foo: ''''' givesKeyword: #foo: arguments: #( '' ).
	self assertPragma: 'foo: ''bar''' givesKeyword: #foo: arguments: #( 'bar' ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 1/20/2006 02:25'!
testCompileSymbol
	self assertPragma: 'foo: #bar' givesKeyword: #foo: arguments: #( bar ).
	self assertPragma: 'foo: #bar:' givesKeyword: #foo: arguments: #( bar: ).
	self assertPragma: 'foo: #bar:zork:' givesKeyword: #foo: arguments: #( bar:zork: ).! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 7/3/2006 15:00'!
testCompileTemps
	"Pragmas should be placeable before and after temps."
	
	self 
		shouldnt: [
			self assert: (self compile: '| temps | <foo>' selector: #zork) 
				pragmas notEmpty ]
		raise: SyntaxErrorNotification.
	self 
		shouldnt: [
			self assert: (self compile: '<foo> | temps |' selector: #zork) 
				pragmas notEmpty ]
		raise: SyntaxErrorNotification.! !

!MethodPragmaTest methodsFor: 'testing-compiler' stamp: 'lr 10/5/2006 09:49'!
testCompileValue
	self assertPragma: 'foo: true' givesKeyword: #foo: arguments: #( true ).
	self assertPragma: 'foo: false' givesKeyword: #foo: arguments: #( false ).
	self assertPragma: 'foo: nil' givesKeyword: #foo: arguments: #( nil )! !

!MethodPragmaTest methodsFor: 'testing-pragma' stamp: 'lr 1/20/2006 00:35'!
testKeyword
	| pragma |
	pragma := Pragma keyword: #foo: arguments: #( 123 ).
	self assert: pragma keyword = #foo:.! !

!MethodPragmaTest methodsFor: 'testing-pragma' stamp: 'lr 1/20/2006 00:36'!
testMessage
	| pragma message |
	pragma := Pragma keyword: #foo: arguments: #( 123 ).
	message := pragma message.
	
	self assert: message selector = #foo:.
	self assert: message arguments = #( 123 ).! !

!MethodPragmaTest methodsFor: 'testing-method' stamp: 'lr 1/20/2006 07:54'!
testMethod
	| pragma |
	pragma := self pragma: 'foo' selector: #bar.
	self assert: pragma method == (self class >> #bar).! !

!MethodPragmaTest methodsFor: 'testing-method' stamp: 'md 2/18/2006 19:59'!
testMethodClass
	| pragma |
	pragma := self pragma: 'foo' selector: #bar.
	self assert: pragma methodClass == self class.! !

!MethodPragmaTest methodsFor: 'testing-compiled' stamp: 'lr 2/6/2006 21:03'!
testNoPragma
	| method |
	method := self compile: '' selector: #foo.
	self assert: method pragmas = #().! !

!MethodPragmaTest methodsFor: 'testing' stamp: 'lr 3/19/2007 11:42'!
testNumArgs
	| pragma |
	pragma := Pragma keyword: #value arguments: #().
	self assert: pragma numArgs = 0.
	
	pragma := Pragma keyword: #+ arguments: #( 1 ).
	self assert: pragma numArgs = 1.
	
	pragma := Pragma keyword: #value:value: arguments: #( 1 2 ).
	self assert: pragma numArgs = 2! !

!MethodPragmaTest methodsFor: 'testing-primitives' stamp: 'lr 1/20/2006 02:31'!
testPrimitiveIndexed1
	"This test useses the #instVarAt: primitive."
	
	self compile: '<primitive: 74> ^ #inst' selector: #inst.
	self assert: self inst = #inst.! !

!MethodPragmaTest methodsFor: 'testing-primitives' stamp: 'MartinMcClure 3/18/2010 21:55'!
testPrimitiveIndexed2
	"This test useses the #identityHash primitive."

	self compile: '<primitive: 75> ^ #idHash' selector: #idHash.
	self assert: self idHash = self basicIdentityHash.! !

!MethodPragmaTest methodsFor: 'testing-primitives' stamp: 'lr 1/20/2006 02:42'!
testPrimitiveNamed1
	"This test useses the #primitiveDirectoryLookup primitive."

	self compile: '<primitive: ''primitiveDirectoryLookup'' module: ''FilePlugin''> ^ #lookup' selector: #lookup.
	self assert: self lookup = #lookup.
	
! !

!MethodPragmaTest methodsFor: 'testing-primitives' stamp: 'lr 1/20/2006 02:41'!
testPrimitiveNamed2
	"This test useses the #primPathNameDelimiter primitive."

	self compile: '<primitive: ''primitiveDirectoryDelimitor'' module: ''FilePlugin''> ^ #delim' selector: #delim.
	self assert: self delim = FileDirectory primPathNameDelimiter.
	
! !

!MethodPragmaTest methodsFor: 'testing-printing-reformating' stamp: 'md 2/3/2007 11:55'!
testReformat


	self assert: (DisplayScreen class compiledMethodAt: #actualScreenDepth) getSource string = 'actualScreenDepth
	<primitive: ''primitiveScreenDepth''>
	^ Display depth'.
	

	self shouldnt: [ DisplayScreen class reformatMethodAt: #actualScreenDepth] raise: Error.

	self assert: (DisplayScreen class compiledMethodAt: #actualScreenDepth) getSource string = 'actualScreenDepth
	<primitive: ''primitiveScreenDepth''>
	^ Display depth'.	
! !

!MethodPragmaTest methodsFor: 'testing-method' stamp: 'lr 1/20/2006 07:54'!
testSelector
	| pragma |
	pragma := self pragma: 'foo' selector: #bar.
	self assert: pragma selector == #bar.! !

!MethodPragmaTest methodsFor: 'testing' stamp: 'lr 3/19/2007 11:38'!
testSendTo
	| pragma wasHere |
	pragma := Pragma keyword: #value:value: arguments: #( 1 2 ).
	self assert: (pragma sendTo: [ :a :b |
		self assert: a = 1; assert: b = 2.
		wasHere := true ]).
	self assert: wasHere! !

!MethodPragmaTest methodsFor: 'testing' stamp: 'lr 3/19/2007 11:38'!
testWithArgumentsDo
	| pragma wasHere |
	pragma := Pragma keyword: #add:after: arguments: #( 1 2 ).
	self assert: (pragma withArgumentsDo: [ :a :b |
		self assert: a = 1; assert: b = 2.
		wasHere := true ]).
	self assert: wasHere! !

TestCase subclass: #MethodPropertiesTest
	instanceVariableNames: 'method'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!MethodPropertiesTest methodsFor: 'running' stamp: 'MikeRoberts 9/26/2009 16:56'!
expectedFailures
	"The new closure compiler does not use MethodProperties.  These tests end up performing
	their assertions on AdditionalMethodState."
	
	^#(testAt testAtIfAbsent testIncludesKey testRemoveKey testRemoveKeyifAbsent)! !

!MethodPropertiesTest methodsFor: 'private' stamp: 'lr 2/6/2006 20:43'!
propertyDictionaryFor: aMethod
	^ aMethod properties instVarNamed: 'properties'.! !

!MethodPropertiesTest methodsFor: 'running' stamp: 'lr 1/20/2006 19:16'!
setUp
	method := Object >> #halt.! !

!MethodPropertiesTest methodsFor: 'running' stamp: 'lr 1/20/2006 19:20'!
tearDown
	Object recompile: #halt from: Object.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'kwl 7/26/2006 11:41'!
testAllMethodsHaveMethodClass
	Smalltalk garbageCollect.
	self assert: (CompiledMethod allInstances
			reject: [:cm | cm literals last isVariableBinding
					and: [cm literals last value isBehavior
							or: [cm literals last value isTrait]]]) isEmpty
			description: 'CompiledMethods must have methodClass literal'! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'kwl 7/26/2006 11:40'!
testAllMethodsHaveNewPropertyFormat
	Smalltalk garbageCollect.
	self assert: (CompiledMethod allInstances
			reject: [:cm | cm hasNewPropertyFormat]) isEmpty
		description: 'CompiledMethods must have new property format'! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'StephaneDucasse 8/30/2009 16:50'!
testAnalogousCodeTo
	"self debug: #testAnalogousCodeTo"
	
	method properties at: #zork put: 'hello'.
	self assert: (method = method).
	! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:21'!
testAt
	self should: [ method properties at: #zork ] raise: Error.
	self assert: (self propertyDictionaryFor: method) isNil.
	method properties at: #zork put: 'hello'.
	self assert: (method properties at: #zork) = 'hello'.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:18'!
testAtIfAbsent
	self assert: (method properties at: #zork ifAbsent: [ 'hello' ]) = 'hello'.
	self assert: (self propertyDictionaryFor: method) isNil.
	method properties at: #zork put: 'hi'.
	self assert: (method properties at: #zork ifAbsent: [ 'hello' ]) = 'hi'.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:18'!
testAtIfAbsentPut
	self assert: (method properties at: #zork ifAbsentPut: [ 'hello' ]) = 'hello'.
	self assert: (method properties at: #zork ifAbsentPut: [ 'hi' ]) = 'hello'.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:18'!
testAtPut
	self assert: (method properties at: #zork put: 'hello') = 'hello'.
	self assert: (method properties at: #zork) = 'hello'.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'Alexandre Bergel 12/21/2009 23:41'!
testAtPutRepeatedly
	"Thanks to Andreas Raab"
	self assert: (method properties at: #zork put: 'hello') = 'hello'.
	self assert: (method properties at: #zork put: 'hello') = 'hello'.
	self assert: (method properties at: #zork) = 'hello'.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:19'!
testIncludesKey
	self deny: (method properties includesKey: #zork).
	self assert: (self propertyDictionaryFor: method) isNil.
	method properties at: #zork put: 123.
	self assert: (method properties includesKey: #zork).! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:18'!
testRemoveKey
	method properties at: #zork put: 'hello'.
	self should: [ method properties removeKey: #halt ] raise: Error.
	self assert: (method properties removeKey: #zork) = 'hello'.
	self assert: (self propertyDictionaryFor: method) isNil.
	self should: [ method properties removeKey: #zork ] raise: Error.
	self assert: (self propertyDictionaryFor: method) isNil.! !

!MethodPropertiesTest methodsFor: 'testing' stamp: 'lr 2/6/2006 22:18'!
testRemoveKeyifAbsent
	method properties at: #zork put: 'hello'.
	self assert: (method properties removeKey: #halt ifAbsent: [ 'hi' ]) = 'hi'.
	self assert: (method properties removeKey: #zork ifAbsent: [ 'hi' ]) = 'hello'.
	self assert: (self propertyDictionaryFor: method) isNil.
	self should: (method properties removeKey: #zork ifAbsent: [ 'hi' ]) = 'hi'.
	self assert: (self propertyDictionaryFor: method) isNil.! !

TestCase subclass: #MonitorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!MonitorTest methodsFor: 'examples' stamp: 'md 3/19/2006 21:15'!
testExample1

	| producer1 producer2  monitor goal work counter goalReached finished |
	goal := (1 to: 1000) asOrderedCollection.
	work := OrderedCollection new.
	counter := 0.
	goalReached := false.
	finished := Semaphore new.
	monitor := Monitor new.

	producer1 := [
       [monitor critical:
             [monitor waitUntil: [counter \\5 = 0].
              goalReached or: [work add: (counter := counter + 1)].
              goalReached := counter >= goal size.
              monitor signal
            ].
           goalReached
          ]
             whileFalse.
         finished signal.
	].

	producer2 := [
         [monitor critical:
                [monitor waitWhile: [counter \\5 = 0].
                 goalReached or: [work add: (counter := counter + 1)].
                 goalReached := counter >= goal size.
                 monitor signal].
         goalReached
       ] whileFalse.
     finished signal
	].

	producer1 forkAt: Processor userBackgroundPriority.
	producer2 forkAt: Processor userBackgroundPriority.

	finished wait; wait.
	self assert: goal = work! !

!MonitorTest methodsFor: 'examples' stamp: 'md 3/19/2006 21:19'!
testExample2
	"Here is a second version that does not use a semaphore to inform the 
	forking process about termination of both forked processes"

	| producer1 producer2  monitor goal work counter goalReached activeProducers|
	goal := (1 to: 1000) asOrderedCollection.
	work := OrderedCollection new.
	counter := 0.
	goalReached := false.
	activeProducers := 0.
	monitor := Monitor new.

  producer1 :=
      [ monitor critical: [activeProducers := activeProducers + 1].
  [monitor critical:
            [monitor waitUntil: [counter \\5 = 0].
      goalReached or: [work add: (counter := counter + 1)].
     " Transcript show: 'P1  '; show: counter printString; show: '  ';
       show: activeProducers printString; cr."
      goalReached := counter >= goal size.
      monitor signal
            ].
           goalReached
          ]
             whileFalse.
         monitor critical: [activeProducers := activeProducers - 1.
        monitor signal: #finish].
 ] .

 producer2 :=
    [monitor critical: [activeProducers := activeProducers + 1].

  [monitor critical:
          [monitor waitWhile: [counter \\5 = 0].
    goalReached or: [work add: (counter := counter + 1)].
    goalReached := counter >= goal size.
    monitor signal].
         goalReached ] whileFalse.
     monitor critical: [
		activeProducers := activeProducers - 1. 
		monitor signal: #finish].
	].

	producer1 forkAt: Processor userBackgroundPriority.
	producer2  forkAt: Processor userBackgroundPriority.


	monitor critical: [
		monitor waitUntil: [activeProducers = 0 & (goalReached)]
				for: #finish.
  	].

	self assert: goal = work
! !

TestCase subclass: #NumberParsingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Numbers'!

!NumberParsingTest commentStamp: 'dtl 11/24/2004 15:35' prior: 0!
Tests to verify parsing of numbers from streams and strings.

Note: ScaledDecimalTest contains related tests for parsing ScaledDecimal.!

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'dtl 11/24/2004 14:29'!
testFloatFromStreamAsNumber
	"This covers parsing in Number>>readFrom:"

	| rs aFloat |
	rs := '10r-12.3456' readStream.
	aFloat := Number readFrom: rs.
	self assert: -12.3456 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456e2' readStream.
	aFloat := Number readFrom: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456e2e2' readStream.
	aFloat := Number readFrom: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs upToEnd = 'e2'.

	rs := '10r-12.3456d2' readStream.
	aFloat := Number readFrom: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '10r-12.3456q2' readStream.
	aFloat := Number readFrom: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '-12.3456q2' readStream.
	aFloat := Number readFrom: rs.
	self assert: -1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '12.3456q2' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1234.56 = aFloat.
	self assert: rs atEnd.

	rs := '12.3456z2' readStream.
	aFloat := Number readFrom: rs.
	self assert: 12.3456 = aFloat.
	self assert: rs upToEnd = 'z2'.
! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'nice 4/28/2006 01:20'!
testFloatFromStreamWithExponent
	"This covers parsing in Number>>readFrom:"

	| rs aFloat |
	rs := '1.0e-14' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs atEnd.

	rs := '1.0e-14 1' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs upToEnd = ' 1'.

	rs := '1.0e-14eee' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1.0e-14 = aFloat.
	self assert: rs upToEnd = 'eee'.

	rs := '1.0e14e10' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1.0e14 = aFloat.
	self assert: rs upToEnd = 'e10'.

	rs := '1.0e+14e' readStream. "Plus sign is not parseable"
	aFloat := Number readFrom: rs.
	self assert: 1.0 = aFloat.
	self assert: rs upToEnd = 'e+14e'.

	rs := '1.0e' readStream.
	aFloat := Number readFrom: rs.
	self assert: 1.0 = aFloat.
	self assert: rs upToEnd = 'e'.! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'dtl 11/24/2004 14:07'!
testFloatFromStringAsNumber
	"This covers parsing in Number>>readFrom:"

	| aFloat |
	aFloat := '10r-12.3456' asNumber.
	self assert: -12.3456 = aFloat.
	aFloat := '10r-12.3456e2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '10r-12.3456d2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '10r-12.3456q2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '-12.3456q2' asNumber.
	self assert: -1234.56 = aFloat.
	aFloat := '12.3456q2' asNumber.
	self assert: 1234.56 = aFloat.
! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'dtl 11/24/2004 14:12'!
testFloatFromStringWithExponent
	"This covers parsing in Number>>readFrom:"

	| aFloat |
	aFloat := '1.0e-14' asNumber.
	self assert: 1.0e-14 = aFloat.
	aFloat := '1.0e-14 1' asNumber.
	self assert: 1.0e-14 = aFloat.
	aFloat := '1.0e-14e' asNumber.
	self assert: 1.0e-14 = aFloat.
	aFloat := '1.0e14e' asNumber.
	self assert: 1.0e14 = aFloat.
	aFloat := '1.0e+14e' asNumber. "Plus sign is not parseable"
	self assert: 1.0 = aFloat.
! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'damiencassou 5/30/2008 11:09'!
testFloatReadWithRadix
	"This covers parsing in Number>>readFrom:
	Note: In most Smalltalk dialects, the radix notation is not used for numbers
	with exponents. In Squeak, a string with radix and exponent can be parsed,
	and the exponent is always treated as base 10 (not the base indicated in the
	radix prefix). I am not sure if this is a feature, a bug, or both, but the
	Squeak behavior is documented in this test. -dtl"
	| aNumber rs |
	aNumber := '2r1.0101e9' asNumber.
	self assert: 672.0 = aNumber.
	self assert: (Number readFrom: '2r1.0101e9') = (1.3125 * (2 raisedTo: 9)).
	rs := '2r1.0101e9e9' readStream.
	self assert: (Number readFrom: rs) = 672.0.
	self assert: rs upToEnd = 'e9'! !

!NumberParsingTest methodsFor: 'tests - Integer' stamp: 'dtl 11/24/2004 14:05'!
testIntegerFromString
	"This covers parsing in Number>>readFrom:
	Trailing decimal points should be ignored."

	self assert: ('123' asNumber == 123).
	self assert: ('-123' asNumber == -123).
	self assert: ('123.' asNumber == 123).
	self assert: ('-123.' asNumber == -123).
	self assert: ('123This is not to be read' asNumber == 123).
	self assert: ('123s could be confused with a ScaledDecimal' asNumber == 123).
	self assert: ('123e could be confused with a Float' asNumber == 123).
! !

!NumberParsingTest methodsFor: 'tests - Integer' stamp: 'damiencassou 5/30/2008 11:09'!
testIntegerReadFrom
	"Ensure remaining characters in a stream are not lost when parsing an integer."
	| rs i s |
	rs := '123s could be confused with a ScaledDecimal' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: 's could be confused with a ScaledDecimal' = s.
	rs := '123.s could be confused with a ScaledDecimal' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: '.s could be confused with a ScaledDecimal' = s.
	rs := '123sA has unary message sA' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: 'sA has unary message sA' = s.
	rs := '123sB has unary message sB' readStream.
	i := Number readFrom: rs.
	self assert: i == 123.
	s := rs upToEnd.
	self assert: 'sB has unary message sB' = s! !

!NumberParsingTest methodsFor: 'tests - Integer' stamp: 'dtl 11/24/2004 18:18'!
testIntegerReadWithRadix
	"This covers parsing in Number>>readFrom:
	Note: In most Smalltalk dialects, the radix notation is not used for numbers
	with exponents. In Squeak, a string with radix and exponent can be parsed,
	and the exponent is always treated as base 10 (not the base indicated in the
	radix prefix). I am not sure if this is a feature, a bug, or both, but the
	Squeak behavior is documented in this test. -dtl"

	| aNumber rs |
	aNumber := '2r1e26' asNumber.
	self assert: 67108864 = aNumber.
	self assert: (Number readFrom: '2r1e26') = (2 raisedTo: 26).
	rs := '2r1e26eee' readStream.
	self assert: (Number readFrom: rs) = 67108864.
	self assert: rs upToEnd = 'eee'
! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'hfm 4/2/2010 13:56'!
testNumberReadExactlyError
	"This covers parsing in Number>>readExactlyFrom:"

	| rs |
	
	rs := '' readStream.
	self should: [Number readFrom: rs] raise: Error.
	
	rs := 'foo' readStream.
	self should: [Number readFrom: rs] raise: Error.

	rs := 'radix' readStream.
	self shouldnt: [Number readFrom: rs] raise: Error.
	
	rs := '.e0' readStream.
	self should: [Number readFrom: rs] raise: Error.
	
	rs := '-.e0' readStream.
	self should: [Number readFrom: rs] raise: Error.
	
	rs := '--1' readStream.
	self should: [Number readFrom: rs] raise: Error.! !

!NumberParsingTest methodsFor: 'tests - Float' stamp: 'NorbertHartl 5/28/2008 09:45'!
testNumberReadOnlyDigit
	"This covers parsing in Number>>readFrom:"

	| rs num |
	rs := '1e' readStream.
	num := Number readFrom: rs.
	self assert: 1 = num.
	self assert: rs upToEnd = 'e'.
	
	rs := '1s' readStream.
	num := Number readFrom: rs.
	self assert: 1 = num.
	self assert: rs upToEnd = 's'.

	rs := '1.' readStream.
	num := Number readFrom: rs.
	self assert: 1 = num.
	self assert: num isInteger.
	self assert: rs upToEnd = '.'.! !

!NumberParsingTest methodsFor: 'tests - ScaledDecimal' stamp: 'nice 8/29/2008 22:04'!
testScaledDecimalWithTrailingZeroes
	"This is a non regression tests for http://bugs.squeak.org/view.php?id=7169"
	
	self assert: (Number readFrom: '0.50s2') = (1/2).
	self assert: (Number readFrom: '0.500s3') = (1/2).
	self assert: (Number readFrom: '0.050s3') = (1/20).! !

TestCase subclass: #ProcessSpecificTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!ProcessSpecificTest commentStamp: 'mvl 3/13/2007 13:52' prior: 0!
A ProcessSpecificTest is a test case for process local and dynamic variables.
!

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 14:13'!
checkDynamic: value
	self assert: TestDynamicVariable value = value! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 14:13'!
checkLocal: value
	self assert: TestLocalVariable value = value! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 15:02'!
testDynamicVariable

	| s1 s2 p1stopped p2stopped |
	s1 := Semaphore new.
	s2 := Semaphore new.
	p1stopped := p2stopped := false.
	[
		TestDynamicVariable value: 1 during:[
			self checkDynamic: 1.
			(Delay forMilliseconds: 30) wait.
			self checkDynamic: 1.
			TestDynamicVariable value: 3 during:[
				(Delay forMilliseconds: 30) wait.
				self checkDynamic: 3
			].
			self checkDynamic: 1.
		].
		self checkDynamic: nil.
		p1stopped := true.
		s1 signal.
	] fork.

	[
		TestDynamicVariable value: 2 during:[
			self checkDynamic: 2.
			(Delay forMilliseconds: 30) wait.
			self checkDynamic: 2.
		].
		self checkDynamic: nil.
		p2stopped := true.
		s2 signal.
	] fork.

	"Set a maximum wait timeout so that the test case will complete 
	 even if the processes fail to signal us."
	s1 waitTimeoutSeconds: 2.
	s2 waitTimeoutSeconds: 2.
	self assert: p1stopped.
	self assert: p2stopped.! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 15:03'!
testLocalVariable

	| s1 s2 p1stopped p2stopped |
	s1 := Semaphore new.
	s2 := Semaphore new.
	p1stopped := p2stopped := false.
	[
		self checkLocal: 0.
		TestLocalVariable value: 1.
		self checkLocal: 1.
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 1.
		TestLocalVariable value: 2.
		self checkLocal: 2.
		p1stopped := true.
		s1 signal.
	] fork.

	[
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 0.
		TestLocalVariable value: 3.
		self checkLocal: 3.
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 3.
		TestLocalVariable value: 4.
		self checkLocal: 4.
		p2stopped := true.
		s2 signal.
	] fork.

	"Set a maximum wait timeout so that the test case will complete 
	 even if the processes fail to signal us."
	s1 waitTimeoutMSecs: 5000.
	s2 waitTimeoutMSecs: 5000.
	self assert: p1stopped.
	self assert: p2stopped.
! !

TestCase subclass: #ProcessTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!ProcessTest commentStamp: 'GiovanniCorriga 8/30/2009 14:56' prior: 0!
A ProcessTest holds test cases for generic Process-related behaviour.!

!ProcessTest methodsFor: 'running' stamp: 'GiovanniCorriga 8/30/2009 15:53'!
tearDown
	Processor activeProcess environmentRemoveKey: #processTests ifAbsent: []! !

!ProcessTest methodsFor: 'testing' stamp: 'GiovanniCorriga 8/30/2009 15:58'!
testEnvironmentAt
	Processor activeProcess environmentAt: #processTests put: 42.
	self assert: (Processor activeProcess environmentAt: #processTests) = 42.
	self should: [Processor activeProcess environmentAt: #foobar] raise: Error! !

!ProcessTest methodsFor: 'testing' stamp: 'GiovanniCorriga 8/30/2009 15:53'!
testEnvironmentAtPut
	self assert: (Processor activeProcess environmentAt: #processTests put: 42) = 42.! !

!ProcessTest methodsFor: 'testing' stamp: 'GiovanniCorriga 8/30/2009 16:03'!
testEnvironmentRemoveKey
	Processor activeProcess environmentAt: #processTests put: 42.
	Processor activeProcess environmentRemoveKey: #processTests.
	self assert: (Processor activeProcess environmentAt: #processTests ifAbsent: []) isNil.
	self should: [Processor activeProcess environmentAt: #processTests] raise: Error! !

!ProcessTest methodsFor: 'testing' stamp: 'NouryBouraqadi 10/1/2009 07:45'!
testIsSelfEvaluating
	self assert: Processor printString = 'Processor'! !

TestCase subclass: #SelfEvaluatingObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Objects'!

!SelfEvaluatingObjectTest methodsFor: 'utilities' stamp: 'sd 7/31/2005 21:43'!
assertCode: code print: aString
	self assert: (self compile: code) printString = aString! !

!SelfEvaluatingObjectTest methodsFor: 'utilities' stamp: 'sd 7/31/2005 21:42'!
compile: aString
	^ Compiler evaluate: aString! !

!SelfEvaluatingObjectTest methodsFor: 'tests' stamp: 'TestRunner 11/23/2009 15:38'!
testArray
	"self run: #testArray"

	self assertCode: '#(1 2 3)' print: '#(1 2 3)'.
	self assertCode: '{1 . 2 . 3}' print: '#(1 2 3)'.
	self assertCode: '{1 + 0 . 2 . 3}' print: '#(1 2 3)'.
	self assertCode: '{1 + 0 . 1 @ 2 . 3}' print: '{1. (1@2). 3}'.
	self assertCode: '{2@3}' print: '{(2@3)}'.
	self assertCode: '{Object new}' print:  'an Array(an Object)'.
	self assertCode: '{Rectangle new . Object new}' print:  'an Array(nil corner: nil an Object)'.
	self assertCode: '{10@10 corner: 20@20 . 100@100 corner: 200@200}' print: '{(10@10 corner: 20@20). (100@100 corner: 200@200)}'! !

!SelfEvaluatingObjectTest methodsFor: 'tests' stamp: 'sd 7/31/2005 22:04'!
testObjects
	"self debug: #testObjects"

	self assert: 10 isSelfEvaluating.
	self assert: $a isSelfEvaluating.
	self assert: 3.14157 isSelfEvaluating.
	self assert: #(1 2 3) isSelfEvaluating.
	self assert: #abc isSelfEvaluating.
	self assert: 'abc' isSelfEvaluating.

	self assert: Object isSelfEvaluating.
	self assert: Object new isSelfEvaluating not.

	self assert: (Array with: 10) isSelfEvaluating.
	self assert: (Array with: Object new) isSelfEvaluating not.

	self assert: true isSelfEvaluating.
	self assert: false isSelfEvaluating.

	self assert: nil isSelfEvaluating.

	self assert: (1 to: 10) isSelfEvaluating.
	self assert: (1->2) isSelfEvaluating.
	self assert: Color red isSelfEvaluating.
	self assert: RunArray new isSelfEvaluating.! !

TestCase subclass: #TimespanDoSpanAYearTest
	instanceVariableNames: 'aTimespan aDuration aDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!TimespanDoSpanAYearTest commentStamp: 'tlk 1/6/2004 17:55' prior: 0!
I am one of several Sunit test Cases intentended to provide complete coverage for the Chronology set of classes as part of the external testing. See DateAndTimeEpochTestCase for a complete list. tlk.
My fixtures include a Timespan that crosses over a year boundary:
aDate = December 25, 2004, midnight
aDuration = 91 days
aTimeSpan= 91 days, starting December 25, 2004, midnight!

!TimespanDoSpanAYearTest methodsFor: 'running' stamp: 'brp 9/26/2004 18:59'!
setUp
	aDate := DateAndTime year: 2004 month: 12 day: 25 hour: 0 minute: 0 second: 0.
	aDuration := Duration days: 91 hours: 0 minutes: 0 seconds: 0 nanoSeconds: 0.

	aTimespan := Timespan starting: aDate duration: aDuration! !

!TimespanDoSpanAYearTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:55'!
testMonthsDo

	| monthArray |

	monthArray := Array
				with: (Month starting: (DateAndTime year: 2004 day: 355) duration: 31 days)
				with: (Month starting: (DateAndTime year: 2005 day: 1) duration: 31 days)
				with: (Month starting: (DateAndTime year: 2005 day: 32) duration: 29 days)
				with: (Month starting: (DateAndTime year: 2005 day: 61) duration: 31 days).
				
	self assert: aTimespan months = monthArray! !

!TimespanDoSpanAYearTest methodsFor: 'testing' stamp: 'brp 1/16/2004 13:55'!
testNext

	self assert: aTimespan next
			= (Timespan
					starting: (DateAndTime
							year: 2005
							month: 3
							day: 26
							hour: 0
							minute: 0
							second: 0)
					duration: aDuration)! !

!TimespanDoSpanAYearTest methodsFor: 'testing' stamp: 'brp 9/26/2004 19:06'!
testWeeksDo
	| weeks weekArray |
	weeks := aTimespan weeks.
	self assert: weeks size = ((aDuration days / 7.0) ceiling + 1).

	weekArray := OrderedCollection new.
	weekArray
		addLast: (Week starting: (DateAndTime year: 2004 month: 12 day: 19) duration: 7 days);
		addLast: (Week starting: (DateAndTime year: 2004 month: 12 day: 26) duration: 7 days).

	2 to: 79 by: 7 do:
		[ :i | weekArray
				addLast: (Week starting: (DateAndTime year: 2005 day: i) duration: 7 days) ].

	weekArray := weekArray asArray.
	self assert: aTimespan weeks = weekArray
! !

!TimespanDoSpanAYearTest methodsFor: 'testing' stamp: 'nk 3/30/2004 11:08'!
testYearsDo
	| yearArray |
	yearArray := Array
				with: (Year
						starting: (DateAndTime
								year: 2004
								month: 12
								day: 25)
						duration: 366 days).
	self assert: aTimespan years = yearArray
! !

TestCase subclass: #TimespanDoTest
	instanceVariableNames: 'aTimespan aDuration aDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!TimespanDoTest commentStamp: 'tlk 1/6/2004 17:55' prior: 0!
I am one of several Sunit test Cases intentended to provide complete coverage for the Chronology set of classes as part of the external testing. See DateAndTimeEpochTestCase for a complete list.  tlk.
My fixtures are:
aDate = January 8, 2003, midnight
aDuration = 91 days
aTimeSpan= 91 days, starting January 8, 2003, midnight
!

!TimespanDoTest methodsFor: 'running' stamp: 'tlk 1/5/2004 13:01'!
setUp
	aDate := DateAndTime
				year: 2003
				month: 01
				day: 07
				hour: 0
				minute: 0
				second: 0.
	aDuration := Duration
				days: 91
				hours: 0
				minutes: 0
				seconds: 0
				nanoSeconds: 0.
	aTimespan := Timespan starting: aDate duration: aDuration! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 18:00'!
testDatesDo
	| dateArray |
	dateArray := OrderedCollection new.
	7
		to: 97
		do: [:each | dateArray
				addLast: (Date year: 2003 day: each)].
	dateArray := dateArray asArray.
	self assert: aTimespan dates = dateArray! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 16:36'!
testDoWith
	| count |
	count := 0.
	aTimespan
		do: [:each | count := count + 1]
		with: (Timespan
				starting: aDate
				duration: 7 days).
	self assert: count = 13! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 16:39'!
testDoWithWhen
	| count |
	count := 0.
	aTimespan
		do: [:each | count := count + 1]
		with: (Timespan starting: aDate duration: 7 days)
		when: [:each | count < 5].
	self assert: count = 5	
! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 15:39'!
testEveryDo
	|count  duration |
	count := 0.
	duration := 7 days.
	(aTimespan
			every: duration
			do: [:each | count := count + 1]).
	self assert: count = 13
			! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 13:05'!
testMonthsDo
	| monthArray |
	monthArray := Array
				with: (Month
						starting: (DateAndTime year: 2003 day: 1)
						duration: 31 days)
				with: (Month
						starting: (DateAndTime year: 2003 day: 32)
						duration: 28 days)
				with: (Month
						starting: (DateAndTime year: 2003 day: 60)
						duration: 31 days)		
				with: (Month
						starting: (DateAndTime year: 2003 day: 91)
						duration: 30 days).
	self assert: aTimespan months = monthArray! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 16:09'!
testNext
	self assert: aTimespan next 
			= (Timespan
					starting: (DateAndTime
							year: 2003
							month: 4
							day: 8
							hour: 0
							minute: 0
							second: 0)
					duration: aDuration)! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 13:07'!
testWeeksDo
	| weekArray |
	weekArray := OrderedCollection new.
	7
		to: 98
		by: 7
		do: [:each | weekArray
				addLast: (Week
						starting: (DateAndTime year: 2003 day: each)
						duration: 7 days)].
	weekArray := weekArray asArray.
	self assert: aTimespan weeks = weekArray
! !

!TimespanDoTest methodsFor: 'testing' stamp: 'tlk 1/5/2004 13:09'!
testYearsDo
	| yearArray |
	yearArray := Array
				with: (Year
						starting: (DateAndTime year: 2003 day: 7)
						duration: 365 days).
	self assert: aTimespan years contents = yearArray contents! !

TestCase subclass: #YearMonthWeekTest
	instanceVariableNames: 'restoredStartDay restoredTimeZone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Chronology'!

!YearMonthWeekTest commentStamp: 'tlk 1/6/2004 17:55' prior: 0!
I am one of several Sunit test Cases intentended to provide complete coverage for the Chronology set of classes as part of the external testing. See DateAndEpochTestCase for a complete list.  tlk.
I have no fixtures but do make sure to restore anything I change.!

!YearMonthWeekTest methodsFor: 'running' stamp: 'brp 9/26/2004 19:26'!
setUp
	restoredStartDay := Week startDay.
	restoredTimeZone := DateAndTime localTimeZone.

	Week startDay: #Sunday.
	DateAndTime localTimeZone: (TimeZone timeZones detect: [:tz | tz abbreviation = 'GMT']).! !

!YearMonthWeekTest methodsFor: 'running' stamp: 'brp 9/26/2004 19:27'!
tearDown
	Week startDay: restoredStartDay.
	DateAndTime localTimeZone: restoredTimeZone.! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 12:51'!
testDaysInMonth
	self assert: (Month daysInMonth: 2 forYear: 2000) = 29.
	self assert: (Month daysInMonth: 2 forYear: 2001) = 28.
	self assert: (Month  daysInMonth: 2 forYear: 2004) = 29.
	self assert: (Month  daysInMonth: 2 forYear: 2100) = 28.
	
	self assert: (Month  daysInMonth: 'January' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'February' forYear: 2003) = 28.
	self assert: (Month  daysInMonth: 'March' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'April' forYear: 2003) = 30.
	self assert: (Month  daysInMonth: 'May' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'June' forYear: 2003) = 30.
	self assert: (Month  daysInMonth: 'July' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'August' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'September' forYear: 2003) = 30.
	self assert: (Month  daysInMonth: 'October' forYear: 2003) = 31.
	self assert: (Month  daysInMonth: 'November' forYear: 2003) = 30.
	self assert: (Month  daysInMonth: 'December' forYear: 2003) = 31.! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 12:37'!
testDaysInYear
	self assert: (Year daysInYear: 2000) = 366.
	self assert: (Year daysInYear: 2001) = 365.
	self assert: (Year daysInYear: 2004) = 366.
	self assert: (Year daysInYear: 2100) = 365.
	self assert: (Year daysInYear: 2003) = 365.! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 13:37'!
testIndexOfDay
	self assert: (Week indexOfDay: 'Friday') = 6.

! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 12:33'!
testIsLeapYear
	self assert: (Year isLeapYear: 2000).
	self deny: (Year isLeapYear: 2001).
	self assert: (Year isLeapYear: 2004).
	self deny: (Year isLeapYear: 2100).
	self deny: (Year isLeapYear: 2002).! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testMonthPrintOn
	| aMonth cs rw |
	aMonth := Month 
		starting: DateAndTime new
		duration: 31 days.
	cs := 'January 1901' readStream.
	rw := ReadWriteStream on: ''.
	aMonth printOn: rw.
	self assert: rw contents = cs contents! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'tlk 1/3/2004 13:34'!
testStartDay
	Week startDay: 'Wednesday'.
	self assert: Week startDay = 'Wednesday'.
	Week startDay: 'Thursday'.
	self assert: Week startDay = 'Thursday'.

! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'PeterHugossonMiller 9/3/2009 11:51'!
testWeekPrintOn
	| aWeek cs rw |
	aWeek := Week starting: (DateAndTime year: 1900 month: 12 day: 31).
	cs := 'a Week starting: 1900-12-30T00:00:00+00:00'.
	rw := String new writeStream.
	aWeek printOn: rw.
	self assert: rw contents = cs! !

!YearMonthWeekTest methodsFor: 'testing' stamp: 'damiencassou 5/30/2008 11:09'!
testYearPrintOn
	| aYear cs rw |
	aYear := Year 
		starting: DateAndTime new
		duration: 365 days.
	cs := 'a Year (1901)' readStream.
	rw := ReadWriteStream on: ''.
	aYear printOn: rw.
	self assert: rw contents = cs contents! !

DynamicVariable subclass: #TestDynamicVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Processes'!

!TestDynamicVariable commentStamp: 'mvl 3/13/2007 13:51' prior: 0!
TestDynamicVariable is a test class using in ProcessSpecificTest.

!

Object subclass: #CollectionCombinator
	instanceVariableNames: 'resultProcessingBlock collectionOfArrays buffer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'KernelTests-Methods'!

!CollectionCombinator commentStamp: '<historical>' prior: 0!
For a collection of collections, enumerate all elements of the cartesian product. The code shows how recursion is used to implement variable nesting of loops.
The cartesian product is usually a huge collection, that should not be kept in memory. Therefore the user of the class has to provide a block with one argument that is called each time a tuple is constructed. When possible, that block should not build a collection of all these tuples, but should immediately drop unsuitable tuples. 
To get a first impression, try this with 'inspect it':

     | result |
     result := OrderedCollection new.
    CollectionCombinator new
         forArrays:  (OrderedCollection with: #(#a #b #c)
                                             with: #(1 2 3 4 5)
                                             with: #('v' 'w' 'x' 'y' 'z')
                                             with: #('one' 'two' 'three')
                         )
         processWith: [:item |result addLast: item].
    result
         !

!CollectionCombinator methodsFor: 'as yet unclassified' stamp: 'BG 12/20/2001 21:33'!
combineFromIdx: myIdx

   "  this method is recursive. Recursion runs from values 1 to  collectionOfArrays size  of parameter myIdx. Each time it is called, this method has the responsiblity to provide all possible values for one index position of the result tuples. That index position is given by the value of  myIdx."

   (collectionOfArrays at: myIdx) do:
     [:item |
       buffer at: myIdx put: item.
       myIdx = collectionOfArrays size
         ifTrue: [resultProcessingBlock value: buffer shallowCopy]
         ifFalse: [self combineFromIdx: myIdx + 1]
    ].

  " The buffer is a shared object and its contents are later changed. It is therefore necessary to make a copy. "! !

!CollectionCombinator methodsFor: 'as yet unclassified' stamp: 'BG 12/20/2001 21:32'!
forArrays: anArray processWith: aBlock

 "  anArray is a kind of a sequenceable collection of arrays.
    aBlock is a block with one argument, that is used to process a  tuple immediately after it is constructed. "
  collectionOfArrays := anArray.
  resultProcessingBlock := aBlock.
  buffer := Array new: anArray size.
  self combineFromIdx: 1
   ! !
